/*
 * @(#)posix_sync_md.c	1.13 00/09/13
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#include "javavm/include/porting/sync.h"
#include "javavm/include/porting/int.h"
#include "javavm/include/porting/doubleword.h"
#include <pthread.h>
#include <sys/time.h>
#include <errno.h>
#include <assert.h>

void * __mutex_tracking = NULL;

CVMBool
POSIXmutexInit(POSIXMutex * m)
{
    return pthread_mutex_init(&m->m, NULL) == 0;
}

void
POSIXmutexDestroy(POSIXMutex * m)
{
  /*
  if (__mutex_tracking == m)
    printf("posixmutexdestroy: destroying %p\n",m);
  */
    pthread_mutex_destroy(&m->m);
}

CVMBool
POSIXmutexTryLock(POSIXMutex * m)
{
  //printf("jvm:trylock mutex(%p) pid(%d)\n",m,getpid());
    return pthread_mutex_trylock(&m->m) == 0;
}

void
POSIXmutexLock(POSIXMutex * m)
{
  //printf("jvm:lock mutex(%p) pid(%d)\n",m,getpid());  
  /*
  if (__mutex_tracking == m)
    printf("posixunlock: locking %p\n",m);
  */
    pthread_mutex_lock(&m->m);
}

void
POSIXmutexUnlock(POSIXMutex * m)
{
  //printf("jvm:unlock mutex(%p) pid(%d)\n",m,getpid());
  /*
  if (__mutex_tracking == m)
    printf("posixunlock: unlocking %p\n",m);
  */
    pthread_mutex_unlock(&m->m);
}

CVMBool
POSIXcondvarInit(POSIXCondVar * c, POSIXMutex * m)
{
    return pthread_cond_init(&c->c, NULL) == 0;
}

void
POSIXcondvarDestroy(POSIXCondVar * c)
{
    pthread_cond_destroy(&c->c);
}

int
POSIXcondvarWait(POSIXCondVar * c, POSIXMutex * m, CVMJavaLong millis)
{
    if (!CVMlongEqz(millis)) {
	struct timespec tm;
	struct timeval tv;
	CVMJavaLong ltmp, ltmp1, one_thousand;
	CVMJavaInt itmp;
 	gettimeofday(&tv, NULL);
	tm.tv_sec = tv.tv_sec;
	tm.tv_nsec = tv.tv_usec * 1000;
	one_thousand = CVMint2Long(1000);
	ltmp = CVMlongDiv(millis, one_thousand);
	itmp = CVMlong2Int(ltmp);
	ltmp1 = CVMint2Long(itmp);
	if (CVMlongEq(ltmp, ltmp1)) {
	    /* no overflow */
	    tm.tv_sec += itmp; /* millis / 1000 */
	    ltmp = CVMlongRem(millis, one_thousand);
	    itmp = CVMlong2Int(ltmp);
	    tm.tv_nsec += itmp * 1000000; /* (millis % 1000) * 1000000 */
	    if (tm.tv_nsec >= 1000000000) {
		++tm.tv_sec;
		tm.tv_nsec -= 1000000000;
	    }
	    assert(tm.tv_nsec < 1000000000);
	    if (tm.tv_sec >= tv.tv_sec) {
		/* no overflow */
		return pthread_cond_timedwait(&c->c, &m->m, &tm);
	    }
	}
    }
    /* timeout is 0 or overflow */
    return pthread_cond_wait(&c->c, &m->m);
}

void
POSIXcondvarNotify(POSIXCondVar * c)
{
    pthread_cond_signal(&c->c);
}

void
POSIXcondvarNotifyAll(POSIXCondVar * c)
{
    pthread_cond_broadcast(&c->c);
}
