#
#  Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
#
# 
#  @(#)defs_linux.mk	1.22 00/11/10
#
# defs for Linux target
#
CVM_TARGET      = linux
J2ME_CLASSLIB   = foundation

uname = $(shell uname)

ifeq ($(uname), SunOS)
# Cross compiling?
HOST_PLATFORM = solaris
endif

ifeq ($(uname), Linux)
HOST_PLATFORM = linux
endif

# use ksh so we can pass in -e like make does. sh doesn't
# support -e as a separate parameter and there's no way to get gnumake
# to pass -ec instead of just -c.
#SHELL=ksh -e

CPPFLAGS = $(CVM_DEFINES)
CPPFLAGS += -D_GNU_SOURCE

#
# platform specific command line flags
#
CPPFLAGS      += -DHAVE_64_BIT_IO

#
# Special TimeSys Linux/RT hack
# (For bits/pthreadtypes.h)
# - Pratik (23rd March 2001)
CPPFLAGS  += -DTIMESYS

#
# All the platform specific build flags we need to keep track of in 
# case they are toggled. Leave the CVM_ prefix off.
#
CVM_FLAGS += \

USE_SUNCC_CLEANUP_ACTION 	= $(DEFAULT_CLEANUP_ACTION)
HAVE_64_BIT_IO_CLEANUP_ACTION	= $(DEFAULT_CLEANUP_ACTION)
GPROF_CLEANUP_ACTION 		= $(DEFAULT_CLEANUP_ACTION)

CVM_BOOTDIR	   = $(JDK_HOME)
# /$(HOST_PLATFORM)
CVM_JAVABIN	   = $(CVM_BOOTDIR)/bin


FILESEP=/

ifneq ($(uname), Linux)
# Cross compiler
CC		= i386-redhat-linux-gcc
AR		= i386-redhat-linux-ar
else
# Native compiler
CC		= egcs
AR		= ar
endif
LIB_FLAGS       = -nostdlib -Wl,-r
AS	        = $(CC)
ZIP             = zip
LD              = $(CC)
LIB 		= $(CC)
AR_CREATE_FLAGS = rc
AR_UPDATE_FLAGS = ts
CVM    = tjvm
CVMLIB = tjvm.o

CVM_FDLIBM_SRCDIR = $(CVM_SHAREROOT)/native/java/lang/fdlibm/src

#
# Platform specific source directories
#
CVM_SRCDIRS   += $(CVM_TOP)/src/porting/ansi_c
CVM_SRCDIRS   += $(CVM_TOP)/src/porting/posix
CVM_SRCDIRS   += $(CVM_TOP)/src/porting/i386
CVM_INCLUDES  += -I$(CVM_TOP)/src
CVM_INCLUDES  += -I$(CVM_TOP)/..
CVM_TZDIR      = $(CVM_TOP)/src/linux/lib
CVM_TZDATAFILE = $(LIBDIR)/tzmappings

#
# rules for running tools
#
ASM_RULE = $(AT)$(AS) -traditional -D_ASM $(CPPFLAGS) -c -o $@ $<
CC_RULE   = $(AT)$(CC) $(CFLAGS) -o $@ $<
CC_RULE_FP = $(AT)$(CC) $(CFLAGS_FP) -o $@ $<
LINK_RULE = $(AT)$(LD)  $(LINKFLAGS) -g -o $@ $^ $(LINKLIBS)
LIB_RULE  = $(AT)$(LIB) $(LIB_FLAGS) -g -o $@ $^ 

#
#  Compiler flags
#

ifeq ($(CVM_OPTIMIZED), true)
# gcc optmized
CCFLAGS     = -O4 -Wall -Werror
CCFLAGS_FP  = -O0 -Wall 
# gcc debug
else
CCFLAGS     = -Wall
endif
CCDEPEND    = -MM

ifeq ($(CVM_SYMBOLS), true)
CCFLAGS += -g
endif

CPPFLAGS += $(CVM_INCLUDES)
CFLAGS   = -c $(CCFLAGS) $(CPPFLAGS)
CFLAGS_FP = -c $(CCFLAGS_FP) $(CPPFLAGS)

LINKFLAGS = -Wl,-export-dynamic
#LINKLIBS  = -lpthreadrt -lm -lnsl -ldl -lrk
LINKLIBS  = -lpthreadrt -lm -lnsl -ldl 

# If building a debug build with JVMDI, link against libjdwp_g to pick
# up debugging symbols automatically at program startup rather than
# when the DSO is opened. Note that this only works properly with dbx.
# NOTE: (kbr) taken out because dbx 5.0 supports preloading of shared
# libraries' symbols.
# ifeq ($(CVM_JVMDI), true)
# ifeq ($(CVM_DEBUG), true)
# LINKLIBS += -L/net/ceres/export4/kbr/jpda/build/solaris/lib/sparc -ljdwp_g
# endif
# endif

ifeq ($(GPROF), true)
LINKFLAGS += -pg
CFLAGS += -g -pg
endif


ifeq ($(CVM_DYNAMIC_LINKING), true)
	CVM_TARGETOBJS += linker_md.o
endif

#
# Include the right configuration and profile specific makefiles
#
include ../linux/defs_cdc_linux.mk
ifneq ($(CLASSLIB), cdc)
include ../linux/defs_$(CLASSLIB)_linux.mk
endif
