#
#  Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
#
#
#  @(#)defs.mk	1.240 00/11/21
#
# Common makefile defs
#

empty:=
space:= $(empty) $(empty)

# We need to check this here because the CVM_JVMDI option overrides many
# others that follows through CVM_DEBUG:
ifeq ($(CVM_JVMDI), true)
        override CVM_DEBUG = true
endif

# Set default options.
# NOTE: These options are officially supported and are documented in
#       docs/guide/build.html:

CVM_CLASSLOADING	= true
CVM_DEBUG               = false 
CVM_DEBUG_ASSERTS	= $(CVM_DEBUG)
CVM_DEBUG_CLASSINFO	= $(CVM_DEBUG)
CVM_DEBUG_DUMPSTACK	= $(CVM_DEBUG)
CVM_DEBUG_STACKTRACES	= true
CVM_JAVAC_DEBUG		= $(CVM_DEBUG)
CVM_JVMDI               = false
CVM_NO_LOSSY_OPCODES    = $(CVM_JVMDI)
CVM_PRELOAD_LIB         = false
CVM_SYMBOLS             = $(CVM_DEBUG)
CVM_TERSEOUTPUT         = true
ifeq ($(CVM_DEBUG), true)
CVM_OPTIMIZED		= false
else
CVM_OPTIMIZED		= true
endif

# NOTE: These options are not officially supported:

CVM_REFLECT		= true
CVM_SERIALIZATION	= true
CVM_DYNAMIC_LINKING	= true
CVM_TEST_GC             = false
CVM_TEST_GENERATION_GC  = false

#
# All build directories relative to CVM_BUILD_TOP
#
CVM_TOP       = ../..

ifeq ($(CVM_DEBUG), true)
CVM_BUILD_TOP = $(CVM_TOP)/build/$(CVM_TARGET)/debug
else
CVM_BUILD_TOP = $(CVM_TOP)/build/$(CVM_TARGET)/release
endif



LIBDIR = $(CVM_BUILD_TOP)/lib

# The name of this J2ME product
ifeq ($(J2ME_CLASSLIB), foundation)
CLASSLIB                = foundation
J2ME_PRODUCT_NAME       = Foundation
else
CLASSLIB                = foundation
J2ME_PRODUCT_NAME       = Foundation
endif

#
# The directory and jar files which the library classes are going to
# be put into.
#
LIB_CLASSESDIR          = $(CVM_BUILD_TOP)/$(CLASSLIB)_classes
LIB_CLASSESJAR          = $(LIBDIR)/$(CLASSLIB).jar

ifeq ($(TIMESYS_INFO), 1)
else
TIMESYS_VERSION_INFO=-custom
endif    

# Release information
J2ME_BUILD_VERSION	= 1_0
J2ME_BUILD_STATUS	= fcs
J2ME_BUILD_ID 		= ar

# To be used for the -version command line argument.
J2ME_BUILD_RELEASE = $(subst _,.,$(J2ME_BUILD_VERSION))
J2ME_BUILD_VERSION_STRING = $(J2ME_BUILD_RELEASE)$(J2ME_BUILD_STATUS)-$(J2ME_BUILD_ID)$(TIMESYS_VERSION_INFO)

CVM_DEFINES += \
	-DJ2ME_VERSION_STRING='"$(J2ME_BUILD_VERSION_STRING)"' \
	-DJ2ME_PRODUCT_NAME='"$(J2ME_PRODUCT_NAME)"' \
        -DJ2ME_CLASSESJAR='"../lib/$(CLASSLIB).jar"'

#
# command line flags
#
ifeq ($(CVM_DEBUG), true)
	CVM_DEFINES   += -DCVM_DEBUG
endif
ifeq ($(CVM_JAVAC_DEBUG), true)
	JAVAC_OPTIONS += -g
else
	JAVAC_OPTIONS += -g:none
endif
ifeq ($(CVM_DEBUG_CLASSINFO), true)
	CVM_DEFINES      += -DCVM_DEBUG_CLASSINFO
endif
ifeq ($(CVM_DEBUG_STACKTRACES), true)
	CVM_DEFINES      += -DCVM_DEBUG_STACKTRACES
endif
ifeq ($(CVM_DEBUG_DUMPSTACK), true)
	CVM_DEFINES      += -DCVM_DEBUG_DUMPSTACK
endif
ifeq ($(CVM_DEBUG_ASSERTS), true)
	CVM_DEFINES      += -DCVM_DEBUG_ASSERTS
endif
ifeq ($(CVM_CLASSLOADING), true)
	CVM_DEFINES      += -DCVM_CLASSLOADING
	CVM_DYNAMIC_LINKING = true
else
        override CVM_PRELOAD_LIB = true
endif
# If reflection is explicitly stated to be false by the user, don't
# allow serialization into the build to override that later
ifneq ($(CVM_REFLECT), true)
	override CVM_SERIALIZATION = false
endif
ifeq ($(CVM_SERIALIZATION), true)
	CVM_DEFINES      += -DCVM_SERIALIZATION
        override CVM_REFLECT = true
endif
ifeq ($(CVM_REFLECT), true)
	CVM_DEFINES      += -DCVM_REFLECT
endif
ifeq ($(CVM_JVMDI), true)
	CVM_DEFINES      += -DCVM_JVMDI
	CVM_DYNAMIC_LINKING = true
	override CVM_NO_LOSSY_OPCODES = true
endif
ifeq ($(CVM_NO_LOSSY_OPCODES), true)
	CVM_DEFINES      += -DCVM_NO_LOSSY_OPCODES
endif
ifeq ($(CVM_INSTRUCTION_COUNTING), true)
	CVM_DEFINES      += -DCVM_INSTRUCTION_COUNTING
endif
# make sure we check CVM_DYNAMIC_LINKING after checking CVM_JVMDI
# and CVM_CLASSLOADING.
ifeq ($(CVM_DYNAMIC_LINKING), true)
	CVM_DEFINES      += -DCVM_DYNAMIC_LINKING
endif
ifeq ($(CVM_TERSEOUTPUT), true)
	AT=@
else
	AT=
endif

ifeq ($(CVM_TEST_GC), true)
        CVM_DEFINES   += -DCVM_TEST_GC
endif

#
# All the build flags we need to keep track of in case they are toggled.
# Leave the CVM_ prefix off. 
#
# WARNING: None of the cleanup actions should try to delete flags
# or the generated directory or problems will occur.
#
CVM_FLAGS += \
	CVM_SYMBOLS \
	CVM_OPTIMIZED \
	CVM_DEBUG \
	CVM_JAVAC_DEBUG \
	CVM_DEBUG_CLASSINFO \
	CVM_DEBUG_STACKTRACES \
	CVM_DEBUG_DUMPSTACK \
	CVM_DEBUG_ASSERTS \
	CVM_CLASSLOADING \
	CVM_NO_LOSSY_OPCODES \
	CVM_INSTRUCTION_COUNTING \
	CVM_GCCHOICE \
	CVM_NO_CODE_COMPACTION \
	CVM_JVMDI \
	CVM_REFLECT \
	CVM_SERIALIZATION \
	CVM_PRELOAD_LIB \
	CVM_DYNAMIC_LINKING \
	CVM_TEST_GC \
	CVM_TEST_GENERATION_GC \
	J2ME_CLASSLIB

CVM_DEFAULT_CLEANUP_ACTION 	= \
	rm -rf $(CVM_OBJDIR)
CVM_JAVAC_DEBUG_CLEANUP_ACTION 	= \
	rm -rf .*classes \
	       .*.list \
	       $(CVM_BUILDTIME_CLASSESDIR) \
	       $(CVM_TEST_CLASSESDIR) \
	       *_classes \
	       $(CVM_BUILDTIME_CLASSESZIP) \
	       $(CVM_TEST_CLASSESZIP)
CVM_DEBUG_CLASSINFO_CLEANUP_ACTION  = \
	rm -rf $(CVM_OBJDIR) $(CVM_ROMJAVA_CPATTERN)*
CVM_NO_LOSSY_OPCODES_CLEANUP_ACTION = \
	rm -rf $(CVM_OBJDIR) $(CVM_ROMJAVA_CPATTERN)*
CVM_CLASSLOADING_CLEANUP_ACTION     = \
	rm -rf $(CVM_OBJDIR) $(CVM_ROMJAVA_CPATTERN)* \
	       $(CVM_BUILDTIME_CLASSESZIP) \
		.buildtimeclasses
CVM_INSTRUCTION_COUNTING_CLEANUP_ACTION = \
        rm -f $(CVM_OBJDIR)/opcodes.o $(CVM_OBJDIR)/executejava.o \
	     $(CVM_OBJDIR)/jni_impl.o
CVM_GCCHOICE_CLEANUP_ACTION 	= \
	mkdir -p $(CVM_DERIVEDROOT)/javavm/include/; \
	echo \\\#include \"javavm/include/gc/$(CVM_GCCHOICE)/gc_config.h\" \
	 	 > $(CVM_DERIVEDROOT)/javavm/include/gc_config.h
CVM_NO_CODE_COMPACTION_CLEANUP_ACTION = \
	rm -rf $(CVM_ROMJAVA_CPATTERN)*

CVM_REFLECT_CLEANUP_ACTION = \
	$(CVM_JAVAC_DEBUG_CLEANUP_ACTION) \
	$(CVM_OBJDIR)/jni_impl.o $(CVM_OBJDIR)/jvm.o $(CVM_OBJDIR)/reflect.o \
	$(CVM_ROMJAVA_CPATTERN)*

CVM_SERIALIZATION_CLEANUP_ACTION = \
	$(CVM_JAVAC_DEBUG_CLEANUP_ACTION) \
	$(CVM_OBJDIR)/jvm.o $(CVM_ROMJAVA_CPATTERN)*

CVM_PRELOAD_LIB_CLEANUP_ACTION = \
	rm -rf $(CVM_ROMJAVA_CPATTERN)* \
	$(CVM_BUILDTIME_CLASSESDIR) \
	$(CVM_BUILDTIME_CLASSESZIP) .buildtimeclasses 

CVM_DYNAMIC_LINKING_CLEANUP_ACTION = \
	rm -f $(CVM_OBJDIR)/jvm.o $(CVM_OBJDIR)/jni_impl.o \
		$(CVM_OBJDIR)/linker_md.o $(CVM_OBJDIR)/common_exceptions.o

CVM_DEBUG_CLEANUP_ACTION 		= $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_OPTIMIZED_CLEANUP_ACTION 		= $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_SYMBOLS_CLEANUP_ACTION 		= $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_DEBUG_STACKTRACES_CLEANUP_ACTION 	= $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_DEBUG_DUMPSTACK_CLEANUP_ACTION 	= $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_DEBUG_ASSERTS_CLEANUP_ACTION 	= $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_JVMDI_CLEANUP_ACTION		= $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_TEST_GC_CLEANUP_ACTION              = $(CVM_DEFAULT_CLEANUP_ACTION)
CVM_TEST_GENERATION_GC_CLEANUP_ACTION   = $(CVM_DEFAULT_CLEANUP_ACTION)
#
# Wipe out objects and classes when J2ME_CLASSLIB changes.
#
J2ME_CLASSLIB_CLEANUP_ACTION            = \
	$(CVM_JAVAC_DEBUG_CLEANUP_ACTION) ; \
	$(CVM_DEFAULT_CLEANUP_ACTION)

# generate header dependency files
GENERATEMAKEFILES = true

# use the generational gc by default. Other possible GC choices are:
#	semispace
# 	marksweep
#CVM_GCCHOICE   = generational
CVM_GCCHOICE = marksweep

ifeq ($(CVM_TEST_GC), true)
#        override CVM_GCCHOICE   = semispace
		override CVM_GCCHOICE = marksweep
endif

#
# There should be a GC-specific makefile.
# This special-case will do for now.
#
ifeq ($(CVM_GCCHOICE), generational)
    CVM_SHAREOBJS += \
        gen_semispace.o \
	gen_markcompact.o
endif

#
# Directories
#
# JDK_HOME         = /usr/local/jdk1.2.2
JDK_HOME	 = /usr/java/jdk1.3.1_01
# JDK_HOME       = /usr/local/java/jdk1.3

CVM_OBJDIR               = $(CVM_BUILD_TOP)/obj
CVM_BINDIR               = $(CVM_BUILD_TOP)/bin
CVM_DERIVEDROOT          = $(CVM_BUILD_TOP)/generated
CVM_BUILDTIME_CLASSESDIR = $(CVM_BUILD_TOP)/btclasses
CVM_TEST_CLASSESDIR      = $(CVM_BUILD_TOP)/testclasses

CVM_SHAREROOT  = $(CVM_TOP)/src/share
CVM_TARGETROOT = $(CVM_TOP)/src/$(CVM_TARGET)

CVM_JAVAC	   = $(CVM_JAVABIN)/javac
CVM_JAVAH	   = $(CVM_JAVABIN)/javah
CVM_JAVA	   = $(CVM_JAVABIN)/java
CVM_JAVADOC        = $(CVM_JAVABIN)/javadoc
CVM_JAR            = $(CVM_JAVABIN)/jar

INSTALLDIR	= $(CVM_TOP)/install

#
# Java source directories. 
#

CVM_TESTCLASSES_SRCDIR    = $(CVM_SHAREROOT)/javavm/test
CVM_CLDCCLASSES_SRCDIR    = $(CVM_SHAREROOT)/classes/cldc
CVM_SHAREDCLASSES_SRCDIR  = $(CVM_SHAREROOT)/classes
CVM_TARGETCLASSES_SRCDIR  = $(CVM_TARGETROOT)/classes

CVM_BUILDTIME_CLASSESZIP = btclasses.zip
CVM_TEST_CLASSESZIP      = testclasses.zip

ifeq ($(CVM_TEST_GC), true)
include ../share/testgc.mk
endif

ifeq ($(CVM_TEST_GENERATION_GC), true)
    CVM_SRCDIRS += \
	$(CVM_SHAREROOT)/javavm/test/GenerationGCTest/Csrc 
    CVM_INCLUDES  += \
	-I$(CVM_SHAREROOT)/javavm/test/GenerationGCTest/Include 
   CVM_SHAREOBJS += \
	 BarrierTest.o
endif

CVM_SRCDIRS    += \
	$(CVM_SHAREROOT)/javavm/runtime \
	$(CVM_TARGETROOT)/javavm/runtime \
	$(CVM_TARGETROOT)/bin \
	$(CVM_DERIVEDROOT)/javavm/runtime \
	$(CVM_SHAREROOT)/javavm/runtime/gc/$(CVM_GCCHOICE) \
	$(CVM_SHAREROOT)/javavm/native/sun/misc \
	$(CVM_SHAREROOT)/javavm/native/java/lang/reflect \
	$(CVM_SHAREROOT)/javavm/native/java/security \
	$(CVM_SHAREROOT)/native/common \
	$(CVM_SHAREROOT)/native/java/lang \
	$(CVM_SHAREROOT)/native/java/lang/ref \
	$(CVM_SHAREROOT)/native/java/lang/reflect \
	$(CVM_SHAREROOT)/native/java/lang/fdlibm/src \
	$(CVM_SHAREROOT)/native/java/util \
	$(CVM_SHAREROOT)/native/java/io \
	$(CVM_SHAREROOT)/native/java/net \
	$(CVM_SHAREROOT)/native/java/util/zip \
	$(CVM_SHAREROOT)/native/java/util/zip/zlib-1.1.3 \
	$(CVM_SHAREROOT)/native/sun/misc \
	$(CVM_TARGETROOT)/native/java/lang \
	$(CVM_TARGETROOT)/native/java/io \
	$(CVM_TARGETROOT)/native/java/net \
	$(CVM_TARGETROOT)/native/java/util \
	$(CVM_TARGETROOT)/native/com/timesys/CoreExtension \
	$(CVM_TARGETROOT)/native/javax/realtime \
	$(CVM_TARGETROOT)/native/com/timesys \

#
# Source bundle building stuff
#

#
# Stuff to exclude from a bundle
#
BUNDLE_EXCLUDE_PATTERNS +=			\
    *SCCS/*					\
    */build/share/bin/				\
    */build/share/bin/*				\
    */cldc/java/				\
    */cldc/java/*				\
    */cldc/java/util/				\
    */cldc/java/util/*				\
    */cldc/java/io/				\
    */cldc/java/io/*				\
    */cldc/com/					\
    */cldc/com/*				\
    */cldc/com/sun/				\
    */cldc/com/sun/*				\
    */cldc/com/sun/cldc/			\
    */cldc/com/sun/cldc/*			\
    */cldc/com/sun/cldc/i18n/			\
    */cldc/com/sun/cldc/i18n/*			\
    */cldc/com/sun/cldc/i18n/j2me/		\
    */cldc/com/sun/cldc/i18n/j2me/*		\
    */cldc/com/sun/cldc/i18n/j2se/		\
    */cldc/com/sun/cldc/i18n/j2se/*		\
    */cldc/com/sun/cldc/io/			\
    */cldc/com/sun/cldc/io/*			\
    */cldc/com/sun/cldc/io/j2me/		\
    */cldc/com/sun/cldc/io/j2me/*		\
    */cldc/com/sun/cldc/io/j2me/events/		\
    */cldc/com/sun/cldc/io/j2me/events/*	\
    */cldc/com/sun/cldc/io/j2me/datagram/	\
    */cldc/com/sun/cldc/io/j2me/datagram/*	\
    */cldc/com/sun/cldc/io/j2me/storage/	\
    */cldc/com/sun/cldc/io/j2me/storage/*	\
    */cldc/com/sun/cldc/io/j2me/serversocket/	\
    */cldc/com/sun/cldc/io/j2me/serversocket/*	\
    */cldc/com/sun/cldc/io/j2me/socket/		\
    */cldc/com/sun/cldc/io/j2me/socket/*	\
    */cldc/com/sun/cldc/io/j2me/debug/		\
    */cldc/com/sun/cldc/io/j2me/debug/*		\
    */cldc/com/sun/cldc/io/j2me/resource/	\
    */cldc/com/sun/cldc/io/j2me/resource/*	\
    */cldc/com/sun/cldc/io/j2me/translate/	\
    */cldc/com/sun/cldc/io/j2me/translate/*	\
    */cldc/com/sun/cldc/io/j2me/http/		\
    */cldc/com/sun/cldc/io/j2me/http/*		\
    */cldc/com/sun/cldc/io/j2se/		\
    */cldc/com/sun/cldc/io/j2se/*		\
    */cldc/com/sun/cldc/io/j2se/socket/		\
    */cldc/com/sun/cldc/io/j2se/socket/*	\
    */cldc/com/sun/cldc/io/j2se/datagram/	\
    */cldc/com/sun/cldc/io/j2se/datagram/*	\
    */cldc/com/sun/cldc/io/j2se/http/		\
    */cldc/com/sun/cldc/io/j2se/http/*		\
    */cldc/com/sun/cldc/io/j2se/storage/	\
    */cldc/com/sun/cldc/io/j2se/storage/*	\
    */cldc/com/sun/cldc/io/j2se/serversocket/	\
    */cldc/com/sun/cldc/io/j2se/serversocket/*	\
    */cldc/com/sun/cldc/io/j2se/translate/	\
    */cldc/com/sun/cldc/io/j2se/translate/*	\
    */cldc/com/sun/cldc/io/palm/		\
    */cldc/com/sun/cldc/io/palm/*		\
    */cldc/com/sun/cldc/io/palm/storage/	\
    */cldc/com/sun/cldc/io/palm/storage/*	\
    */cldc/com/sun/cldc/j2me/			\
    */cldc/com/sun/cldc/j2me/*			\
    */cldc/com/sun/cldc/util/			\
    */cldc/com/sun/cldc/util/*			\
    */cldc/com/sun/cldc/util/j2me/		\
    */cldc/com/sun/cldc/util/j2me/*

#
# All bundles include these directories
#
BUNDLE_INCLUDE_LIST += 				\
	cdcfoundation/doc				\
	cdcfoundation/ext/jpda/src/share		\
	cdcfoundation/ext/jpda/build/share	\
	cdcfoundation/jdk_build/share		\
	cdcfoundation/build/share					\
	cdcfoundation/src/porting			\
	cdcfoundation/src/share

#
# The VxWorks and linux bundles are default. 
# Add the additional stuff necessary
#
BUNDLE_INCLUDE_LIST +=						\
	cdcfoundation/ext/jpda/src/vxworks			\
	cdcfoundation/ext/jpda/build/vxworks			\
	cdcfoundation/jdk_build/vxworks				\
	cdcfoundation/src/vxworks					\
	cdcfoundation/build/vxworks/GNUmakefile			\
	cdcfoundation/build/vxworks/defs_cdc_vxworks.mk		\
	cdcfoundation/build/vxworks/defs_foundation_vxworks.mk	\
	cdcfoundation/build/vxworks/defs_vxworks.mk		\
	cdcfoundation/build/vxworks/rules_vxworks.mk		\
	cdcfoundation/build/vxworks/config_vxworks.mk		\
	cdcfoundation/build/vxworks/target

#
# Additional stuff necessary for Linux source bundle
#
BUNDLE_INCLUDE_LIST += 						\
	cdcfoundation/ext/jpda/src/linux				\
	cdcfoundation/ext/jpda/build/linux			\
	cdcfoundation/jdk_build/linux				\
	cdcfoundation/src/linux					\
	cdcfoundation/build/linux/GNUmakefile			\
	cdcfoundation/build/linux/defs_cdc_linux.mk		\
	cdcfoundation/build/linux/defs_foundation_linux.mk	\
	cdcfoundation/build/linux/defs_linux.mk			\
	cdcfoundation/build/linux/rules_linux.mk

#
# Additional stuff to exclude when building the vxWorks bundle
#
BUNDLE_EXCLUDE_PATTERNS += 					\
	cdcfoundation/build/vxworks/target/PC/default/*		\
	cdcfoundation/build/vxworks/target/PC/*.lst		\
	cdcfoundation/build/vxworks/target/PC-boot/*.o		\
	cdcfoundation/build/vxworks/target/PC-boot/vxWorks*	\
	cdcfoundation/build/vxworks/target/PC-boot/ctdt.*		\
	cdcfoundation/build/vxworks/target/PC-boot/depend.*

#
# some build directories that need to be created.
#
CVM_BUILDDIRS  += \
	$(CVM_OBJDIR) \
	$(CVM_BINDIR) \
	$(CVM_DERIVEDROOT)/javavm/runtime \
	$(CVM_DERIVEDROOT)/javavm/include \
	$(CVM_DERIVEDROOT)/jni \
	$(CVM_DERIVEDROOT)/cni \
	$(CVM_DERIVEDROOT)/offsets \
	$(CVM_DERIVEDROOT)/flags \
	$(CVM_BUILDTIME_CLASSESDIR) \
	$(CVM_TEST_CLASSESDIR) \
	$(LIBDIR) \
	$(LIB_CLASSESDIR)

#
# C include directories
#
CVM_INCLUDES   += \
	-I$(CVM_SHAREROOT) \
	-I$(CVM_TARGETROOT) \
	-I$(CVM_BUILD_TOP) \
	-I$(CVM_SHAREROOT)/com/timesys/CoreExtension \

#
# These are for the convenience of external code like
# JDK native method libraries that like to #include
# "jni.h" and "java_lang_String.h", etc.  We should
# only need these for those .c files, but gnumake
# doesn't support target-specific macros.
#
CVM_INCLUDES  += \
	-I$(CVM_SHAREROOT)/javavm/export \
	-I$(CVM_SHAREROOT)/native/common \
	-I$(CVM_SHAREROOT)/native/java/lang \
	-I$(CVM_SHAREROOT)/native/java/lang/fdlibm/include \
	-I$(CVM_SHAREROOT)/native/java/net \
	-I$(CVM_SHAREROOT)/native/java/io \
	-I$(CVM_SHAREROOT)/native/java/util/zip/zlib-1.1.3 \
	-I$(CVM_TARGETROOT)/native/java/net \
	-I$(CVM_TARGETROOT)/native/java/util \
	-I$(CVM_DERIVEDROOT)/jni \
	-I$(CVM_SHAREROOT)/native/javax/realtime \
	-I$(CVM_TARGETROOT)/native/javax/realtime \

#
# Classes to build
#
CVM_TEST_CLASSES += \
	ClassLoaderTest \
	ClassLink \
	ClassisSubclassOf \
	HelloWorld \
	EllisGC_ST \
	ExceptionTest \
	StaticFieldTest \
	MTGC \
	FastSync \
	InterruptTest \
	ThreadsAndSync \
	ThreadSuspend \
	DaemonThreadTest \
	Test \
	ManyFieldsAndMethods \
	ExceptionThrowingOpcodesTest \
	UncaughtExceptionTest \
	StringSignatureTest \
	InterfaceTest \
	DebuggerTest \
	setTime \
	TimeZoneTest \
	CVMadddec 

#
# Add JavaCodeCompact in only for classloading builds. Otherwise the
# ROMization of the test classes takes too long.
#
ifeq ($(CVM_CLASSLOADING), true)
	CVM_TEST_CLASSES += JavaCodeCompact
endif

# Note: this test removed now that it is known to work. If you want to
# add it back in you must change the CVM_DYNAMIC_LINKING_CLEANUP_ACTION
# above.
# ifeq ($(CVM_DYNAMIC_LINKING), true)
# 	CVM_TEST_CLASSES += DynLinkTest
# endif

ifeq ($(CVM_REFLECT), true)
	CVM_TEST_CLASSES += \
		ReflectionTest \
		ReflectionTestSamePackageHelper \
		cvmtest.ReflectionTestOtherPackageHelper \
		ReflectionClinitTest \
		ReflectionSecurity \
		ReflectionStackOverflowTest
endif

#
# Classes with CNI native methods
#
CVM_CNI_CLASSES += sun.misc.CVM \
		   java.security.AccessController \
		   java.lang.reflect.Constructor \
		   java.lang.reflect.Field \
		   java.lang.reflect.Method

#
# Classes that the VM needs to have field offsets for.
#
CVM_OFFSETS_CLASSES += \
	java.lang.String \
	java.lang.Throwable \
	java.lang.Class \
	java.lang.Thread \
	java.lang.Boolean \
	java.lang.Byte \
	java.lang.Character \
	java.lang.Short \
	java.lang.Integer \
	java.lang.Long \
	java.lang.Float \
	java.lang.Double \
	java.lang.ref.Reference

ifeq ($(CVM_CLASSLOADING), true)
CVM_OFFSETS_CLASSES += \
	java.lang.ClassLoader 
endif

ifeq ($(CVM_REFLECT), true)
	CVM_OFFSETS_CLASSES += \
		java.lang.reflect.AccessibleObject \
		java.lang.reflect.Constructor \
		java.lang.reflect.Field \
		java.lang.reflect.InvocationTargetException \
		java.lang.reflect.Method
endif

#
# The objects that make up zlib-1.1.3
#
ZLIBOBJS = \
	CRC32.o \
	ZipFile.o \
	ZipEntry.o \
	zadler32.o \
	zcrc32.o \
	deflate.o \
	trees.o \
	zutil.o \
	inflate.o \
	infblock.o \
	inftrees.o \
	infcodes.o \
	infutil.o \
	inffast.o \
	zip_util.o

#
# The objects that make up the 'fdlibm' math library
#
MATHOBJS = \
	e_acos.o \
	e_acosh.o \
	e_asin.o \
	e_atan2.o \
	e_atanh.o \
	e_cosh.o \
	e_exp.o \
	e_fmod.o \
	e_gamma.o \
	e_gamma_r.o \
	e_hypot.o \
	e_j0.o \
	e_j1.o \
	e_jn.o \
	e_lgamma.o \
	e_lgamma_r.o \
	e_log.o \
	e_log10.o \
	e_pow.o \
	e_rem_pio2.o \
	e_remainder.o \
	e_scalb.o \
	e_sinh.o \
	e_sqrt.o \
	k_cos.o \
	k_rem_pio2.o \
	k_sin.o \
	k_standard.o \
	k_tan.o \
	s_asinh.o \
	s_atan.o \
	s_cbrt.o \
	s_ceil.o \
	s_copysign.o \
	s_cos.o \
	s_erf.o \
	s_expm1.o \
	s_fabs.o \
	s_finite.o \
	s_floor.o \
	s_frexp.o \
	s_ilogb.o \
	s_isnan.o \
	s_ldexp.o \
	s_lib_version.o \
	s_log1p.o \
	s_logb.o \
	s_matherr.o \
	s_modf.o \
	s_nextafter.o \
	s_rint.o \
	s_scalbn.o \
	s_signgam.o \
	s_significand.o \
	s_sin.o \
	s_tan.o \
	s_tanh.o \
	w_acos.o \
	w_acosh.o \
	w_asin.o \
	w_atan2.o \
	w_atanh.o \
	w_cosh.o \
	w_exp.o \
	w_fmod.o \
	w_gamma.o \
	w_gamma_r.o \
	w_hypot.o \
	w_j0.o \
	w_j1.o \
	w_jn.o \
	w_lgamma.o \
	w_lgamma_r.o \
	w_log.o \
	w_log10.o \
	w_pow.o \
	w_remainder.o \
	w_scalb.o \
	w_sinh.o \
	w_sqrt.o

#
# Objects to build
#
ifeq ($(CVM_CLASSLOADING), true)
CVM_SHAREOBJS += \
	classlink.o \
	classverify.o \
	constantpool.o \
	mangle.o \
	quicken.o \
	verifycode.o
endif

CVM_SHAREOBJS += \
	bag.o \
	basictypes.o \
	bcattr.o \
	classinitialize.o \
	classcreate.o \
	classload.o \
	classlookup.o \
	classtable.o \
	classes.o \
	common_exceptions.o \
	cstates.o \
	executejava.o \
	fixedspace.o \
	gc_common.o \
	gc_impl.o \
	globals.o \
	globalroots.o \
	indirectmem.o \
	interpreter.o \
	jni_impl.o \
	jni_util.o \
	jvm.o \
	jvmdi.o \
	jvmdi_jni.o \
	jvmpi.o \
	loadercache.o \
	localroots.o \
	named_sys_monitor.o \
	objsync.o \
	opcodelen.o \
	opcodes.o \
	packages.o \
	preloader.o \
	reflect.o \
	stackmaps.o \
	stacks.o \
	stackwalk.o \
	string.o \
	sync.o \
	typeid.o \
	utils.o \
        porting_debug.o \
	verifyformat.o \
	weakrefs.o \
	xrun.o \
	\
	Object.o \
	Class.o \
	ClassLoader.o \
	CVM.o \
	DatagramPacket.o \
	Finalizer.o \
	Float.o \
	GC.o \
	Double.o \
	Package.o \
	Runtime.o \
	Shutdown.o \
	System.o \
	SecurityManager.o \
	TimeZone.o \
	Thread.o \
	Throwable.o \
	StrictMath.o \
	Array.o \
	Field.o \
	Method.o \
	Proxy.o \
	Constructor.o \
	FileDescriptor.o \
	FileInputStream.o \
	FileOutputStream.o \
	ObjectInputStream.o \
	ObjectStreamClass.o \
	ObjectOutputStream.o \
	InetAddress.o \
	AccessController.o \
	ResourceBundle.o \
	CRC32.o \
	ZipFile.o \
	ZipEntry.o \
	String.o \
	Inflater.o \
	jvmcei.o \
	RTThreadAdaptor.o \
	RTMonitorAdapter.o \
	RawMemoryAccess.o \
	HeapMemoryAdapter.o \
	MemoryArea.o \
	ImmortalMemory.o \
	ScopedMemory.o \
	mem_registry.o \
	MarkAndSweepCollector.o \
	AsynchronouslyInterruptedException.o \
	RealtimeSystem.o \
    SizeEstimator.o \
    HighResolutionTime.o \
    PhysicalMemoryManager.o


# Note: this test removed now that it is known to work. If you want to
# add it back in you must change the CVM_DYNAMIC_LINKING_CLEANUP_ACTION
# above.
# ifeq ($(CVM_DYNAMIC_LINKING), true)
# 	# Pick up the native code for the dynamic linking test
# 	CVM_SHAREOBJS += DynLinkTest.o
# endif

CVM_SHAREOBJS += $(ZLIBOBJS)
CVM_SHAREOBJS += $(MATHOBJS)

#
# CDC is the basis for all profiles. Include unconditionally.
#
include ../share/defs_cdc.mk

#
# And the profile definitions
#
ifneq ($(CLASSLIB), cdc)
include ../share/defs_$(CLASSLIB).mk
endif

#
# After including the profile defs files, our object lists are complete.
# Create the full path object list.
#
CVM_OBJECTS0 = $(CVM_SHAREOBJS) $(CVM_TARGETOBJS)

CVM_OBJECTS  = $(patsubst %.o,$(CVM_OBJDIR)/%.o,$(CVM_OBJECTS0))
