#
#  Copyright 1998-2000 Sun Microsystems, Inc. All Rights Reserved.
#
#
#  @(#)rules.mk	1.72 00/11/03
#
#  Common makefile rules
#

# speed up makefile by not using implicit suffixes
.SUFFIXES:

all: $(CLASSLIB)

$(INSTALLDIR)/cdcfoundation:
	@mkdir -p $@ ;
	@ln -ns $(CVM_TOP)/* $@;

#########################################################
# If CVM_PRELOAD_LIB is set, classes are listed in CDC
# (and a profile) will be added into CVM_BUILDTIME_CLASSES.
#########################################################
ifeq ($(CVM_PRELOAD_LIB), true)
    CVM_BUILDTIME_CLASSES += $(CLASSLIB_CLASSES)
endif

#
# Compiling .java files.
#

#
# Classes for the class libraries (CDC runtime + profile)
#
LIBCLASSES_SRCDIRS = $(PROFILE_SRCDIR) \
    $(CVM_SHAREDCLASSES_SRCDIR) $(CVM_TARGETCLASSES_SRCDIR) \
    $(CVM_CLDCCLASSES_SRCDIR)
$(LIB_CLASSESDIR)/%.class: $(PROFILE_SRCDIR)/%.java
#	@echo "../../linux/classes/javax/realtime/EDFScheduler.java" >>.libclasses.list
	@echo $? >>.libclasses.list
	@touch .libclasses
$(LIB_CLASSESDIR)/%.class: $(CVM_TARGETCLASSES_SRCDIR)/%.java
	@echo $? >>.libclasses.list
	@touch .libclasses
$(LIB_CLASSESDIR)/%.class: $(CVM_SHAREDCLASSES_SRCDIR)/%.java
	@echo $? >>.libclasses.list
	@touch .libclasses
$(LIB_CLASSESDIR)/%.class: $(CVM_CLDCCLASSES_SRCDIR)/%.java
	@echo $? >>.libclasses.list
	@touch .libclasses

#
# Build-time classes
#
# Allow overriding of build-time classes with foundation classes
#
BTCLASSES_SRCDIRS += $(PROFILE_SRCDIR) \
	    $(CVM_SHAREDCLASSES_SRCDIR) $(CVM_TARGETCLASSES_SRCDIR)
$(CVM_BUILDTIME_CLASSESDIR)/%.class: $(PROFILE_SRCDIR)/%.java
	@echo $? >>.btclasses.list
	@touch .btclasses
$(CVM_BUILDTIME_CLASSESDIR)/%.class: $(CVM_SHAREDCLASSES_SRCDIR)/%.java
	@echo $? >>.btclasses.list
	@touch .btclasses
$(CVM_BUILDTIME_CLASSESDIR)/%.class: $(CVM_TARGETCLASSES_SRCDIR)/%.java
	@echo $? >>.btclasses.list
	@touch .btclasses

ifeq ($(CVM_PRELOAD_LIB), true)
$(CVM_BUILDTIME_CLASSESDIR)/%.class: $(CVM_CLDCCLASSES_SRCDIR)/%.java
	@echo $? >>.btclasses.list
	@touch .btclasses
endif

#
# Test classes
#
TESTCLASSES_SRCDIRS = \
    $(CVM_TESTCLASSES_SRCDIR)
$(CVM_TEST_CLASSESDIR)/%.class: $(CVM_TESTCLASSES_SRCDIR)/%.java
	@echo $? >>.testclasses.list
	@touch .testclasses

#
# This is the javac command to be used in the compilation of all classes
#
JAVAC_CMD = $(CVM_JAVAC) $(JAVAC_OPTIONS) -J-Xms32m -J-Xmx64m

#
# Convert the class lists to names of class files
# so they can be javac'ed.
#
# First convert . to /, then prepend the classes directory and append .class
#
CLASSLIB_CLASS0 = $(subst .,/,$(CLASSLIB_CLASSES))
CLASSLIB_CLASS_FILES = \
	$(patsubst %,$(LIB_CLASSESDIR)/%.class,$(CLASSLIB_CLASS0))

BUILDTIME_CLASS0 = $(subst .,/,$(CVM_BUILDTIME_CLASSES))
BUILDTIME_CLASS_FILES = \
	$(patsubst %,$(CVM_BUILDTIME_CLASSESDIR)/%.class,$(BUILDTIME_CLASS0))

TEST_CLASS0 = $(subst .,/,$(CVM_TEST_CLASSES))
TEST_CLASS_FILES = \
	$(patsubst %,$(CVM_TEST_CLASSESDIR)/%.class,$(TEST_CLASS0))

# Convert list of Java source directories to colon-separated paths
LIBCLASSES_SRCPATH0 = $(subst $(space),:,$(strip $(LIBCLASSES_SRCDIRS)))
LIBCLASSES_SRCPATH  = $(subst $(space),:,$(strip $(LIBCLASSES_SRCPATH0)))

BTCLASSES_SRCPATH0 = $(subst $(space),:,$(strip $(BTCLASSES_SRCDIRS)))
BTCLASSES_SRCPATH  = $(subst $(space),:,$(strip $(BTCLASSES_SRCPATH0)))

TESTCLASSES_SRCPATH0 = $(subst $(space),:,$(strip $(TESTCLASSES_SRCDIRS)))
TESTCLASSES_SRCPATH  = $(subst $(space),:,$(strip $(TESTCLASSES_SRCPATH0)))

.delete.libclasses.list:
	@$(RM) .libclasses.list

.delete.btclasses.list:
	@$(RM) .btclasses.list

.delete.testclasses.list:
	@$(RM) .testclasses.list

.report.libclasses.list:
	@echo "Checking for $(CLASSLIB) classes to compile ..."

.report.btclasses.list:
	@echo "Checking for build-time classes to compile ..."

.report.testclasses.list:
	@echo "Checking for test classes to compile ..."

.compile.libclasses:
	$(AT)if [ -s .libclasses.list ] ; then				\
		echo "Compiling $(CLASSLIB) classes...";		\
		$(JAVAC_CMD)						\
			-d $(LIB_CLASSESDIR) 				\
			-bootclasspath $(CVM_BUILDTIME_CLASSESDIR) 	\
			-classpath $(LIB_CLASSESDIR) 			\
			-sourcepath $(LIBCLASSES_SRCPATH)		\
			`cat .libclasses.list` ;			\
	fi

.compile.btclasses:
	$(AT)if [ -s .btclasses.list ] ; then				\
		echo "Compiling build-time classes...";			\
		$(JAVAC_CMD)						\
			-d $(CVM_BUILDTIME_CLASSESDIR)			\
			-bootclasspath $(CVM_BUILDTIME_CLASSESDIR) 	\
			-classpath $(CVM_BUILDTIME_CLASSESDIR)		\
			-sourcepath $(BTCLASSES_SRCPATH)		\
			`cat .btclasses.list` ;				\
	fi

.compile.testclasses:
	$(AT)if [ -s .testclasses.list ] ; then				\
		echo "Compiling test classes...";			\
		$(JAVAC_CMD)						\
			-d $(CVM_TEST_CLASSESDIR)			\
			-bootclasspath 					\
			   $(CVM_BUILDTIME_CLASSESDIR):$(LIB_CLASSESDIR)\
			-classpath $(CVM_TEST_CLASSESDIR)		\
			-sourcepath $(TESTCLASSES_SRCPATH)		\
			`cat .testclasses.list` ;			\
	fi

#
# The rules to compile our three kinds of classes: 
#     library (CDC or CDC+profile)
#     build-time classes (to be pre-loaded by JavaCodeCompact)
#     test classes
# 
$(CLASSLIB)classes: .delete.libclasses.list .report.libclasses.list $(CLASSLIB_CLASS_FILES) .compile.libclasses

testclasses: .delete.testclasses.list .report.testclasses.list $(TEST_CLASS_FILES) .compile.testclasses

btclasses: .delete.btclasses.list .report.btclasses.list $(BUILDTIME_CLASS_FILES) .compile.btclasses

#
# 1) Initialize build. 
# 2) Build the build-time classes
# 3) Build the library classes using the build-time ones in the bootclasspath
# 4) Build the VM (which includes native method object files)
#
$(CLASSLIB): initbuild btclasses $(CVM_BUILDTIME_CLASSESZIP) $(CLASSLIB)classes $(LIB_CLASSESJAR) testclasses $(CVM_BINDIR)/$(CVM) $(CVM_TZDATAFILE)

include ../share/jcc.mk

#####################################
# include all of the dependency files
#####################################

files := $(foreach file, $(wildcard $(CVM_OBJDIR)/*.d), $(file))
ifneq ($(strip $(files)),)
    include $(files)
endif


vpath %.c      $(CVM_SRCDIRS)
vpath %.S      $(CVM_SRCDIRS)
vpath %.java   $(CVM_SRCJAVA_PATH) $(CVM_SRCTESTJAVA_PATH)

# make empty.mk depend on CVM_SRCDIRS
# This will cause make re-start if it needs to create
# any dynamically created files or directories, so that the above 
# vpath directive will work
$(CVM_DERIVEDROOT)/empty.mk:  $(CVM_SRCDIRS)
	touch $(CVM_DERIVEDROOT)/empty.mk
include $(CVM_DERIVEDROOT)/empty.mk


#
# See if any build flags have toggled since the last build. If so, then
# delete anything that might be dependent on the build flags. 
#
checkflags:: $(CVM_DERIVEDROOT) $(CVM_FLAGS)


# Make sure all of the build flags files are up to date. If not
# then do the requested cleanup action.
$(CVM_FLAGS):: $(CVM_DERIVEDROOT)/flags
	@if [ ! -f $(CVM_DERIVEDROOT)/flags/$@.$($@) ]; then \
		echo "Flag $@ changed. Cleaning up."; \
		rm -f $(CVM_DERIVEDROOT)/flags/$@.*; \
		touch $(CVM_DERIVEDROOT)/flags/$@.$($@); \
		$($@_CLEANUP_ACTION); \
	fi
#
# create directories
#
$(CVM_BUILDDIRS):
	@echo ... mkdir $@
	@if [ ! -d $@ ]; then mkdir -p $@; fi

#
# Make sure opcodes.h gets generated before executejava.o gets built
#
$(CVM_OBJDIR)/executejava.o: $(CVM_DERIVEDROOT)/javavm/include/opcodes.h

$(CVM_OBJDIR)/%.o: %.c
	@echo "... $@"
	$(CC_RULE)
ifeq ($(GENERATEMAKEFILES), true)
	@$(CC) $(CCDEPEND) $(CPPFLAGS) $< 2> /dev/null | \
	    sed 's!$*\.o!$(dir $@)&!g' > $(@:.o=.d)
endif

$(CVM_OBJDIR)/%.o: %.S
	@echo "... $@"
	$(ASM_RULE)

# object files that are useful for getting size information
sizelibs : $(CVM_OBJDIR)/$(CVMLIB)
$(CVM_OBJDIR)/$(CVMLIB) : $(CVM_OBJECTS)
	@echo "Linking $@"
	$(LIB_RULE)

#
# Initialize the environment for the build process: 
#
initbuild: checkflags $(CVM_BUILDDIRS)

$(CVM_BINDIR)/$(CVM) :: $(CVM_ROMJAVA_LIST) # force needed headers to build 1st

ifeq ($(CVM_CLASSLOADING), true)
$(CVM_BINDIR)/$(CVM) :: $(CVM_TEST_CLASSESZIP)
$(CVM_BINDIR)/$(CVM) :: headers
endif

$(CVM_BINDIR)/$(CVM) :: $(CVM_OBJECTS) $(CVM_OBJDIR)/$(CVM_ROMJAVA_O)
	@echo "Linking $@"
	$(LINK_RULE)

clean:: 
	rm -rf $(INSTALLDIR)
	rm -rf .libclasses .btclasses .testclasses
	rm -rf .*.list
	rm -rf $(CVM_BUILDTIME_CLASSESDIR) \
	       $(CVM_TEST_CLASSESDIR) *_classes
	rm -rf $(CVM_BUILDTIME_CLASSESZIP) \
	       $(CVM_TEST_CLASSESZIP)
	rm -rf $(LIBDIR)
	rm -rf $(CVM_BUILDDIRS) $(CVM_BINDIR)/$(CVM)
	rm -rf $(CVM_JCC_CLASSPATH)
	rm -rf $(CVM_DERIVEDROOT)

#
# Zip up everthing in the classes and testclasses directories.
# Use -x if you want to omit any directories
#
$(CVM_BUILDTIME_CLASSESZIP): .btclasses
	@echo ... $@
	(cd $(CVM_BUILDTIME_CLASSESDIR); $(ZIP) -r -0 -q - * ) \
		> .classes.tmp
	@mv -f .classes.tmp $@
	@cp -f $(CVM_BUILDTIME_CLASSESZIP) $(CVM_BUILD_TOP)/$(CVM_BUILDTIME_CLASSESZIP)

$(CVM_TEST_CLASSESZIP): .testclasses
	@echo ... $@
	(cd $(CVM_TEST_CLASSESDIR); $(ZIP) -r -0 -q - * ) \
		> .classes.tmp
	@mv -f .classes.tmp $@
	@cp -f $(CVM_TEST_CLASSESZIP) $(CVM_BUILD_TOP)/$(CVM_TEST_CLASSESZIP)

$(LIB_CLASSESJAR): .libclasses
	@echo ... $@
	(cd $(LIB_CLASSESDIR); $(CVM_JAR) cf ../../$@ *; )

#
# If this is a classloading build, then we need to use javah to create
# the jni headers for the runtime classes since JCC won't have seen these.
#
# NOTE: we could probably fix JCC so that we can generate
# offset and CNI header files separately too.
#
ifeq ($(CVM_CLASSLOADING), true)
headers: $(CVM_DERIVEDROOT)/jni/.time.stamp
$(CVM_DERIVEDROOT)/jni/.time.stamp : $(LIB_CLASSESJAR)
	@echo ... generating jni class headers
	@$(CVM_JAVAH) -jni \
		-d $(CVM_DERIVEDROOT)/jni \
		-classpath $(LIB_CLASSESJAR) \
		-bootclasspath $(CVM_BUILDTIME_CLASSESZIP) \
		$(CLASSLIB_CLASSES)
	@touch $@
endif

################################################
# Rules for building source bundles
################################################
lib-src: src.zip
src.zip:: $(INSTALLDIR)/cdcfoundation
	@echo ">>>Making "$@" ..." ;
	(cd $(INSTALLDIR); \
	 $(ZIP) -r -q - $(BUNDLE_INCLUDE_LIST) \
		 -x $(BUNDLE_EXCLUDE_PATTERNS)) > $(INSTALLDIR)/src.zip;
	@rm -rf $<;
	@echo "<<<Finished "$@" ..." ;

#
# Rules for building documentation
#
update_docs:: lib-src
	@echo ">>>Updating "$@" ..." ;
	(cd $(INSTALLDIR); $(ZIP) -r -u -q install/src.zip doc -x $(EXCLUDE_PATTERNS) ) ;
	@echo "<<<Finished "$@" ..." ;

$(INSTALLDIR)/javadoc:
	@echo ... mkdir $@
	@mkdir -p $@

javadoc.zip: javadoc-$(CLASSLIB)
	@echo ">>>Making "$@" ..." ;
	(cd $(INSTALLDIR); \
	$(ZIP) -r -q - javadoc) \
	> $(INSTALLDIR)/javadoc.zip;
	@echo "<<<Finished "$@" ..." ;

#
# CDC is the basis for all profiles. Include unconditionally.
#
include ../share/rules_cdc.mk

#
# And any profile-specific rules.
#
ifneq ($(CLASSLIB), cdc)
include ../share/rules_$(CLASSLIB).mk
endif
