/* sysEl3c90xEnd.c - system configuration module for el3c90xEnd driver */
 
/* Copyright 1984-1999 Wind River Systems, Inc. */
#include "copyright_wrs.h"
 
/* @(#)sysEl3c90xEnd.c	1.1 00/04/18 */

/*
modification history
--------------------
01b,12mar99, jkf renamed boardResource and pciResources to dev specific.
01a,12mar99, cn  created.
*/
 
/*
DESCRIPTION
This is the WRS-supplied configuration module for the VxWorks
el3c90xEnd (lnPci) END driver.
 
It initializes the resources for all the 3c90x devices found.

It performs the dynamic parameterization of the el3c90xEnd driver.
This technique of 'just-in-time' parameterization allows driver
parameter values to be declared as any other defined constants rather
than as static strings.
*/

#include "vxWorks.h"
#include "config.h"
 
#include "vmLib.h"
#include "stdio.h"
#include "sysLib.h"
#include "logLib.h"
#include "stdlib.h"
#include "string.h"
#include "end.h"
#include "intLib.h"
#include "drv/pci/pciIntLib.h"
 
#include "drv/end/el3c90xEnd.h"

#ifdef INCLUDE_EL_3C90X_END

/* imports */
 
IMPORT STATUS           sysMmuMapAdd (void * address, UINT len,
                                      UINT initialStateMask,
                                      UINT initialState);

IMPORT END_OBJ *	el3c90xEndLoad (char *);

/* defines */

/*
 * default values if PCI_CFG_TYPE defined to be PCI_CFG_FORCE
 * note: memory addresses must be aligned on MMU page boundaries
 */

#define EL_3C90X_IO_ADR0   	0xf400
#define EL_3C90X_MEM_ADR0  	0xfd000000
#define EL_3C90X_INT_LVL0  	0x0b
#define EL_3C90X_INT_VEC0  	0

#define EL_3C90X_IO_ADR1   	0xf420
#define EL_3C90X_MEM_ADR1  	0xfd200000
#define EL_3C90X_INT_LVL1  	0x05
#define EL_3C90X_INT_VEC1  	0

#define EL_3C90X_IO_ADR2   	0xf440
#define EL_3C90X_MEM_ADR2  	0xfd300000
#define EL_3C90X_INT_LVL2  	0x0c
#define EL_3C90X_INT_VEC2  	0

#define EL_3C90X_IO_ADR3   	0xf460
#define EL_3C90X_MEM_ADR3  	0xfd400000
#define EL_3C90X_INT_LVL3  	0x9
#define EL_3C90X_INT_VEC3  	0

#define EL_3C90X_MAX_DEV 	4

#define EL_3C90X_END_FLAGS	0
#define EL_3C90X_BUFF_MTPLR	NONE

#define EXT_INTERRUPT_BASE      INT_NUM_IRQ0    /* irq0     */
 
#define PCI_DEV_MMU_MSK         (~(VM_PAGE_SIZE - 1))   /* Mask MMU page */
 
#define PCI_DEV_ADRS_SIZE       VM_PAGE_SIZE    /* one page */
 
/* Offsets of PCI resources in host space */
 
#ifndef CPU_PCI_IO_ADRS
#define CPU_PCI_IO_ADRS         0               /* base of PCI I/O addr */
#endif /* CPU_PCI_IO_ADRS */
 
#ifndef CPU_PCI_MEM_ADRS
#define CPU_PCI_MEM_ADRS        0               /* base of PCI mem address */
#endif /* CPU_PCI_MEM_ADRS */
 
#ifndef PCI2DRAM_BASE_ADRS
#define PCI2DRAM_BASE_ADRS      0               /* DRAM base addrs from PCI */
#endif /* PCI2DRAM_BASE_ADRS */
 
/* typedefs */
 
typedef struct el3c90xPciResource              /* EL_3C90X_PCI_RESOURCES */
    {
    UINT32      iobaseCsr;              /* Base Address Register 0 */
    UINT32      membaseCsr;             /* Base Address Register 1 */
    char        irq;                    /* Interrupt Request Level */
    UINT32      irqvec;                 /* Interrupt Request vector */
    UINT32      configType;             /* type of configuration */
    UINT32      boardType;              /* type of LAN board for this unit */
    UINT32      pciBus;                 /* PCI Bus number */
    UINT32      pciDevice;              /* PCI Device number */
    UINT32      pciFunc;                /* PCI Function number */
    } EL_3C90X_PCI_RESOURCES;
 
typedef struct el3c90xBoardResource            /* EL_3C90X_BOARD_RESOURCES */
    {
    UINT32      type;                   /* type of the board */
    UINT32      vendorId;               /* Vendor ID */
    UINT32      deviceId;               /* Device ID */
    UINT32      lnUsrFlags;             /* LN driver user flags */
    FUNCPTR     mediaSelectFunc;        /* media select routine */
    } EL_3C90X_BOARD_RESOURCES;
 
/* locals */
 
/*
 * This array defines the board-specific PCI resources, the base address
 * register configuration mode and the Ethernet adapter type. It's indexed
 * using the device number returned from pciFindDevice().
 */

LOCAL EL_3C90X_PCI_RESOURCES el3c90xPciResrcs [] =
    {
    {EL_3C90X_IO_ADR0, EL_3C90X_MEM_ADR0, EL_3C90X_INT_LVL0, EL_3C90X_INT_VEC0,
     PCI_CFG_TYPE, 0, 0, 0, 0},
    {EL_3C90X_IO_ADR1, EL_3C90X_MEM_ADR1, EL_3C90X_INT_LVL1, EL_3C90X_INT_VEC1,
     PCI_CFG_TYPE, 0, 0, 0, 0},
    {EL_3C90X_IO_ADR2, EL_3C90X_MEM_ADR2, EL_3C90X_INT_LVL2, EL_3C90X_INT_VEC2,
     PCI_CFG_TYPE, 0, 0, 0, 0},
    {EL_3C90X_IO_ADR3, EL_3C90X_MEM_ADR3, EL_3C90X_INT_LVL3, EL_3C90X_INT_VEC3,
     PCI_CFG_TYPE, 0, 0, 0, 0}
    };

/* 
 * this table will be filled up with the actual resources used by this
 * device.
 */

EL_3C90X_PCI_RESOURCES * 	pEl3c90xActualRsrc [EL_3C90X_MAX_DEV];	

/*
 * This array defines board-specific vendor and device ids, flags to pass to
 * the drive load routine and the function used to select the media.
 */

/* 
 * Fix from WindRiver for TSR #152340. Added by cjp 2/14/00.
 * See also el3c90xEnd.h.
 */    
#if 1
LOCAL EL_3C90X_BOARD_RESOURCES el3c90xBrds [] =
{
{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_10BT, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_10BT_COMBO, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_10_100BT, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_100BT4, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10BT, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10BT_COMBO, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100BT, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100BT4, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100FX, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100BT_SERV, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_KRAKATOA_10BT_TPC, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10FL, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100_COMBO, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_TORNADO_10_100BT, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_TORNADO_10_100BT_SERV, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_HURRICANE_SOHO100TX, 0, NULL},
{0, TC_VENDORID, TC_DEVICEID_TORNADO_HOMECONNECT, 0, NULL},

};
#else
LOCAL EL_3C90X_BOARD_RESOURCES el3c90xBrds [] =
    {
	{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_10BT, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_10BT_COMBO, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_10_100BT, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_BOOMERANG_100BT4, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10BT, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10BT_COMBO, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100BT, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100BT4, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100FX, 0, NULL},
		
	{0, TC_VENDORID, TC_DEVICEID_CYCLONE_10_100BT_SERV, 0, NULL}
    };
#endif

/* forward declarations */

LOCAL STATUS	sysEl3c90xPciInit (void);

/*******************************************************************************
*
* sysEl3c90xPciInit - prepare LAN adapter for EL_3C90X initialization
*
* This routine finds out the PCI device, maps its memory and IO address.
* It must be done prior to initializing the driver.  Also
* must be done prior to MMU initialization, usrMmuInit().
*
* RETURNS: OK/ERROR
*/

STATUS sysEl3c90xPciInit (void)
    {
    UINT32 		membaseCsr;
    UINT32 		iobaseCsr;
    char 		irq;
    int 		pciBus;
    int 		pciDevice;
    int 		pciFunc;
    int 		unit;
    int 		found = 0;
    int 		netEndUnits = 0;
    UINT16		tmpWord;
    UINT32		tmpLong;
    UINT32		tmpLong2;
 

    /* for all the support network devices find if some of them exist */
    
    for (unit = 0; unit < NELEMENTS (el3c90xBrds); unit++)
	{
	if (pciFindDevice (el3c90xBrds [unit].vendorId,
                           el3c90xBrds [unit].deviceId,
                           netEndUnits, &pciBus, &pciDevice, &pciFunc) == OK)
	    {
             /* board detected */
             /* for now, we're ignoring the possibility of duplicates */

	     found = TRUE;

             /* load up the PCI device table */
             
             pEl3c90xActualRsrc [unit] = el3c90xPciResrcs + netEndUnits; 

             pEl3c90xActualRsrc [unit]->pciBus    = pciBus;
             pEl3c90xActualRsrc [unit]->pciDevice = pciDevice;
             pEl3c90xActualRsrc [unit]->pciFunc   = pciFunc;

             netEndUnits++;   	/* number of units found */
             }
        }

    if ((found != TRUE) || (pciDevice > PCI_MAX_DEV))
	return (ERROR);

    /* Now initialize all the units we found */

    for (unit = 0; ((unit < netEndUnits) &&
                    (unit < NELEMENTS (el3c90xPciResrcs)));
         unit++)
	{
        /* Fill in the resource entry */

        pEl3c90xActualRsrc [unit] = el3c90xPciResrcs + unit;

        if (pEl3c90xActualRsrc [unit]->configType == PCI_CFG_FORCE) 
            {
            /* write the iobase, membase, and irq */

            pciConfigOutLong (pEl3c90xActualRsrc [unit]->pciBus, 
			      pEl3c90xActualRsrc [unit]->pciDevice, 
			      pEl3c90xActualRsrc [unit]->pciFunc,
                              PCI_CFG_BASE_ADDRESS_0, 
			      pEl3c90xActualRsrc [unit]->iobaseCsr |
                              PCI_BASE_IO);

            pciConfigOutLong (pEl3c90xActualRsrc [unit]->pciBus, 
			      pEl3c90xActualRsrc [unit]->pciDevice, 
			      pEl3c90xActualRsrc [unit]->pciFunc,
                              PCI_CFG_BASE_ADDRESS_1, 
			      pEl3c90xActualRsrc [unit]->membaseCsr);

            pciConfigOutByte (pEl3c90xActualRsrc [unit]->pciBus, 
			      pEl3c90xActualRsrc [unit]->pciDevice, 
			      pEl3c90xActualRsrc [unit]->pciFunc,
                              PCI_CFG_DEV_INT_LINE, 
			      pEl3c90xActualRsrc [unit]->irq);
            }
	    
        /*
         * get memory base address and IO base address
         * Note: we read it in again, even if we just wrote it out because the 
         * device can change what we wrote
         */

        pciConfigInLong (pEl3c90xActualRsrc [unit]->pciBus, 
			 pEl3c90xActualRsrc [unit]->pciDevice, 
			 pEl3c90xActualRsrc [unit]->pciFunc,
	                 PCI_CFG_BASE_ADDRESS_0, &iobaseCsr);

        pciConfigInLong (pEl3c90xActualRsrc [unit]->pciBus, 
			 pEl3c90xActualRsrc [unit]->pciDevice, 
			 pEl3c90xActualRsrc [unit]->pciFunc,
                         PCI_CFG_BASE_ADDRESS_1, &membaseCsr);

        pciConfigInByte (pEl3c90xActualRsrc [unit]->pciBus, 
			 pEl3c90xActualRsrc [unit]->pciDevice, 
			 pEl3c90xActualRsrc [unit]->pciFunc,
                         PCI_CFG_DEV_INT_LINE, &irq);

	/* 
	 * we want to make sure the device implements memory BARs 
	 * first, we disable PCI memory decode by the device 
	 */

        pciConfigInWord (pEl3c90xActualRsrc [unit]->pciBus, 
			 pEl3c90xActualRsrc [unit]->pciDevice, 
			 pEl3c90xActualRsrc [unit]->pciFunc,
	                 PCI_CFG_COMMAND, &tmpWord);
 

        pciConfigOutWord (pEl3c90xActualRsrc [unit]->pciBus, 
			  pEl3c90xActualRsrc [unit]->pciDevice, 
			  pEl3c90xActualRsrc [unit]->pciFunc,
	                  PCI_CFG_COMMAND, (tmpWord & (~PCI_CMD_MEM_ENABLE)));
 
	/* Read and save the BAR */

        pciConfigInLong (pEl3c90xActualRsrc [unit]->pciBus, 
			 pEl3c90xActualRsrc [unit]->pciDevice, 
			 pEl3c90xActualRsrc [unit]->pciFunc,
	                 PCI_CFG_BASE_ADDRESS_1, &tmpLong);
 
	/* find out the memory space requirements of this BAR */

        pciConfigOutLong (pEl3c90xActualRsrc [unit]->pciBus, 
			  pEl3c90xActualRsrc [unit]->pciDevice, 
			  pEl3c90xActualRsrc [unit]->pciFunc,
	                  PCI_CFG_BASE_ADDRESS_1, 0xffffffff);
 
        pciConfigInLong (pEl3c90xActualRsrc [unit]->pciBus, 
			 pEl3c90xActualRsrc [unit]->pciDevice, 
			 pEl3c90xActualRsrc [unit]->pciFunc,
	                 PCI_CFG_BASE_ADDRESS_1, &tmpLong2);
 
	if (tmpLong2 != 0)
	    {
	    /* this is a legitimate memory BAR */

	    pciConfigOutLong (pEl3c90xActualRsrc [unit]->pciBus, 
			     pEl3c90xActualRsrc [unit]->pciDevice, 
			     pEl3c90xActualRsrc [unit]->pciFunc,
			     PCI_CFG_BASE_ADDRESS_1, tmpLong);
	    }
	else
	    {
	    /* the driver will use IO-mapped BAR */

	    membaseCsr = NONE;
	    }
 
	/* re-enable PCI memory decode by the device */

        pciConfigOutWord (pEl3c90xActualRsrc [unit]->pciBus, 
			  pEl3c90xActualRsrc [unit]->pciDevice, 
			  pEl3c90xActualRsrc [unit]->pciFunc,
	                  PCI_CFG_COMMAND, (tmpWord));
        /*
         * mask off registers. IO base needs to be masked off because bit0
         * will always be set to 1
         */

        iobaseCsr &= PCI_IOBASE_MASK;
	iobaseCsr += CPU_PCI_IO_ADRS;

	if (membaseCsr != NONE)
            {
	    membaseCsr &= PCI_MEMBASE_MASK;
	    membaseCsr += CPU_PCI_MEM_ADRS;

#ifdef INCLUDE_MMU_BASIC
	    if (sysMmuMapAdd ((void *)(membaseCsr & PCI_DEV_MMU_MSK),
			      PCI_DEV_ADRS_SIZE,
			      VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE |
			      VM_STATE_MASK_CACHEABLE,
			      VM_STATE_VALID      | VM_STATE_WRITABLE |
			      VM_STATE_CACHEABLE_NOT
			      ) == ERROR)
		{
		 /* for now, exit, but later break and stop where we're at 
		    when we're detecting multiple units */
		 return (ERROR);
		}
#else
#warning INCLUDE_MMU_BASIC not defined
#endif /* INCLUDE_MMU_BASIC */
            }

	/* over write the resource table with values read */

	pEl3c90xActualRsrc [unit]->membaseCsr   = membaseCsr;
	pEl3c90xActualRsrc [unit]->iobaseCsr    = iobaseCsr;
	pEl3c90xActualRsrc [unit]->irq          = irq;
        pEl3c90xActualRsrc [unit]->irqvec       = irq + EXT_INTERRUPT_BASE;

	/* enable mapped memory and IO addresses */

        pciConfigOutWord (pEl3c90xActualRsrc [unit]->pciBus, 
			  pEl3c90xActualRsrc [unit]->pciDevice, 
			  pEl3c90xActualRsrc [unit]->pciFunc,
                          PCI_CFG_COMMAND, PCI_CMD_IO_ENABLE |
                          PCI_CMD_MEM_ENABLE | PCI_CMD_MASTER_ENABLE);

        /* disable sleep mode */

        pciConfigOutWord (pEl3c90xActualRsrc [unit]->pciBus, 
			  pEl3c90xActualRsrc [unit]->pciDevice, 
			  pEl3c90xActualRsrc [unit]->pciFunc, 
			  PCI_CFG_MODE, 
                          SLEEP_MODE_DIS);
        }

    return (OK); 
    }

/*******************************************************************************
*
* sysEl3c90xIntEnable - enable Lan97x interrupts
*
* This routine enables ln7997x interrupts.  This may involve operations on
* interrupt control hardware.
*
* RETURNS: OK or ERROR for invalid arguments.
*/

STATUS sysEl3c90xIntEnable
    (
    int	level		/* level number */
    )
    {
    return (sysIntEnablePIC(level));
    }

/*******************************************************************************
*
* sysLanIntDisable - disable ln970 interrupts
*
* This routine disables ln970 interrupts.  This may involve operations on
* interrupt control hardware.
*
* RETURNS: OK or ERROR for invalid arguments.
*/

STATUS sysEl3c90xIntDisable
    (
    int	level		/* level number */
    )
    {
    return (sysIntDisablePIC(level));
    }

/******************************************************************************
*
* sysEl3c90xEndLoad - load an istance of the el3c90xEnd driver
*
* This routine loads the el3c90xEnd driver with the parameters specified by
* the resource table for the device, and some default values.
*
* The END device load string formed by this routine is in the following
* following format.
* <devMemAddr>:<devIoAddr>:<pciMemBase>:<vecnum>:<intLvl>:<memAdrs>
* :<memSize>:<memWidth>:<flags>:<buffMultiplier>
*
* .IP <devMemAddr>
* Device register base memory address
* .IP <devIoAddr>
* Device register base IO address
* .IP <pciMemBase>
* Base address of PCI memory space
* .IP <vecNum>
* Interrupt vector number.
* .IP <intLvl>
* Interrupt level.
* .IP <memAdrs>
* Memory pool address or NONE.
* .IP <memSize>
* Memory pool size or zero.
* .IP <memWidth>
* Memory system size, 1, 2, or 4 bytes (optional).
* .IP <flags>
* Device specific flags.
* .IP <buffMultiplier>
* Buffer Multiplier or NONE. If NONE is specified, it defaults to 2
*
* This routine only loads and initializes instance zero of the device.
* If the user wishes to use more than one el3c90x devices, this routine
* should be changed.
*
* RETURNS: pointer to END object or ERROR.
*
* SEE ALSO: el3c90xEndLoad()
*/
 
END_OBJ * sysEl3c90xEndLoad
    (
    char * pParamStr,   /* ptr to initialization parameter string */
    void * unused       /* unused optional argument */
    )
    {
    /*
     * The el3c90xEnd driver END_LOAD_STRING should be:
     * <devMemAddr>:<devIoAddr>:<pciMemBase>:<vecnum>:<intLvl>:<memAdrs>
     * :<memSize>:<memWidth>:<flags>:<buffMultiplier>
     * Note that unit string is prepended by the mux, so we
     * don't put it here.
     */
 
    char * pStr = NULL;
    char paramStr [200];
    static char el3c90xParamTemplate [] = 
			"0x%x:0x%x:0x%x:%d:%d:-1:-1:-1:0x%x:0x%x";
    END_OBJ * pEnd;
 
    if (strlen (pParamStr) == 0)
        {
        /*
         * muxDevLoad() calls us twice.  If the string is
         * zero length, then this is the first time through
         * this routine.
         */
 
        pEnd = (END_OBJ *) el3c90xEndLoad (pParamStr);
        }
    else
        {
        /*
         * On the second pass through here, we actually create
         * the initialization parameter string on the fly.
         * Note that we will be handed our unit number on the
         * second pass and we need to preserve that information.
         * So we use the unit number handed from the input string.
         */
 
        pStr = strcpy (paramStr, pParamStr);
 
        /* Now, we get to the end of the string */
 
        pStr += strlen (paramStr);
 
        /* finish off the initialization parameter string */
 
        sprintf (pStr, el3c90xParamTemplate,
		 (UINT) pEl3c90xActualRsrc [0]->membaseCsr,
		 (UINT) pEl3c90xActualRsrc [0]->iobaseCsr,
		 PCI2DRAM_BASE_ADRS,  	
		 pEl3c90xActualRsrc [0]->irqvec, 
		 pEl3c90xActualRsrc [0]->irq, 		
		 EL_3C90X_END_FLAGS,	
		 EL_3C90X_BUFF_MTPLR	
		 );
 
        if ((pEnd = (END_OBJ *) el3c90xEndLoad (paramStr)) == (END_OBJ *)ERROR)
            {
            logMsg ("Error: el3c90xEndLoad failed to load driver\n",
		    0, 0, 0, 0, 0, 0);
            }
        }
 
    return (pEnd);
    }
#endif /* INCLUDE_EL_3C90X_END */

