/*
 * @(#)syncClockToTimePort.c	1.3 00/11/10
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
* The following code is used to intialize VxWorks idea of time-of-day.
* This may not be needed in a system with a built-in real-time clock that
* is supported int the board support package.
*/

#include "vxWorks.h"
#include "time.h"
#include "sockLib.h"
#include "inetLib.h"
#include "stdioLib.h"
#include "hostLib.h"
#include "ioLib.h"

STATUS showTimeOfDay(void);

/*
* Connect to the time port of the specified host, and use the value returned
* to set the local time-of-day clock. Should be accurate to one second
* relative to the host providing the time.
*/

STATUS
syncClockToTimePort(char *serverName)
{

    struct sockaddr_in  serverAddr;
    int			sockAddrSize;
    int			sFd;
    struct tm		timeOfDay;
    struct timespec	currentTime = {0, 0};
    uint32_t		timeInSeconds;

    /* create client's socket */
    if ((sFd = socket(AF_INET, SOCK_STREAM, 0)) == ERROR)
    {
	perror("syncClockToTimePort - socket");
	return ERROR;
    }

    /* build server socket address */
    sockAddrSize = sizeof (struct sockaddr_in);
    bzero((char*)&serverAddr, sockAddrSize);
    serverAddr.sin_family = AF_INET;
    serverAddr.sin_len = (u_char)sockAddrSize;
    serverAddr.sin_port = htons(37);

    if (((serverAddr.sin_addr.s_addr = inet_addr(serverName)) == ERROR) &&
	((serverAddr.sin_addr.s_addr = hostGetByName(serverName)) == ERROR))
    {
        perror("syncClockToTimePort - server address");
        close(sFd);
	return ERROR;
    }

    /* connect to server */
    if (connect(sFd, (struct sockaddr*)&serverAddr, sockAddrSize) == ERROR)
    {
	perror("syncClockToTimePort - connect");
	close(sFd);
	return ERROR;
    }


    if (read(sFd, (char*)&timeInSeconds, sizeof (timeInSeconds)) < 0)
    {
	perror("syncClockToTimePort - read");
	close(sFd);
	return ERROR;
    }

    /* time service delivers time in GMT */
    timeInSeconds = ntohl(timeInSeconds);
    gmtime_r(&timeInSeconds, &timeOfDay);

    timeOfDay.tm_isdst = 0;  /* never dst on GMT */

    /* server sends time in seconds from 1/1/1900, gmtime() assumes time
     * in seconds from 1/1/1970. This causes the timeOfDay to be off by
     * 70 years. So we tweak it. */
    timeOfDay.tm_year -= 70;

    currentTime.tv_sec = mktime(&timeOfDay);

    clock_settime(CLOCK_REALTIME, &currentTime);

    return showTimeOfDay();
}


STATUS
showTimeOfDay(void)
{
    char timeStr[40] = "";
    time_t theTime = time(0);
    int timeStrSize = sizeof (timeStr);

    ctime_r(&theTime, timeStr, &timeStrSize);

    printf("Time is %s\n", timeStr);
    return OK;
}
