/*
 * @(#)linker_md.c	1.8 00/11/12
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
 * Adapted from JDK 1.2 linker_md.c v1.37. NOTE: (kbr) Requires native
 * threads. VxWorks-specific.
 */

#include <ioLib.h>
#include <loadLib.h>
#include <moduleLib.h>
#include <symLib.h>
#include <sysSymTbl.h>

#include "path_md.h"

/*
 * create a string for the JNI native function name by adding the
 * appropriate decorations.
 */
int
dbgsysBuildFunName(char *name, int nameLen, int args_size, int encodingIndex)
{
  /* On VxWorks, there is only one encoding method. */
    if (encodingIndex == 0)
        return 1;
    return 0;
}

/*
 * create a string for the dynamic lib open call by adding the
 * appropriate pre and extensions to a filename and the path
 */
void
dbgsysBuildLibName(char *holder, int holderlen, char *pname, char *fname)
{
    const int pnamelen = pname ? strlen(pname) : 0;
    char *suffix;

#ifdef DEBUG   
    suffix = "_g";
#else
    suffix = "";
#endif 

    /* Quietly truncate on buffer overflow.  Should be an error. */
    if (pnamelen + strlen(fname) + 10 > holderlen) {
        *holder = '\0';
        return;
    }

    if (pnamelen == 0) {
        sprintf(holder, "lib%s%s.o", fname, suffix);
    } else {
        sprintf(holder, "%s/lib%s%s.o", pname, fname, suffix);
    }
}

void *
dbgsysLoadLibrary(const char *name, char *err_buf, int err_buflen)
{
    MODULE_ID mid;
    int fd;
    static const char *openErrorString = "Error opening library";
    static const char *loadErrorString = "Error loading library";

    fd = open((char *) name, O_RDONLY, 0);
    if (fd == ERROR) {
	strncpy(err_buf, openErrorString, err_buflen - 2);
	err_buf[err_buflen-1] = 0;
	return NULL;
    }
    mid = loadModule(fd, LOAD_GLOBAL_SYMBOLS);
    close(fd);
    if (mid == NULL) {
	strncpy(err_buf, loadErrorString, err_buflen - 2);
	err_buf[err_buflen-1] = 0;
    }
    return mid;
}

void dbgsysUnloadLibrary(void *handle)
{
    unldByModuleId(handle);
}

struct CallbackInfo {
    char *name;
    UINT16 group;
    void *val;
};

static BOOL
findSymbolCB(char *name,
	     int val,
	     SYM_TYPE type,
	     int arg,
	     UINT16 group)
{
    struct CallbackInfo *info = (struct CallbackInfo *) arg;
    
    if ((!strcmp(name, info->name)) &&
	(info->group == group)) {
	/* Found the symbol we're looking for. Returning FALSE from
	   this callback will cause symEach() to return the current
	   symbol to its caller. */
	info->val = (void *) val;
	return FALSE;
    }
    return TRUE;
}

void * dbgsysFindLibraryEntry(void *handle, const char *name)
{
    MODULE_ID mid;
    MODULE_INFO mInfo;
    struct CallbackInfo info;
    SYMBOL *sym;

    mid = (MODULE_ID) handle;
    if (moduleInfoGet(mid, &mInfo) == ERROR)
	return NULL;
    info.name = (char *) name;
    info.group = (UINT16) mInfo.group;
    sym = symEach(sysSymTbl, (FUNCPTR) &findSymbolCB, (int) &info);
    if (sym == NULL)
	return NULL;
    return info.val;
}
