package com.timesys;
import javax.realtime.*;

// Dispatcher works by spawning a java thread, then calling a native
// method that sigwaits on the set of recieved signals.
/**
   Dispatcher works by spawning a java thread, which then sigwaits on
   the set of POSIX signals (1 to 63).
  */
public final class Dispatcher
{
  //The dispatcher can only be started once.
  private static boolean started = false;
  private static Object startLock = new Object();

  //The dispatcher's process ID is set by runDispatcher
  private static int dispPID = -1;
  private static Object dispPIDlock = new Object();
  private static NoHeapRealtimeThread DispThread = null;

  //Don't let anyone instanciate this class.    
  public Dispatcher()
  {
  }

  // Start the dispatcher with default values.
    /**
       Start the dispatcher with default values.
    */
  public static synchronized void startDispatcher() throws IllegalDispatcherStateException
  {
    startDispatcher(new PriorityParameters(PriorityScheduler.instance().getMaxPriority()),
		    null, null, ImmortalMemory.instance(), null);
  }

    /**
       Start the dispatcher with specified values
    */
  public static synchronized void startDispatcher(SchedulingParameters scheduling,
						  ReleaseParameters release,
						  MemoryParameters memory,
						  MemoryArea area,
						  ProcessingGroupParameters group)
    throws IllegalDispatcherStateException
  {
    synchronized(startLock){
      if(started){
	throw new IllegalDispatcherStateException();
      }
      else{
	started = true;
      }
    }
    
    DispThread = new NoHeapRealtimeThread(scheduling, release, memory,
					  area, group, null){
	public void run(){
	  runDispatcher();
	  started = false;
	}
      };
    
    DispThread.setDaemon(true);
    DispThread.start();
  }
  
  private static native void runDispatcher();

  private static void killDispatcher()
  {
    System.out.print("killing dispatcher\n");
  }

    /**
       Returns true if the dispatcher is running, false otherwise.
    */
  public static boolean isStarted()
  {
    return started;
  }

  // threads attempting to check the dispatchers pid
    /**
       Return the UNIX process ID of the dispatcher thread with getpid() 
       (as defined in unistd.h)
    */
  public static int getPID() throws IllegalDispatcherStateException, java.lang.InterruptedException
  {
    if(!started)
      throw new IllegalDispatcherStateException();
    
    synchronized(dispPIDlock){
      if(dispPID<0){
	dispPIDlock.wait();
      }
    }
    return(dispPID);
  }

  // called by the native-side dispatcher code to set it's pid
  private static void setPID(int PID)
  {
    synchronized(dispPIDlock){
      dispPID=PID;
      dispPIDlock.notifyAll();
    }
    return;
  }
}
