/*
 * -----------------------------------------------------------------
 * HeapMemory.java : Java class that represents the heap used by the
 * VM.
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

package javax.realtime;

/**
 * The <code>HeapMemory</code> class is a singleton object that allows
 * logic within other scoped memory to allocate objects in the Java heap.
 *
 * @author Pratik Solanki
 */
public final class HeapMemory extends MemoryArea
{
    private static HeapMemory heap = initHeap();
    
    private HeapMemory()
    {
		//super(Runtime.getRuntime().totalMemory());
        super(4*1024*1024);
        //initHeap();
    }
  
    /**
     * Returns a pointer to the singleton <code>HeapMemory</code> space.
     * @return The singleton <code>HeapMemory</code> object.
     */
    public static HeapMemory instance()
    {
		return heap;
    }
    

	public long memoryRemaining()
	{
		return Runtime.getRuntime().freeMemory();		
	}

    public long memoryConsumed()
    {
		return Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();
    }

    private native static HeapMemory initHeap();
}

