/*
 * -----------------------------------------------------------------
 * ImmortalMemory.java : ImmortalMemory is used by all threads and 
 * represents memory that is never garbage collected.
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

package javax.realtime;

import java.util.Vector;

/** 
 * <code>ImmortalMemory</code> is a memory resource that is shared among all
 * threads.  Objects allocated in the immortal memory live until the end
 * of the application.  Objects in immortal memory are never subject to
 * garbage collection, although some GC algorithms may require a scan of
 * the immortal memory.  An <em>immortal</em> object may only contain 
 * reference to other immortal objects or to heap objects.  Unlike 
 * standard Java heap objects, immortal objects continue to exist even
 * after there are no other references to them.
 * 
 * @author Pratik Solanki
 */
public final class ImmortalMemory extends MemoryArea
{
    
    private static final int MEM_SIZE = 2*1024*1024; //size of immortal memory

    /**
     * Returns a pointer to the singleton <code>ImmortalMemory</code> 
     * space.
     * 
     * @return The singleton <code>ImmortalMemory</code> object.
     */
  	public static ImmortalMemory instance()
    {
		return immortal;
    }

    private ImmortalMemory()
    {
        super(MEM_SIZE);
		//initImmortal(MEM_SIZE);
    }

    private static ImmortalMemory immortal = initImmortal(MEM_SIZE);
    private native static ImmortalMemory initImmortal(long memSize);
	
}

