/*
 * -----------------------------------------------------------------
 * ImportanceParameters.java :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

package javax.realtime;

/** 
  Importance is an additional scheduling metric that may be used by some 
  priority-based scheduling algorithms during overload conditions to differentiate execution
  order among threads of the same priority.

  <p>In some real-time systems an external physical process determines
  the period of many threads. If rate-monotonic priority assignment is
  used to assign priorities many of the threads in the system may have the same
  priority because their periods are the same.  However, it is conceivable that some
  threads may be more important than others and in an overload situation importance
  can help the scheduler decide which threads to execute first.

  The base scheduling algorithm represented by {@link PriorityScheduler} is not 
  required to use importance.  However, the RTSJ strongly suggests to implementers
  that a fairly simple subclass of {@link PriorityScheduler} that uses importance can
  offer value to some real-time applications.
  */

public class ImportanceParameters extends PriorityParameters
{
    int importance;
    
    /** 
      Create an instance of <code>ImportanceParameters</code>.
      @param priority   The priority assigned to a thread.
                        This value is used in place of java.lang.Thread.priority.
      @param importance  The importance value assigned to a thread.
     */
    public ImportanceParameters(int priority, int importance)
    {
	super(priority);
	this.importance = importance;
    }
    
    /** 
      Get the importance value.
     */
    public int getImportance()
    {
	return importance;
    }
    
    /** 
      Set the importance.
     */
    public void setImportance(int importance)
    {
	this.importance = importance;
    }

    public java.lang.String toString()
    {
	return new String(super.toString()+":"+Integer.toString(importance));
    }
}
