/*
 * -----------------------------------------------------------------
 * MarkAndSweepCollector.java: A java class representing the Mark
 * and sweep collector used by the VM.
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

package javax.realtime;

/**
 * This class represents the mark and sweep garbage collector 
 * used in the system.
 *
 * @author Pratik Solanki
 */
public class MarkAndSweepCollector extends GarbageCollector
{

  /**
   * Preemption latency is a measure of the maximum time a RealtimeThread may
   * have to wait for the collector to reach a preemption-safe point.
   *
   * @return The preempting latency of the garbage collector.
   */
  public RelativeTime getPreemptionLatency()
  {
  	 return new RelativeTime(getPreemptionLatency0(), 0);
  }

  /**
   * This function returns the amount of time a RealtimeThread
   * may have to wait for the collector to reach a preemption-safe 
   * point. Currently only millisecond resolution is supported.
   * 
   * @return The preemption latency in milliseconds. 
   */
  private native int getPreemptionLatency0();
}
