/*
 * -----------------------------------------------------------------
 * MonitorControl.java :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

package javax.realtime;

import java.util.*;

/**
  Abstract superclass for all monitor control policy objects.
  */
public abstract class MonitorControl {

  /* Dio: This two variables should be the same
   * as the ones in the rk.h file
   * ToDo: move it to querying the class of the monitorControl
   * from C (through JNI)
   */

    final static int PI =1;
    final static int PCP=2;
    static MonitorControl defaultMonitorControl = null;

    static native int setMonitorControl0(java.lang.Object monitor, int protocol, int ceiling);
    static native void setMonitorControl1(int protocol, int ceiling);
    static native java.lang.Object getMonitorControl0(java.lang.Object monitor);
    static native java.lang.Object getMonitorControl1();

    public MonitorControl()
    {
    }

  public static MonitorControl getMonitorControl(Object monitor)
  {
      MonitorControl tmp = (MonitorControl) getMonitorControl0(monitor);
      if (tmp == null)
	  return getMonitorControl();

      return tmp;
  }

  public static MonitorControl getMonitorControl()
  {
    return (MonitorControl) getMonitorControl1();
  }

    /**
      Control the default monitor behavior for object monitors
      used by synchronized statements and methods in the system.
      The type of the policy object determines the type of behavior.
      Conforming implementations must support priority ceiling emulation
      and priority inheritance for fixed priority preemptive threads.
      @param policy The new monitor control policy.  If null nothing happens.
      */
    public static void setMonitorControl(MonitorControl defaultMonCtl)
    {
	if (defaultMonCtl == null)
	    return;

	defaultMonitorControl = defaultMonCtl;
	if (defaultMonCtl instanceof PriorityCeilingEmulation)
	    {
		setMonitorControl1(PCP, ((PriorityCeilingEmulation)defaultMonCtl).getDefaultCeiling());
	    }
	else
	    {
		setMonitorControl1(PI,0);
	    }
    }
    
    /**
      Has the same effect as <code>setMonitorControl()</code>, except that the policy
      only affects the indicated object monitor.
      @param monitor The monitor for which the new policy will be in use.
      The policy will take effect on the first attempt to lock the monitor after the
      completion of this method. If null nothing will happen.
      @param policy The new policy for the object. If null nothing will happen.
      */
    public static void setMonitorControl(java.lang.Object monitor, MonitorControl monCtl)
    {
	if ((monitor == null) || (monCtl == null))
	    return;

	if (monCtl instanceof PriorityCeilingEmulation)
	    {
		setMonitorControl0(monitor,PCP,((PriorityCeilingEmulation)monCtl).getDefaultCeiling());
	    }
	else
	    {
		setMonitorControl0(monitor,PI,0);
	    }
    }
}
