/*
 * -----------------------------------------------------------------
 * RealtimeClock.java :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

package javax.realtime;


class RealtimeClock extends Clock{

	private int nanos;
	private long millis;
	
	public RealtimeClock(){
		millis = 0;
		nanos = 0;
	
	}

 	/**
  	 * Return the resolution of the clock -- the interval between ticks.
  	 * @return A RelativeTime object representing the resolution of this
  	 */
	public RelativeTime getResolution(){

		return new RelativeTime(1,0);
	}
	

  	/**
   	 * Return the current time in an existing object. The time represented by
   	 * the given AbsoluteTime is changed some time between the invocation of
   	 * the method and the return of the method
   	 * @param time   The AbsoluteTime object which will have its time changed.
   	 *			if null then nothing happens.
   	 */
	public void getTime(AbsoluteTime time){

		if(time==null)
			return;

		long totalnanos = System.currentTimeMillis()*1000000;
		long currentmillis = totalnanos/1000000;
		int currentnanos = (int)(totalnanos - (currentmillis*1000000));
	
		time.set(currentmillis,currentnanos);
	}

  	/**
   	 * Set the resolution of this. For some hardwhere clocks setting resolution
   	 * impossible and if called on those nothing happens.
   	 * @param resolution The new resolution of this
   	 */
	public void setResolution(RelativeTime resolution){
		return;
	}


}


