/*
 * -----------------------------------------------------------------
 * VTMemory.java : VTMemory represents a memory area that may take a 
 * variable amount of tiem for allocation.
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

package javax.realtime;

/**
 * The executin time of an allocation from a <code>VTMemory</code> area
 * may take a variable amount of time. However, since <code>VTMemory</code>
 * areas are not subject to garbage collection and objects within it may 
 * not be moved, these areas can be used by instances of 
 * {@link NoHeapRealtimeThread}.
 *
 * @author Pratik Solanki
 */
public class VTMemory extends ScopedMemory
{
    long initialSize;
    long maximumSize;
    
    /**
     * Creates a <code>VTMemory</code> of the given size.
     *
     * @param initialSizeInBytes The size in bytes of the memory to initially 
     *          allocate for this area.
     * @param maximumSizeInBytes The maximum size in bytes this memory area can 
     *          grow to.
     */
    public VTMemory(long initialSizeInBytes, long maxSizeInBytes)
    {
        this(initialSizeInBytes, maxSizeInBytes, null);
    }

    public VTMemory(long initialSizeInBytes, long maxSizeInBytes, Runnable logic)
    {
        super(initialSizeInBytes, logic);
		
        if(initialSizeInBytes > maxSizeInBytes)
            System.out.println("LTMemory - initial size is greater than maximum size. Ignoring maximum size.");
        
        memorySize = (maxSizeInBytes>initialSizeInBytes)?maxSizeInBytes:initialSizeInBytes;
        
        initialSize = initialSizeInBytes;
        maximumSize = memorySize;
        
		// The following is a ScopedMemory function that I am calling.
		// This should ideally be in VTMemory but I did this so that I didn't
		// have to duplicate the function in LT and VT Memory.
		// - Pratik
		initScoped(memorySize);
    }
    
    
    public VTMemory(SizeEstimator initial, SizeEstimator maximum)
    {
        this(initial.getEstimate(), maximum.getEstimate());
    }
    
    public VTMemory(SizeEstimator initial, SizeEstimator maximum, Runnable logic)
    {
        this(initial.getEstimate(), maximum.getEstimate(), logic);
    }

    public long getMaximumSize()
    {
        return (int)maximumSize;
    }
    
    public String toString()
    {
        return new String("(VTMemory) " + super.toString());
    }
}

