/*
 * -----------------------------------------------------------------
 *  sync_md.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying TimeSys Community Source License Agreement from TimeSys
 * in file TCSL.
 *
 * This file is derived from software distributed under the following terms:
 *
 * Sun Community Source License
 * Sun Microsystems
 * For details, see http://www.sun.com/981208/scsl/principles.html
 *
 * Your use of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


/*
 * @(#)sync_md.h	1.5 00/09/30
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
 * Machine-dependent synchronization definitions.
 */

#ifndef _LINUX_SYNC_MD_H
#define _LINUX_SYNC_MD_H

#include "porting/posix/sync.h"
#include "javavm/include/porting/vm-defs.h"

/*
 * In LinuxThreads pthreads, it is not safe to use a signal handler
 * and longjmp to break out of a condvar wait.  The mutex will not
 * be reacquired, and the pthreads data structures will not be
 * cleaned up, leaving the thread on the wait queue.  So we use
 * a "semaphore" per thread and explicitly wakeup the thread we want
 * to, just like HotSpot and win32 JDK.
 */

#define CVMmutexInit(m)		POSIXmutexInit(&(m)->pmtx)
#define CVMmutexDestroy(m)	POSIXmutexDestroy(&(m)->pmtx)
#define CVMmutexTryLock(m)	POSIXmutexTryLock(&(m)->pmtx)
#define CVMmutexLock(m)		POSIXmutexLock(&(m)->pmtx)
#define CVMmutexUnlock(m)	POSIXmutexUnlock(&(m)->pmtx)

#define CVM_FASTLOCK_TYPE CVM_FASTLOCK_MICROLOCK

#define CVM_MICROLOCK_TYPE CVM_MICROLOCK_DEFAULT

CVMUint32 atomicCmpSwap(CVMUint32 new, volatile CVMUint32 *addr, CVMUint32 old);

#define CVMatomicCompareAndSwap(a, n, o)	\
	atomicCmpSwap((n), (a), (o))

CVMUint32 atomicSwap(CVMUint32 new, volatile CVMUint32 *addr);

#define CVMatomicSwap(a, n)	\
        atomicSwap((n), (a))

void linuxSyncInit(void);
void linuxSyncInterruptWait(CVMThreadID *thread);
void linuxSyncSuspend(CVMThreadID *thread);
void linuxSyncResume(CVMThreadID *thread);

struct CVMMutex {
    POSIXMutex pmtx;
};

struct CVMCondVar {
    POSIXCondVar pcv;
    CVMThreadID *waiters;
    CVMThreadID **last_p;
};

#endif /* _LINUX_SYNC_MD_H */
