/*
 * -----------------------------------------------------------------
 *  threads_md.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying TimeSys Community Source License Agreement from TimeSys
 * in file TCSL.
 *
 * This file is derived from software distributed under the following terms:
 *
 * Sun Community Source License
 * Sun Microsystems
 * For details, see http://www.sun.com/981208/scsl/principles.html
 *
 * Your use of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


/*
 * @(#)threads_md.h	1.13 00/11/13
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
 * Machine-dependent thread definitions.
 */

#ifndef _LINUX_THREADS_MD_H
#define _LINUX_THREADS_MD_H

#include "porting/posix/threads.h"
#include "javavm/include/porting/sync.h"

#include <pthread.h>
#include <semaphore.h>
#include <setjmp.h>

struct CVMThreadID {
    pthread_t pthreadCookie;
    void *stackTop;
    pthread_mutex_t lock;
    /* support for CVMmutexSetOwner */
    CVMMutex *targetMutex;
    CVMInt32 lastPriority;
    /* support for CVMthreadSuspend */
    int suspended;
    int suspended_in_wait;
    int in_sigsuspend;
    pthread_cond_t suspend_cv;
    /* support for Thread.interrupt */
    CVMBool interrupted;
    CVMBool in_wait;
    CVMBool notified;
    /* emulate a semaphore */
    pthread_mutex_t wait_mutex;
    pthread_cond_t wait_cv;
    int value;

    /* Dio: trying to add a waiting queue */
    int sharedTimerReferenceCount;
    int waitUntilDataValid;
    long long waitUntilTime;    
    pthread_mutex_t wait_until_mtx;
    pthread_cond_t wait_until_cv;
    CVMThreadID *waitingOn;
    CVMThreadID *waitingQnext;
    CVMThreadID **waitingQprev_p;


    /* support for close */
    int fd;	/* only valid while thread is on I/O queue! */
    /* IO or wait queue */
    CVMThreadID *next;
    CVMThreadID **prev_p;
};

#define POSIX_COOKIE(t)		((t)->pthreadCookie)

#define CVMthreadCreate		POSIXthreadCreate
#define CVMthreadCreateEx	POSIXthreadCreateEx
#define CVMthreadSelf		POSIXthreadGetSelf

#define CVMthreadGetPriority	POSIXthreadGetPriority

#define CVMthreadIOInterrupt	POSIXthreadSendSig


/* NOTE: The current redzone and stack sizes below are taken straight from the
         the debug version of Solaris port.  These numbers will probably fine 
	 for the Linux port but may not be optimum.  To obtain the optimum numbers, 
	 one will have to run a static stack analysis written for analyzing the
         assembly codes of CVM on the Linux platform. 
*/
#define CVM_THREAD_MIN_C_STACK_SIZE				(320 * 1024)

#define CVM_REDZONE_ILOOP                                       115920 /* 11.32KB */
#define CVM_REDZONE_CVMimplementsInterface                       47520 /*  5.14KB */
#define CVM_REDZONE_CVMCstackCVMpc2string                        47520 /*  5.14KB */
#define CVM_REDZONE_CVMCstackCVMID_objectGetClassAndHashSafe     53920 /*  5.27KB */
#define CVM_REDZONE_CVMclassLookupFromClassLoader               135000 /* 13.18KB */
#define CVM_REDZONE_CVMclassLink                                 74560 /*  7.28KB */
#define CVM_REDZONE_CVMCstackmerge_fullinfo_to_types             58000 /*  5.66KB */
#define CVM_REDZONE_CVMsignalErrorVaList                         84000 /*  8.20KB */

#endif /* _LINUX_THREADS_MD_H */
