/*
 * -----------------------------------------------------------------
 *  sync_md.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying TimeSys Community Source License Agreement from TimeSys
 * in file TCSL.
 *
 * This file is derived from software distributed under the following terms:
 *
 * Sun Community Source License
 * Sun Microsystems
 * For details, see http://www.sun.com/981208/scsl/principles.html
 *
 * Your use of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

/*
 * @(#)sync_md.c	1.12 00/10/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#include "javavm/include/porting/sync.h"
#include "javavm/include/porting/threads.h"
#include "javavm/include/porting/float.h"	/* for setFPMode() */
#include <pthread.h>
#include <semaphore.h>
#include <sys/time.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <assert.h>
#include <stdio.h>

#include "javavm/include/interpreter.h"

//#include  "javavm/include/interpreter.h"

static sem_t setowner_sem;

/*
 * Call by signal handler to process set-owner request.
 */
static void setOwner(CVMThreadID *self)
{
	CVMBool success = CVMmutexTryLock(self->targetMutex);
	assert(success), (void) success;


	self->targetMutex = NULL;
	/*
	 * According to the man page, sem_post is reentrant and
	 * signal-handler safe on 486 and above.  But
	 * self->lock should prevent our sem_wait from being
	 * sent a signal for this handler.
	 */
	sem_post(&setowner_sem);

	/* reset priority */

	/* Dio this is not the place to do
	 * PI. This interfere with our PI so
	 * we comment it out
	 */

	//CVMthreadSetPriority(self, self->lastPriority);
}

/*
 * signal handler for CVMmutexSetOwner() or
 * CVMthreadSuspend() request
 */

/* This is a SIGUSR2 handler used to defer unblocking of io calls if they are not called from AI methods -Kalpesh. */
static void sig_iounblock(int sig){
	
  CVMExecEnv* ee  =  CVMgetEE();
   
  if(ee->dirtyAie<=0){
  	ee->dirtyAie = -1;
  }
//  printf("IO unblocked\n");    

  return;

}

static void sigfunc0(int sig)
{
    CVMThreadID *self = CVMthreadSelf();
    assert(POSIX_COOKIE(self) == pthread_self());

    if (self->in_sigsuspend) {
	return;
    }
    if (self->targetMutex != NULL) {
	setOwner(self);
    }
    if (self->suspended) {
	sigset_t sig_set;

	pthread_sigmask(SIG_SETMASK, NULL, &sig_set);
	sigdelset(&sig_set, SIGUSR1);

	do {
	    /* wait for thread resume or set-owner request */
	    self->in_sigsuspend = 1;
	    sigsuspend(&sig_set);
	    self->in_sigsuspend = 0;
	    if (self->targetMutex != NULL) {
		setOwner(self);
	    }
	} while (self->suspended);
    }
}

void
CVMmutexSetOwner(CVMThreadID *self, CVMMutex *m, CVMThreadID *target)
{
    /* block out other suspends, etc... */
    pthread_mutex_lock(&target->lock);

    {
	int prio;
	CVMthreadGetPriority(self, &prio);
	CVMthreadGetPriority(target, &target->lastPriority);
	if (prio > target->lastPriority) {
	  /* Dio this is not the place to do
	   * PI. This interfere with our PI so
	   * we comment it out
	   */

	  //CVMthreadSetPriority(target, prio);
	}
    }

    target->targetMutex = m;

    /* send signal */
    pthread_kill(POSIX_COOKIE(target), SIGUSR1);

    {
	int status;
	do {
	    status = sem_wait(&setowner_sem);
	} while (status == -1 && errno == EINTR);
    }
  
    assert(target->targetMutex == NULL);

    pthread_mutex_unlock(&target->lock);
}

/* Dio: This are converted to priority queues to have a predictable
 * notification about who should be notified first.
 * 
 * This is going to rely only on rt_priority given that that is the way 
 * the priority inheritance is implemented in RK and that the regular
 * priority is "boosted" and therefore should not be used when we want
 * "predictable" behavior in a real-time task set.
 */

/* Dio: we move the priority ordering to dequeue time
 */

#if 0

static void
enqueue(CVMCondVar *c, CVMThreadID *t)
{
  CVMThreadID *p = c->waiters;
  struct sched_param t_schp, p_schp;
  int t_policy, p_policy;

  if (p ==NULL)
    {
      c->waiters = p;
      *c->last_p = t;
      t->prev_p = c->last_p;
      c->last_p = &t->next;
      t->next = NULL;
    }
  else
    {
      /* search for smaller priority */
      pthread_getschedparam(t->pthreadCookie,&t_policy, &t_schp);
      pthread_getschedparam(p->pthreadCookie, &p_policy, &p_schp);

      while ( (p != NULL) && (p_schp.sched_priority  > p_schp.sched_priority))
	{
	  p = p->next;
	  if (p != NULL)
	    pthread_getschedparam(p->pthreadCookie, &p_policy, &p_schp);	  
	}
	  
      if (p == NULL)
	{
	  /* insert as the last */
	  *c->last_p = t;
	  t->prev_p = c->last_p;
	  c->last_p = &t->next;
	  t->next = NULL;
	}
      else
	{
	  /* insert it before p */
	  if (p == c->waiters)
	    {
	      /* first one */
	      t->next = c->waiters;
	      t->next->prev_p = &t->next;
	      c->waiters = t;
	      t->prev_p = &c->waiters;
	    }
	  else
	    {
	      /* in the middle */
	      *p->prev_p = t;
	      t->prev_p = p->prev_p;
	      t->next = p;
	      p->prev_p = &t->next;
	    }
	}
    }
}

#endif

static void
enqueue(CVMCondVar *c, CVMThreadID *t)
{
    *c->last_p = t;
    t->prev_p = c->last_p;
    c->last_p = &t->next;
    t->next = NULL;
}

static void
dequeue_me(CVMCondVar *c, CVMThreadID *t)
{
    *t->prev_p = t->next;

    if (t->next != NULL) {
	t->next->prev_p = t->prev_p;
    }

    if (c->last_p == &t->next) {
	c->last_p = &c->waiters;
    }

    t->next = NULL;
    t->prev_p = NULL;
}


/* Dio: we are doing the priority ordering at dequeue time
 * to take into account any changed to the priority that could
 * happened while the threads were blocked
 */

static CVMThreadID *
dequeue(CVMCondVar *c)
{
    struct sched_param c_schp, h_schp;
    int c_policy, h_policy;
    CVMThreadID *ct,*h;
    h = c->waiters;

    if (h != NULL) {
      
      ct = h->next; 
      pthread_getschedparam(h->pthreadCookie, &h_policy, &h_schp);
       
      while (ct != NULL) 
	{
	  pthread_getschedparam(ct->pthreadCookie,&c_policy, &c_schp);
	  
	  if (c_schp.sched_priority > h_schp.sched_priority)
	    {
	      /* new highest priority*/
	      h = ct;
	      h_schp.sched_priority = c_schp.sched_priority; // note this is not a full copy of the structure
	    }
	  ct = ct->next;
	}
      dequeue_me(c, h);
    }
    return h;
}

#if 0
static CVMThreadID *
dequeue(CVMCondVar *c)
{
    CVMThreadID *t = c->waiters;
    if (t != NULL) {
	c->waiters = t->next;

	if (t->next != NULL) {
	    t->next->prev_p = t->prev_p;
	}

	if (c->last_p == &t->next) {
	    c->last_p = &c->waiters;
	}
	t->next = NULL;
	t->prev_p = NULL;
    }
    return t;
}
#endif



CVMBool
CVMcondvarWait(CVMCondVar* c, CVMMutex* m, CVMJavaLong millis)
{
    CVMBool waitForever;
    CVMThreadID * self = CVMthreadSelf();
    struct timespec timeout;

    if (millis == 0LL) {
	waitForever = CVM_TRUE;
    } else {
	struct timeval tv;
	long secs;
	long long tmp;
        gettimeofday(&tv, NULL);

	waitForever = CVM_FALSE;
	tmp = millis / 1000LL;
	secs = (long)tmp;
	if (secs != tmp) {
	    /* overflow */
	    waitForever = CVM_TRUE;
	} else {
	    timeout.tv_sec = tv.tv_sec + secs;
	    timeout.tv_nsec = tv.tv_usec * 1000;
	    timeout.tv_nsec += (millis % 1000) * 1000000;
            if (timeout.tv_nsec >= 1000000000) {
                ++timeout.tv_sec;
                timeout.tv_nsec -= 1000000000;
            }
            assert(timeout.tv_nsec < 1000000000);
            if (timeout.tv_sec < tv.tv_sec) {
                /* overflow */
		waitForever = CVM_TRUE;
            }
	}
    }

    pthread_mutex_lock(&self->lock);

    assert(self->value == 0);

    if (self->interrupted) {
	self->interrupted = CVM_FALSE;
	pthread_mutex_unlock(&self->lock);
	return CVM_FALSE;
    }

    self->in_wait = CVM_TRUE;
    self->notified = CVM_FALSE;

    /* add ourself to the wait queue that notify and notifyAll look at */
    enqueue(c, self);

    CVMmutexUnlock(m);

    /* notifies can happen now */

    pthread_mutex_unlock(&self->lock);

    /* thread can be interrupted now */

    /* emulate semaphore wait */
    pthread_mutex_lock(&self->wait_mutex);

    while (self->value == 0) {
	if (waitForever) {
	    pthread_cond_wait(&self->wait_cv, &self->wait_mutex);
	} else {
	    int status = pthread_cond_timedwait(&self->wait_cv,
		&self->wait_mutex, &timeout);
	    if (status == ETIMEDOUT) {
		break;
	    }
	}
    }

    pthread_mutex_unlock(&self->wait_mutex);

    /* Make sure we're not supposed to be still suspended before getting on
       with reacquiring the mutex lock: */
    pthread_mutex_lock(&self->lock);
    while (self->suspended) {
        pthread_cond_wait(&self->suspend_cv, &self->lock);
    }
    self->in_wait = CVM_FALSE;
    pthread_mutex_unlock(&self->lock);

    /* reacquire condvar mutex */
    CVMmutexLock(m);

    /* no more notifications after this point */

    {
	/* find out why we were woken up */
	if (self->notified) {
	    /* normal notify or notifyAll */
	    assert(self->value == 1);
	    /* removed from wait queue */ 
	    assert(self->next == NULL && self->prev_p == NULL);
	} else {
	    /*
	     * We were woken up by linuxSyncInterruptWait() posting on
	     * the semaphore, or the timed wait timed out.
	     */
	    assert(self->interrupted || !waitForever);
	    dequeue_me(c, self);
	    /* removed from wait queue */ 
	    assert(self->next == NULL && self->prev_p == NULL);
	}
    }

    /*
     * Workaround for Linux kernel bug exposed by LinuxThreads.
     */
    setFPMode();

    self->value = 0;

    if (self->interrupted) {
	self->interrupted = CVM_FALSE;
	return CVM_FALSE;
    }

    return CVM_TRUE;

}

/* emulate a semaphore post */
#define SEM_POST(t) 				\
do {						\
    pthread_mutex_lock(&(t)->wait_mutex);	\
    (t)->value = 1;				\
    pthread_cond_signal(&(t)->wait_cv);		\
    pthread_mutex_unlock(&(t)->wait_mutex);	\
}while (0)					



#undef CVMcondvarInit
#undef CVMcondvarDestroy

CVMBool CVMcondvarInit(CVMCondVar* c ,CVMMutex* m)
{
   
    c->waiters = NULL;
    c->last_p = &c->waiters;

    return POSIXcondvarInit(&(c)->pcv, &(m)->pmtx);
   
}


void 
CVMcondvarDestroy(CVMCondVar * c)
{
    POSIXcondvarDestroy(&(c)->pcv);
}

void
CVMcondvarNotify(CVMCondVar * c)
{
    CVMThreadID *t;
    if ((t = dequeue(c)) != NULL) {
	t->notified = CVM_TRUE;
	SEM_POST(t);
    }
}

void
CVMcondvarNotifyAll(CVMCondVar * c)
{
    CVMThreadID *t;
    while ((t = dequeue(c)) != NULL) {
	t->notified = CVM_TRUE;
	SEM_POST(t);
    }
}

void
linuxSyncInterruptWait(CVMThreadID *thread)
{
    pthread_mutex_lock(&thread->lock);
    if (thread->in_wait) {
	/* thread->notified is not set */
	/* thread will dequeue itself from wait queue */
	SEM_POST(thread);
    }
    pthread_mutex_unlock(&thread->lock);
}

void
linuxSyncSuspend(CVMThreadID *t)
{
    if (POSIX_COOKIE(t) == thr_self()) {
	t->suspended = 1;
	pthread_kill(POSIX_COOKIE(t), SIGUSR1);
    } else {
	mutex_lock(&t->lock);
	if (!t->suspended) {
	    t->suspended = 1;
	    /*
	     * If the thread is performing a wait, we do special handling,
	     * to avoid the use of signals.
	     */
	    if (t->in_wait) {
		t->suspended_in_wait = 1;
	    } else {
		pthread_kill(POSIX_COOKIE(t), SIGUSR1);
	    }
	}
	mutex_unlock(&t->lock);
    }
}

void
linuxSyncResume(CVMThreadID *t)
{
    pthread_mutex_lock(&t->lock);
    if (t->suspended) {
	t->suspended = 0;
	if (t->suspended_in_wait) {
	    t->suspended_in_wait = 0;
	    pthread_cond_signal(&t->suspend_cv);
	} else {
	    pthread_kill(POSIX_COOKIE(t), SIGUSR1);
	}
    }
    pthread_mutex_unlock(&t->lock);
}

/*
 * Linux doesn't handle MT core dumps very well, so suspend here
 * for the debugger.
 */
static void crash(int sig)
{
    fprintf(stderr, "Process received signal %d, suspending\n", sig);
    kill(getpid(), SIGSTOP);
}

void
linuxSyncInit(void)
{
    struct sigaction sa;
    sa.sa_handler = sigfunc0;	/* mutex set owner and suspend */
    sa.sa_flags = 0;
    sigemptyset(&sa.sa_mask);
    sigaction(SIGUSR1, &sa, NULL);

    {
	int signals[] = {SIGSEGV, SIGBUS, SIGILL, SIGFPE};
	int i;

	for (i = 0; i < (sizeof signals / sizeof signals[0]); ++i) {
	    struct sigaction sa;
	    sa.sa_handler = crash;
	    sa.sa_flags = 0;
	    sigemptyset(&sa.sa_mask);
	    sigaction(signals[i], &sa, NULL);
	}
    }

    sem_init(&setowner_sem, 0, 0);

    /*temporary -Kalpesh*/
    if(signal(SIGUSR2,sig_iounblock)==SIG_ERR)
 	printf("Error in setting signal handler\n");
    if(siginterrupt(SIGUSR2,1)<0)
 	printf("IO can not be interrupted.\n");


}



