/*
 * -----------------------------------------------------------------
 * RTMonitorAdapter.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


#include "com_timesys_CoreExtension_RTMonitorAdapter.h"
#include <stdio.h>
#include <unistd.h>

#include "jni.h"
#include "jvmcei.h"

#include "javavm/include/defs.h"
#include "javavm/include/sync.h"
#include "javavm/include/indirectmem.h"
#include "javavm/include/common_exceptions.h"
#include "javavm/include/objects.h"

#include <rk/rk.h>
#include <rk/rk_error.h>
//#include <rk/timespec.h>
#include <pthread.h>
#include "rtmonitoradapter.h"

/* force monitor debug */
#if 0
#define CVM_DEBUG_MONITOR_1 true
#endif

static JNIEnv *theEnv;
static struct jvmcei * theJVMCei;

/* We have a global variable for the default monitor control because we cannot have it 
 * in a java class given that to access a java class we need to have the system monitors in 
 * place. Once they are in place, we can use this to change this default.
 */

static int defaultMonitorControlType = PRIORITY_INHERITANCE; //NONE;
static int defaultCeiling = 0;

int maximumConcurrentLocks = -1;
int concurrentLocks = 0;
int reservedExceptionsLocks =0;

void monitorAdapterSetMaximumConcurrentLocks(int num)
{
  maximumConcurrentLocks = num;
}

int monitorAdapterGetMaximumConcurrentLocks()
{
  return maximumConcurrentLocks;
}

int monitorAdapterGetConcurrentLocks()
{
  return concurrentLocks;
}

int monitorAdapterSetDefaultMonControl(int type, int ceiling)
{
  defaultMonitorControlType = type;
  defaultCeiling = ceiling;
  return 0;
}

int monitorAdapterGetDefaultMonControl()
{
  return defaultMonitorControlType;
}

int monitorAdapterGetMonitorCeiling()
{
  return defaultCeiling;
}

#if 0
int afterCreateObjectFilterHandler(void *p)
{      
 jclass clazz;
 jmethodID mID;

 jobject monControlObject;

 jobject obj;

 struct  arg_t {CVMObjectICell *object } *arg;

 arg = (struct arg_t *) p;

 obj = (jobject) arg->object;

 clazz = (*theEnv)->FindClass(theEnv, "javax/realtime/MonitorControl");
 
 if (clazz == NULL)
   {
     printf("RTMonitorAdapter: cannot find MonitorControl class\n");
     return -1;
   }

 mID = (*theEnv)->GetStaticMethodID(theEnv, clazz, "getDefaultMonitorControl", "()Ljavax/realtime/MonitorControl;");

 if(mID == NULL)
     {
	 printf("RTMonitorAdapter: Could not find static method ID: MonitorControl.getDefaultMonitorControl\n");
	 return -1;
     }

 monControlObject = (*theEnv)->CallStaticObjectMethod(theEnv, clazz, mID);


 mID = (*theEnv)->GetStaticMethodID(theEnv, clazz, "setMonitorControl","(Ljava/lang/Object;Ljavax/realtime/MonitorControl;)V");

 (*theEnv)->CallStaticVoidMethod(theEnv, clazz,mID, obj, monControlObject);
 
 
 return JVMCEI_CONTINUE_CURRENT_CALL;
}

#endif

struct arg_bcm_t { JNIEnv *env; CVMObjMonitor **pmon; CVMObject *obj;} *arg_bcm_p;

/* Whenever a particular monitor control is set for a specific object, the monitor is going to
 * be created at the association event, meaning whenever setMonitorControl is called.
 * In any other case we just need to check at the default MonitorControl to create the proper 
 * mutex.
 */


int beforeCreateMonitorFilterHandler(void *p)
{
  CVMObject *obj;
  CVMObjMonitor **mp;
  //jmethodID mID;
  JNIEnv *env;

  arg_bcm_p = (struct arg_bcm_t *) p;
  env = arg_bcm_p->env;
  obj =  arg_bcm_p->obj;
  mp = arg_bcm_p->pmon;

  /* we only need to take into account the case were the defaultMonitorControl is 
   * not equal to NONE. If an object is assigned a separate policy, then the
   * monitor will be created at assignment time and will remain attached to
   * the object for its lifetime
   */

  if (defaultMonitorControlType != NONE)
    {

#ifdef CVM_DEBUG_MONITOR_1
      printf("RTMonitorAdapter: creating PI monitor\n");
#endif

      /* If reached maximum concurrent locks let it continue to
       * fail later
       */
      if ((maximumConcurrentLocks != -1) && (concurrentLocks +1 > maximumConcurrentLocks))
	{
	  CVMExecEnv*    ee  = CVMjniEnv2ExecEnv(env);
	  CVMthrowResourceLimitError(ee, "Maximum Concurrent Locks Exceeded");
	  return JVMCEI_CONTINUE_CURRENT_CALL;
	}

      (*mp) = calloc(1,sizeof(CVMObjMonitor));
      (*mp)->mon.monitorControl = defaultMonitorControlType;
      (*mp)->mon.ceiling = defaultCeiling;		    
      CVMobjMonitorInit((*mp), NULL, 0);
      (*mp)->state = CVM_OBJMON_BOUND;

      /* Make it sticky so it will not be scavenged */

      (*mp)->sticky = 1;
      (*mp)->obj = obj;

#ifdef CVM_DEBUG
      (*mp)->owner = NULL;
      (*mp)->magic = CVM_OBJMON_MAGIC;
#endif

      concurrentLocks++;

      return JVMCEI_ABORT_CURRENT_CALL;
    }
  else
    {
      return JVMCEI_CONTINUE_CURRENT_CALL;
    }
}

struct arg_bim_t { CVMSysMonitor **pmon;} *arg_bim_p;

int beforeInitMonitorFilterHandler(void *p)
{
  CVMSysMonitor **mp;
  arg_bim_p = (struct arg_bim_t *) p;
  mp = arg_bim_p->pmon;

#ifdef CVM_DEBUG_MONITOR_1
  printf("RTMonitorAdapter: BEFORE_INIT_MONITOR_FILTER monitorControl(%d)\n",
	 (*mp)->monitorControl);
#endif

  if ((*mp)->monitorControl == PTHREAD_PRIO_INHERIT)
    {
      /* init the mutex with the proper protocol */
      pthread_mutexattr_t mtx_attr;

#ifdef CVM_DEBUG_MONITOR_1
      printf("RTMonAdaptor: initializing mutex %p protocol(%d) sysMonitor(%p)\n",
	     &((*mp)->rmutex.mutex), (*mp)->monitorControl, (*mp) );
      __mutex_tracking = &((*mp)->rmutex.mutex);
#endif

      pthread_mutexattr_init(&mtx_attr);	
      pthread_mutexattr_setprotocol(&mtx_attr, (*mp)->monitorControl);
      pthread_mutexattr_settype(&mtx_attr, PTHREAD_MUTEX_NORMAL);
      
      pthread_mutex_init((pthread_mutex_t *) &((*mp)->rmutex.mutex),&mtx_attr);     
      return JVMCEI_ABORT_CURRENT_CALL;
    }
  else
    {
      return JVMCEI_CONTINUE_CURRENT_CALL;
    }
}

void RTMonAdaptorStaticFilterInitPIMutex(CVMSysMutex *p, char *s, int num)
{
  pthread_mutexattr_t mtx_attr;
  pthread_mutexattr_init(&mtx_attr);	
  pthread_mutexattr_setprotocol(&mtx_attr, PTHREAD_PRIO_INHERIT);
  pthread_mutexattr_settype(&mtx_attr, PTHREAD_MUTEX_NORMAL);
      
  pthread_mutex_init((pthread_mutex_t *) &(p->rmutex.mutex),&mtx_attr);     

  p->name = s;
  p->rank = num;
#ifdef CVM_DEBUG
  p->nextOwned = NULL;
#endif
}

struct arg_bfm_t { CVMObjMonitor **pmon;} *arg_bfm_p;

int beforeFreeMonitorFilterHandler(void *p)
{
  CVMObjMonitor **mp;
  arg_bfm_p = (struct arg_bfm_t *) p;
  mp = arg_bfm_p->pmon;

  if ((*mp)->mon.monitorControl != 0)
    {
      CVMobjMonitorDestroy((*mp));
      return JVMCEI_ABORT_CURRENT_CALL;
    }
  else
    {
      return JVMCEI_CONTINUE_CURRENT_CALL;
    }
}

int beforeMonitorOperationFilterHandler(void *p)
{
  CVMExecEnv *ee;
  CVMObject *obj;
  int retVal = JVMCEI_CONTINUE_CURRENT_CALL;
  CVMObjMonitor *mon;
  struct arg_t {CVMExecEnv *ee; CVMObject *obj;} *parg;
  parg = (struct arg_t *) p;
  
  ee = parg->ee;
  obj = parg->obj;

  {

    if (CVMobjMonitorState(obj) != CVM_LOCKSTATE_MONITOR)
      {
	/* Create the monitor */
	mon = calloc(1, sizeof(CVMObjMonitor));
	if (mon == NULL)
	  {
	    retVal = JVMCEI_ABORT_CURRENT_CALL;
	  }
	else
	  {
	    mon->mon.monitorControl = defaultMonitorControlType;
	    mon->mon.ceiling = defaultCeiling;


	    CVMobjMonitorInit(mon, NULL, 0);
	    mon->state = CVM_OBJMON_BOUND;
	    mon->bits = obj->hdr.various32;
	    mon->obj = obj;

	    CVMobjMonitorSet(obj, mon, CVM_LOCKSTATE_MONITOR);
	  }
      }
  }
  
  return JVMCEI_CONTINUE_CURRENT_CALL;
}

JNIEXPORT jint JNICALL Java_com_timesys_CoreExtension_RTMonitorAdapter_bootNative
  (JNIEnv * env, jclass clazz)
{
  theEnv = env;

  theJVMCei = (*env)->GetJVMCeiInterface();

  /* install the filter */

  //  theJVMCei->installFilter(JVMCEF_AFTER_CREATE_OBJECT, afterCreateObjectFilterHandler);
  theJVMCei->installFilter(JVMCEF_BEFORE_CREATE_MONITOR, beforeCreateMonitorFilterHandler);
  theJVMCei->installFilter(JVMCEF_BEFORE_INIT_MONITOR, beforeInitMonitorFilterHandler);
  theJVMCei->installFilter(JVMCEF_BEFORE_FREE_MONITOR, beforeFreeMonitorFilterHandler);
  theJVMCei->installFilter(JVMCEF_BEFORE_MONITOR_OPERATION, beforeMonitorOperationFilterHandler);

  /* set up done. Everything else is taken care of by the filter
   * handlers
   */
  return 0;
}
