/*
 * -----------------------------------------------------------------
 * Dispatcher.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

#include "com_timesys_Dispatcher.h"
#include "AsyncEvent.h"
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>


JNIEXPORT void JNICALL Java_com_timesys_Dispatcher_runDispatcher
  (JNIEnv* env, jobject this)
{
  int signum=0;
  int status;
  jclass Dispatcher, AsyncEvent;
  jmethodID eventHappened;
  jmethodID setPID;
  sigset_t signal_set;


#ifdef CVM_DEBUG
  printf("Dispatcher native code started\n");
#endif
  
  
  /****************************/
  /* Cache AsyncEvent methods */

  /* retrieve a ref to the java method we call when we recieve */
  /* an event for which there are handlers */
  AsyncEvent = (*env)->FindClass(env, "javax/realtime/AsyncEvent");
  if(!AsyncEvent){
    printf("ERROR: callback jclass NULL\n");
    return;
  }

  eventHappened = (*env)->GetStaticMethodID(env,AsyncEvent,"eventHappened","(I)V");
  if(eventHappened == NULL){
    printf("ERROR: callback jmethodID NULL\n");
    return;
  }

  /***********************/
  /* Set up  signal sets */

  sigemptyset(&signal_set);

  /* add signals 1 to N_HAPPS (equivelant to _NSIG) */
  for(signum=1;signum<=N_HAPPS;signum++)
  {
      if(signum != SIGUSR1)
        sigaddset(&signal_set, signum);
  }
  
  /* this is the non-obvious part of using sigwait, see `man sigwait` */
  status = pthread_sigmask(SIG_BLOCK, &signal_set, NULL);
  if(status!=0){
    printf("ERROR: pthread_sigmask returned %d\n",status);
    return;
  }

#ifdef CVM_DEBUG
  printf("Dispatcher pid: %d\n",getpid());
  printf("Dispatcher suspending...\n");
#endif


  /**************************/
  /* Record dispatchers pid */
  
  /* retrieve a ref to the private setPID method in the */
  /* com.timesys.Dispatcher class and give it our pid */
  Dispatcher = (*env)->FindClass(env, "com/timesys/Dispatcher");
  if(!Dispatcher){
    printf("ERROR: DispatcherInfo not found\n");
    return;
  }
  
  setPID = (*env)->GetStaticMethodID(env,Dispatcher,"setPID","(I)V");
  if(!setPID){
    printf("ERROR: Dispatcher setPID not found\n");
    return;
  }
  
  (*env)->CallStaticVoidMethod(env,Dispatcher,setPID,(jint)getpid());
  if((*env)->ExceptionCheck(env))
    (*env)->ExceptionDescribe(env);
  
  /**************************************************/
  /* Wait for event and dispatch it when it happens */

  while(1)
    {
#ifdef CVM_DEBUG
      printf("sigwaiting\n");
#endif
      /* suspend Dispatcher thread (us) until signal reception */
      sigwait(&signal_set, &signum);
 
#ifdef CVM_DEBUG
      printf("recieved signal %d\n",signum);
#endif
      (*env)->CallStaticVoidMethod(env,AsyncEvent,eventHappened,signum);
#ifdef CVM_DEBUG
      printf("back to native code\n");
#endif
      if((*env)->ExceptionCheck(env)){
#ifdef CVM_DEBUG
	printf("exception occured during callback to eventHappened\n");
#endif
	return;
      }
    }
}
