/*
 * -----------------------------------------------------------------
 * RMScheduler.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

#include "com_timesys_RMScheduler.h"
#include <jni.h>
#include <rk/rk.h>
#include <rk/rk_error.h>
#include <stdio.h>

JNIEXPORT jint JNICALL Java_com_timesys_RMScheduler_createResourceSet0
  (JNIEnv *env, jobject this)
{
  jint res;
  //printf("RMScheduler: creating the resource set\n");
  res = (jint) rk_resource_set_create("java");
  //printf("RMScheduler: after creating the resource set rs(%x)\n", (unsigned int) res);
  return res;
}

JNIEXPORT jint JNICALL Java_com_timesys_RMScheduler_deleteResourceSet0
  (JNIEnv *env, jobject this, jint rs)
{
  return rk_resource_set_destroy((rk_resource_set_t)rs);
}

JNIEXPORT jint JNICALL Java_com_timesys_RMScheduler_createCPUReserve0
  (JNIEnv *env, jobject this, jint rs, jlong start_milliseconds, jint start_nanoseconds , jlong period_milliseconds, jint period_nanoseconds, jlong cost_milliseconds, jint cost_nanoseconds, jlong deadline_milliseconds, jint deadline_nanoseconds)
{
    jint res;
    cpu_reserve_attr_data_t attr;

    attr.start_time.tv_sec = start_milliseconds / 1000;
    attr.start_time.tv_nsec = (start_milliseconds % 1000) * 1000 * 1000 + start_nanoseconds;

    attr.compute_time.tv_sec = cost_milliseconds / 1000;
    attr.compute_time.tv_nsec = (cost_milliseconds % 1000) * 1000 * 1000 + cost_nanoseconds;

    attr.period.tv_sec = period_milliseconds / 1000;
    attr.period.tv_nsec = (period_milliseconds % 1000) * 1000 * 1000 + period_nanoseconds;

    attr.deadline.tv_sec = deadline_milliseconds / 1000;
    attr.deadline.tv_nsec = (deadline_milliseconds % 1000) * 1000 * 1000 + deadline_nanoseconds;

    attr.blocking_time.tv_sec = 0;
    attr.blocking_time.tv_nsec = 0;

    /* To do: we need an interface to allow the programmer to set these parameters */
    attr.reserve_type.enf_mode = 
	attr.reserve_type.rep_mode =
	attr.reserve_type.sch_mode = RSV_HARD;

    CVMdebugPrintf(("RMScheduler: creating the cpu reserve\n"));
    CVMdebugPrintf(("\trs(%x), c_sec(%d), c_nsec(%d), t_sec(%d), t_nsec(%d)\n",
	   rs, attr.compute_time.tv_sec,
	   attr.compute_time.tv_nsec,
	   attr.period.tv_sec,
	   attr.period.tv_nsec));

    res = (jint) rk_cpu_reserve_create((rk_resource_set_t)rs, &attr);

    //printf("RMScheduler: after creating the cpu reserve\n");

    return res;
}



