/*
 * @(#)java_props_md.c	1.9 00/11/09
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#include <pwd.h>
#include <locale.h>
/*
 DMRLNX: force ARCH definition to i386 string.
 Force ARCH to be set to i386 for now
 sys/systeminfo.h does not appear to exist...
*/
#define ARCH	"i386"
#ifndef ARCH
#include <sys/systeminfo.h>	/* For os_arch */
#endif
#include <sys/utsname.h>	/* For os_name and os_version */
#include <langinfo.h>           /* For nl_langinfo */
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/param.h>
#include <time.h>

#include "locale_str.h"
#include "java_props.h"

/* Following commented out to get rid of compiler warning
 *
 * ../../src/linux/native/java/lang/java_props_md.c:36: warning: `altzone' defined but not used
 *
 * - Pratik (10th Sep 2001)
 */
//#ifdef HAVE_ALTZONE
//#else
//static long altzone = 0;
//#endif

#ifdef ALT_CODESET_KEY
#define CODESET ALT_CODESET_KEY
#endif

#ifndef __USE_XOPEN
#define CODESET _NL_CTYPE_CODESET_NAME
#else
#if __GLIBC__ == 2 && __GLIBC_MINOR__ == 0
#warning [jk] glibc 2.0 defines __USE_XOPEN but doesn"'"t have CODESET
#define CODESET _NL_CTYPE_CODESET_NAME
#endif
#endif

/* Take an array of string pairs (map of key->value) and a string (key).
 * Examine each pair in the map to see if the first string (key) matches the
 * string.  If so, store the second string of the pair (value) in the value and
 * return 1.  Otherwise do nothing and return 0.  The end of the map is
 * indicated by an empty string at the start of a pair (key of "").
 */
static int
mapLookup(char* map[], const char* key, char** value) {
    int i;
    for (i = 0; strcmp(map[i], ""); i += 2){
        if (!strcmp(key, map[i])){
            *value = map[i + 1];
            return 1;
        }
    }
    return 0;
}

#ifndef P_tmpdir
#define P_tmpdir "/var/tmp"
#endif

void
CVMgetJavaProperties(java_props_t *sprops)
{
    const char *v; /* tmp var */

    memset(sprops, 0, sizeof *sprops);
    if (sprops->user_dir) {
        return;
    }

    /* tmp dir */
    sprops->tmp_dir = P_tmpdir;

    /* Printing properties */
    sprops->printerJob = "sun.awt.motif.PSPrinterJob";

    /* Java 2D properties */
    sprops->graphics_env = "sun.awt.X11GraphicsEnvironment";
    sprops->awt_toolkit = NULL;

    v = getenv("JAVA_FONTS");
    /*
    sprops->font_dir = v ? v :
    "/usr/openwin/lib/X11/fonts/Type1:/usr/openwin/lib/X11/fonts/TrueType";
    */

    /* If JAVA_FONTS is not set, the system-dependent fontpath will
     * be evaluated later
     */
    sprops->font_dir = v ? v : "";

#ifdef SI_ISALIST
    /* supported instruction sets */
    {
        char list[258];
        sysinfo(SI_ISALIST, list, sizeof(list));
        sprops->cpu_isalist = strdup(list);
    }
#else
    sprops->cpu_isalist = NULL;
#endif

    /* endianness of platform */
    {
        unsigned int endianTest = 0xff000000;
        if (((char*)(&endianTest))[0] != 0)
            sprops->cpu_endian = "big";
        else
            sprops->cpu_endian = "little";
    }

    /* os properties */
    {
        struct utsname name;
	//char arch[12];
	uname(&name);
	sprops->os_name = strdup(name.sysname);
	sprops->os_version = strdup(name.release);

#ifdef ARCH
	sprops->os_arch = ARCH;
#else
	sysinfo(SI_ARCHITECTURE, arch, sizeof(arch));
	if (strcmp(arch,"sparc") == 0 ) {
	    sprops->os_arch = "sparc";
	} else if (strcmp(arch,"i386") == 0 ) {
	    /* use x86 to match the value received on win32 */
	    sprops->os_arch = "x86";
	} else if (strcmp(arch,"ppc") == 0 ) {
	    sprops->os_arch = "ppc";
	} else {
            sprops->os_arch = "Unknown";
	}
#endif
    }

    /* Determing the language, country, and encoding from the host,
     * and store these in the user.language, user.region, and
     * file.encoding system properties. */
    {
        char *lc;
        lc = setlocale(LC_CTYPE, "");
#ifndef __linux__
        if (lc == NULL) {
            /*
             * 'lc == null' means system doesn't support user's environment
             * variable's locale.
             */
          setlocale(LC_ALL, "C");
          sprops->language = "en";
          sprops->encoding = "8859_1";
        } else {
#else
        if (lc == NULL || !strcmp(lc, "C") || !strcmp(lc, "POSIX")) {
            lc = "en_US";
        }
        {
#endif
            /*
             * locale string format in Solaris is
             * <language name>_<region name>.<encoding name>
             * <region name> and <encoding name> are optional.
             */
            char temp[64], *language = NULL, *region = NULL, *encoding = NULL;
            char *std_language = NULL, *std_region = NULL,
                 *std_encoding = NULL;
            char region_variant[64], *variant = NULL, *std_variant = NULL;
            char *p, encoding_variant[64];

            strcpy(temp, lc);
            setlocale(LC_ALL, lc);

            /* Parse the language, region, encoding, and variant from the
             * locale.  Any of the elements may be missing, but they must occur
             * in the order language_region.encoding@variant, and must be
             * preceded by their delimiter (except for language).
             *
             * If the locale name (without .encoding@variant, if any) matches
             * any of the names in the locale_aliases list, map it to the
             * corresponding full locale name.  Most of the entries in the
             * locale_aliases list are locales that include a language name but
             * no country name, and this facility is used to map each language
             * to a default country if that's possible.  It's also used to map
             * the Solaris locale aliases to their proper Java locale IDs.
             */
            if ((p = strchr(temp, '.')) != NULL) {
                strcpy(encoding_variant, p); /* Copy the leading '.' */
                *p = '\0';
            } else if ((p = strchr(temp, '@')) != NULL) {
                 strcpy(encoding_variant, p); /* Copy the leading '@' */
                 *p = '\0';
            } else {
                *encoding_variant = '\0';
            }
            
            if (mapLookup(locale_aliases, temp, &p)) {
                strcpy(temp, p);
            }
            
            language = temp;
            if ((region = strchr(temp, '_')) != NULL) {
                *region++ = '\0';
            }
            
            p = encoding_variant;
            if ((encoding = strchr(p, '.')) != NULL) {
                p[encoding++ - p] = '\0';
                p = encoding;
            }
            if ((variant = strchr(p, '@')) != NULL) {
                p[variant++ - p] = '\0';
            }

            /* Normalize the language name */
            std_language = "en";
            if (language != NULL) {
                mapLookup(language_names, language, &std_language);
            }
            sprops->language = std_language;

            /* Normalize the variant name.  Do this BEFORE handling the region
             * name, since the variant name will be incorporated into the
             * user.region property.
             */
            if (variant != NULL) {
                mapLookup(variant_names, variant, &std_variant);
            }

            /* Normalize the region name.  If there is a std_variant, then
             * append it to the region.  This applies even if there is no
             * region, in which case the empty string is used for the region.
             * Note that we only use variants listed in the mapping array;
	     * others are ignored.
             */
            *region_variant = '\0';
            if (region != NULL) {
                std_region = region;
                mapLookup(region_names, region, &std_region);
                strcpy(region_variant, std_region);
            }
            if (std_variant != NULL) {
               strcat(region_variant, "_");
               strcat(region_variant, std_variant);
            }
            if ((*region_variant) != '\0') {
               sprops->region = strdup(region_variant);
            }

            /* Normalize the encoding name.  Note that we IGNORE the string
             * 'encoding' extracted from the locale name above.  Instead, we 
             * use the more reliable method of calling nl_langinfo(CODESET).  
             * This function returns an empty string if no encoding is set for
             * the given locale (e.g., the C or POSIX locales); we use the
             * default ISO 8859-1 converter for such locales.  We don't need 
             * to map from the Solaris string to the Java identifier at this 
             * point because that mapping is handled by the character
             * converter alias table in CharacterEncoding.java.
             */
            p = nl_langinfo(CODESET);
	    if (*p == '\0' || strcmp(p, "ANSI_X3.4-1968") == 0) {
		std_encoding = "8859_1";
	    } else {
		std_encoding = p;
	    }
#ifdef __linux__
            /*  
             * Remap the encoding string to a different value for japanese
             * locales on linux so that customized converters are used instead
             * of the default converter for "EUC-JP". The customized converters
             * omit support for the JIS0212 encoding which is not supported by
             * the variant of "EUC-JP" encoding used on linux
             */
            if (strcmp(p, "EUC-JP") == 0) {
                std_encoding = "EUC-JP-LINUX";
            }
#endif
            sprops->encoding = std_encoding;
        }
    }
    
    sprops->unicode_encoding = "UnicodeLittle";

    /* user properties */
    {
        struct passwd *pwent = getpwuid(getuid());
	sprops->user_name = pwent ? strdup(pwent->pw_name) : "?";
	sprops->user_home = pwent ? strdup(pwent->pw_dir) : "?";
    }

    /* User TIMEZONE */
    {
	/*
	 * We defer setting up timezone until it's actually necessary.
	 * Refer to TimeZone.getDefault(). However, the system
	 * property is necessary to be able to be set by the command
	 * line interface -D. Here temporarily set a null string to
	 * timezone.
	 */
	tzset();	/* for compatibility */
	sprops->timezone = "";
    }

    /* Current directory */
    {
        char buf[MAXPATHLEN];
        getcwd(buf, sizeof(buf));
	sprops->user_dir = strdup(buf);
    }

    sprops->file_separator = "/";
    sprops->path_separator = ":";
    sprops->line_separator = "\n";

    /*
     * Now for some path information
     *
     * For the defaults, assume that we are
     * in <cvmroot>/build/<platform>/bin
     */

    /* System classes */
    sprops->sysclasspath = J2ME_CLASSESJAR;    /* System classes */

    sprops->ext_dirs     = "../extclasses.zip";   /* Extension classes */
    sprops->dll_dir      = "../lib";              /* Where to find libraries */
    sprops->library_path = "../lib";              /* Where to find libraries */
    sprops->java_home    = "..";                  /* Installation directory */
}
