/*
 * -----------------------------------------------------------------
 * AsynchronouslyInterruptedException.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


#include "javax_realtime_AsynchronouslyInterruptedException.h"
#include "javavm/include/interpreter.h"

#include "javavm/include/globals.h"
#include "javavm/include/indirectmem.h"
#include "javavm/include/sync.h"
#include "javavm/include/gc_common.h"

#include <sys/types.h>
#include <signal.h>

#include "generated/offsets/java_lang_Thread.h"

CVMExecEnv* ThreadObj2ExecEnv(JNIEnv *env, jobject thread);

/* Inaccessible static: genericAie */


/*
 * Class:     javax_realtime_AsynchronouslyInterruptedException
 * Method:    postAIE
 * Signature: (Ljavax/realtime/RealtimeThread;)Z
 */

JNIEXPORT jboolean JNICALL Java_javax_realtime_AsynchronouslyInterruptedException_postAIE
  (JNIEnv * env, jobject this, jobject target)
{
	CVMExecEnv* tee;
	CVMExecEnv* ee = CVMjniEnv2ExecEnv(env);
        tee = ThreadObj2ExecEnv(env,target);

	if(tee==NULL)
		return 0;	
	CVMD_gcUnsafeExec(ee,{
		CVMgcUnsafeSetPendingAIException(ee,tee,
			CVMID_icellDirect(ee,this));
	     });

	/*If thread is in wait, join then invoke
	 * if (isIOBlocked)
	 * 	peek in stack of target ee to se whether its called from AI
	 * 		if true then send signal to that thread
	 */
	 if(tee->isIOBlocked==CVM_TRUE){
	     CVMthreadIOInterrupt(&tee->threadInfo);	
	 }		
		
	
	return 1;

}

/*
 * Class:     javax_realtime_AsynchronouslyInterruptedException
 * Method:    getStackDepth
 * Signature: (Ljavax/realtime/RealtimeThread;)I
 */
JNIEXPORT jint JNICALL Java_javax_realtime_AsynchronouslyInterruptedException_getStackDepth
  (JNIEnv * env, jobject this)
{
	CVMExecEnv* ee = CVMjniEnv2ExecEnv(env);
	return ee->currentStackDepth;

}

/*
 * Class:     javax_realtime_AsynchronouslyInterruptedException
 * Method:    getPendingAIE
 * Signature: (Ljavax/realtime/RealtimeThread;)Ljavax/realtime/AsynchronouslyInterruptedException;
 */
JNIEXPORT jthrowable JNICALL Java_javax_realtime_AsynchronouslyInterruptedException_getPendingAIE
  (JNIEnv * env,jclass thisclass, jobject target)
{
	CVMExecEnv* tee;
	tee = ThreadObj2ExecEnv(env,target);
	if(CVMpendingAIException(tee)){
		return CVMpendingAIExceptionICell(tee);
	}
	return NULL;
}

/*
 * Class:     javax_realtime_AsynchronouslyInterruptedException
 * Method:    makeNonPending
 * Signature: (Ljavax/realtime/RealtimeThread;)V
 */
JNIEXPORT void JNICALL Java_javax_realtime_AsynchronouslyInterruptedException_makeNonPending
  (JNIEnv * env, jobject this, jobject target)
{
	CVMExecEnv* tee;
	tee = ThreadObj2ExecEnv(env,target);

	CVMclearPendingAIException(tee);
}


CVMExecEnv* ThreadObj2ExecEnv(JNIEnv *env, jobject thread)
{
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
    CVMExecEnv *targetEE;
    CVMJavaLong eetopVal;

    if(!CVMisNoHeapRealtimeThread(ee)){
      CVMsysMutexLock(ee, &CVMglobals.threadLock);
    }

    CVMID_fieldReadLong(ee, thread,
			CVMoffsetOfjava_lang_Thread_eetop,
			eetopVal);
    targetEE = (CVMExecEnv *)CVMlong2VoidPtr(eetopVal);

    if(!CVMisNoHeapRealtimeThread(ee)){
      CVMsysMutexUnlock(ee, &CVMglobals.threadLock);
    }

    return targetEE; 
}
    




