/*
 * -----------------------------------------------------------------
 * ImmortalMemory.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


#include "javax_realtime_ImmortalMemory.h"
#include "javax_realtime_ImmortalPhysicalMemory.h"

#include "javavm/include/indirectmem.h"
#include "javavm/include/globalroots.h"
#include "javavm/include/utils.h"
#include "javavm/include/indirectmem.h"
#include "javavm/include/directmem.h"

#include "javavm/include/mem_registry.h"

#include <stdlib.h>

jobject jtImmortalMemory;
int immortalInitialized=0;

/*
 * Class:     javax_realtime_ImmortalMemory
 * Method:    initImmortal
 * Signature: (J)Ljavax/realtime/ImmortalMemory;
 */
JNIEXPORT jobject JNICALL Java_javax_realtime_ImmortalMemory_initImmortal
  (JNIEnv *env, jclass immortalClazz, jlong memSize)
{
    
  CVMExecEnv *ee;
  CVMClassBlock *cbImmortal;
  char *immortalMemBase;
  jmethodID constructorID;
  int immortalObjSize;
  char *envIMSize = NULL;  
  
  ee = CVMjniEnv2ExecEnv(env);
  cbImmortal = CVMgcSafeClassRef2ClassBlock(ee, immortalClazz);
  immortalObjSize = CVMcbInstanceSize(cbImmortal);
  
  // First check if the environment variable has been set. If so
  // allocate that many bytes.
  envIMSize = getenv("IMMORTAL_SIZE");
  if(envIMSize != NULL)
  {
      int size = atoi(envIMSize);

      // It should be big enough to hold ImmortalMemory and HeapMemory objects.
      if(size < immortalObjSize*2)       
      {
          printf("Immortal Memory Size is too less. Reverting to default size\n");
      }
      else
      {
          memSize = size;
      }
  }

  immortalMemBase = (char *) malloc(memSize);
  
  if(immortalMemBase == NULL)
    {
      //Throw OutOfMemoryError and halt JVM
      jclass clazz = (*env)->FindClass(env, "java/lang/OutOfMemoryError");
      (*env)->ThrowNew(env, clazz, "Could not allocate initial memory for javax.realtime.ImmortalMemory");
	  return NULL;
    }
  // register with central database

  /*jtImmortalMemory = (*env)->NewGlobalRef(env, thisObj);

  CVMJTregistryRegisterMemoryArea(CVMjniEnv2ExecEnv(env), thisObj,
	  (jlong) immortalMemBase, memSize, 2);*/

  // Pratik:
  // 
  // Heavy duty magic needs to take place here. Reason being that I 
  // can't let the ImmortalMemory java object get created in Heap. So
  // I have to make sure that the java object gets created in 
  // ImmortalMemory too. (Kind of like the chicken and the egg problem).
  //
  // 
  // Allocate object here, do all initialisation that typically gets 
  // dones in CVMgcAllocNewInstance, do constructor initialisation, etc.
  // Then register memory area.
  
  jtImmortalMemory = CVMID_getGlobalRoot(ee);
  
  CVMD_gcUnsafeExec(ee,
          {
            CVMObject *immortalObject = (CVMObject*) immortalMemBase;
            
            memset((void*)immortalObject, 0, immortalObjSize);
            immortalObject->hdr.clas = cbImmortal;
            CVMobjMonitorSet(immortalObject, 0, CVM_LOCKSTATE_UNLOCKED);

            CVMassert(CVMobjectVariousWord(immortalObject) ==
                        CVM_OBJECT_DEFAULT_VARIOUS_WORD);
  
            // I cannot add ImmortalMemory to the list of finalizable 
            // objects.

            // Object created. Now put it in jtImmortalMemory
            CVMID_icellSetDirect(ee, jtImmortalMemory, immortalObject);

            // Now I have an ICell reference to immortal memory.
          });
  
  // Now invoke the contructor on jtImmortalMemory.
  constructorID = (*env)->GetMethodID(env, immortalClazz, "<init>", "()V");
  if(constructorID == NULL)
  {
      //Throw InternalError and halt JVM
      jclass clazz = (*env)->FindClass(env, "java/lang/InternalError");
      (*env)->ThrowNew(env, clazz, "Could not find constructor of javax.realtime.ImmortalMemory!!!");
	  return NULL;
  }
  
  (*env)->CallVoidMethod(env, jtImmortalMemory, constructorID);
  
  // Call a special registry function that will register 
  // immortal memory and mark its first immortalObjSize bytes
  // as being used.
  // CAVEAT: This means that ImmortalMemory should be ATLEAST
  // immortalObjSize bytes big.
            
//  Initialise the memory registry
  initialiseMemoryRegistry(ee);
  CVMJTregistryRegisterImmortal(ee, jtImmortalMemory, (jlong) ((int) immortalMemBase), (jlong) memSize, immortalObjSize);
  
  immortalInitialized = 1;
  return jtImmortalMemory;
}


/*** ImmortalPhysicalMemory ****/
/*
 * Class:     javax_realtime_ImmortalPhysicalMemory
 * Method:    initImmortal
 * Signature: ()Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_javax_realtime_ImmortalPhysicalMemory_getGlobalRefToSelf
  (JNIEnv *env, jobject thisObj)
{
  return (*env)->NewGlobalRef(env, thisObj);
}
