/*
 * -----------------------------------------------------------------
 * MonitorControl.c :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

#include "javax_realtime_MonitorControl.h"
#include "jni.h"
#include "javavm/include/defs.h"
#include "javavm/include/sync.h"
#include "javavm/include/indirectmem.h"

#include <rk/rk.h>
#include <pthread.h>

#include "native/com/timesys/CoreExtension/rtmonitoradapter.h"

/* force monitor debug */
#if 0
#ifdef CVM_DEBUG
#define CVM_DEBUG_MONITOR_1 true
#endif
#endif

int monitorAdapterSetDefaultMonControl(int type, int ceiling);

JNIEXPORT void JNICALL Java_javax_realtime_MonitorControl_setMonitorControl1
  (JNIEnv *env, jclass clazz, jint protocol, jint ceiling)
{
  monitorAdapterSetDefaultMonControl(protocol, ceiling);
}

static void setMagicAndOwner(CVMObjMonitor *m)
{
#ifdef CVM_DEBUG
  m->magic = CVM_OBJMON_MAGIC;
  m->owner = NULL;
#endif
}

JNIEXPORT jobject JNICALL Java_javax_realtime_MonitorControl_getMonitorControl1
  (JNIEnv *env, jclass argClazz)
{
  jclass clazz;
  jobject monitorTypeObject;
  jmethodID mid;
  int monitorControlType;
  int ceiling;


  monitorControlType = monitorAdapterGetDefaultMonControl();
  ceiling = monitorAdapterGetMonitorCeiling();


  if (monitorControlType == 1)
    {
      clazz = (*env)->FindClass(env, "javax/realtime/PriorityInheritance");
      if (clazz == NULL)
	printf("MonitorControl.c: error getting class\n");
      mid = (*env)->GetMethodID(env, clazz, "<init>", "()V");
      if (mid == NULL)
	printf("MonitorControl.c: error getting constructor\n");	
      
      monitorTypeObject = (*env)->NewObject(env, clazz, mid);
    }
  else if (monitorControlType == 2)
    {
      clazz = (*env)->FindClass(env, "javax/realtime/PriorityCeilingEmulation");
      if (clazz == NULL)
	printf("MonitorControl.c: error getting class\n");
      mid = (*env)->GetMethodID(env, clazz, "<init>", "(I)V");
      if (mid == NULL)
	printf("MonitorControl.c: error getting constructor\n");	
      
      monitorTypeObject = (*env)->NewObject(env, clazz, mid, ceiling);
    }
  else
    {
      // NONE
      monitorTypeObject = NULL;
    }
  return monitorTypeObject;
}

JNIEXPORT jobject JNICALL Java_javax_realtime_MonitorControl_getMonitorControl0
  (JNIEnv *env, jclass argClazz, jobject monitor)
{
  jclass clazz;
  jobject monitorTypeObject;
  jmethodID mid;
  int monitorControlType;
  int ceiling = -1;

  CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);

  CVMD_gcUnsafeExec(ee,
  {
    //CVMObjMonitor *m;
    CVMObject *obj;

    /* Verify that we don't have a monitor already initialized.
     * Once initialized, it is unsafe to change the locking protocol so we don't 
     * allow it.
     */
    
    obj = CVMID_icellDirect(ee, (CVMObjectICell*) monitor);

    if (CVMobjMonitorState(obj) == CVM_LOCKSTATE_MONITOR)
      {
	CVMObjMonitor *mon = CVMobjMonitor(obj);
	monitorControlType = mon->mon.monitorControl;
	ceiling = mon->mon.ceiling;
      }
    else
      {
	monitorControlType = 0;
      }
  });

  if (monitorControlType == 1)
    {
      clazz = (*env)->FindClass(env, "javax/realtime/PriorityInheritance");
      if (clazz == NULL)
	printf("MonitorControl.c: error getting class\n");
      mid = (*env)->GetMethodID(env, clazz, "<init>", "()V");
      if (mid == NULL)
	printf("MonitorControl.c: error getting constructor\n");	
      
      monitorTypeObject = (*env)->NewObject(env, clazz, mid);
    }
  else if (monitorControlType == 2)
    {
      clazz = (*env)->FindClass(env, "javax/realtime/PriorityCeilingEmulation");
      if (clazz == NULL)
	printf("MonitorControl.c: error getting class\n");
      mid = (*env)->GetMethodID(env, clazz, "<init>", "(I)V");
      if (mid == NULL)
	printf("MonitorControl.c: error getting constructor\n");	
      
      monitorTypeObject = (*env)->NewObject(env, clazz, mid, ceiling);
    }
  else
    {
      // NONE
      monitorTypeObject = NULL;
    }
  return monitorTypeObject;

}

JNIEXPORT int JNICALL Java_javax_realtime_MonitorControl_setMonitorControl0
  (JNIEnv *env, jclass clazz, jobject monitor, jint protocol, jint ceiling)
{
  /* This function should create the monitor and associated with the object.
   * For this we are factoring out assign_monitor_to_object inside objsync.c
   * so we can reuse it from here.
   */

  //int debugMode = 0;

  CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);

  CVMD_gcUnsafeExec(ee,
  {
    CVMObjMonitor *m;
    CVMObject *obj;

    /* Verify that we don't have a monitor already initialized.
     * Once initialized, it is unsafe to change the locking protocol so we don't 
     * allow it.
     */
    
    obj = CVMID_icellDirect(ee, (CVMObjectICell*) monitor);

    if (CVMobjMonitorState(obj) == CVM_LOCKSTATE_UNLOCKED)
      {
	m = calloc(1,sizeof(CVMObjMonitor));
	if (m != NULL)
	  {
	    m->mon.monitorControl = protocol;
	    m->mon.ceiling = ceiling;

	    m->state = CVM_OBJMON_BOUND;
	    m->bits = obj->hdr.various32;
	    m->obj = obj;

	    /* Sticky so they are not be scavenged */
	    m->sticky = 1;

	    setMagicAndOwner(m);

	    CVMobjMonitorSet(obj, m, CVM_LOCKSTATE_MONITOR);

	    CVMsysMonitorInit(&m->mon, NULL, 0);
	    /* printf("MonitorControl: SysMonitor %p set\n",&m->mon); */
	  }
      }
  }); 

  return 0;
}

