/*
 * -----------------------------------------------------------------
 * RealtimeThread.c : The implementation of native methods of
 * the RealtimThread class.
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying Common Public License in file CPLicense.  Your use
 * of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */

#include "javax_realtime_RealtimeThread.h"
#include <rk/rk.h>
#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#include <pthread.h>
#include <sched.h>

#include <sys/time.h>
#include <unistd.h>

#include "generated/offsets/java_lang_Thread.h"

#include "javavm/include/mem_registry.h"
#include "javavm/include/directmem.h"
#include "javavm/include/indirectmem.h"
#include "javavm/include/utils.h"           // For CVMdebugPrintf()
#include "javavm/include/objects.h"
#include "javavm/include/classes.h"

extern CVMExecEnv * ThreadObj2ExecEnv(JNIEnv *env, jobject thread);

pthread_mutex_t timerQmtx;
pthread_mutex_t threadQmtx;
CVMThreadID *TQhead;

void javax_realtime_realtimethread_sleepUntil(CVMThreadID *self, long long time, int tnum)
{
  CVMThreadID *tmp;
  int newTimer = 0;
#ifdef CVM_DEBUG
  printf("\n\n--- SLEEP UNTIL BEGIN: time(%Ld) tid(%d) ---\n",time, tnum);
#endif
  pthread_mutex_lock(&timerQmtx);

  /* look for a timer already in place*/
  tmp = TQhead;

#ifdef CVM_DEBUG
  printf("BEFORE looking for timer\n");
#endif

  while ((tmp != NULL) && (tmp->waitUntilTime != time))
    tmp = tmp->waitingQnext;
#ifdef CVM_DEBUG
  printf("AFTER looking for timer tid(%d)\n", tnum);
#endif

  if (tmp == NULL)
    {
#ifdef CVM_DEBUG
      printf("Adding timer tid(%d)\n", tnum);
#endif
      /* Add it */
      self->waitUntilTime = time;
      self->waitingQnext = TQhead;
      if (TQhead != NULL)
	{
	  TQhead->waitingQprev_p = &self->waitingQnext;
	}

      TQhead = self;
      self->waitingQprev_p = &TQhead;

      self->waitingOn = self;
      newTimer  = 1;
      self->waitUntilDataValid = 1;
    }
  else
    self->waitingOn = tmp;

  pthread_mutex_unlock(&timerQmtx);

  if (newTimer)
    {
      struct timeval tv;
      long long diff;
      unsigned long ldiff;
      CVMThreadID *tmp;

      gettimeofday(&tv , NULL);


      diff = (long long) ((long long) tv.tv_sec * (long long) 1000 * (long long) 1000);
      diff = diff + (long long) tv.tv_usec;

      diff = self->waitingOn->waitUntilTime - diff ;
#ifdef CVM_DEBUG
      printf("sleeping for %Ld usecs tid(%d)\n",diff, tnum);
#endif

      if (diff >0)
	{
	  do
	    {
	      ldiff = (unsigned long) diff;
#ifdef CVM_DEBUG
	      printf("[SLEEPUNTIL] sleeping for %lu \n",ldiff);
#endif
	      usleep(ldiff);

	      gettimeofday(&tv , NULL);
	      
	      diff = (long long) ((long long) tv.tv_sec * (long long) 1000 * (long long) 1000);
	      diff = diff + (long long) tv.tv_usec;
	      
	      diff = self->waitingOn->waitUntilTime - diff ;
	    }
	  while (diff >0 );
	}

      tmp = self->waitingOn;

#ifdef CVM_DEBUG
      printf("[SLEEPUNTIL]: after usleep before lock(waitingOn->mutex) tid(%d)\n",tnum);
#endif
      pthread_mutex_lock(&self->waitingOn->wait_until_mtx);

#ifdef CVM_DEBUG
      printf("[SLEEPUNTIL]: before broadcast(waitingOn->waituntilcv) tid(%d)\n", tnum);
#endif

      pthread_cond_broadcast(&self->waitingOn->wait_until_cv);

      self->waitingOn->waitUntilDataValid = 0;

#ifdef CVM_DEBUG
      printf("[SLEEPUNTIL]:  usleep before lock(timerQmtx) tid(%d)\n", tnum);
#endif

      pthread_mutex_lock(&timerQmtx);

#ifdef CVM_DEBUG
      printf("[SLEEPUNTIL]: after lock(timerQmtx) tid(%d)\n", tnum);
#endif
      if (self->waitingOn->waitingQnext != NULL)
	{
	  self->waitingOn->waitingQnext->waitingQprev_p = self->waitingOn->waitingQprev_p;
	}

      *(self->waitingOn->waitingQprev_p) = self->waitingOn->waitingQnext;

      pthread_mutex_unlock(&timerQmtx);

#ifdef CVM_DEBUG
      printf("[SLEEPUNTIL]: after unlock(timerQmtx) tid(%d)\n", tnum);
#endif

      pthread_mutex_unlock(&tmp->wait_until_mtx);

      /* yield  the processor for highest priority threads */
      sched_yield();

#ifdef CVM_DEBUG
      printf("[SLEEPUNTIL]: after unlock(wait_until_mtx) tid(%d)\n", tnum);
#endif 
    }
  else
    {
	pthread_mutex_lock(&self->waitingOn->wait_until_mtx);
	if(self->waitingOn->waitUntilDataValid)
	    {
#ifdef CVM_DEBUG
		printf("[SLEEP UNTIL]: waiting for the other thread to wake me up tid(%d)\n", tnum);
#endif

		self->waitingOn->sharedTimerReferenceCount++;
		
#ifdef CVM_DEBUG
		{
		  struct sched_param p;
		  
		  sched_getparam(getpid(),&p);
		  
		  printf("[SLEEP UNTIL]: before cond_wait tid(%d) rt_priority(%d) pid(%d)\n", tnum, p.sched_priority, getpid());
		}
#endif
	  
		pthread_cond_wait(&self->waitingOn->wait_until_cv, &self->waitingOn->wait_until_mtx);
		
#ifdef CVM_DEBUG
		{
		  struct sched_param p;

		  sched_getparam(getpid(),&p);
		  printf("[SLEEP UNTIL]: after cond_wait tid(%d) rt_priority(%d) pid(%d)\n", tnum, p.sched_priority, getpid());
		}
#endif
		
		self->waitingOn->sharedTimerReferenceCount--;


#ifdef CVM_DEBUG
		printf("[SLEEP UNTIL]: after mutex unlock tid(%d)\n", tnum);
#endif
	    }
	else
#ifdef CVM_DEBUG
	    printf("[SLEEP UNTIL]: timer data was invalid (already expired) tid(%d)\n", tnum);
#endif

	pthread_mutex_unlock(&self->waitingOn->wait_until_mtx);

	/* yield  the processor for highest priority threads */
	sched_yield();
    }

#ifdef CVM_DEBUG
  {
      struct sched_param p;
      struct timeval tv;
      long long diff;

      gettimeofday(&tv , NULL);

      diff = (long long) ((long long) tv.tv_sec * (long long) 1000 * (long long) 1000);
      diff = diff + (long long) tv.tv_usec;

      sched_getparam(getpid(),&p);

      printf("\n --- SLEEP UNTIL END --- tid(%d) pid(%d) rt_priority(%d) now(%Ld)\n\n", tnum, getpid(), p.sched_priority, diff);
  }
#endif
}


void javax_realtime_realtimethread_initSleepQ()
{
  pthread_mutexattr_t attr;

  pthread_mutexattr_init(&attr);

  pthread_mutexattr_setprotocol(&attr, PTHREAD_PRIO_INHERIT);
  pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_NORMAL);

  pthread_mutex_init(&timerQmtx, &attr);

  TQhead = NULL;
}

JNIEXPORT void JNICALL Java_javax_realtime_RealtimeThread_yield0
  (JNIEnv *env, jclass clazz)
{
  sched_yield();
}

/*ISHIN*/

#define EDF_PRI  40
#define EDF_MIN  40
#define EDF_MAX 100

extern int JavaScheduler;

JNIEXPORT void JNICALL Java_javax_realtime_RealtimeThread_setDeadline
  (JNIEnv *env, jobject thisobj, jlong millis, jlong nanos)
{
    struct timespec r_deadline;

    /* initialize variable in ExecEnv */
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
    CVMExecEnv *threadEE;

    /*struct sched_param p;*/
    CVMJavaLong eetopVal;

    CVMID_fieldReadLong(ee, thisobj,
			CVMoffsetOfjava_lang_Thread_eetop,
			eetopVal);
    threadEE = (CVMExecEnv *)CVMlong2VoidPtr(eetopVal);

    //threadEE = (CVMExecEnv *) execEnv;

    ee = threadEE;


     r_deadline.tv_sec = millis / 1000;
     r_deadline.tv_nsec = (millis % 1000) * 1000 * 1000 + nanos;
     ee->deadline = 0;
     ee->deadline += ((long long)r_deadline.tv_sec * (long long)1000000) ;
     ee->deadline += ((long long)r_deadline.tv_nsec / (long long )1000);

     printf ("*** setDeadline to -------------------------------\n");

      printf("     =================================================================\n");
      printf ("      EDF Thread %d with deadline (%Ld) and priority (%d)\n",
	      ee->threadID, ee->deadline % 1000000000, ee->threadInfo.lastPriority);
      printf("     -----------------------------------------------------------------\n");
      printf("      ThreadID  pthreadID  priority   nextPeriod     deadline\n");
      printf("         %2d        %d        %3d     %9Ld   %9Ld(%Ld)\n",
		ee->threadID, (int)ee->threadInfo.pthreadCookie,
		ee->threadInfo.lastPriority,
		ee->startNextPeriodMicros % 1000000000,
		(ee->startNextPeriodMicros + ee->deadline) % 1000000000,
		ee->deadline % 1000000000);

}

/*ISHIN*/


//static int getEligibleRange (JNIEnv *env, jobject this)
static int getEligiblePriority (JNIEnv *env, jobject this)
{
   CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
   CVMExecEnv **firstPtr = &CVMglobals.threadList;
   CVMExecEnv *ptr = *firstPtr;
   int P_Array[256], i;


   /* struct sched_param p; */
   CVMInt32 priority;
   jlong deadline, td;
   //long long c_time;
   int max=EDF_MAX, min=EDF_MIN;
  // struct timeval tv;
/*
   gettimeofday(&tv, NULL);

   c_time = (long long) ((long long) tv.tv_sec * (long long) 1000 * (long long) 1000);
   c_time = c_time + (long long) tv.tv_usec;
*/
   //printf ("     current time : %9Ld\n", c_time);

   for (i=0; i<256; i++) P_Array[i] = 0;

   deadline = ee->startNextPeriodMicros + ee->deadline;

   ptr = *firstPtr;

   printf("     -----------------------------------------------------------------\n");
   printf("      ThreadID  pthreadID  priority   nextPeriod     deadline\n");

   while (ptr != 0) {
      CVMthreadGetPriority(&(ptr->threadInfo), &priority);
      if (ptr->deadline > 0 && ee->threadID != ptr->threadID) {
//       && priority >= EDF_MIN) {
	  printf("*          %d        %d        %3d     %9Ld   %9Ld(%Ld)\n",
		ptr->threadID, (int)ptr->threadInfo.pthreadCookie,
		priority,
		ptr->startNextPeriodMicros % 1000000000,
		(ptr->startNextPeriodMicros + ptr->deadline) % 1000000000,
		ptr->deadline % 1000000000);
	  td = ptr->startNextPeriodMicros + ptr->deadline;

	  if (td == deadline && priority >= min) {max = min = priority; break;}
	  else if (deadline < td && priority >= min) min = priority+1;
	  else if (deadline > td && priority <= max) max = priority-1;
	  P_Array[priority]++;
      }
      ptr = ptr->nextEE;
   }


   printf("     ---- max : %d, min : %d -----------------------------------------\n",
   	max, min);

   if (max > min) priority = min + 1;
   else if (min > EDF_MIN && max < EDF_MAX && max == min) priority = min;
   else { //priority = min + 1;
      int nEmpty=0, nMove=0, pEmpty, pMove, j;
      double ratio;

      i = min-1;
      while (i > EDF_MIN) {
         if (P_Array[i] == 0) break;
	 else {i--; nMove++;}
      }
      pMove = i+1;

      j=i;
      if (i>EDF_MIN) {
          while (i > EDF_MIN) {
             if (P_Array[i] == 1) break;
	     else {i--; nEmpty++;}
          }

          pEmpty = i+1;

	  nMove++;
          nEmpty = nEmpty + nMove;

          ratio = (double)nEmpty / (nMove+1);
	  /*
          printf (" nMove - %d, nEmpty - %d, ratio - %f\n", nMove, nEmpty, ratio);
          printf (" pMove - %d, pEmpty - %d, ratio - %f\n", pMove, pEmpty, ratio);
	  */
          for (i=0; i<nMove-1; i++) {
	      ptr = *firstPtr;
              while (ptr != 0) {
                  CVMthreadGetPriority(&(ptr->threadInfo), &priority);
                  if (priority == pMove) {
		      int inc;
		      struct sched_param p;
		      inc = ratio * (2*i+1) * 0.5;
//		      if (inc == 0) inc = 1;
                      p.sched_priority = pEmpty + inc;
		      printf (" * %d * Change priority from %d to %d\n",
	              (int)ptr->threadInfo.pthreadCookie, pMove,
		      p.sched_priority);
                      pthread_setschedparam(ptr->threadInfo.pthreadCookie,
		                            SCHED_FIFO, &p);
		      break;
		  }
                  ptr = ptr->nextEE;
              }
              pMove++;
          }
	  nMove--;
	  priority = pEmpty + ratio * (2*(nMove)+1) * 0.5;
      }
      else {
          int nEmpty=0, nMove=0, pEmpty, pMove, j;
          double ratio;

  	  i = max+1;
          while (i < EDF_MAX) {
              if (P_Array[i] == 0) break;
	      else {i++; nMove++;}
          }
          pMove = i-1;

          j = i;
	  while (i < EDF_MAX) {
             if (P_Array[i] == 1) break;
	     else {i++; nEmpty++;}
          }
	  pEmpty = i-1;
          nEmpty = nEmpty + nMove;

          ratio = (double)nEmpty / (nMove+2);

          printf (" nMove - %d, nEmpty - %d, ratio - %f\n", nMove, nEmpty, ratio);
          printf (" pMove - %d, pEmpty - %d, ratio - %f\n", pMove, pEmpty, ratio);

          for (i=0; i<nMove; i++) {
	      ptr = *firstPtr;
              while (ptr != 0) {
                  CVMthreadGetPriority(&(ptr->threadInfo), &priority);
                  if (priority == pMove) {
		      int dec;
		      struct sched_param p;
		      dec = ratio * (2*i+1) * 0.5;
                      p.sched_priority = pEmpty - dec;
		      printf (" * %d * Change priority from %d to %d (dec:%d)\n",
	              (int)ptr->threadInfo.pthreadCookie, pMove,
		      p.sched_priority, dec);
                      pthread_setschedparam(ptr->threadInfo.pthreadCookie,
		                            SCHED_FIFO, &p);
		      break;
		  }
                  ptr = ptr->nextEE;
              }
              pMove--;
          }
	  priority = pEmpty - ratio * (2*nMove+1) * 0.5;
      }
      ptr = *firstPtr;
      while (ptr != 0) {
         int pri;
         CVMthreadGetPriority(&(ptr->threadInfo), &pri);
         if (ptr->deadline > 0 && ee->threadID != ptr->threadID ) {
	     printf("*          %d        %d        %3d     %9Ld   %9Ld(%Ld)\n",
		ptr->threadID, (int)ptr->threadInfo.pthreadCookie,
		pri,
		ptr->startNextPeriodMicros % 1000000000,
		(ptr->startNextPeriodMicros + ptr->deadline) % 1000000000,
		ptr->deadline % 1000000000);
         }
         ptr = ptr->nextEE;
      }
      printf("     -----------------------------------------------------------------\n");
   }

   return priority;
}


JNIEXPORT void JNICALL Java_javax_realtime_RealtimeThread_sleepMicrosUntil0
  (JNIEnv *env, jobject this, jlong time, jint num)
{
  CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);

  /************* ISHIN *************/
  {
    struct sched_param p;
    CVMInt32 pri;
    jlong deadline;
    int min=EDF_MIN, max=EDF_MAX;


    int scheduler = JavaScheduler;
    printf ("     ------------ Current Java Scheduler : ");
    switch (JavaScheduler) {
       case 1: printf (" Priority Scheduler ------------\n"); break;
       case 2: printf (" EDF Scheduler ------------\n"); break;
    }
    CVMsysMutexLock(ee, &CVMglobals.threadLock);

    ee->startNextPeriodMicros = time;


    if (scheduler == 2 && ee->deadline > 0) { // EDF == 2 // && ee->threadInfo.lastPriority == EDF_PRI) {

      CVMthreadGetPriority(&(ee->threadInfo), &pri);
      deadline = ee->startNextPeriodMicros + ee->deadline;

      printf("     =================================================================\n");
      printf ("      EDF Thread %d with deadline (%Ld) and priority (%d)\n",
	      ee->threadID, deadline % 1000000000, pri);
      //      printf("     ThreadID pthreadID priority     deadline\n");

      pri = getEligiblePriority(env, this);
      if (pri > EDF_MAX) printf (" ------------------\n");

      printf ("      EDF thread (%d) priority is set to %d (%d, %d)\n",
	      ee->threadID, pri, min+1, max);
      printf("     =================================================================\n");

      p.sched_priority = pri;
      pthread_setschedparam(ee->threadInfo.pthreadCookie, SCHED_FIFO, &p);
      CVMsysMutexUnlock(ee, &CVMglobals.threadLock);
    }

  }
  /************* ISHIN *************/

  javax_realtime_realtimethread_sleepUntil(&ee->threadInfo, time, num);
}

JNIEXPORT jlong JNICALL Java_javax_realtime_RealtimeThread_startNextPeriodMicros0
  (JNIEnv *env, jobject this)
{
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
    return (jlong) ee->startNextPeriodMicros;
}


/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    waitForNextPeriod0
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_javax_realtime_RealtimeThread_waitForNextPeriod0
  (JNIEnv *env, jobject this, jlong periodMicros)
{
  long long diff;
  struct timeval t;
  CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);

  gettimeofday(&t,NULL);

  diff = ee->startNextPeriodMicros - (t.tv_sec * 1000000 + t.tv_usec);
  if (diff < 0)
    {
      long long periodsMissed = ((-diff / periodMicros) + 1);
      ee->startNextPeriodMicros += (periodsMissed * periodMicros) + diff;
      diff = (periodsMissed * periodMicros) + diff;
    }
#ifdef CVM_DEBUG
  printf("Waiting for %lld usecs\n",diff);
#endif
  usleep(diff);
  ee->startNextPeriodMicros += periodMicros;
  return 0;
}

JNIEXPORT jboolean JNICALL Java_javax_realtime_RealtimeThread_canChangeScheduler0
  (JNIEnv *env, jobject this)
{
    CVMJavaLong eetopVal;
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);

    CVMID_fieldReadLong(ee, this,
			CVMoffsetOfjava_lang_Thread_eetop,
			eetopVal);

    ee = (CVMExecEnv *)CVMlong2VoidPtr(eetopVal);


    //CVMExecEnv *ee = (CVMExecEnv *) execEnv;

    if (ee == (CVMExecEnv *)0)
	return JNI_TRUE;

    if (ee->canChangeScheduler)
	return JNI_TRUE;
  
    return JNI_FALSE;
}

/* gc-unsafe function */
MemoryAreaInfo* outerScopedMemory(CVMExecEnv *ee, jclass classScopedMemory)
{
    PerThreadMemoryInfo *ptmi = ee->perThreadStack;
    PerThreadMemoryInfo *traverser = ptmi;
    CVMClassBlock *cbScoped = CVMgcUnsafeClassRef2ClassBlock(ee, classScopedMemory);
      
    // First traverse all the way up to my current memory area
    while(traverser != NULL)
    {
        CVMObject* memoryAreaObj = CVMID_icellDirect(ee, traverser->info->memoryArea);
        if(CVMisAssignable(ee, CVMobjectGetClass(memoryAreaObj), cbScoped))
        {
            return traverser->info;
        }
        traverser = traverser->upInStack;
    }
    
    return NULL;

}

/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    popMemoryArea 
 * Signature: ()Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_javax_realtime_RealtimeThread_popMemoryArea
  (JNIEnv *env, jobject thisobj)
{
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
	CVMObjectICell *popped = NULL; 
	PerThreadMemoryInfo *ptmi = ee->perThreadStack;
    MemoryAreaInfo *info;
    CVMdebugPrintf((" ******* popMemoryArea got called\n")); 
    // Theoraticaly, this should never happen. pop should never get 
    // called without push having being called first i.e. the stack
    // should not be empty. But just in case.... 
    if(ptmi == NULL)
    {
        CVMdebugPrintf(("RealtimeThread.c:popMemoryArea() - pop got called with an empty stack. :(\n"));
        return NULL;
    }
   // CVMdebugPrintf(("\t\tpopMemoryArea() - got called with perThreadStack = %x\n", ptmi));
    info = ptmi->info;
    
    CVMD_gcUnsafeExec(ee, {
		ee->perThreadStack = ptmi->upInStack;
		if(ee->perThreadStack != NULL)
		{
            CVMdebugPrintf((" ******* popMemoryArea - ee->perThreadSTack != NULL\n")); 
		    ee->perThreadStack->downInStack = NULL;
		    CVMID_icellAssignDirect(ee, CVMmemoryAreaICell(ee), ee->perThreadStack->info->memoryArea);
	   	    popped = ptmi->info->memoryArea;
		}
		else
		{
            CVMdebugPrintf((" ******* popMemoryArea - ee->perThreadStack == NULL\n")); 
            //popped = &CVMID_nullICell;
		//    extern CVMObjectICell *jtHeapMemoryArea;
		//    popped = NULL;
		    CVMID_icellSetNull(CVMmemoryAreaICell(ee));
		}
		CVMJTregistryDeletePTMI(ptmi);
	});

    if(popped != NULL && !CVMID_icellIsNull(popped))
    {
        jboolean isScoped = JNI_FALSE;
        jfieldID refCountID;
        int refCountVal;
        //int result;
        jclass classScopedMemory = (*env)->FindClass(env, "javax/realtime/ScopedMemory");

        if(classScopedMemory == NULL)
        {
            //exception already thrown
            return NULL;
        }
        isScoped = (*env)->IsInstanceOf(env, info->memoryArea, classScopedMemory);
        if(isScoped)
        {
            refCountID = (*env)->GetFieldID(env, classScopedMemory, "referenceCount", "I");
            refCountVal = (*env)->GetIntField(env, info->memoryArea, refCountID);
            if(refCountVal == 0)
            {
                CVMdebugPrintf((" ******* popMemoryArea - removing edge\n")); 
                CVMD_gcUnsafeExec(ee, {
                        CVMJTtreeRemoveEdge(ee, info, outerScopedMemory(ee, classScopedMemory));
                    });
            }
        }
    }
     
    CVMdebugPrintf((" ******* popMemoryArea returning\n")); 
	return (*env)->NewLocalRef(env, popped);
}

JNIEXPORT void JNICALL Java_javax_realtime_RealtimeThread_setNativeRealtimePriority
(JNIEnv *env, jobject thisobj, jint priority)
{
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
    CVMExecEnv *threadEE;
    struct sched_param p;
    CVMJavaLong eetopVal;


    p.sched_priority = priority;

    CVMID_fieldReadLong(ee, thisobj,
			CVMoffsetOfjava_lang_Thread_eetop,
			eetopVal);
    threadEE = (CVMExecEnv *)CVMlong2VoidPtr(eetopVal);

    //threadEE = (CVMExecEnv *) execEnv;

    if (threadEE != NULL)
      pthread_setschedparam(threadEE->threadInfo.pthreadCookie, SCHED_FIFO, &p);

    /*************** ISHIN **************/
    printf ("ISHIN - steNativeRealtimePriority - %d\n", priority);
    threadEE->threadInfo.lastPriority = priority;
    /*************** ISHIN **************/


}

CVMBool cycleExists(CVMExecEnv *ee, MemoryAreaInfo *info)
{
    PerThreadMemoryInfo *ptmi = ee->perThreadStack;
    PerThreadMemoryInfo *traverser = ptmi;
    //CVMBool result = CVM_FALSE;
    
    if(ptmi == NULL)
    {
      return CVM_FALSE;
    }

    // First traverse all the way up to my current memory area
    while(traverser != NULL)
    {
        if(traverser->info == info)
            return CVM_TRUE;
        traverser = traverser->upInStack;
    }
    return CVM_FALSE;
}



/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    pushMemoryArea 
 * Signature: (Ljava/lang/Object;)V
 */
JNIEXPORT void JNICALL Java_javax_realtime_RealtimeThread_pushMemoryArea
  (JNIEnv *env, jobject thisobj, jobject memoryArea)
{
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
	MemoryAreaInfo *info = CVMJTregistryLookupMemoryArea(ee, memoryArea);
    jboolean isScoped = JNI_FALSE;
    jfieldID refCountID;
    int refCountVal=0;
    int result;
    jclass classScopedMemory = (*env)->FindClass(env, "javax/realtime/ScopedMemory");
    if(classScopedMemory == NULL)
    {
        //exception already thrown
        return;
    }
    isScoped = (*env)->IsInstanceOf(env, memoryArea, classScopedMemory);
    refCountID = (*env)->GetFieldID(env, classScopedMemory, "referenceCount", "I");
    if(isScoped)
        refCountVal = (*env)->GetIntField(env, memoryArea, refCountID);

//    CVMdebugPrintf(("pushMemoryArea called with thisObj = %I, memoryArea = %I\n", thisobj, memoryArea));
    
    
	CVMD_gcUnsafeExec(ee, {
        result = 0;
        if(isScoped)
        {
            // Its a scoped memory area.
            // Check cycles
            if(!cycleExists(ee, info))
            {
                if(CVMJTtreeAddEdge(ee, info, outerScopedMemory(ee, classScopedMemory), refCountVal))
                    result = 1;
                else
                    result = -2;
            }
            else
                result = -1;
        }
        else
            result = 1;
        if(result==1)
        {
            PerThreadMemoryInfo *ptmi = CVMJTregistryCreatePTMI(ee);
            ptmi->info = info;
            ptmi->downInStack = NULL;
            ptmi->upInStack = ee->perThreadStack;
            if(ee->perThreadStack != NULL)
                ee->perThreadStack->downInStack = ptmi;
            CVMID_icellAssignDirect(ee, CVMmemoryAreaICell(ee), info->memoryArea);
            ee->perThreadStack = ptmi;
        }
    });
    if(result != 1)
    {
        //throw ScopedCycleException
        jclass exception = (*env)->FindClass(env, "javax/realtime/ScopedCycleException");
        if(exception != NULL)
        {
            if(result == -1)    //cycle exists on stack
                (*env)->ThrowNew(env, exception, "Entering this memory area causes a cycle on memory area stack");
            else
            {
                CVMdebugPrintf(("Unable to add edge in tree.\n"));
                (*env)->ThrowNew(env, exception, "Illegal to enter this memory area from current memory area");
            }
        }
        else
        {
            CVMdebugPrintf(("Unable to find ScopedCycleException so I'm throwing InternalError\n"));
            (*env)->ExceptionClear(env);
            exception = (*env)->FindClass(env, "java/lang/InternalError");
            (*env)->ThrowNew(env, exception, "Could not find ScopedCycleException class. Throwing InternalError instead");
        }
    }
    return;
}


JNIEXPORT void JNICALL Java_javax_realtime_RealtimeThread_postAIE
  (JNIEnv * env, jobject target, jobject genericAIE)
{
	
	CVMExecEnv* ee = CVMjniEnv2ExecEnv(env);
	CVMExecEnv* tee = ThreadObj2ExecEnv(env,target);

	if(tee==NULL)
		return;	
	CVMD_gcUnsafeExec(ee,{
		CVMgcUnsafeSetPendingAIException(ee,tee,
			CVMID_icellDirect(ee,genericAIE));
	     });

	 if(tee->isIOBlocked==CVM_TRUE){
	     CVMthreadIOInterrupt(&tee->threadInfo);	
	 }		

}


/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    getOuterMemoryArea0 
 * Signature: (Ljava/lang/Object;)Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_javax_realtime_RealtimeThread_getOuterMemoryArea0
  (JNIEnv *env, jobject thisobj, jobject memoryArea)
{
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
    PerThreadMemoryInfo *ptmi = ee->perThreadStack;
    PerThreadMemoryInfo *traverser = ptmi;
    CVMBool result = CVM_FALSE;
      
    if(ptmi == NULL)
    {
     // printf("ERROR: getOuterMemoryArea() - Per  Thread stack is null!!\n");
      return NULL;
    }

    // First traverse all the way up to my current memory area
    while(traverser != NULL)
    {
        CVMID_icellSameObject(ee, traverser->info->memoryArea, memoryArea, result);
        if(result == CVM_TRUE)
            break;
        traverser = traverser->upInStack;
    }
    // Now we are at our location. Keep moving up until we reach the next higher ScopedMemory
    if(result == CVM_TRUE)
    {
        traverser = traverser->upInStack;
        while(traverser != NULL && traverser->info->memType != MEMTYPE_SCOPED)
        {
           traverser = traverser->upInStack;
        }
        if(traverser == NULL)
            return NULL;

        //else return a reference to that scoped memory.
        return (*env)->NewLocalRef(env, traverser->info->memoryArea);
    }
    
    // Couldn't find the passed memory area on the stack. This is an ERROR.
    CVMdebugPrintf(("ERROR: getOuterMemoryArea() - Couldn't find the memory area passed\n"));

    
    return NULL;
}


int getMemoryAreaStackDepth0(CVMExecEnv *ee)
{
    int depth = 0;
    PerThreadMemoryInfo *traverser = ee->perThreadStack;
    for( ; traverser != NULL; depth++, traverser = traverser->upInStack);
    return depth;
}

/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    getMemoryAreaStackDepth0
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_javax_realtime_RealtimeThread_getMemoryAreaStackDepth0
  (JNIEnv *env, jclass thisClazz)
{
    /*int depth = 0;
    PerThreadMemoryInfo *traverser = ee->perThreadStack;
    for( ; traverser != NULL; depth++, traverser = traverser->upInStack);
    return depth;*/
    return getMemoryAreaStackDepth0(CVMjniEnv2ExecEnv(env));
}

/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    getOuterMemoryArea1
 * Signature: (I)Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_javax_realtime_RealtimeThread_getOuterMemoryArea1
  (JNIEnv *env, jclass thisClazz, jint indexPassed)
{
//    CVMBool result = CVM_FALSE;
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
    PerThreadMemoryInfo *traverser = ee->perThreadStack;
    PerThreadMemoryInfo *secondPtr;
    jint index = indexPassed;
    /*while(traverser != NULL)
    {
        CVMID_icellSameObject(ee, traverser->info->memoryArea, CVMmemoryAreaICell(ee), result);
        if(result)
        {
            break;
        }
        traverser = traverser->upInStack;
    }*/
    
    if(traverser == NULL)
        return NULL;
    
    secondPtr = traverser;
    
    // go index steps up
    for( ; secondPtr!=NULL && index>=0; index--)
    {
        secondPtr = secondPtr->upInStack;    
    }
    
    if(index >= 0)
    {
        CVMdebugPrintf(("Returning NULL because index >= 0\n")); 
        return NULL;
    }

    //Now go all the way to the top
    while(secondPtr != NULL)
    {
        secondPtr = secondPtr->upInStack;
        traverser = traverser->upInStack;
    }
    //CVMdebugPrintf(("getouterMemoryArea() - %d is %I\n", indexPassed, traverser->info->memoryArea));
    return (*env)->NewLocalRef(env, traverser->info->memoryArea);
}


/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    getMemoryAreaStack
 * Signature: (Ljava/lang/Object;)V
 */
JNIEXPORT void JNICALL Java_javax_realtime_RealtimeThread_getMemoryAreaStack
  (JNIEnv *env, jclass thisClazz, jobjectArray stack)
{
    int arrayLength=0, i=0;
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
    //int depth = getMemoryAreaStackDepth0(ee) ;
    PerThreadMemoryInfo *traverser = ee->perThreadStack;

    arrayLength = (*env)->GetArrayLength(env, stack);

    // This algorithm could do with some optimization.
    // Its a poor algorithm but fast to implement.
    //
    // First go all the way to the top
    while(traverser->upInStack != NULL)
    {
        traverser = traverser->upInStack;
    }
        
    // Now start filling the array
    while(i<arrayLength && traverser != NULL)
    {
        (*env)->SetObjectArrayElement(env, stack, i, traverser->info->memoryArea);
        if((*env)->ExceptionCheck(env))
            return;
        i++;
        traverser = traverser->downInStack;
    }
}

/*
 * Class:     javax_realtime_RealtimeThread
 * Method:    getCurrentMemoryArea0
 * Signature: ()Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_javax_realtime_RealtimeThread_getCurrentMemoryArea0
  (JNIEnv *env, jclass thisClazz)
{
    return (*env)->NewLocalRef(env, CVMmemoryAreaICell(CVMjniEnv2ExecEnv(env)));
}
