import java.awt.*;
import java.applet.*;
import java.io.*;
import java.net.URL;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;



//import graph.*;
/*************************************************************************
**
**    Applet example2a
**                                              Version 1.0   August 1996
**
**************************************************************************
**    Copyright (C) 1996 Leigh Brookshaw
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**************************************************************************
**
**    This is a simple Applet that creates a Scroll Chart using the 
**    Graph2D class library and double buffering
**
*************************************************************************/

public class TimeSys extends Panel implements Runnable 
{

	Graph2D graph;
    Label title;
    DataSet data1 = new DataSet();

    Axis    yaxis_right;

    Image    osi = null;
    Graphics osg = null;
    int iwidth  = 0;
    int iheight = 0;

    Thread runner = null;
	Pipe p_guireader;
	PipedInputStream pin;
	PipedOutputStream pout;
    int count = 0,rp=0,noheap=1;

    int period      = 250;
    int maximum     = 25;
    URL markersURL;

/*
 *  Initialize the applet. The Parameters passed are the title of the plot
 *  the marker file to use and the update period in milliseconds
 */
	public void init(String titleF, String server, int server_port,int a_noheap) 
	{
		int i;
    	int j;

	/*
 	 *  Create the Graph instance and modify the default behaviour
 	 */

		noheap = a_noheap;

    	graph = new Graph2D();
    	graph.zerocolor = new Color(0,255,0);
    	graph.borderTop    = 50;
    	graph.borderBottom = 50;
    	graph.setDataBackground(Color.black);
	/*
 	 *  Create the Title
 	 */
    	title = new Label(titleF, Label.CENTER);
    	title.setFont(new Font("TimesRoman",Font.PLAIN,25));

	/*
 	 *  Load a file containing Marker definitions
 	 */
    	try {
    		graph.setMarkers(new Markers(new FileInputStream("marker.txt")));
    	} catch(Exception e) {
        	/*System.err.println("Failed to create Marker URL!");*/
    	}
 

	/*
 	 *  Modify the default Data behaviour
     */
		if(noheap == 1)
    		data1.linecolor   = Color.yellow;
		else
    		data1.linecolor   = Color.green;

    	data1.marker      = 1;
    	data1.markercolor = Color.gray;
	/*
 	 *  Setup the Axis. Attach it to the Graph2D instance, 
     *	and attach the data to it.
     */
    	yaxis_right = graph.createAxis(Axis.RIGHT);
    	yaxis_right.attachDataSet(data1);
    	yaxis_right.setLabelFont(new Font("Helvetica",Font.PLAIN,20));

    	graph.attachDataSet(data1);
		
		ServerThread sthr = new ServerThread(server,server_port,noheap);
		sthr.start();
		
		p_guireader = Pipe.getInstance();
		if(noheap == 1){
			pin = p_guireader.getNoHeapPipeIn();
			pout = p_guireader.getNoHeapPipeOut();
		}else {
			pin = p_guireader.getSimplePipeIn();
			pout = p_guireader.getSimplePipeOut();
		}
	
    	this.setLayout( new BorderLayout() );
    	this.add("North",  title);
    	this.add("Center", graph);
	}

	public void start() 
	{
		if(runner == null) {
    		runner = new Thread(this);
        	runner.start();
    	}
	}


	public void stop() 
	{
		if(runner != null) {
    		runner.stop();
        	runner = null;
    	}
	}

	public void run() 
	{
		int i =0;
    	double data[] = new double[2];
    	Graphics g;
    	g = graph.getGraphics();
		if( osi == null || iwidth != graph.size().width
    		|| iheight != graph.size().height  ) {
    		iwidth = graph.size().width;
        	iheight = graph.size().height;
        	osi = graph.createImage(iwidth,iheight);
        	osg = osi.getGraphics();
    	}

    	osg.setColor(this.getBackground());
    	osg.fillRect(0,0,iwidth,iheight);
    	osg.setColor(g.getColor());

    	osg.clipRect(0,0,iwidth,iheight);
            
	
		try {
			rp = pin.read();
		}catch(Exception e){
			System.err.println("Exception in reading pipe");
			e.printStackTrace();
		}

    	while(true) {
    		count++;
			if(count >= maximum) data1.delete(0,0);
        	data[1] = (rp);
            data[0] = count;

            try {
            	data1.append(data,1);
		    } catch (Exception e) {
            	System.out.println("Error appending Data!");
            }


			data1.yaxis.maximum =  100.0;
        	data1.yaxis.minimum =  0.0;
            
			graph.update(osg);

            g.drawImage(osi,0,0,graph);
		

			/*if(noheap == 1)
				rp = p_guireader.noheappipe_read();
			else
				rp = p_guireader.pipe_read();*/
			
			try {
				rp = pin.read();
			}catch(Exception e){
				System.err.println("Exception in reading pipe");
				e.printStackTrace();
			}
            
			try {  
				Thread.sleep(period); 
			} catch(Exception e) { }

	     }
	}

	public static void print_usage()
	{
		System.out.println("usage: java TimeSys <no_of_thr> <server_name>");
		System.out.println("no_of_thr: 1) NoHeapRealtime 2) Simple 3) Both");
		System.exit(0);
	}

	public static void main(String[] arg)
	{
		TimeSys obj1,obj2 ;
		TimeFrame mainF;
		
		if(arg.length != 2)
			print_usage();

		obj1 = new TimeSys();
		obj2 = new TimeSys();

		int no_thr = new Integer(arg[0]).intValue();

		if(no_thr == 1 || no_thr == 3)
			obj1.init("NoHeapRealTime",arg[1],8192, 1);

		if(no_thr == 2 || no_thr == 3)
			obj2.init("SimpleThread",arg[1], 8194, 0);
	
		mainF = new TimeFrame("TimeSys");
		if(no_thr == 3)	
    		mainF.setLayout( new GridLayout(2,1) );
		else
			mainF.setLayout(new GridLayout(1,1));
		if(no_thr == 1 || no_thr == 3)
    		mainF.add(obj1);
		if(no_thr == 2 || no_thr == 3)
    		mainF.add(obj2);
		mainF.setSize(900,700);
		mainF.setResizable(false);
		mainF.show();

		if(no_thr == 1 || no_thr == 3)
			obj1.start();
		if(no_thr == 2 || no_thr == 3)
			obj2.start();

		while(true){
			try{
				Thread.sleep(100000);
			}catch(Exception e){}
		}
	}
}

class TimeFrame extends Frame
{

	public TimeFrame(String name)
	{
		super(name);
		  addWindowListener(new WindowAdapter(){
	         public void windowClosing(WindowEvent e) {
					System.exit(0);
	         }
     });
	}
}
