import javax.realtime.*;

/* 
   This is the third in a series of basic AIE tests designed as a starting
   point for more comprehensive testing. Each program tests an individual
   feature of the the AsynchronouslyInterruptedException (AIE) class and
   it's features.

   AIE3.java tests synchronozation deferring of remote AI exceptions in
   explicitly synchronized code blocks (which use the monitorenter and
   monitorexit bytecodes). The parent thread sleeps for two seconds then
   interrupts it's child with the system generic AIE. Meanwhile, the child
   is spinning (not sleeping, as sleep has special behaviour with AIE's) in a
   synchronized block which it exits after the exception is posted.

   Depending on the testing machine, the nature of the build and other timing
   factors, Target3.count may have to be adjusted to ensure the AIE gets
   posted during the synchronized block.

   The correct output is as follows:

   * AIE3 Test commencing
   * starting target thread
   * Parent sleeping for two seconds
   * Target thread started
   * Target thread aquiring object lock
   * Target thread busey waiting in synchronized block
   * Parent interrupting Target
   * Target thread releasing object lock
   * Target caught system generic AIE
   javax.realtime.AsynchronouslyInterruptedException
	   at javax.realtime.MemoryArea.newInstance0(Ljava/lang/Class;)Ljava/lang/Object;(Native Method)
	   at javax.realtime.MemoryArea.newInstance(Ljava/lang/Class;)Ljava/lang/Object;(Unknown Source)
	   at javax.realtime.AsynchronouslyInterruptedException.getGeneric()Ljavax/realtime/AsynchronouslyInterruptedException;(Unknown Source)
	   at javax.realtime.RealtimeThread.interrupt()V(Unknown Source)
	   at AIE3.main([Ljava/lang/String;)V(Unknown Source)
   * Test completed successfully
   
*/


public class AIE3
{
  public static void main(String[] argv) throws InterruptedException
  {
    RealtimeThread t = new javax.realtime.RealtimeThread(null, null, null, null, null,
					  new Target3());
    
    System.out.print("* AIE3 Test commencing\n");
    System.out.print("* starting target thread\n");
    
    t.start();
    
    System.out.print("* Parent sleeping for two seconds\n");
    Thread.sleep(2000);

    System.out.print("* Parent interrupting Target\n");
    t.interrupt();
  }
}

class Target3 implements Runnable
{

  public int count = 50000000;

  public Object lock = new Object();

  public void run()
  {
    System.out.print("* Target thread started\n");
    try{
      loop();
    }
    catch(Throwable e){
      if(e == AsynchronouslyInterruptedException.getGeneric()){
	System.out.print("* Target caught system generic AIE\n");
	e.printStackTrace();
	((AsynchronouslyInterruptedException)e).happened(false);
	System.out.print("* Test completed successfully\n");
      }
      else{
	System.out.print("* Target did not catch generic AIE\n");
	e.printStackTrace();
	System.out.print("* Test failed\n");
      }
    }
  }

  public void loop() throws AsynchronouslyInterruptedException
  {
    System.out.print("* Target thread aquiring object lock\n");
    synchronized(lock)
      {
	System.out.print("* Target thread busey waiting in synchronized block\n");
	for(int i=0; i<count; i++)
	  ;
	System.out.print("* Target thread releasing object lock\n");
      }
    System.out.print("* Target thread has left synchronized block without throwing aie.\n");
    System.out.print("* Target thread spinning in interruptible method\n");
    while(true);
  }  
}
