import javax.realtime.*;

/* 
   This is the seventh in a series of basic AIE tests designed as a starting
   point for more comprehensive testing. Each program tests an individual
   feature of the the AsynchronouslyInterruptedException (AIE) class and
   it's features.

   AIE7.java tests AIE interruption of Thread.stop(). The parent thread
   sleeps for two seconds then interrupts it's child with the system generic
   AIE. Meanwhile, the child is sleeping.

   Depending on the testing machine, the nature of the build and other timing
   factors, Target7.count may have to be adjusted to ensure the AIE gets
   posted during the 'defer' method.

   The correct output is as follows:

   * AIE7 Test commencing
   * starting target thread
   * Parent sleeping for two seconds
   * Target thread started
   * Target thread attempting to sleep for five seconds
   * Parent interrupting Target
   * Target caught system generic AIE
   javax.realtime.AsynchronouslyInterruptedException
           at javax.realtime.MemoryArea.newInstance0(Ljava/lang/Class;)Ljava/lang/Object;(Native Method)
	   at javax.realtime.MemoryArea.newInstance(Ljava/lang/Class;)Ljava/lang/Object;(Unknown Source)
	   at javax.realtime.AsynchronouslyInterruptedException.getGeneric()Ljavax/realtime/AsynchronouslyInterruptedException;(Unknown Source)
	   at javax.realtime.RealtimeThread.interrupt()V(Unknown Source)
	   at AIE7.main([Ljava/lang/String;)V(Unknown Source)
   * Test completed successfully

*/


public class AIE7
{
  public static void main(String[] argv) throws InterruptedException
  {
    RealtimeThread t = new RealtimeThread(null, null, null, null, null,
					  new Target7());
    
    System.out.print("* AIE7 Test commencing\n");
    System.out.print("* starting target thread\n");
    
    t.start();
    
    System.out.print("* Parent sleeping for two seconds\n");
    Thread.sleep(2000);

    System.out.print("* Parent interrupting Target\n");
    t.interrupt();
  }
}

class Target7 implements Runnable
{

  public int count = 50000000;

  public void run()
  {
    System.out.print("* Target thread started\n");
    try{
      loop();
    }
    catch(Throwable e){
      if(e == AsynchronouslyInterruptedException.getGeneric()){
	System.out.print("* Target caught system generic AIE\n");
	e.printStackTrace();
	((AsynchronouslyInterruptedException)e).happened(false);
	System.out.print("* Test completed successfully\n");
      }
      else{
	System.out.print("* Target did not catch generic AIE\n");
	e.printStackTrace();
	System.out.print("* Test failed\n");
      }
    }
  }

  public void loop() throws InterruptedException
  {
    System.out.print("* Target thread attempting to sleep for five seconds\n");
    Thread.sleep(5000);
    System.out.print("* Target thread spinning in interruptible method\n");
    while(true);
  }
}

