import javax.realtime.*;

/* 
   This is the seventh in a series of basic AIE tests designed as a starting
   point for more comprehensive testing. Each program tests an individual
   feature of the the AsynchronouslyInterruptedException (AIE) class and
   it's features.

   AIE8.java tests AIE interruption of Object.wait(). The parent thread
   sleeps for two seconds then interrupts it's child with the system generic
   AIE. Meanwhile, the child is waiting on an object semaphore.

   The correct output is as follows:

   * AIE8 Test commencing
   * starting target thread
   * Parent sleeping for two seconds
   * Target thread started
   * Target thread attempting to wait on object semaphore
   * Parent interrupting Target
   * Target caught system generic AIE
   javax.realtime.AsynchronouslyInterruptedException
           at javax.realtime.MemoryArea.newInstance0(Ljava/lang/Class;)Ljava/lang/Object;(Native Method)
	   at javax.realtime.MemoryArea.newInstance(Ljava/lang/Class;)Ljava/lang/Object;(Unknown Source)
	   at javax.realtime.AsynchronouslyInterruptedException.getGeneric()Ljavax/realtime/AsynchronouslyInterruptedException;(Unknown Source)
	   at javax.realtime.RealtimeThread.interrupt()V(Unknown Source)
	   at AIE8.main([Ljava/lang/String;)V(Unknown Source)
   * Test completed successfully

*/


public class AIE8
{
  public static void main(String[] argv) throws InterruptedException
  {
    RealtimeThread t = new RealtimeThread(null, null, null, null, null,
					  new Target8());
    
    System.out.print("* AIE8 Test commencing\n");
    System.out.print("* starting target thread\n");
    
    t.start();
    
    System.out.print("* Parent sleeping for two seconds\n");
    Thread.sleep(2000);

    System.out.print("* Parent interrupting Target\n");
    t.interrupt();
  }
}

class Target8 implements Runnable
{
    
  public Object lock = new Object();

  public void run()
  {
    System.out.print("* Target thread started\n");
    try{
      loop();
    }
    catch(Throwable e){
      if(e == AsynchronouslyInterruptedException.getGeneric()){
	System.out.print("* Target caught system generic AIE\n");
	e.printStackTrace();
	((AsynchronouslyInterruptedException)e).happened(false);
	System.out.print("* Test completed successfully\n");
      }
      else{
	System.out.print("* Target did not catch generic AIE\n");
	e.printStackTrace();
	System.out.print("* Test failed\n");
      }
    }
  }

  public void loop() throws InterruptedException
  {
    System.out.print("* Target thread attempting to wait on object semaphore\n");
    synchronized(lock)
      {
	lock.wait();
      }
    System.out.print("* Target thread spinning in interruptible method\n");
    while(true);
  }
}

