//package javax.realtime.calltests;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 2000  All Rights Reserved
 */


/*				AsyncEventTest
 
Subtest 1:
	"public AsyncEvent()"

Subtest 2:
	"public synchronized void addHandler(AsyncEventHandler handler)" 
	where handler is null

Subtest 3:
	"public synchronized void addHandler(AsyncEventHandler handler)"

Subtest 4:
	"public void bindTo(java.lang.String happening)

Subtest 5:
	"public ReleaseParameters createReleaseParameters()"

Subtest 6:
	"public boolean handledBy(AsyncEventHandler target)"

Subtest 7:
	"public boolean handledBy(AsyncEventHandler target)" where target is 
	not the handler

Subtest 8:
	"public boolean handledBy(AsyncEventHandler target)" where target is 
	null

Subtest 9:
	"public synchronized void removeHandler(AsyncEventHandler handler)

Subtest 10:
	"public synchronized void removeHandler(AsyncEventHandler handler) 
	where handler not associated with AsyncEvent

Subtest 11:
	"public synchronized void removeHandler(AsyncEventHandler handler) 
	where handler is null

Subtest 12:
	"public synchronized void setHandler(AsyncEventHandler handler)"

Subtest 13:
	"public synchronized void setHandler(AsyncEventHandler handler)" where
	handler is null

Subtest 14:
	"public synchronized void fire()"
*/

import javax.realtime.*;
import com.timesys.*;

public class AsyncEventTest 
{
	
    public static void run() {
	AsyncEvent ae = null;
	Object o = null;
	int pid;
	
	BoundAsyncEventHandler h = new BoundAsyncEventHandler(null, null,null,
			          new LTMemory(1048576,1048576),null,true){
		public void handleAsyncEvent(){
		    //				for(int i=0; i<10000000;i++);
		    System.out.println("**** Inside h.handleAsyncEvent()");
		}
	    };
	
	BoundAsyncEventHandler h2 = new BoundAsyncEventHandler(null, null,null,
			          new LTMemory(1048576,1048576),null,true){
		public void handleAsyncEvent(){
		    //				for(int i=0; i<10000000;i++);
		    System.out.println("**** Inside h2.handleAsyncEvent()");
		}
	    };
	
	Tests.newTest("AsyncEventTest");

	/* Subtest 1:
	** Constructor "public AsyncEvent()"
	*/
	Tests.increment();

	try {
	    System.out.println("AsyncEventTest: AsyncEvent()");
	    ae = new AsyncEvent();
	    if( !(ae instanceof AsyncEvent) )
		throw new Exception("Return object is not instanceof "+
				    "AsyncEvent");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: AsyncEvent() failed");
	    Tests.fail("new AsyncEvent()",e);
	}
		
	/* Subtest 2:
	** Method "public synchronized void addHandler(AsyncEventHandler 
	** handler)" where handler is null
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: addHandler("+
			       "AsyncEventHandler)");
	    BoundAsyncEventHandler nh = null;
	    ae = new AsyncEvent();
	    ae.addHandler(nh);
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: addHandler("+
			       "AsyncEventHandler) failed");
	    Tests.fail("ae.addHandler(null)",e);
	}

	/* Subtest 3:
	** Method "public synchronized void addHandler(AsyncEventHandler 
	** handler)"
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: addHandler("+
			       "AsyncEventHandler)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: addHandler("+
			       "AsyncEventHandler) failed");
	    Tests.fail("ae.addHandler(h)",e);
	}

	/* Subtest 4:
	** Method "public void bindTo(java.lang.String happening)
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: bindTo(String)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    ae.bindTo("63");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: bindTo(String) failed");
	    Tests.fail("ae.bindTo(63)",e);
	}

	/* Subtest 5:
	** Method "public ReleaseParameters createReleaseParameters()"
	*/

	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: createReleaseParameters()");
	  ae = new AsyncEvent();
	  o = ae.createReleaseParameters();
	  if (! (o instanceof ReleaseParameters))
	      throw new Exception("Return object is not instanceof "+
				  "ReleaseParameters");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: createReleaseParameters()"+
			       "failed");
	    Tests.fail("ae.createReleaseParameters()",e);
	}
	
	/* Subtest 6:
	** Method "public boolean handledBy(AsyncEventHandler target)"
	where target is the handler
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: handledBy(AsyncEventHandler)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    boolean hb = ae.handledBy(h);
	    if (hb == false)
		throw new Exception("Handler not recognized");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: handledBy(AsyncEventHandler) "+
			       "failed");
	    Tests.fail("ae.handledBy(h)",e);
	}
		
	/* Subtest 7:
	** Method "public boolean handledBy(AsyncEventHandler target)" where 
	** target is not the handler
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: handledBy(AsyncEventHandler)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    boolean hb = ae.handledBy(h2);
	    if (hb == true)
		throw new Exception("Wrong handler recognized");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: handledBy(AsyncEventHandler) "+
			       "failed");
	    Tests.fail("ae.handledBy(h)",e);
	}

	/* Subtest 8:
	** Method "public boolean handledBy(AsyncEventHandler target)" where 
	** target is null
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: handledBy(AsyncEventHandler"+
			       "=null)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    boolean hb = ae.handledBy(null);
	    if (hb == true)
		throw new Exception("handledBy() method returned true for a "+
				    "null target");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: handledBy(AsyncEventHandler"+
			       "=null) failed");
	    Tests.fail("ae.handledBy(null)",e);
	}

	/* Subtest 9:
	** Method "public synchronized void removeHandler(AsyncEventHandler 
	** handler)
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: removeHandler()");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    ae.removeHandler(h);
	    if (ae.handledBy(h)==true)
		throw new Exception("Handler was not removed");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: removeHandler() failed");
	    Tests.fail("ae.removeHandler(h)",e);
	}

	/* Subtest 10:
	** Method "public synchronized void removeHandler(AsyncEventHandler 
	** handler) where handler not associated with AsyncEvent
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: removeHandler(AsyncEvent"+
			       "Handler)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    ae.removeHandler(h2);
	    if (ae.handledBy(h)==false)
		throw new Exception("Wrong handler was removed");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: removeHandler(AsyncEvent"+
			       "Handler) failed");
	    Tests.fail("ae.removeHandler(h)",e);
	}

	/* Subtest 11:
	** Method "public synchronized void removeHandler(AsyncEventHandler 
	** handler) where handler is null
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: removeHandler(null)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    ae.removeHandler(null);
	    if (ae.handledBy(h) == false)
		throw new Exception("Handler was removed");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: removeHandler(null) failed");
	    Tests.fail("ae.removeHandler(null)",e);
	}

	/* Subtest 12:
	** Method "public synchronized void setHandler(AsyncEventHandler 
	** handler)"
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: setHandler("+
			       "AsyncEventHandler)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    ae.setHandler(h2);
	    if (ae.handledBy(h) == true)
		throw new Exception("Handler was not set properly");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: setHandler("+
			       "AsyncEventHandler) failed");
	    Tests.fail("ae.setHandler(h2)",e);
	}

	/* Subtest 13:
	** Method "public synchronized void setHandler(AsyncEventHandler 
	** handler)" where handler is null
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: setHandler(null)");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    ae.setHandler(null);
	    if (ae.handledBy(h) == true)
		throw new Exception("Handler was not removed");
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: setHandler(null) failed");
	    Tests.fail("ae.setHandler(h2)",e);
	}

	/* Subtest 14:
	** Method "public synchronized void fire()"
	*/
	Tests.increment();
	try {
	    System.out.println("AsyncEventTest: fire()");
	    ae = new AsyncEvent();
	    ae.addHandler(h);
	    ae.addHandler(h2);
	    ae.fire();
	} catch (Exception e) {
	    System.out.println("AsyncEventTest: fire() failed");
	    Tests.fail("ae.fire()",e);
	}

	Tests.printSubTestReportTotals("AsyncEventTest");
    }
}
