import javax.realtime.*;
import java.util.*;

// FIXME what to do about ImmortalMemory ? It will keep getting created, and
// eventually grind the system to a screeching halt

public class MemoryFactory extends RealtimeThread {

    final static public int HEAP_MEMORY=0;
    final static public int IMMORTAL_MEMORY=1;
    final static public int VT_MEMORY=2;
    final static public int LT_MEMORY=3;
    
    final static int MAX_SIZE=40000;

    int quit;

    Random r;

    sanity_checker san_chk;

    static final int CHANCE_MEM_AREA_CREATE=10;

    public MemoryFactory() {
	r=new Random();
	quit=0;
    }

    public void run() {

	System.out.println("*** MemoryFactory is running");

	while(quit!=1) {

	    System.out.println("*** MemoryFactory about to lock");
	    san_chk.lock(this);
	    System.out.println("*** MemoryFactory done locking");

	    if((getRand()%100 + san_chk.getNumMemAreas()) <
	       CHANCE_MEM_AREA_CREATE) {
		
		int type_to_create=getRand()%4;
		MemData md=createArea(type_to_create);

		System.out.println("*** MemFactory created memory");
		md.dump();

	    }

	    System.out.println("*** MemoryFactory about to unlock");
	    san_chk.unlock(this);
	    System.out.println("*** MemoryFactory done unlocking");
	}

    }

    public void set_sanity_checker(sanity_checker sc) { san_chk=sc; }

    public int getRand() {
	int rand=r.nextInt();
	if(rand < 0) rand *=-1;

	return rand;
    }

    MemData createArea(int type) {

	MemData md=new MemData(san_chk.uniqueId(), san_chk);
	MemoryArea ma=null;
	int init=0, tot=0, err_occured=0;

	switch(type) {
	case HEAP_MEMORY :
	    ma=new MyHeapMemory(0);
	    System.out.println("*** MemoryFactory creating a HeapMemory");
	    break;
	case IMMORTAL_MEMORY :
	    ma=new MyImmortalMemory(0);
	    System.out.println("*** MemoryFactory creating an ImmortalMemory");
	    break;
	case VT_MEMORY :
	    tot=r.nextInt()%MAX_SIZE;
	    if(tot < 0) tot*=-1;
	    init=r.nextInt()%MAX_SIZE;
	    if(init < 0) init*=-1;
	    System.out.println("*** MemoryFactory creating a VTMemory of "+
			       "size init : "+init+" tot : "+tot);

	    try {
		ma=new MyVTMemory(init, tot);
	    } catch(Throwable t) {
		System.out.println("***** MemoryFactory Error occured in "+
				   "VT_Memory creation"+t);
		err_occured=1;
	    }

	    if(err_occured==1) {
		if(init > tot) {
		    return null;
		} else {
		    System.out.println("ERROR VT_Memory create failed. Why?");
		    san_chk.destroyAll();
		}
	    } 

	    md.setInitial(init);
	    md.setMaximum(tot);
	    break;

	case LT_MEMORY :
	    tot=r.nextInt()%MAX_SIZE;
	    if(tot < 0) tot*=-1;
	    init=r.nextInt()%MAX_SIZE;
	    if(init < 0) init*=-1;
	    System.out.println("*** MemoryFactory creating a LTMemory of "+
			       "size init : "+init+" tot : "+tot);

	    try {
		ma=new MyLTMemory(init, tot);
	    } catch(Throwable t) {
		System.out.println("*** MemoryFactory Error occured in "+
				   "LT_Memory creation"+t);
		err_occured=1;
	    }

	    if(err_occured==1) {
		if(init > tot) {
		    return null;
		} else {
		    System.out.println("ERROR LT_Memory create failed. Why?");
		    san_chk.destroyAll();
		}
	    } 

	    md.setInitial(init);
	    md.setMaximum(tot);
	    break;

	}

	md.setType(type);
	md.setMemoryArea(ma);
	san_chk.addMemData(md);

	System.out.println("There was a MemoryArea created");
	md.dump();

	return(md);

    }

    public MemData getMemData() {

	MemData md=null;

	System.out.println("Inside getMemData - tot = "+san_chk.num_mem_areas);

	if(san_chk.num_mem_areas == 0) {
	    while(md==null) {
		int type=r.nextInt()%4;
		if(type < 0) type*=-1;
		md=createArea(type);
	    }
	}

	int index=r.nextInt()%san_chk.num_mem_areas;
	if(index < 0) index*=-1;
	md=san_chk.getMemDataIndex(index);

	System.out.println("*** Using mem_data index - "+index);
	
	System.out.println("***MemFactory  getMemData dump - ");
	md.dump();

	return md;

    }

    public void setQuit() {quit = 1;}

}
