import java.util.*;

public class ObjectData extends Object {

    // Unique identifier
    int id;

    // Number of threads that hold a reference to obj
    int reference_count;
    
    // If this is 0, then no new threads may get a reference to obj
    int is_assignable;
    
    // The memory area that obj was created in
    MemData md;

    // These are all the threads that hold a reference to this object
    Vector thread_data;

    sanity_checker san_chk;

    public ObjectData(int i_d, sanity_checker sc) {
	id=i_d;
	is_assignable=1;
	reference_count=0;
	san_chk=sc;
    }

    // Just a data dump
    public void dump(int caller) {

	System.out.println("-----Begin ObjectData Dump-----");

	System.out.println("Id : "+id+" mem area : "+md.getId()+" reference "+
			   "count : "+reference_count+" is_assignable : "+
			   is_assignable);

	if(caller != sanity_checker.MEMORY) {
	    System.out.println("MemData");
	    md.dump(sanity_checker.OBJECT);
	}

	if(caller != sanity_checker.THREAD) {
	    System.out.println("Threads");
	    for(int x=0; x<reference_count; x++) {
		((ThreadData)thread_data.elementAt(x)).dump(
						    sanity_checker.OBJECT);
	    }
	}

	System.out.println("-----End ObjectData Dump-----");

    }

    public void dump() {
	dump(-1);
    }

    // When sanity_checker sees that an object's reference count is
    // 0, and is_assignable == 0, it should remove all reference to the 
    // ObjectData, and call releaseObject() (for Immortal memory?)
    public void selfDestruct() { 
	is_assignable=0;
    }

    // This is the memory that the object is created in
    public void setMemData(MemData memd) { md=memd; }
    public MemData getMemData() { return md; }
    

    // This is the objects unique identifier
    public int getId() { return id; }

    // This will release the final reference to the object so that the MyObject
    // that is present here can be garbage collected.
    public void releaseObject() { md.release(id); }

    public int getIsAssignable() { return is_assignable; }
    public int getReferenceCount() { return reference_count; }

    public void addThreadData(ThreadData td) { 
	thread_data.addElement(td); 
	reference_count++;
    }

    public void removeThreadData(ThreadData td) {

	for(int x=0; x<reference_count; x++) {
	    ThreadData tmp=((ThreadData)thread_data.elementAt(x));
	    if(tmp==td) {
		thread_data.remove(x);
		return;
	    }
	}

    }

}





