import javax.realtime.*;
import java.util.*;

public class ObjectFactory extends RealtimeThread {

    Random r; 

    sanity_checker san_chk;

    MemoryFactory mf;

    final static int CHANCE_OBJECT_CREATE=30;
    final static int CHANCE_OBJECT_DESTROY=20;

    int quit;

    public ObjectFactory() {
	r=new Random();
	quit=0;
    }

    int getRand() {
	int ret=r.nextInt()%100;
	if(ret<0) ret*=-1;
	return ret;
    }

    ObjectData createObject() {

	System.out.println("***ObjectFactory creating an object");

	MyObject mo=null;
	MemData md=mf.getMemData();

	System.out.println("*** ObjectFactory using MemData");
	md.dump();

	int id=san_chk.uniqueId();
	try {
	    MemoryArea ma=md.getMemArea();
	    if(ma instanceof MyHeapMemory) {
		((MyHeapMemory)ma).createObject(id);
	    } else if(ma instanceof MyImmortalMemory) {
		((MyImmortalMemory)ma).createObject(id);
	    } else if(ma instanceof MyLTMemory) {
		((MyLTMemory)ma).createObject(id);
	    } else if(ma instanceof MyVTMemory) {
		((MyVTMemory)ma).createObject(id);
	    } else {
		System.out.println("ERROR : ObjectFactory create with mem. "+
				   "area of unknown type");
		san_chk.destroyAll();
	    }
	} catch(Throwable t) {
	    System.out.println("Mem data = "+md+" memarea = "
			       +md.getMemArea());
	    System.out.println("***ObjectFactory Error creating an object "+t);
	    san_chk.destroyAll();
	}

	ObjectData od=new ObjectData(id,san_chk);
	san_chk.addObjectData(od);
	san_chk.attach(md.getMemArea(),id);
	return od;

    }

    void destroyObject() {
	int des=getRand()%san_chk.getNumObjects();
	ObjectData od=san_chk.getObjectIndex(des);
	od.selfDestruct();

	System.out.println("*** destroyObject called on");
	od.dump();

    }

    public void run() {

	while(quit != 1) {

	    san_chk.lock(this);

	    if((getRand()+san_chk.getNumObjects()) < CHANCE_OBJECT_CREATE) {
		System.out.println("*** ObjectFactory creating an object");
		createObject();
	    }

	    if((getRand() < CHANCE_OBJECT_DESTROY) && 
	       san_chk.getNumObjects()>0){
		System.out.println("*** ObjectFactory destroying an object");
		destroyObject();
	    }

	    san_chk.unlock(this);

	}

    }

    // If there are no current objects, this will have one created, otherwise
    // a random object is selected from the pool of objects and a reference to
    // it is returned
    public ObjectData getObjectData() {		

	if(san_chk.getNumObjects() == 0) createObject();
	int des=getRand()%san_chk.getNumObjects();
	ObjectData od=san_chk.getObjectIndex(des);
	return od;

    }

    public void set_sanity_checker(sanity_checker sc) { san_chk=sc; }

    public void setMemoryFactory(MemoryFactory memf) { mf=memf; }

    public void setQuit() { quit=1; }

}
