import javax.realtime.*;
import java.util.*;

public class ThreadFactory extends RealtimeThread {

    Random r;

    MemoryFactory mem_fact;
    ObjectFactory obj_fact;

    sanity_checker san_chk;

    // This is also the number of threads that will be alive in the system
    static final int CHANCE_THREAD_CREATE = 20;

    static final public int USER_THREAD=0;
    static final public int RT_THREAD=1;
    static final public int NHRT_THREAD=2;

    int quit;

    public ThreadFactory() {
	r=new Random();
	quit=0;
    }

    public void setMemoryFactory(MemoryFactory mf) { mem_fact=mf; }
    public void setObjectFactory(ObjectFactory of) { obj_fact=of; }
    public void set_sanity_checker(sanity_checker sc) { san_chk=sc; }

    public int getRand() {
	int rand=r.nextInt();
	if(rand < 0) rand *=-1;

	return rand;
    }

    public void run() {

	System.out.println("*** ThreadFactory is running");

	ThreadData td;

	// quit is the mechanism that sanity_checker uses to terminate all
	// threads when something goes wrong
	while(quit!=1) {

	    td=null;
	    
	    System.out.println("*** ThreadFactory about to lock");

	    // to insure stepwise execution of all threads
	    san_chk.lock(this);
	    System.out.println("*** ThreadFactory has locked");

	    // CHANCE_THREAD_CREATE == max number of threads as well
	    if((getRand()%100 + san_chk.getNumThreads()) < 
	       CHANCE_THREAD_CREATE) {

		int type_to_create=getRand() % 3;

		switch(type_to_create) {

		case 0 : 
		    System.out.println("*** ThreadFactory is creating a "+
				       "Thread");
		    td=createThread();
		    break;
		case 1 : 
		    System.out.println("*** ThreadFactory is creating a "+
				       "RealtimeThread");
		    td=createRealtimeThread();
		    break;
		case 2 : 
		    System.out.println("*** ThreadFactory is creating a "+
				       "NoHeapRealtimeThread");
		    td=createNoHeapRealtimeThread();
		    break;

		}

		if(td != null) {
		    System.out.println("Created :");
		    td.dump();
		}

	    }

	    System.out.println("ThreadFactory about to unlock");
	    san_chk.unlock(this);
	    System.out.println("ThreadFactory unlocked");

	    if(td!=null) td.getThread().start();

	}

    }

    ThreadData createThread() {

	ThreadData td=new ThreadData(san_chk.uniqueId());

	td.type=USER_THREAD;
	td.setThread(new MyThread(san_chk));
	san_chk.addThreadData(td);

	return td;

    }

    // This function will create a realtime thread, set up it's 
    // corresponding ThreadData structure, and add the ThreadData to san_chk
    // so that a reference to this thread is somewhat global to all classes
    ThreadData createRealtimeThread() {

	ThreadData td=new ThreadData(san_chk.uniqueId());

	SchedulingParameters sp=new PriorityParameters(mem_tester.BASE_PRIO);

	td.type=RT_THREAD;
	td.setThread(new MyRealtimeThread());

	((RealtimeThread)td.getThread()).setSchedulingParameters(sp);

	((MyRealtimeThread)td.getThread()).set_sanity_checker(san_chk);
	((MyRealtimeThread)td.getThread()).setObjectFactory(obj_fact);
	((MyRealtimeThread)td.getThread()).setMemoryFactory(mem_fact);

	san_chk.addThreadData(td);

	return td;

    }

    ThreadData createNoHeapRealtimeThread() {

	int err_occured=0;

	ThreadData td=new ThreadData(san_chk.uniqueId());

	td.type=NHRT_THREAD;

	SchedulingParameters sp=new PriorityParameters(mem_tester.BASE_PRIO);
	MemData md=mem_fact.getMemData();

	if(md==null) {
	    System.out.println("MemData == null");
	    san_chk.destroyAll();
	}

	MyNoHeapRealtimeThread nhrtt=null;

	System.out.println("*** ThreadFactory is using MemData");
	md.dump();

	MemoryArea tmpma=null;

	System.out.println("After Here");

	if(md.getMemoryArea() instanceof MyHeapMemory) {
	    tmpma=((MyHeapMemory)md.getMemoryArea()).getSelf();
	} else if(md.getMemoryArea() instanceof MyImmortalMemory) {
	    tmpma=((MyImmortalMemory)md.getMemoryArea()).getSelf();
	} else if(md.getMemoryArea() instanceof MyLTMemory) {
	    tmpma=((MyLTMemory)md.getMemoryArea()).getSelf();
	} else if(md.getMemoryArea() instanceof MyVTMemory) {
	    tmpma=((MyVTMemory)md.getMemoryArea()).getSelf();
	}

	System.out.println("After Here2");

	try {
	    nhrtt=new MyNoHeapRealtimeThread(san_chk, sp, tmpma);
	} catch(Throwable t) {
	    System.out.println("*** ThreadFactory There was an exception "+
			       "that occured : "+t);
	    err_occured=1;
	}

	System.out.println("*** Still running");

	if(err_occured==1) {
	    if(md.type == MemoryFactory.HEAP_MEMORY) {
		return null;
	    } else {
		System.out.println("ERROR ThreadFactory : There was an "+
				   "exception in creating "+
				   "a NoHeapRealtimeThread that should not "+
				   "have occured");
		san_chk.destroyAll();
	    } 
	} else {
	    if(md.type == MemoryFactory.HEAP_MEMORY) {
		System.out.println("ERROR ThreadFactory : A heap memory "+
                                   "object was given to a NHRTT and there "+
                                   "was no exception thrown");
		san_chk.destroyAll();
	    }
	}

	td.setThread(nhrtt);
	san_chk.addThreadData(td);
	san_chk.attach(td.getThread(), md.getMemoryArea(), 
		       sanity_checker.STATIC);

	return td;

    }

    public void setQuit() { 
	System.out.println("*** ThreadFactory calling setQuit");
	quit=1; 
    }

}
