import java.lang.*;
import java.io.*;
import javax.realtime.*;
import java.util.*;

public class MyThread extends RealtimeThread {

    public static final int STARTING_RUN=0;
    public static final int ENDING_RUN=1;
    public static final int LOCKING=2;
    public static final int UNLOCKING=3;

    public static final int WAITING=0;
    public static final int RUNNABLE=1;
    public static final int RUNNING=2;
    public static final int BLOCKING=3;

    public static final int MAX_HOLDS=4;

    public static final int BASE_PRIO=11;

    public static final int NUM_THREADS=2;

    public int index, priority, state, num_locks, quit;

    public AbsoluteTime trigger_edge, start;

    int is_overrun;

    PipedInputStream pinstr;
    PipedOutputStream poutstr;
    
    Random r;

    public int locking;

    static mutex mutexes[];
    static MyThread mt[];

    int num_blocking;

    sync_tester st;

    MyThread() {}

    MyThread(int ind, PipedInputStream pis, PipedOutputStream pos, 
	     mutex mutexe[], MyThread m[], sync_tester s) {

	quit=0;

	st=s;
	trigger_edge=new AbsoluteTime();

	mutexes=mutexe;
	mt=m;
	r=new Random();

	System.out.println("MyThread : Creating thread # "+ind);
	
	state=RUNNABLE;
	
	priority=1;
	index=ind;
	setName(Integer.toString(index));
	pinstr=pis;
	poutstr=pos;

	num_locks=0;

	is_overrun=0;                                                  

	num_blocking=0;

    }

    public void setquit() {quit=1;}

    public void check_priority() {
	int running_priority=priority;
	int higher_thread;

	for(int x=0; x<NUM_THREADS; x++) {
	    if((mt[x].state==BLOCKING) && 
	       (mt[x].path_to(index)==1)) {

		if(mt[x].priority > running_priority) {
		    running_priority=mt[x].priority;
		}

	    }
	}

	for(int x=0; x<NUM_THREADS; x++) {
	    if(mt[x].state==RUNNABLE) {
		if(mt[x].priority > running_priority) {
		    System.out.println("\n\n(*&^(*&^(*&^ERROR ::: Thread "+x+
				       "had a higher priority of "+
				       mt[x].priority+" and "+index+" still"+
				       "ran\n\n");
		    /* Dio: This method is not implemented in
		     * CVM so don't use it.
		     */
		   st.killAll();
		}
	    }
	}

    }

    public int path_to(int x) {

	int next_thread=index;
	int next_mutex;

	while(true) {

	    next_mutex=mt[next_thread].locking;
	    next_thread=mutexes[next_mutex].holder;
	    
	    if(next_thread==x) return 1;
	    if(mt[next_thread].state!=BLOCKING) return 0;

	}

    }

    public void blockingInc() {num_blocking++;}

    public void wakeup() {
	b_write(index);
	b_write(STARTING_RUN);
	b_write(0);
	
	System.out.println("MyThread : thread "+index+" is running at "+
			   "base priority "+((PriorityParameters) this.
			     getSchedulingParameters()).getPriority());
	
	start=Clock.getRealtimeClock().getTime();
	System.out.print("MyThread : thread "+index+" Current time = ");
	strungOut(start); 
	System.out.print("MyThread : thread "+index+" Period beginning "+
			 "time = ");
	strungOut(trigger_edge);
	System.out.print("MyThread : difference = ");
	timeDiff(trigger_edge, start);
	System.out.println("Timediff done ?");
	trigger_edge.set(
			 (
			  (
			   ((PeriodicParameters)this.getReleaseParameters()).
			   getPeriod().getMilliseconds()*(num_blocking+1)
			  )
			  +trigger_edge.getMilliseconds()
			 ),
			 (
			  (
			   ((PeriodicParameters)this.getReleaseParameters()).
			    getPeriod().getNanoseconds()*(num_blocking+1)
			  )+trigger_edge.getNanoseconds()
			 )
			);
	
	num_blocking=0;

	check_priority();
	
	if(state!=RUNNABLE) {
	    System.out.println("\n\n\nERROR ::: woken up thread that "+
			       "wasn't RUNNABLE\n\n");
	    dump_threads();
	    dump_mutexes();
	    st.killAll();
	}
	
	state=RUNNING;
	
	dump_threads();
	dump_mutexes();

    }

    public void waitnow() {
	b_write(index);
	b_write(ENDING_RUN);
	b_write(0);

	System.out.println("MyThread : thread "+index+" is done running");
	
	state=WAITING;

	waitForNextPeriod();

	if(is_overrun == 1) {
	    System.out.println("MyThread : Calling an extra wFNP");
	    waitForNextPeriod();
            is_overrun=0;
	}

    }

    public void locking_(int mtl) {
	b_write(index);
	b_write(LOCKING);
	b_write(mtl);

	state=BLOCKING;

    }

    public void unlocking(int mtu) {

	System.out.println("MyThread : thread "+index+" is unlocking "+mtu);

	b_write(index);
	b_write(UNLOCKING);
	b_write(mtu);
    }

    public void setPrio(int x) {
	priority=x;
    }

    public int getPrio() {
	return priority;
    }

    void b_write(int x) {

	try {
	    poutstr.write(x);
	} catch (Exception e) {
	    System.out.println("***"+e);

	    st.killAll();
	}

    }

    public void dump_mutexes() {

	System.out.println("***Dumping mutexes (thread "+index+")***");
	for(int x=0; x < mutex.NUM_MUTEX; x++) {
	    mutexes[x].dump(x);
	}

    }

    public void dump() {

	System.out.print("Thread "+index+": prio - "+priority+" overrun - "+
			 is_overrun+" state ");

	if(state==RUNNABLE) System.out.println("RUNNABLE");
	else if(state==WAITING) System.out.println("WAITING");
	else if(state==RUNNING) System.out.println("RUNNING");
	else if(state==BLOCKING) System.out.println("BLOCKING on "+locking);
	else System.out.println("ERROR-unknown state "+state);

    }

    public void dump_threads() {

	System.out.println("***Dumping threads (thread "+index+")***");
	for(int x=0; x < NUM_THREADS; x++) {
	    mt[x].dump();
	}

    }

    public int cycle_check(int curr_index, int orig_index) {

	if(mutexes[mt[curr_index].locking].locked==0) return 0;

	if(mt[mutexes[mt[curr_index].locking].holder].state!=BLOCKING) {
	    return 0;
	}

	if(mutexes[mt[curr_index].locking].holder==orig_index) {
	    return -1;
	} else return(cycle_check(mutexes[mt[curr_index].locking].holder,
				  orig_index));
    }

    public int lock() {

	int mutex_to_lock=r.nextInt();
	if(mutex_to_lock < 0) mutex_to_lock=mutex_to_lock*-1;
	mutex_to_lock=mutex_to_lock%(mutex.NUM_MUTEX);

	locking = mutex_to_lock;

	if(cycle_check(index, index)==0) {
	    
	    locking_(mutex_to_lock);

	    System.out.println("MyThread : thread "+index+" locking mutex # "+
			       locking+", "+mutexes[mutex_to_lock]);
	    
	    mutexes[mutex_to_lock].lock(this);
	    
	}

	return 0;

    }

    void strungOut(AbsoluteTime at) {

	System.out.println("("+at.getMilliseconds()+", "+
			   at.getNanoseconds()+")");

    }

    void timeDiff(AbsoluteTime te, AbsoluteTime ct) {

	System.out.println("("+(ct.getMilliseconds()-te.getMilliseconds())+
			   ", "+(ct.getNanoseconds()-te.getNanoseconds())+")");

    }

    public void run() {

	int inr=0, outr=0;
	boolean succeed;

	while(quit!=1) {
	    
	    wakeup();
	    
	    if(r.nextInt() < mutex.CHANCE_NEWLOCK) {
		lock();
	    }
	    
	    waitnow();
	    
	}
	
    }

    public int getOverrun() {
	return(is_overrun);
    }

    public void setOverrun(int x) {
	System.out.println("MyThread : thread "+index+" setting overrun/miss "+
			   "correction state to "+x);
	is_overrun=x;
    }

}




