/*
 * @(#)doubleword.h	1.30 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#ifndef _GCC_32_BIT_DOUBLEWORD_H
#define _GCC_32_BIT_DOUBLEWORD_H

/*
 * See src/share/javavm/include/porting/doubleword.h for full documentation.
 */

/*
 * Two versions, one assuming a platform double-word instruction that
 * can allow unaligned access, and the other not allowing it.  
 */

#ifdef CAN_DO_UNALIGNED_INT64_ACCESS

#define CVMjvm2Long(l)		(*(CVMJavaLong *)(l))
#define CVMlong2Jvm(l, v)	(*(CVMJavaLong *)(l) = (v))

#else /* !CAN_DO_UNALIGNED_INT64_ACCESS */

#define CVMjvm2Long(location)	\
({				\
    union {			\
	CVMJavaLong l;		\
	CVMUint32 v[2];		\
    } res;			\
    res.v[0] = (location)[0];	\
    res.v[1] = (location)[1];	\
    res.l;			\
})

#define CVMlong2Jvm(location, val)	\
{					\
    union {				\
	CVMJavaLong l;			\
	CVMUint32 v[2];			\
    } tmp;				\
    tmp.l = (val);			\
    (location)[0] = tmp.v[0];		\
    (location)[1] = tmp.v[1];		\
}

#endif /* CAN_DO_UNALIGNED_INT64_ACCESS */

#ifdef CAN_DO_UNALIGNED_DOUBLE_ACCESS

#define CVMjvm2Double(location)		\
    (*(CVMJavaDouble *)(location))

#define CVMdouble2Jvm(location, val)		\
    (*(CVMJavaDouble *)(location) = (val))

#else /* !CAN_DO_UNALIGNED_DOUBLE_ACCESS */

#define CVMjvm2Double(location)		\
({					\
    union {				\
	CVMJavaDouble d;		\
	CVMUint32 v[2];			\
    } res;				\
    res.v[0] = (location)[0];		\
    res.v[1] = (location)[1];		\
    res.d;				\
})

#define CVMdouble2Jvm(location, val)	\
{					\
    union {				\
	CVMJavaDouble d;		\
	CVMUint32 v[2];			\
    } tmp;				\
    tmp.d = (val);			\
    (location)[0] = tmp.v[0];		\
    (location)[1] = tmp.v[1];		\
}

#endif /* CAN_DO_UNALIGNED_DOUBLE_ACCESS */

#if defined(CAN_DO_UNALIGNED_DOUBLE_ACCESS) && defined(COPY_64_AS_DOUBLE)

/* CVMmemCopy64(CVMUint32* destination, CVMUint32* source) */
#define CVMmemCopy64(destination, source) \
    (*(CVMJavaDouble *)(destination) = *(CVMJavaDouble *)(source))

#elif defined(CAN_DO_UNALIGNED_INT64_ACCESS) && defined(COPY_64_AS_INT64)

/* CVMmemCopy64(CVMUint32* destination, CVMUint32* source) */
#define CVMmemCopy64(destination, source) \
    (*(CVMJavaLong *)(destination) = *(CVMJavaLong *)(source))

#else

/* CVMmemCopy64(CVMUint32* destination, CVMUint32* source) */
#define CVMmemCopy64(destination, source) \
    ((destination)[0] = (source)[0], (destination)[1] = (source)[1])

#endif

/*
 * Long Conversions:
 */

#define CVMint2Long(val)	((CVMJavaLong)(val))

#define CVMlong2Int(val)	((CVMJavaInt)(val))

#define CVMlong2Float(val)	((CVMJavaFloat)(val))

#define CVMlong2Double(val)	((CVMJavaDouble)(val))

#define CVMlong2VoidPtr(val)	((void *)(int)(val))

#define CVMvoidPtr2Long(val)	((CVMJavaLong)(int)(val))

#ifdef HAVE_DOUBLE_BITS_CONVERSION
# ifdef NORMAL_DOUBLE_BITS_CONVERSION
#  define CVMlongBits2Double(val)	(*((CVMJavaDouble *) &(val)))
#  define CVMdouble2LongBits(val)	(*((CVMJavaLong *) &(val)))
# endif
#else
# error Must port CVMlongBitsToDouble and CVMdoubleToLongBits to your platform
#endif

/*
 * Long Arithmetic:
 */

#define CVMlongAdd(op1, op2)	((op1) + (op2))
#define CVMlongSub(op1, op2)	((op1) - (op2))
#define CVMlongMul(op1, op2)	((op1) * (op2))
#define CVMlongDiv(op1, op2)	((op1) / (op2))
#define CVMlongRem(op1, op2)	((op1) % (op2))
#define CVMlongAnd(op1, op2)	((op1) & (op2))
#define CVMlongOr(op1, op2)	((op1) | (op2))
#define CVMlongXor(op1, op2)	((op1) ^ (op2))

/*
 * The shift operations are somewhat unlike the above. Special-case them
 */
#define CVMlongUshr(op1, n) \
    (((unsigned long long)(op1)) >> ((n) & 0x3F))

#define CVMlongShl(op1, n) \
    ((op1) << ((n) & 0x3F))

#define CVMlongShr(op1, n) \
    ((op1) >> ((n) & 0x3F))

/*
 * Long Unary operations
 */

#define CVMlongNeg(op)	(-(op))
#define CVMlongNot(op)	(~(op))

/*
 * Long Comparisons (returning an CVMBool: CVM_TRUE or CVM_FALSE)
 */

/* To 0: */

#define CVMlongLtz(op)	((op) < 0LL)
#define CVMlongGez(op)	((op) >= 0LL)
#define CVMlongEqz(op)	((op) == 0LL)

/*
 * Long OperationBetween operands:
 */

#define CVMlongEq(op1, op2)	((op1) == (op2))
#define CVMlongNe(op1, op2)	((op1) != (op2))
#define CVMlongGe(op1, op2)	((op1) >= (op2))
#define CVMlongLe(op1, op2)	((op1) <= (op2))
#define CVMlongLt(op1, op2)	((op1) < (op2))
#define CVMlongGt(op1, op2)	((op1) > (op2))

/*
 * Long Compare operation 
 */
#define CVMlongCompare(op1, op2) \
    (CVMlongLt((op1), (op2)) ? -1 : \
     CVMlongGt((op1), (op2)) ? 1 : \
     0)

/*
 * Long Constants (CVMJavaLong's each)
 */
#define CVMlongConstZero()    (0LL)
#define CVMlongConstOne()     (1LL)

/*
 * Double Conversions:
 */

#define CVMdouble2Long(val) \
    double2Long((val))

/*
 * JAVA_COMPLIANT_d2l, NAN_CHECK_d2l, or neither (but not both) may be
 * defined.
 *
 * If NAN_CHECK_d2l is defined, then BOUNDS_CHECK_d2l may be defined.
 */

#ifdef JAVA_COMPLIANT_d2l

#define double2Long(val)   (CVMJavaLong)((val))

#elif (defined(NAN_CHECK_d2l))

#define double2Long(val)   ((val) != (val) ? 0LL : double2Long0(val))

#else

extern CVMJavaLong double2Long(CVMJavaDouble d);

#endif

#ifdef BOUNDS_CHECK_d2l

#define double2Long0(val) \
    (  ((val) <= -9223372036854775808.0) ? longMin : \
                   ((val) >= 9223372036854775807.0) ? longMax : \
                   (CVMJavaLong)(val) )

#else

#define double2Long0(val) ((CVMJavaLong)(val))

#endif

#endif /* _GCC_32_BIT_DOUBLEWORD_H */
