/*
 * -----------------------------------------------------------------
 *  globals.h :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying TimeSys Community Source License Agreement from TimeSys
 * in file TCSL.
 *
 * This file is derived from software distributed under the following terms:
 *
 * Sun Community Source License
 * Sun Microsystems
 * For details, see http://www.sun.com/981208/scsl/principles.html
 *
 * Your use of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


/*
 * @(#)globals.h	1.92 00/11/13
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#ifndef _INCLUDED_GLOBALS_H
#define _INCLUDED_GLOBALS_H

#include "javavm/include/defs.h"
#include "javavm/include/objects.h"
#include "javavm/include/classes.h"
#include "javavm/include/interpreter.h"
#include "javavm/include/stacks.h"
#include "javavm/include/porting/globals.h"
#include "javavm/include/sync.h"
#include "javavm/include/cstates.h"
#include "javavm/include/jni_impl.h"
#include "javavm/include/packages.h"
#include "javavm/include/xrun.h"
#include "javavm/include/jvmdi_impl.h"

/*
 * This file is generated from the GC choice given at build time.
 */
#include "generated/javavm/include/gc_config.h"

typedef void (*pProc)(void);
typedef struct exit_proc {
    pProc proc;
    struct exit_proc *next;
} * exit_procPtr;

struct CVMOptions {
    void *vfprintfHook;
    void *exitHook;
    void *abortHook;
    void *safeExitHook;

    const char *heapSizeStr;
    const char *nativeStackSizeStr;
    const char *gcAttributesStr;

#ifdef CVM_DEBUG
    const char *traceFlagsStr;
#endif
#ifdef CVM_JVMDI
    CVMBool debugging;
#endif
#ifdef CVM_CLASSLOADING
    CVMUint16 classVerificationLevel;
#endif
#ifdef CVM_HAVE_PROCESS_MODEL
    CVMBool fullShutdownFlag;
#endif
};

struct CVMGlobalState {
    /*
     * Global variables go here.
     */

    /* shared data for consistent states */
    CVMCState cstate[CVM_NUM_CONSISTENT_STATES];

    CVMSysMutex globalRootsLock; /* protect globalRoots */
    CVMStack globalRoots;	 /* stack for allocating global roots,
				    used for JNI global refs and CVM
				    global roots. */

    CVMSysMutex weakGlobalRootsLock; /* protect weakGlobalRoots */
    CVMStack weakGlobalRoots;	 /* stack for allocating JNI weak
                                    global refs */

    /*
     * Class and ClassLoader global roots:
     * 
     * The CVMcbJavaInstance and CVMcbClassLoader fields can't be
     * regular global roots, or gc would have to make special checks
     * on each global root in order to make class unloading work. It
     * would need to check if the global root is a Class or ClassLoader
     * instance, and skip them on full gc's (which do class unloading).
     *
     * The classGlobalRoots are for CVMcbJavaInstance fields and the
     * classLoaderGlobalRoots are for CVMcbClassLoader fields. The latter
     * only has one allocated per ClassLoader, which will be shared among
     * all classes loaded by the ClassLoader. Access to both types of roots
     * is protected using the classTable lock.
     */
    CVMStack classGlobalRoots;	      /* for allocating Class global roots */
    CVMStack classLoaderGlobalRoots;  /* for allocating ClassLoader global */
				    
    /*
     * Table of all dynamically loaded classes. Protected by the 
     * classTable lock.
     */
    CVMStack    classTable;

    /*
     * Lists of classes to and ClassLoader globa roots to free. These lists
     * are setup by CVMclassDoClassUnloadingPass1() and processed by
     * CVMclassDoClassUnloadingPass2().
     */
    CVMClassBlock*       freeClassList;
    CVMClassLoaderICell* freeClassLoaderList;

    /*
     * ClassTable lock - It not only protects the classTable, but also
     * the classGlobalRoots and  classLoaderGlobalRoots.
     */
    CVMSysMutex classTableLock; 
#define CVM_CLASSTABLE_LOCK(ee)   \
    CVMsysMutexLock(ee, &CVMglobals.classTableLock)
#define CVM_CLASSTABLE_UNLOCK(ee) \
    CVMsysMutexUnlock(ee, &CVMglobals.classTableLock)
#define CVM_CLASSTABLE_ASSERT_LOCKED(ee) \
    CVMassert(CVMsysMutexIAmOwner(ee, &CVMglobals.classTableLock));

    /*
     * Cache of all <Class,ClassLoader> pairs that we have loaded. Protected
     * by the loaderCache lock.
     */
    CVMLoaderCacheEntry** loaderCache;

    /*
     * Database of loader constraints. See loaderconstraints.c for details.
     * Protected by the loaderCache lock.
     */
#ifdef CVM_CLASSLOADING
    CVMLoaderConstraint** loaderConstraints;
#endif

    /*
     * loaderCache lock - It  protects the loaderCache and loaderConstraints
     */
    CVMSysMutex loaderCacheLock; 
#define CVM_LOADERCACHE_LOCK(ee)   \
    CVMsysMutexLock(ee, &CVMglobals.loaderCacheLock)
#define CVM_LOADERCACHE_UNLOCK(ee) \
    CVMsysMutexUnlock(ee, &CVMglobals.loaderCacheLock)
#define CVM_LOADERCACHE_ASSERT_LOCKED(ee) \
    CVMassert(CVMsysMutexIAmOwner(ee, &CVMglobals.loaderCacheLock));
#define CVM_LOADERCACHE_ASSERT_UNLOCKED(ee) \
    CVMassert(!CVMsysMutexIAmOwner(ee, &CVMglobals.loaderCacheLock));

    CVMSysMutex heapLock;	 /* The memory-related lock */


#ifdef CVM_CLASSLOADING
    CVMSysMutex constantpoolLock;    /* The constantpool rewriting lock */
    CVMSysMutex codeLock;	     /* The code rewriting lock */
    CVMSysMutex nullClassLoaderLock; /* The NULL classloader lock */
#endif
    
#ifdef CVM_JVMDI
    CVMSysMutex debuggerLock;
#endif

    /*
     * Target global variables
     */
    CVMTargetGlobalState target;

    /*
     * GC attributes, to be parsed by the GC implementation
     */
    CVMUint32 numGCOptions;
    char** gcOptions;

    /*
     * GC specific global state
     */
    CVMGCGlobalState gc;

    /*
     * JNI JavaVM state
     */
    CVMJNIJavaVM javaVM;

    /*
     * The execution environment for the main thread.
     */
    CVMExecEnv mainEE;

    /*
     * Thread list
     */
    CVMSysMutex threadLock;	/* protect threadList */
    CVMExecEnv *threadList;	/* linked-list of threads */

    CVMUint32 userThreadCount;
    CVMUint32 threadCount;
    CVMCondVar threadCountCV;

    CVMUint32 threadIDCount;
#ifdef CVM_DEBUG

#endif

    CVMSysMutex syncLock;		/* Object monitor "inflation" */

    CVMObjMonitor *objLocksBound;	/* bound to object */
    CVMObjMonitor *objLocksUnbound;
    CVMObjMonitor *objLocksFree;	/* free locks */
    CVMOwnedMonitor *objLocksFreeOwned;

#if CVM_MICROLOCK_TYPE == CVM_MICROLOCK_DEFAULT
    CVMMutex objMicroLock[CVM_NUM_OBJ_MICROLOCKS];
    CVMMutex sysMicroLock[CVM_NUM_SYS_MICROLOCKS];
#endif

    /*
     * Well-known ROMizer generated structures
     */

    /*
     * Well-known types and methods
     */
    CVMMethodTypeID initTid;
    CVMMethodTypeID clinitTid;
    CVMMethodTypeID finalizeTid;
    CVMMethodTypeID cloneTid;

    /* The Finalizer.register() static method is called on all non-trivially
       finalizable objects. */
    CVMMethodBlock* java_lang_ref_Finalizer_register;
    
	/* This is for finalizable objects created in a memory area other than
	 * the heap.
	 * -Pratik
     * (deprecated. I have to get rid of this.
     *   - Pratik 19th March 2001)
	 */
    //CVMMethodBlock* javax_realtime_Finalizer_register;
    
    /* This is to allow budgeted allocation in RealtimeThreads
     * - Pratik.
     */
    CVMMethodBlock* javax_realtime_RealtimeThread_allowAllocation;
    
	/*
     * The lock for synchronizing with the reference handler thread 
     */
	CVMJavaVal32*   java_lang_ref_Reference_lock;

    /*
     * The list of java.lang.ref.Reference's waiting to be enqueued
     */
    CVMJavaVal32*   java_lang_ref_Reference_pending;

    /*
     * Does the reference handler thread need to do any work after GC?
     */
    CVMBool         referenceWorkTODO;

    CVMMethodBlock* java_lang_Class_runStaticInitializers;
    CVMMethodBlock* java_lang_Class_newInstance;
#ifdef CVM_REFLECT
    CVMMethodBlock* java_lang_reflect_Constructor_newInstance;
    CVMMethodBlock* java_lang_reflect_Method_invoke;
#endif /* CVM_REFLECT */
#ifdef CVM_DEBUG_STACKTRACES
    CVMMethodTypeID printlnTid;
    CVMMethodBlock* java_lang_Throwable_fillInStackTrace;
#ifdef CVM_DEBUG
    CVMFieldBlock*  java_lang_System_out;
#endif
#endif /* CVM_DEBUG_STACKTRACES */
#ifdef CVM_CLASSLOADING
    CVMMethodBlock* java_lang_ClassLoader_NativeLibrary_getFromClass;
    CVMMethodBlock* java_lang_ClassLoader_addClass;
    CVMMethodBlock* java_lang_ClassLoader_loadClass;
    CVMMethodBlock* java_lang_ClassLoader_findNative;
    CVMMethodBlock* java_lang_ClassLoader_checkPackageAccess;
#endif /* CVM_CLASSLOADING */

    /*
     * Shutdown-related stuff
     */
    CVMMethodBlock* java_lang_Shutdown_shutdown;
    CVMMethodBlock* sun_misc_SystemThread_waitAllSystemThreadsExit;

    /*
     * Other stuff needed for class loading.
     */
#ifdef CVM_CLASSLOADING
    CVMClassLoaderICell* systemClassLoader;
    char*                bootClassPathString;
    CVMClassPathEntry*   bootClassPathEntries;
    CVMUint16            numBootClassPathEntries;
    CVMUint16            classVerificationLevel;
    void*                cvmDynHandle;
#endif /* CVM_CLASSLOADING */

    /*
     * Simple hash table used to keep track of packages loaded by the system
     * and the directory or JAR files that they were loaded from. Used in
     * java.lang.Package to maintain version information for loaded system
     * packages.
     */
    CVMPackage* packages[CVM_PKGINFO_HASHSIZE];
    CVMUint16   numPackages;

#ifdef CVM_DEBUG
    CVMUint32 debugFlags;
#endif /* CVM_DEBUG */

#ifdef CVM_JVMDI
    CVMBool jvmdiDebuggingEnabled;
    CVMInt32 jvmdiSingleStepping; /* set ONLY by jvmdi.c. */
    /* are one or more fields being watched?
     * these flags are accessed by the interpreter to determine if
     * jvmdi should be notified.
     */
    CVMBool jvmdiWatchingFieldAccess; /* set ONLY by jvmdi.c */
    CVMBool jvmdiWatchingFieldModification; /* set ONLY by jvmdi.c */

    CVMObjectICell* systemThreadGroupICell; 
#endif
    CVMObjectICell* mainThreadGroupICell; 

    /*
     * Type system globals.
     * There is only one lock (for now) governing
     * all access to it.
     */
    CVMSysMutex typeidLock; /* protect table insertion */
    CVMUint32	typeIDscalarSegmentSize;
    CVMUint32	typeIDmethodTypeSegmentSize;
    CVMUint32	typeIDmemberNameSegmentSize;

    /*
     * The lock on the intern table (which is now a C data structure)
     * To protect the table during search/mutation, including GC
     * sweeping!
     */
    CVMSysMutex	internLock;
    CVMUint32	stringInternSegmentSizeIdx;

    /*
     * The one word memory for the random number generator
     */
    CVMInt32  lastRandom;

    /*
     * Some desperation exception objects we preallocate.
     */
    CVMThrowableICell* preallocatedOutOfMemoryError;
    CVMThrowableICell* preallocatedStackOverflowError;

    /*
     * Default sizes
     */

    CVMUint32 nativeStackSize;
    CVMUint32 javaStackSize;	/* %comment dlong001 */

    /*
     * Hooks
     */
    void *vfprintfHook;
    void (*exitHook)(int);
    void (*abortHook)();

    void (*safeExitHook)(int);

    /*
     * Shutdown flags
     */
    CVMBool fullShutdown;

    /*
     * A list to keep track of shared native library loaded by the
     * -Xrun command argument. It contains the shared library object
     * handle and the function pointer of calling JVM_onUnLoad at VM
     * exit.
     */
    CVMXrunTable onUnloadTable;

    /*
     * exit_procs contains exitHandle() function pointer to exit 
     *            JVMDI event at VM exit.
     */
    exit_procPtr exit_procs;

#ifdef CVM_JVMDI
    /* JVMDI-specific globally defined static variables */
    JVMDI_Static jvmdiStatics;
#endif
};

#define CVM_CSTATE(x)  (&CVMglobals.cstate[(x)])

/*
 * Defined in share/javavm/runtime/globals.c
 */
extern CVMGlobalState CVMglobals;
extern CVMBool CVMinitVMGlobalState(CVMGlobalState *, CVMOptions *options);
extern void CVMdestroyVMGlobalState(CVMExecEnv *ee, CVMGlobalState *);

#endif /* _INCLUDED_GLOBALS_H */
