/*
 * @(#)preloader_impl.h	1.23 00/10/25
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
 * This file defines the contract between the ROMizer and
 * the code in preloader.c, which is its interface to the
 * rest of the VM. This file is essentially a private
 * communications between romclasses.c and preloader.c
 * Any other use indicates a problem!
 */

#ifndef _INCLUDED_PRELOADER_IMPL_H
#define _INCLUDED_PRELOADER_IMPL_H

#include "javavm/include/objects.h"

/* These must exactly match settings in objects.h */

/* In the following, we must use + instead of | or compiler thinks expression is
   not constant.  Will this work with all compilers? */
#define CVM_OBJHDR_CLASS_INIT(cb) \
    ((CVMClassBlock*)(((int)(cb)) + CVM_OBJECT_IN_ROM_MASK + CVM_DEFAULT_MARKED_TYPE))
#define CVM_OBJHDR_VARIOUS_INIT(hashCode) \
    (((hashCode) << 2) | CVM_LOCKSTATE_UNLOCKED)

#define CVM_ROM_OBJECT_HDR_INIT0(cbName,hashCode)		\
    {CVM_OBJHDR_CLASS_INIT(&(cbName)),				\
     CVM_OBJHDR_VARIOUS_INIT((hashCode))}

#define CVM_ROM_OBJECT_HDR_INIT(cb,hashCode)			\
    CVM_ROM_OBJECT_HDR_INIT0(cb##_Classblock, hashCode)

/*
 * Struct declaration for the java.lang.Class proxy objects
 * that the ROMizer instantiates, along with the initialization macro
 * NOTE: The field, classBlockPointer, appears in the Java class
 * as an "int", even though it is actually a C pointer. IT IS NOT
 * A JAVA REF!
 */
struct java_lang_Class {
    CVMObjectHeader    hdr;
    CVMClassBlock*     classBlockPointer;
    CVMObject*         classLoader;
    CVMArrayOfRef*     signers;
};

#define CVM_CLASS_INIT(cbname,hashCode) \
    {CVM_ROM_OBJECT_HDR_INIT(java_lang_Class,hashCode),	\
     (CVMClassBlock*)(cbname), \
     NULL,  \
     NULL   \
    }

/*
 * Used by the preloader initialization code
 * to copy initialized data into writable space
 */
struct CVM_preloaderInitTriple {
    const void *	from;
    void *		to;
    int			bytecount;
};


/*
 * Finally, the data structures exported from romjava and used
 * by the preloader interface code.
 * The CVM_ROMClasses is now sorted by typeid.
 * The first part, non-array classes, can be used for direct
 * access with just a little computing. The rest should be
 * binsearched. Since there are far more 1-dimensional than
 * 2-dimensional arrays, they can be handled separately.
 * The constants CVM_{first,last}ROMVectorClass mark the
 * divisions.
 * Primitive classes, defined by the VM, are sorted before
 * any other classes. They precede CVM_firstROMNonPrimitiveClass;
 */

extern const int CVM_nROMClasses;
extern struct java_lang_Class CVM_ROMClasses[];
extern const int CVM_firstROMNonPrimitiveClass;
extern const int CVM_firstROMVectorClass;
extern const int CVM_lastROMVectorClass;

extern const int CVM_nROMStrings;
extern struct java_lang_String CVM_ROMStrings[];

extern const struct CVM_preloaderInitTriple CVM_preloaderInitMap[];
extern const int    CVM_nPreloaderMapEntries;

extern const int CVM_nROMClassesRuntimeFlags;
extern CVMUint8 CVMROMClassesRuntimeFlags[];


#endif /* _INCLUDED_PRELOADER_IMPL_H */
