/*
 * @(#)sync.h	1.35 00/09/29
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#ifndef _INCLUDED_SYNC_H
#define _INCLUDED_SYNC_H

#include "javavm/include/defs.h"
#include "javavm/include/porting/defs.h"
#include "javavm/include/porting/sync.h"

/*
 * Global micro locks
 */

typedef enum {
    CVM_CODE_MICROLOCK,
    CVM_NUM_SYS_MICROLOCKS
} CVMSysMicroLock;

void
CVMsysMicroLock(CVMExecEnv *ee, CVMSysMicroLock lock);
void
CVMsysMicroUnlock(CVMExecEnv *ee, CVMSysMicroLock lock);

#ifndef CVM_DEBUG
#define CVMsysMicroLock(ee, l)         CVMsysMicroLockImpl((l))
#define CVMsysMicroUnlock(ee, l)       CVMsysMicroUnlockImpl((l))
#endif

#if CVM_MICROLOCK_TYPE == CVM_MICROLOCK_SCHEDLOCK
#define CVMsysMicroLockImpl(l)		CVMschedLock()
#define CVMsysMicroUnlockImpl(l)	CVMschedUnlock()
#else
#define CVMsysMicroLockImpl(l)		\
    CVMmutexLock(&CVMglobals.sysMicroLock[(l)])
#define CVMsysMicroUnlockImpl(l)	\
    CVMmutexUnlock(&CVMglobals.sysMicroLock[(l)])
#endif

/*
 * Reentrant mutexes
 */

struct CVMReentrantMutex {
    CVMExecEnv *owner;
    CVMUint32 count;
    CVMMutex mutex;
};

extern void    CVMreentrantMutexInit   (CVMReentrantMutex *rm,
					CVMExecEnv *owner, CVMUint32 count);
extern void    CVMreentrantMutexDestroy(CVMReentrantMutex *rm);
extern CVMBool CVMreentrantMutexTryLock(CVMExecEnv *ee, CVMReentrantMutex *rm);
extern void    CVMreentrantMutexLock   (CVMExecEnv *ee, CVMReentrantMutex *rm);
extern void    CVMreentrantMutexUnlock (CVMExecEnv *ee, CVMReentrantMutex *rm);
/*
 * CVM_FALSE is returned to indicate that the wait was interrupted,
 * otherwise CVM_TRUE is returned.
 */
extern CVMBool CVMreentrantMutexWait   (CVMExecEnv *ee, CVMReentrantMutex *rm,
					CVMCondVar *c, CVMInt64 millis);

/* Returns TRUE if the execution environment EE currently owns the
   Reentrant Mutex RM */
extern CVMBool CVMreentrantMutexIAmOwner(CVMExecEnv *ee, CVMReentrantMutex *rm);
/* This macro returns an (CVMMutex *) */
#define CVMreentrantMutexGetMutex(/* CVMReentrantMutex * */ m) (&((m)->mutex))
#define CVMreentrantMutexOwner(/* CVMReentrantMutex * */ m) ((m)->owner)
#define CVMreentrantMutexCount(/* CVMReentrantMutex * */ m) ((m)->count)
#define CVMreentrantMutexSetOwner(ee, m, targetEE)	\
	CVMmutexSetOwner(CVMexecEnv2threadID((ee)),	\
	    &(m)->mutex, CVMexecEnv2threadID((targetEE)))


extern void * __mutex_tracking;

#define CVMreentrantMutexLockUnsafe(ee, rm)	\
{						\
    CVMassert(CVMD_isgcUnsafe(ee));		\
    /*printf("lockunsafe:owner (%p), ee(%p),  count(%d) rm(%p)\n",(rm)->owner, ee, (rm)->count, rm); */ \
    if ((rm)->owner == ee) {			\
        ++(rm)->count;				\
    } else {					\
	CVMD_gcSafeExec((ee), {			\
	    CVMmutexLock(&(rm)->mutex);		\
	});					\
        (rm)->owner = ee;			\
        (rm)->count = 1;			\
    }						\
}


/*
 * Deadlock-detecting VM mutexes
 */

struct CVMSysMutex {
    const char *name;
    CVMUint8 rank;
    CVMReentrantMutex rmutex;
#ifdef CVM_DEBUG
    CVMSysMutex *nextOwned;
#endif
};

extern void       CVMsysMutexInit   (CVMSysMutex* m,
				     const char *name,	/* client managed */
				     CVMUint8 rank);	/* deadlock detection */
extern void       CVMsysMutexDestroy(CVMSysMutex* m);

extern CVMBool    CVMsysMutexTryLock(CVMExecEnv *ee, CVMSysMutex* m);
extern void       CVMsysMutexLock   (CVMExecEnv *ee, CVMSysMutex* m);
extern void       CVMsysMutexUnlock (CVMExecEnv *ee, CVMSysMutex* m);
/*
 * CVM_FALSE is returned to indicate that the wait was interrupted,
 * otherwise CVM_TRUE is returned.
 */
extern CVMBool	  CVMsysMutexWait   (CVMExecEnv *ee, CVMSysMutex* m,
				     CVMCondVar *c, CVMInt64 millis);

/* This macro returns an (CVMReentrantMutex *) */
#define CVMsysMutexGetReentrantMutex(/* CVMSysMutex * */ m) \
	(&((m)->rmutex))
#define CVMsysMutexOwner(m) \
	CVMreentrantMutexOwner(&(m)->rmutex)
#define CVMsysMutexCount(m) \
	CVMreentrantMutexCount(&(m)->rmutex)
#define CVMsysMutexIAmOwner(ee, m) \
	(CVMsysMutexOwner((m)) == (ee))

/*
 * "raw" object-less monitor
 */

typedef enum {
    CVM_WAIT_OK,
    CVM_WAIT_INTERRUPTED,
    CVM_WAIT_NOT_OWNER
} CVMWaitStatus;

struct CVMSysMonitor {
    CVMReentrantMutex rmutex;
    CVMCondVar condvar;
    int monitorControl;
    int ceiling;
};

extern void
CVMsysMonitorInit(CVMSysMonitor* m, CVMExecEnv *owner, CVMUint32 count);

extern void
CVMsysMonitorDestroy(CVMSysMonitor* m);

extern void
CVMsysMonitorEnter(CVMExecEnv *ee, CVMSysMonitor* m);

extern void
CVMsysMonitorExit(CVMExecEnv *ee, CVMSysMonitor* m);

extern CVMBool
CVMsysMonitorNotify(CVMExecEnv *ee, CVMSysMonitor* m);

extern CVMBool
CVMsysMonitorNotifyAll(CVMExecEnv *ee, CVMSysMonitor* m);

extern CVMWaitStatus
CVMsysMonitorWait(CVMExecEnv *ee, CVMSysMonitor* m, CVMInt64 millis);

#define CVMsysMonitorOwner(m) CVMreentrantMutexOwner(&(m)->rmutex)
#define CVMsysMonitorCount(m) CVMreentrantMutexCount(&(m)->rmutex)

#define CVMsysMonitorSetMutexOwner(ee, m, o) \
	CVMreentrantMutexSetOwner((ee), &(m)->rmutex, (o))

#define CVMsysMonitorTryEnter(ee, m)			\
	CVMreentrantMutexTryLock((ee), &(m)->rmutex)

#define CVMsysMonitorEnterUnsafe(ee, m)			\
{							\
    CVMreentrantMutexLockUnsafe((ee), &(m)->rmutex);	\
}

#define CVMsysMonitorEnterSafe(ee, m)			\
{							\
    CVMassert(CVMD_isgcSafe((ee)));			\
    CVMreentrantMutexLock((ee), &(m)->rmutex);		\
}

#define CVMsysMonitorEnter(ee, m)	CVMsysMonitorEnterSafe((ee), (m))

#define CVMsysMonitorExit(ee, m)			\
{							\
    CVMreentrantMutexUnlock((ee), &(m)->rmutex);	\
}


/*
 * GC support
 */

extern void
CVMdeleteUnreferencedMonitors(CVMExecEnv *ee,
			      CVMRefLivenessQueryFunc isLive,
			      void* isLiveData,
			      CVMRefCallbackFunc transitiveScanner,
			      void* transitiveScannerData);


extern void
CVMscanMonitors(CVMExecEnv *ee, CVMRefCallbackFunc refCallback, void* data);

extern void
CVMeeSyncInit(CVMExecEnv *ee);
extern void
CVMeeSyncDestroy(CVMExecEnv *ee);
extern void
CVMeeSyncDestroyGlobal(CVMExecEnv *ee);

/*
 * Support for debug stubs.
 */

#if defined(CVM_DEBUG) && !defined(CVM_PORTING_DEBUG_IMPL)

#undef CVMmutexInit
#define CVMmutexInit(m) \
        CVMmutexInitStub((m))
CVMBool CVMmutexInitStub(CVMMutex* m);

#undef CVMmutexDestroy
#define CVMmutexDestroy(m)      \
        CVMmutexDestroyStub((m))
CVMBool CVMmutexDestroyStub(CVMMutex* m);

#undef CVMcondvarInit
#define CVMcondvarInit(c,m)     \
        CVMcondvarInitStub((c),(m))
CVMBool CVMcondvarInitStub(CVMCondVar* c, CVMMutex* m);

#undef CVMcondvarDestroy
#define CVMcondvarDestroy(c)    \
        CVMcondvarDestroyStub((c))
CVMBool CVMcondvarDestroyStub(CVMCondVar* c);

#endif

#endif /* _INCLUDED_SYNC_H */
