/*
 * @(#)utils.h	1.60 00/10/14
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */


/*
 * Header file for various utility functions and macros.
 */

#ifndef CVM_UTILS_H
#define CVM_UTILS_H

#include "javavm/include/defs.h"
#include "javavm/include/clib.h"
#include "javavm/include/porting/ansi/stdarg.h"
#include "generated/cni/sun_misc_CVM.h"
#include "javavm/export/jni.h"

/*
 * Console I/O. Same as printf, but supports special formats like
 * %C, %P, and %F. See CVMformatStringVaList for details.
 */
void CVMconsolePrintf(const char* format, ...);


/*
 * Functions for accessing the debugging flags. All of the following functions
 * return the previous state of the flag.
 *
 * You can pass in more than one flag at a time to any of the functions.
 */

#ifdef CVM_DEBUG

#define CVM_DEBUGFLAG(flag) ((CVMUint32)sun_misc_CVM_DEBUGFLAG_##flag)

extern CVMUint32
CVMcheckDebugFlags(CVMUint32 flags);

extern CVMUint32
CVMsetDebugFlags(CVMUint32 flags);

extern CVMUint32
CVMclearDebugFlags(CVMUint32 flags);

extern CVMUint32
CVMrestoreDebugFlags(CVMUint32 flags, CVMUint32 oldvalue);

#endif /* CVM_DEBUG */

/*
 * Debug I/O
 */
#ifdef CVM_DEBUG
#define CVMdebugPrintf(x) CVMconsolePrintf x
#define CVMtrace(flags, x) \
    (CVMcheckDebugFlags(flags) != 0 ? CVMconsolePrintf x : (void)0)
#else
#define CVMdebugPrintf(x)
#define CVMtrace(flags, x)
#endif /* CVM_DEBUG */

#define CVMtraceOpcode(x) 	CVMtrace(CVM_DEBUGFLAG(TRACE_OPCODE), x)
#define CVMtraceStatus(x) 	CVMtrace(CVM_DEBUGFLAG(TRACE_STATUS), x)
#define CVMtraceFastLock(x) 	CVMtrace(CVM_DEBUGFLAG(TRACE_FASTLOCK), x)
#define CVMtraceDetLock(x)   	CVMtrace(CVM_DEBUGFLAG(TRACE_DETLOCK), x)
#define CVMtraceSysMutex(x)    	CVMtrace(CVM_DEBUGFLAG(TRACE_MUTEX), x)
#define CVMtraceCS(x)        	CVMtrace(CVM_DEBUGFLAG(TRACE_CS), x)
#define CVMtraceGcStartStop(x)  CVMtrace(CVM_DEBUGFLAG(TRACE_GCSTARTSTOP) | \
					 CVM_DEBUGFLAG(TRACE_GCSCAN), x)
#define CVMtraceGcScan(x)    	CVMtrace(CVM_DEBUGFLAG(TRACE_GCSCAN), x)
#define CVMtraceGcAlloc(x)     	CVMtrace(CVM_DEBUGFLAG(TRACE_GCALLOC), x)
#define CVMtraceGcCollect(x)   	CVMtrace(CVM_DEBUGFLAG(TRACE_GCCOLLECT), x)
#define CVMtraceGcSafety(x)   	CVMtrace(CVM_DEBUGFLAG(TRACE_GCSAFETY), x)
#define CVMtraceClinit(x)   	CVMtrace(CVM_DEBUGFLAG(TRACE_CLINIT), x)
#define CVMtraceExceptions(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_EXCEPTIONS), x)
#define CVMtraceMisc(x)		CVMtrace(CVM_DEBUGFLAG(TRACE_MISC), x)
#define CVMtraceBarriers(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_BARRIERS), x)
#define CVMtraceStackmaps(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_STACKMAPS), x)
#define CVMtraceClassLoading(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_CLASSLOADING), x)
#define CVMtraceClassLookup(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_CLASSLOOKUP), x)
#define CVMtraceTypeID(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_TYPEID), x)
#define CVMtraceVerifier(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_VERIFIER), x)
#define CVMtraceWeakrefs(x)	CVMtrace(CVM_DEBUGFLAG(TRACE_WEAKREFS), x)
#define CVMtraceClassUnloading(x) CVMtrace(CVM_DEBUGFLAG(TRACE_CLASSUNLOAD), x)
#define CVMtraceClassLink(x)    CVMtrace(CVM_DEBUGFLAG(TRACE_CLASSLINK), x)

/*
 * Long utility routines:
 */

extern void CVMlong2String(CVMInt64, char*, char*);

/*
 * Java class utility routines:
 */

#ifdef CVM_DEBUG
extern void
CVMdumpObject(CVMObject* directObj);
#endif

#ifdef CVM_DEBUG
void
CVMdumpThread(JNIEnv* env);

/* Prints the thread name with NO additional characters to stdout.
   Prints "<unnamed thread>" if thread has NULL name. */
void
CVMprintThreadName(JNIEnv* env, CVMObjectICell* threadICell);
#endif

extern char*
CVMclassname2String(CVMClassTypeID classTypeID, char *dst, int size);

/*
 * UTF8/Unicode
 */
/*
 * Number of Unicode characters represented by the null-terminated,
 * UTF8-encoded C string
 */
extern CVMSize 
CVMutfLength(const char *);

/*
 * counted copy from UTF8 string to unicode array.
 * count is number of Unicode characters to copy.
 * returned value is number of UTF8 characters consumed.
 */
extern int
CVMutfCountedCopyIntoCharArray(
    const char * utf8Bytes, CVMJavaChar* unicodeChars, CVMSize  unicodeLength );

extern void 
CVMutfCopyIntoCharArray(const char* utf8Bytes, CVMJavaChar* unicodeChars);

extern char* 
CVMutfCopyFromCharArray(const CVMJavaChar* unicodeChars,
			char* utf8Bytes, CVMInt32 count);
extern CVMJavaChar
CVMutfNextUnicodeChar(const char** utf8String_p);

#define CVMunicodeChar2UtfLength(ch)				\
        (((ch != 0) && (ch <= 0x7f)) ? 1 /* 1 byte */		\
        : (ch <= 0x7FF) ? 2		 /* 2 byte character */	\
        : 3)				 /* 3 byte character */


/*
 * Format a string.  Implements VM-specific formats plus a subset of
 * snprintf formats.  See utils.c for details.
 */

size_t
CVMformatString(char *buf, size_t bufSize, const char *format, ...);

size_t
CVMformatStringVaList(char *buf, size_t bufSize,
    const char *format, va_list ap);

/*
 * Java String utils
 */
extern void
CVMnewStringUTF(CVMExecEnv* ee, CVMStringICell* resultICell,
		const char* utf8Bytes);
extern void
CVMnewString(CVMExecEnv* ee, CVMStringICell* resultICell,
	     const CVMJavaChar* unicodeChars, CVMUint32 len);

/*
 * Alignment
 */
#define CVMalignDoubleWordUp(val)    (((CVMUint32)(val) + 7) & ~7)
#define CVMalignWordUp(val)          (((CVMUint32)(val) + 3) & ~3)
#define CVMalignDoubleWordDown(val)  ((CVMUint32)(val) & ~7)
#define CVMalignWordDown(val)        ((CVMUint32)(val) & ~3)

/*
 * Returns true if the two classes are in the same package. 
 */
CVMBool CVMisSameClassPackage(CVMExecEnv* ee,
			      CVMClassBlock* class1, CVMClassBlock* class2); 

/* This requires that CLAZZ be a non-NULL CVMClassICell pointer and
   point to a valid Java class object. */
extern CVMClassBlock*
CVMgcSafeClassRef2ClassBlock(CVMExecEnv* ee, CVMClassICell *clazz);

/* This requires that CLAZZ be a non-NULL CVMClassICell pointer and
   point to a valid Java class object. */
extern CVMClassBlock *
CVMgcUnsafeClassRef2ClassBlock(CVMExecEnv *ee, CVMClassICell *clazz);

/*
 * Random number generator
 */

/*
 * Initialize the seed
 */
extern void
CVMrandomInit();

/*
 * Get the next random number
 */
extern CVMInt32
CVMrandomNext();


/* Utility function to wrap jvalues (jni.h) in Java objects like
   Integer and Double. This version should only be called from GC-safe
   code; there is no such thing as a GC-unsafe wrapping function,
   since it allocates memory internally and must therefore become
   GC-safe. V (a union) has the appropriate version of its fields set.
   FROMTYPE, an CVMBasicType (basictypes.h) indicates the type of V.
   RESULT must be a valid ICell. Upon success CVM_TRUE is returned,
   and RESULT points either to a newly-created wrapper object for V's
   value or to NULL in the case of a void return value.  Upon failure
   RESULT is set to NULL, CVM_FALSE is returned, and an
   OutOfMemoryError is thrown in the VM. */
CVMBool
CVMgcSafeJavaWrap(CVMExecEnv* ee, jvalue v,
		  CVMBasicType fromType,
		  CVMObjectICell* result);

/* Utility function to unwrap Java objects like Integer and Double
   into jvalues (jni.h). This version should only be called from
   GC-safe code. OBJ must be an appropriately-typed Java object.  The
   appropriate field of V is set with the bits of OBJ's contents, and
   TOTYPE is set to the basic type corresponding to the type of the
   object. Returns CVM_TRUE upon success. Returns CVM_FALSE upon
   failure and throws an exception in the VM. The exception type is
   either the exceptionCb or, if this is NULL,
   IllegalArgumentException (if the conversion failed) or
   NullPointerException (if OBJ was NULL). Note that both V and TOTYPE
   may be mutated in the case of failure. */
CVMBool
CVMgcSafeJavaUnwrap(CVMExecEnv* ee, CVMObjectICell* obj,
		    jvalue* v, CVMBasicType* toType,
		    CVMClassBlock* exceptionCb);

/* Utility function to unwrap Java objects like Integer and Double
   into jvalues (jni.h). This version should only be called from
   GC-unsafe code and is guaranteed not to become GC-safe internally.
   OBJ must be an appropriately-typed Java object. The appropriate
   field of V is set with the bits of OBJ's contents, and TOTYPE is
   set to the basic type corresponding to the type of the
   object. Returns CVM_TRUE upon success. Returns CVM_FALSE upon
   failure and throws an exception in the VM. The exception type is
   either the exceptionCb or, if this is NULL,
   IllegalArgumentException (if the conversion failed) or
   NullPointerException (if OBJ was NULL). Note that both V and TOTYPE
   may be mutated in the case of failure. */
CVMBool
CVMgcUnsafeJavaUnwrap(CVMExecEnv* ee, CVMObject* obj,
		      jvalue* v, CVMBasicType* toType,
		      CVMClassBlock* exceptionCb);

/*
 * Property support
 */

/*
 * Assign value 'val' to property 'key' in properties object 'props'.
 * Use method 'putID' to put the property in.
 *
 * return CVM_TRUE on success, CVM_FALSE on any failure.
 */
CVMBool
CVMputProp(JNIEnv* env, jmethodID putID, 
	   jobject props, const char* key, const char* val);

/*
 * Assign value 'val' to property 'key' in properties object 'props'.
 * Use method 'putID' to put the property in.
 *
 * Do the conversion of 'key' and 'value' using the platform specific
 * C string conversion.
 *
 * return CVM_TRUE on success, CVM_FALSE on any failure.
 */
CVMBool
CVMputPropForPlatformCString(JNIEnv* env, jmethodID putID, 
			     jobject props, const char* key, const char* val);

/*
 * Parse a size from options such as -Xms and -Xss.
 * String format is [0-9]+([kKmM])
 *
 * Return the specified size, or -1 if parsing fails.
 */
CVMInt32
CVMatoml(const char *s);

#endif /* CVM_UTILS_H */
