/*
 * @(#)DependenceAnalyzer.java	1.9 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package dependenceAnalyzer;
import util.ClassFileFinder;
import util.Set;
import java.util.Enumeration;
import java.util.Dictionary;
import java.util.Hashtable;

/*
 * A DependenceAnalyzer is the engine that actually does the work
 * and is the root of a dependence graph. You will subclass this,
 * depending on the level of analysis you want, but I believe
 * that the functions provided here are common.
 */
public abstract class DependenceAnalyzer {

    /*
     * This analyzer maintains a name-to-node dictionary of
     * its graph. This lets you lookup nodes by name. It will
     * return null if the named entity is not in the graph.
     */
    public abstract DependenceNode nodeByName( Object name );

    /*
     * Way(s) to access the whole graph.
     */
    public abstract Enumeration allNodes();
    // too hard for now.
    //public Enumeration unanalyzedNodes();
    //public Enumeration errorNodes();

    /*
     * To conditionally add a node to the graph. It is not
     * an error to add something that's already in the graph.
     * The return value is a DependenceNode. It may be
     * in any state, but if it was not in the graph previous to
     * this call, it will certainly be DependenceNode.UNANALYZED.
     * In other words, this call doesn't cause analysis.
     */
    public abstract DependenceNode addNodeByName( Object name );

    /*
     * To cause the state of a node to move from UNANALYZED,
     * one hopes to ANALYZED, but perhaps to ERROR.
     * Return value is the new state of the node.
     */
    public abstract int analyzeDependences( DependenceNode node );

    /*
     * To complete analysis of the whole graph. Return value
     * is true if all nodes in the graph end in state
     * ANALYZED, else false.
     * Any nodes not ANALYZED will be in state ERROR.
     */
    public boolean analyzeAllDependences(){
	Enumeration nodeList;
	boolean changed;
	boolean succeed = true;
	do {
	    changed = false;
	    nodeList = allNodes();
	    while ( nodeList.hasMoreElements() ){
		DependenceNode dep = (DependenceNode)(nodeList.nextElement());
		if ( dep.state() == DependenceNode.UNANALYZED ){
		    analyzeDependences( dep );
		    changed = true;
		}
		if ( dep.state() == DependenceNode.ERROR )
		    succeed = false;
	    }
	} while ( changed );
	return succeed;
    }
}
