/*
 * @(#)ArrayClassInfo.java	1.19 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package vm;
import Const;
import components.*;
import util.*;

/*
 * An array is a class.
 * It is a subclass of java.lang.Object.
 * It has all the class-related runtime data structures, or at least
 * may of them.
 * It is not read in from class files, but is made up on-the-fly
 * when the classresolver sees a reference to a classname starting with
 * "[".
 *
 * In order to resolve such references early, we must do likewise here.
 */
public
class ArrayClassInfo extends ClassInfo {

    private static int nFake = 0;
    public int    	 arrayClassNumber;
    public int    	 depth;
    public int    	 baseType;
    public String        baseName;
    public ClassConstant baseClass;
    public int           elemClassAccess;
    public ClassConstant subarrayClass;
    

    /*
     * Given signature s,
     * fill in
     * depth ( i.e. number of opening [ )
     * basetype ( i.e. thing after the last [ )
     * and baseClass ( if basetype is a classtype )
     */
    private void fillInTypeInfo( String s ) throws DataFormatException {
	int index = 0;
	char c;
	while( ( c = s.charAt( index ) ) == Const.SIGC_ARRAY )
	    index++;
	depth = index;
	switch( c ){
	case Const.SIGC_INT:
	    baseType = Const.T_INT;
	    baseName = "Int";
	    baseClass = new ClassConstant(new UnicodeConstant("int"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_LONG:
	    baseType = Const.T_LONG;
	    baseName = "Long";
	    baseClass = new ClassConstant(new UnicodeConstant("long"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_FLOAT:
	    baseType = Const.T_FLOAT;
	    baseName = "Float";
	    baseClass = new ClassConstant(new UnicodeConstant("float"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_DOUBLE:
	    baseType = Const.T_DOUBLE;
	    baseName = "Double";
	    baseClass = new ClassConstant(new UnicodeConstant("double"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_BOOLEAN:
	    baseType = Const.T_BOOLEAN;
	    baseName = "Boolean";
	    baseClass = new ClassConstant(new UnicodeConstant("boolean"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_BYTE:
	    baseType = Const.T_BYTE;
	    baseName = "Byte";
	    baseClass = new ClassConstant(new UnicodeConstant("byte"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_CHAR:
	    baseType = Const.T_CHAR;
	    baseName = "Char";
	    baseClass = new ClassConstant(new UnicodeConstant("char"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_SHORT:
	    baseType = Const.T_SHORT;
	    baseName = "Short";
	    baseClass = new ClassConstant(new UnicodeConstant("short"));
	    elemClassAccess = Const.ACC_PUBLIC;
	    break;
	case Const.SIGC_CLASS:
	    baseType = Const.T_CLASS;	    
	    int start = ++index;
	    while (s.charAt(index) != Const.SIGC_ENDCLASS) {
		index++;
	    }

	    baseClass = new ClassConstant(
		new UnicodeConstant( s.substring( start, index ) ) 
	    );
	    ClassInfo baseClassInfo = baseClass.find();
	    if (baseClassInfo != null) {
		// The base class' access should propagate to all arrays with
		// that base class, so elemClassAccess really is equivalent
		// to the the access flags of the base class
		elemClassAccess = baseClassInfo.access;
	    } else {
		elemClassAccess = Const.ACC_PUBLIC;
	    }
	    break;

	default:
	    throw new DataFormatException(Localizer.getString("arrayclassinfo.malformed_array_type_string.dataformatexception",s));
	}
	// For CVM, we want to know the sub-class, which is different from
	// the base class.
	if ( depth > 1 ){
	    subarrayClass = new ClassConstant( new UnicodeConstant( s.substring(1) ) );
	} else if ( depth == 1 ) {
	    subarrayClass = baseClass;
	}
    }


    public ArrayClassInfo( boolean v, String s ) throws DataFormatException {
	super(v);
	constants = new ConstantObject[0];
	arrayClassNumber = nFake++;
	fillInTypeInfo( s );
	className = s;
	thisClass = new ClassConstant( new UnicodeConstant( s ) );
	superClassInfo = lookupClass(/*NOI18N*/"java/lang/Object");
	superClass = superClassInfo.thisClass;
	access = Const.ACC_FINAL|Const.ACC_ABSTRACT;
	access |= (elemClassAccess & Const.ACC_PUBLIC);
	methods = new MethodInfo[0];
	fields  = new FieldInfo[0];
	enterClass();
    }

    protected String createGenericNativeName() { 
        return /*NOI18N*/"fakeArray" + arrayClassNumber;
    }
}
