/*
 * @(#)ClassClass.java	1.43 00/11/09
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package vm;
/*
 * VM-specific internal representation of
 * a class. Target-machine independent.
 * There is a references from each instance of components.ClassInfo to 
 * one of these, and a reference back as well.
 *
 * See also JDKVM for VM-specific info not associated directly with a class.
 */
import components.*;
import util.*;
import Const;
import jcc.Str2ID;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import java.util.StringTokenizer;

public abstract class
ClassClass {
    public ClassInfo		ci;
    public InterfaceMethodTable	inf;
    public boolean		impureConstants = false;
    public int			nGCStatics = 0;
    public boolean              hasStaticInitializer = false;

    public static int finalizerID = Str2ID.sigHash.getID(/*NOI18N*/"finalize", /*NOI18N*/"()V" );

    public MethodConstant findFinalizer( ){
	if ( ci == null ) return null;
	if ( ci.refMethodtable == null ) return null;
	int n = ci.refMethodtable.length;
	for ( int i = 0; i < n ; i++ ){
	    MethodConstant mc= ci.refMethodtable[i];
	    if ( mc.getID() == finalizerID ){
		if ( mc.find().parent.superClass == null ){
		    /*
		     * since we know that java.lang.Object's
		     * finalizer does nothing, we can ignore
		     * its existence!
		     */
		    return null;
		}
		return mc;
	    }
	}
	return null;
    }

    public static void setTypes(){
	classFactory.setTypes();
    }


    /*
     * Make the class list into a vector
     * ordered s.t. superclasses precede their
     * subclasses.
     */
    private static ClassClass[]cvector;
    private static int         cindex;
    private static VMClassFactory classFactory;

    // this is the recursive part
    private static void insertClassElement( ClassInfo e ){
	if ( e.vmClass != null ) return; // already in place.
	ClassInfo sup = e.superClassInfo;
	// make sure our super precedes us.
	if ( (sup != null) && (sup.vmClass == null) ) insertClassElement( sup );
	ClassClass newVmClass = classFactory.newVMClass( e );
	cvector[ cindex++ ] = newVmClass;
	//
	// If the superclass of class C has a <clinit> method, C must
	// be marked as having a static initializer too.
	//
	if (!newVmClass.hasStaticInitializer &&
	    (sup != null) && sup.vmClass.hasStaticInitializer) {
	    newVmClass.hasStaticInitializer = true;
	}
    }

    // this is the entry point for vector building.
    public static ClassClass[] getClassVector( VMClassFactory ftry ){
	if ( cvector != null ) return cvector; // just once, at most.
	classFactory = ftry;
	cvector = new ClassClass[ ClassInfo.nClasses() ];
	cindex  = 0;
	Enumeration classlist = ClassInfo.allClasses();
	while( classlist.hasMoreElements() ){
	    ClassInfo e =  (ClassInfo)classlist.nextElement();
	    if (e.vmClass == null) insertClassElement( e );
	}
	return cvector;
    }

    public static void appendClassElement( ClassInfo c ){
	// foo. Have a cvector in place, must now 
	// add a new entry at the end. "c" is it.
	if ( cvector == null ) return; // ...never mind
	ClassClass[] oldCvector = cvector;
	cvector = new ClassClass[ cindex+1 ];
	System.arraycopy( oldCvector, 0, cvector,0, cindex );
	cvector[ cindex ] = classFactory.newVMClass( c );
    }

    /**
     * Size of an instance in WORDS.
     */
    public int instanceSize(){
	FieldConstant rft[] = ci.refFieldtable;
	if ( rft == null || rft.length == 0 ) return 0;
	FieldInfo lastField = rft[rft.length-1].find();
	return (lastField.instanceOffset+lastField.nSlots);
    }

    public boolean isInterface() {
	return (ci.access&Const.ACC_INTERFACE) != 0;
    }

    public boolean hasMethodtable(){
	return ((!isInterface()) && (ci.refMethodtable != null));
    }

    public boolean isArrayClass(){
	return (ci instanceof ArrayClassInfo);
    }

    public boolean isPrimitiveClass() { 
	return (ci instanceof PrimitiveClassInfo);
    }

    public int nmethods(){
	return (ci.methods==null) ? 0 : ci.methods.length;
    }

    public int nfields(){
	return (ci.fields==null)  ? 0 : ci.fields.length;
    }

    /*
     * A concrete ClassClass needs to declare what needs to be done
     * to a constant pool containing unresolved references.
     * Some need to add UTF string references. Some may not.
     * Returns: TRUE for pure (nothing symbolic left to resolve)
     *	        FALSE otherwise.
     */
    abstract public boolean adjustSymbolicConstants( Hashtable missingClasses );

    public static boolean 
    isPartiallyResolved( ConstantObject[] consts ){
        if ( consts == null ) return false; // no const!
        int nconst = consts.length;
        if ( nconst == 0 ) return false; // no const!

        // first see if we have anything that needs our attention.
        int nsymbolic = 0;
        for( int i = 1; i < nconst; i += consts[i].nSlots ){
            ConstantObject o = consts[i];
            if (!o.isResolved()) {
                return true;
	    }
        }
	return false;
    }

    /*
     * The concrete ClassClass can define this to return an inlining for
     * all its methods
     */
    abstract public void getInlining();

}

