/*
 * -----------------------------------------------------------------
 * executejava.c : The main interpreter loop.
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying TimeSys Community Source License Agreement from TimeSys
 * in file TCSL.
 *
 * This file is derived from software distributed under the following terms:
 *
 * Sun Community Source License
 * Sun Microsystems
 * For details, see http://www.sun.com/981208/scsl/principles.html
 *
 * Your use of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


/*
 * @(#)executejava.c	1.339 00/11/10
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

#include "javavm/include/defs.h"
#include "javavm/include/objects.h"
#include "javavm/include/directmem.h"
#include "javavm/include/indirectmem.h"
#include "generated/javavm/include/opcodes.h"
#include "javavm/include/interpreter.h"
#include "javavm/include/classes.h"
#include "javavm/include/stacks.h"
#include "javavm/include/globalroots.h"
#include "javavm/include/gc_common.h"
#include "javavm/include/globals.h"
#include "javavm/include/preloader.h"
#include "javavm/include/utils.h"
#include "javavm/include/common_exceptions.h"
#include "javavm/include/clib.h"

#include "javavm/include/porting/doubleword.h"
#include "javavm/include/porting/float.h"
#include "javavm/include/porting/int.h"
#include "javavm/include/porting/jni.h"

#ifdef CVM_JVMDI
#include "javavm/include/jvmdi_impl.h"
#include "generated/offsets/java_lang_Thread.h"
#endif

/* *** RTSJ *** */
/* Added by Pratik for reference checking support */
#include "javavm/include/mem_registry.h"
/* *** JSTR *** */

/* ROBBINS SEZ: added to make rempote exceptions work */
#define CVM_REMOTE_EXCEPTIONS_SUPPORTED

/*
 * GET_INDEX - Macro used for getting an unaligned unsigned short from
 * the byte codes.
 */
#undef  GET_INDEX
#define GET_INDEX(ptr) (CVMgetUint16(ptr))

/* %comment cjp001 */
#define CVMisROMPureCode(cb) (CVMcbIsInROM(cb))

/*
 * USELABELS - If using GCC, then use labels for the opcode dispatching
 * rather -then a switch statement. This improves performance because it
 * gives us the oportunity to have the instructions that calculate the
 * next opcode to jump to be intermixed with the rest of the instructions
 * that implement the opcode (see UPDATE_PC_AND_TOS_AND_CONTINUE macro).
 */
#undef USELABELS
#ifdef __GNUC__
#define USELABELS
#endif

#undef CASE
#ifdef USELABELS
#define CASE
#define DEFAULT opc_DEFAULT
#else
#define CASE case
#define DEFAULT default
#endif

/*
 * PREFETCH_OPCCODE - Some compilers do better if you prefetch the next 
 * opcode before going back to the top of the while loop, rather then having
 * the top of the while loop handle it. This provides a better opportunity
 * for instruction scheduling. Some compilers just do this prefetch
 * automatically. Some actually end up with worse performance if you
 * force the prefetch. Solaris gcc seems to do better, but cc does worse.
 */
#undef PREFETCH_OPCCODE
#ifdef __GNUC__
#define PREFETCH_OPCCODE
#endif

/*
 * Tracing macros
 */
#undef TRACE
#undef TRACEIF
#undef TRACESTATUS
#undef TRACE_METHOD_CALL
#undef TRACE_METHOD_RETURN

#define TRACE(a) CVMtraceOpcode(a)
#define TRACESTATUS()							\
    CVMtraceStatus(("stack=%p frame=%p locals=%p tos=%p pc=%p\n",	\
		    stack, frame, locals, topOfStack, pc));

#ifdef CVM_DEBUG
#define TRACE_METHOD_CALL(frame, isJump)   CVMtraceMethodCall(ee, frame, isJump)
#define TRACE_METHOD_RETURN(frame)         CVMtraceMethodReturn(ee, frame)
#define TRACEIF(o, a)				\
    if (opcode == o) {				\
        TRACE(a);				\
    }
#else
#define TRACE_METHOD_CALL(frame, isJump)
#define TRACE_METHOD_RETURN(frame)
#define TRACEIF(o, a)
#endif

/*
 * CVM_JAVA_ERROR - Macro for throwing a java exception from
 * the interpreter loop. It makes sure that the PC is flushed
 * before CVMsignalError is called.
 */
#define CVM_JAVA_ERROR(exception_, error_)			\
{								\
    DECACHE_PC();	/* required for handle_exception */	\
    CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);	\
    CVMthrow##exception_##(ee, error_);		\
    goto handle_exception;					\
}

/* Dio: handling the exception without throwing it
 */

#define CVM_JAVA_ERROR_PASS                                    \
{\
    DECACHE_PC();	/* required for handle_exception */	\
    CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);	\
    goto handle_exception;					\
}

#define CVM_JAVA_ERROR_WITH_CLASSNAME(exception_, cb_)		\
{								\
    DECACHE_PC();	/* required for handle_exception */	\
    CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);	\
    CVMthrow##exception_##(ee, "%C", cb_);	\
    goto handle_exception;					\
}

#define CVM_JAVA_ERROR_WITH_METHODNAME(exception_, mb_)		\
{								\
    DECACHE_PC();	/* required for handle_exception */	\
    CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);	\
    CVMthrow##exception_##(ee, "%C.%M", 			\
		           CVMmbClassBlock(mb_), mb_);		\
    goto handle_exception;					\
}

#define CVM_JAVA_ILLEGAL_ACCESS_ERROR(cb_, mb_)			\
{								\
    DECACHE_PC();	/* required for handle_exception */	\
    CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);	\
    CVMthrowIllegalAccessError(					\
        ee, 							\
	"access non-public method %C.%M through an interface",	\
	cb_, mb_);						\
    goto handle_exception;					\
}


/* 
 * INIT_ROMCLASS_IF_NEEDED - Makes sure that static initializers are
 * run for a class. Assumes the class is in ROM, unless we are in
 * lossless mode, in which case the class may be in RAM since 
 * get/putstatic_checkinit_quick are used (for convenience) as the
 * lossless quick opcodes.
 */
#undef INIT_ROMCLASS_IF_NEEDED
#undef INIT_ROM_CHECK

#ifdef CVM_NO_LOSSY_OPCODES
#define INIT_DO_CHECK(cb)	(CVMcbRomIndex((cb)) != -2)
#define NON_ROM_CHECK(cb)	(CVMcbRomIndex((cb)) == -1)
#else
#define INIT_DO_CHECK(cb)	(CVMassert(CVMcbIsInROM((cb))), CVM_TRUE)
#define NON_ROM_CHECK(cb)	CVM_FALSE
#endif

#define INIT_ROM_CHECK(cb) (INIT_DO_CHECK((cb)) &&			\
			    (NON_ROM_CHECK((cb)) ||			\
			     !CVMcbCheckROMRuntimeFlag(cb, INITIALIZED)))

#define INIT_ROMCLASS_IF_NEEDED(ee, cb)			\
if (INIT_ROM_CHECK(cb) && CVMcbClinitEE(cb) != ee) {	\
        goto init_class;				\
}

#ifdef CVM_DEBUG
static void CVMtraceMethodReturn(CVMExecEnv *ee, CVMFrame* frame)
{
    CVMtrace(CVM_DEBUGFLAG(TRACE_METHOD),
	     ("<%d> Rtrn from: %s method %P\n", ee->threadID,
	      (CVMmbIs(frame->mb, NATIVE) ? "JNI " : "Java"),
	      frame));

    frame = CVMframePrev(frame);
    if (frame->mb == NULL) {
	CVMtrace(CVM_DEBUGFLAG(TRACE_METHOD),
		 ("<%d>      to:   (JNI Local Frame)\n", ee->threadID));
    } else {
	CVMtrace(CVM_DEBUGFLAG(TRACE_METHOD),
		 ("<%d>      to:   %s method %P\n", ee->threadID,
		  (CVMframeIsTransition(frame) 
		   ? "Tran"
		   : (CVMmbIs(frame->mb, NATIVE) ? "JNI " : "Java")), 
		  frame));
    }
}

static void CVMtraceMethodCall(CVMExecEnv *ee, CVMFrame* frame, CVMBool isJump)
{
    CVMFrame* savedFrame = frame;

    frame = CVMframePrev(frame);
    if (frame->mb == NULL) {
	CVMtrace(CVM_DEBUGFLAG(TRACE_METHOD),
		 ("<%d> Call from: (JNI Local Frame)\n", ee->threadID));
    } else {
	CVMtrace(CVM_DEBUGFLAG(TRACE_METHOD),
		 ("<%d> %s from: %s method %P\n", ee->threadID,
		  (isJump ? "Jump" : "Call"),
		  (CVMframeIsTransition(frame)
		   ? "Tran"
		   : (CVMmbIs(frame->mb, NATIVE) ? "JNI " : "Java")),
		  frame));
    }

    frame = savedFrame;
    CVMtrace(CVM_DEBUGFLAG(TRACE_METHOD),
	     ("<%d>      to:   %s method %P\n", ee->threadID,
	      (CVMframeIsTransition(frame)
	       ? "Tran"
	       : (CVMmbIs(frame->mb, NATIVE) ? "JNI " : "Java")), 
	      frame));
}
#endif

#ifdef CVM_INSTRUCTION_COUNTING

#include "generated/javavm/include/opcodeSimplification.h"

/*
 * Instruction counting. CVMbigCount is incremented every 1M
 * insns. CVMsmallCount keeps track of the small change.  
 */
static CVMUint32  CVMsmallCount = 0;
static CVMUint32  CVMbigCount = 0;

/*
 * Maximum length of sequences tracked
 */
#define CVM_SEQ_DEPTH 3

/*
 * An information node. Incidence of byte-codes at this depth, and pointers
 * to information nodes at the next depth.
 */
struct CVMRunInfo {
    int incidence[256];
    struct CVMRunInfo* followers[256];
};
typedef struct CVMRunInfo CVMRunInfo;

/*
 * The sliding window of byte-codes most recently executed.
 */
static CVMOpcode CVMopcodeWindow[CVM_SEQ_DEPTH];
static int CVMopcodeWindowIdx;

/*
 * The root of all sequence information nodes.
 */
static CVMRunInfo* CVMseqInfo;

/*
 * Record window of last executed opcodes, starting from index idx.
 * Lazily create information nodes.
 */ 
static void
CVMrecordWindow(int idx)
{
    int i;
    CVMRunInfo* record   = CVMseqInfo;
    CVMOpcode currOpcode = CVMopcodeWindow[idx];

    /*
     * Traverse information node tree, creating nodes as needed.
     */
    for (i = 1; i < CVM_SEQ_DEPTH; i++) {
	CVMUint32   theIdx;
	CVMRunInfo* nextRecord;

	record->incidence[currOpcode]++;
	nextRecord = record->followers[currOpcode];
	if (nextRecord == 0) {
	    nextRecord = calloc(1, sizeof(CVMRunInfo));
	    record->followers[currOpcode] = nextRecord;
	}
	record = nextRecord;
	theIdx = (idx + i) % CVM_SEQ_DEPTH;
	currOpcode = CVMopcodeWindow[theIdx];
    }
    /*
     * We are now at the right level. Record this opcode.
     */
    record->incidence[currOpcode]++;
}

static void
CVMupdateStats(CVMOpcode opcode)
{
    CVMopcodeWindow[CVMopcodeWindowIdx] = CVMopcodeSimplification[opcode];
    CVMopcodeWindowIdx++;
    CVMopcodeWindowIdx %= CVM_SEQ_DEPTH;
    CVMrecordWindow(CVMopcodeWindowIdx);
}

void 
CVMinitStats()
{
    CVMopcodeWindowIdx = 0;
    CVMseqInfo = calloc(1, sizeof(CVMRunInfo));
}

static void
CVMdumpStats2()
{
    CVMOpcode opcode1, opcode2;
    CVMconsolePrintf("Instruction count=%dM[,%d]\n",
		     CVMbigCount, CVMsmallCount);
    CVMconsolePrintf("\n\n\n\nSequences:\n");
    for (opcode1 = 0; opcode1 < 255; opcode1++) {
	for (opcode2 = 0; opcode2 < 255; opcode2++) {
	    CVMRunInfo* info = CVMseqInfo->followers[opcode1];
	    if (info != 0) {
		if (info->incidence[opcode2] != 0) {
		  CVMconsolePrintf("%d Seq: <%s>,<%s>\n",
			       info->incidence[opcode2],
			       CVMopnames[opcode1],
			       CVMopnames[opcode2]);
		}
	    }
	}
    }
}

static void
CVMdumpStats3()
{
    CVMOpcode opcode1, opcode2, opcode3;
    CVMconsolePrintf("Instruction count=%dM[,%d]\n",
		     CVMbigCount, CVMsmallCount);
    CVMconsolePrintf("\n\n\n\nSequences:\n");
    for (opcode1 = 0; opcode1 < 255; opcode1++) {
	for (opcode2 = 0; opcode2 < 255; opcode2++) {
	    for (opcode3 = 0; opcode3 < 255; opcode3++) {
		CVMRunInfo* info = CVMseqInfo->followers[opcode1];
		if (info != 0) {
		    info = info->followers[opcode2];
		    if (info != 0) {
			if (info->incidence[opcode3] != 0) {
			  CVMconsolePrintf("%d Seq: <%s>,<%s>,<%s>\n",
				       info->incidence[opcode3],
				       CVMopnames[opcode1],
				       CVMopnames[opcode2],
				       CVMopnames[opcode3]);
			}
		    }
		}
	    }
	}
    }
}

static void
CVMdumpStats4()
{
    CVMOpcode opcode1, opcode2, opcode3, opcode4;
    CVMconsolePrintf("Instruction count=%dM[,%d]\n",
		     CVMbigCount, CVMsmallCount);
    CVMconsolePrintf("\n\n\n\nSequences:\n");
    for (opcode1 = 0; opcode1 < 255; opcode1++) {
	for (opcode2 = 0; opcode2 < 255; opcode2++) {
	    for (opcode3 = 0; opcode3 < 255; opcode3++) {
		for (opcode4 = 0; opcode4 < 255; opcode4++) {
		    CVMRunInfo* info = CVMseqInfo->followers[opcode1];
		    if (info != 0) {
			info = info->followers[opcode2];
			if (info != 0) {
			    info = info->followers[opcode3];
			    if (info != 0) {
				if (info->incidence[opcode4] != 0) {
				    CVMconsolePrintf("%d Seq: <%s>,<%s>,<%s>,<%s>\n",
					       info->incidence[opcode4],
					       CVMopnames[opcode1],
					       CVMopnames[opcode2],
					       CVMopnames[opcode3],
					       CVMopnames[opcode4]);
				}
			    }
			}
		    }
		}
	    }
	}
    }
}

void
CVMdumpStats()
{
    CVMdumpStats2();
    CVMdumpStats3();
/*      CVMdumpStats4(); */
}

#define UPDATE_INSTRUCTION_COUNT(opcode)		\
{							\
    CVMsmallCount++;					\
    if (CVMsmallCount % 1000000 == 0) {			\
	CVMbigCount++;					\
        CVMsmallCount = 0;				\
    }							\
    CVMupdateStats(opcode);				\
}
#else
#define UPDATE_INSTRUCTION_COUNT(opcode)
#endif

#ifdef CVM_JVMDI
/* NOTE: This macro must be called AFTER the PC has been
   incremented. CVMjvmdiNotifyDebuggerOfSingleStep may cause a
   breakpoint opcode to get inserted at the current PC to allow the
   debugger to coalesce single-step events. Therefore, we need to
   refetch the opcode after calling out to JVMDI. */
#define DEBUGGER_SINGLE_STEP_NOTIFY()					   \
{									   \
    if (CVMglobals.jvmdiSingleStepping > 0) {				   \
	CVMJavaInt res;							   \
	CVMD_fieldReadInt(CVMID_icellDirect(ee, CVMcurrentThreadICell(ee)),\
			  CVMoffsetOfjava_lang_Thread_single_step,	   \
			  res);						   \
	if (res != 0) {							   \
	    DECACHE_PC();						   \
	    DECACHE_TOS();						   \
	    CVMD_gcSafeExec(ee, {					   \
		CVMjvmdiNotifyDebuggerOfSingleStep(ee, pc);		   \
	    });								   \
	    /* Refetch opcode. See above. */				   \
	    opcode = *pc;						   \
	}								   \
    }									   \
}
#else
#define DEBUGGER_SINGLE_STEP_NOTIFY()
#endif

/*
 * CONTINUE - Macro for executing the next opcode.
 */
#undef CONTINUE
#ifdef USELABELS
#define CONTINUE {				\
        opcode = *pc;				\
	UPDATE_INSTRUCTION_COUNT(opcode);	\
        DEBUGGER_SINGLE_STEP_NOTIFY();		\
	goto *opclabels[opcode];		\
    }
#else
#ifdef PREFETCH_OPCCODE
#define CONTINUE {				\
        opcode = *pc; 				\
	UPDATE_INSTRUCTION_COUNT(opcode);	\
	DEBUGGER_SINGLE_STEP_NOTIFY();		\
	continue;				\
    }
#else
#define CONTINUE {				\
        UPDATE_INSTRUCTION_COUNT(opcode);	\
        DEBUGGER_SINGLE_STEP_NOTIFY();		\
        continue;				\
    }
#endif
#endif

/*
 * UPDATE_PC_AND_TOS - Macro for updating the pc and topOfStack.
 */
#undef UPDATE_PC_AND_TOS
#define UPDATE_PC_AND_TOS(opsize, stack) \
    {pc += opsize; topOfStack += stack;}


/* ROBBINS SEZ:
 * The following macro is intended to be executed between every bytecode
 * to allow for fine grain remote-exception interuption. It is basically 
 * a copy of the code at the check_for_exception label modified for remote
 * exceptions only.
 */


/* ROBBINS SEZ:
 * We check for both remote and AI exceptions. I have unilaterally
 * decided that remote exceptions trump AI exceptions and are
 * handled immeadiatly.
 */

/* ROBBINS SEZ:
 * We must be exceedingly careful of race conditions involved with
 * handling AI exceptions, synchronizations WILL have to be done
 * after the check for AIExceptionPending.
 */

#define CHECK_FOR_REMOTE_AND_AI_EXCEPTIONS()                              \
{                                                                         \
  /* All these conditionals should/will be combined into a single flag */ \
  if(CVMremoteExceptionOccurred(ee)){                                     \
    CVMdebugPrintf(("REMOTE EXCEPTION OCCURRED\n"));                      \
    CVMdebugPrintf(("HANDLING EXCEPTION\n"));                             \
    goto handle_exception;                                                \
  }                                                                       \
  if(CVMATCPermission(frame)==NONDEFERRED){                               \
    if(CVMpendingAIException(ee)){                                        \
      CVMdebugPrintf(("AI EXCEPTION PENDING IN NON-DEFERED STATE\n"));    \
      CVMdebugPrintf(("HANDLING EXCEPTION\n"));                           \
      DECACHE_PC();                                                       \
      goto handle_exception;                                              \
    }                                                                     \
  }                                                                       \
}


/* ROBBINS SEZ:
 * The following three macro's have been modified to include the 
 * CHECK_FOR_REMOTE_AND_AI_EXCEPTIONS macro so that we check for remote
 * and AI exceptions basically between every bytecode. The original
 * code had us checking (for remote exceptions) only when we execute
 * a return instruction. This is probably a serious performance hit.
 */

/*
 * UPDATE_PC_AND_TOS_AND_CONTINUE - Macro for updating the pc and topOfStack,
 * and executing the next opcode. It's somewhat similar to the combination
 * of UPDATE_PC_AND_TOS and CONTINUE, but with some minor optimizations.
 */
#undef UPDATE_PC_AND_TOS_AND_CONTINUE
#ifdef USELABELS
#define UPDATE_PC_AND_TOS_AND_CONTINUE(opsize, stack) {		\
/*	printf("%d",frame->ATCflags.val.permission);*/		\
	opcode = pc[opsize]; pc += opsize; topOfStack += stack;	\
	UPDATE_INSTRUCTION_COUNT(opcode); 			\
        DEBUGGER_SINGLE_STEP_NOTIFY();				\
        CHECK_FOR_REMOTE_AND_AI_EXCEPTIONS();                   \
        goto *opclabels[opcode];				\
    }
#else
#ifdef PREFETCH_OPCCODE
#define UPDATE_PC_AND_TOS_AND_CONTINUE(opsize, stack) {		\
	opcode = pc[opsize]; pc += opsize; topOfStack += stack;	\
	UPDATE_INSTRUCTION_COUNT(opcode);			\
	DEBUGGER_SINGLE_STEP_NOTIFY();				\
        CHECK_FOR_REMOTE_AND_AI_EXCEPTIONS();                           \
	continue;						\
    }
#else
#define UPDATE_PC_AND_TOS_AND_CONTINUE(opsize, stack) {	\
	pc += opsize; topOfStack += stack;		\
	UPDATE_INSTRUCTION_COUNT(opcode);		\
	DEBUGGER_SINGLE_STEP_NOTIFY();			\
        CHECK_FOR_REMOTE_AND_AI_EXCEPTIONS();                   \
	continue;					\
    }
#endif
#endif

/*
 * For those opcodes that need to have a GC point on a backwards branch
 */
#undef UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK
#define UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(skip, stack) { \
    if ((skip) <= 0) {							   \
	CVMD_gcSafeCheckPoint(ee, {					   \
	    DECACHE_PC();						   \
	    DECACHE_TOS();						   \
	}, {});								   \
    }									   \
    UPDATE_PC_AND_TOS_AND_CONTINUE((skip), stack);			   \
}

/*
 * Macros for maintaining  Deferred/Nondeferred state.
 * -Kalpesh.
 */

#define ATC_MONITOR_ENTER(_frame){					\
									\
	if(!(CVMmbIs(mb,NATIVE))){					\
									\
		if(CVMATCSynchCounterIsZero(_frame)){			\
			CVMATCSavePermission(_frame);			\
			CVMATCSetPermission(_frame,SYNCH_DEFERRED);	\
		}							\
		CVMATCSynchCounterIncreament(_frame);			\
	}								\
									\
}									\

#define ATC_MONITOR_EXIT(_frame){				\
								\
	if(!(CVMmbIs(mb,NATIVE))){				\
		CVMATCSynchCounterDecreament(_frame);		\
		if(CVMATCSynchCounterIsZero(_frame)){		\
			CVMATCRestorePermission(_frame);	\
		}						\
								\
	}							\
								\
}								\

/*
 * Macros for accessing the stack.
 */
#undef STACK_INFO
#undef STACK_INT
#undef STACK_FLOAT
#undef STACK_ICELL
#undef STACK_ADDR
#undef STACK_OBJECT
#undef STACK_DOUBLE
#undef STACK_LONG
#define STACK_SLOT(offset)    (topOfStack[offset].s)
#define STACK_INFO(offset)    (STACK_SLOT(offset).j)
#define STACK_ADDR(offset)    (STACK_SLOT(offset).a)
#define STACK_INT(offset)     (STACK_INFO(offset).i)
#define STACK_FLOAT(offset)   (STACK_INFO(offset).f)
#define STACK_ICELL(offset)   (STACK_INFO(offset).r)
#define STACK_OBJECT(offset)  (CVMID_icellDirect(ee, &STACK_ICELL(offset)))

#define STACK_DOUBLE(offset) \
        CVMjvm2Double(&STACK_INFO(offset).raw)
#define STACK_LONG(offset)   \
        CVMjvm2Long(&STACK_INFO(offset).raw)

#ifdef CVM_DEBUG
#undef GET_LONGCSTRING
#define GET_LONGCSTRING(number) \
        (CVMlong2String(number, trBuf, trBuf+sizeof(trBuf)), trBuf)
#endif

/*
 * Macros for caching and flushing the interpreter state. Some local
 * variables need to be flushed out to the frame before we do certain
 * things (like pushing frames or becomming gc safe) and some need to 
 * be recached later (like after popping a frame). We could use one
 * macro to cache or decache everything, but this would be less then
 * optimal because we don't always need to cache or decache everything
 * because some things we know are already cached or decached.
 */
#undef DECACHE_TOS
#undef CACHE_TOS
#undef CACHE_PREV_TOS
#define DECACHE_TOS()    frame->topOfStack = topOfStack;
#define CACHE_TOS()   	 topOfStack = frame->topOfStack;
#define CACHE_PREV_TOS() topOfStack = CVMframePrev(frame)->topOfStack;

#undef DECACHE_PC
#undef CACHE_PC
#define DECACHE_PC()	CVMframePc(frame) = pc;
#define CACHE_PC()      pc = CVMframePc(frame);

#undef CACHE_FRAME
#define CACHE_FRAME()   frame = stack->currentFrame;
 
/*
 * CHECK_NULL - Macro for throwing a NullPointerException if the object
 * passed is a null ref.
 */
#undef CHECK_NULL
#define CHECK_NULL(obj_) 				\
    if ((obj_) == 0) {   				\
        CVM_JAVA_ERROR(NullPointerException, "");	\
    }

/*
 * The following 4 structures are static java bytecodes
 * that contain the transition code for various types of method
 * invocations. They are used to handle the invocation of
 * the mb passed into executeJavaMethod.
 */
static const CVMUint8 CVMinvokeStaticTransitionCode[] = {
    opc_invokestatic_quick, 0, 1, opc_exittransition
};
static const CVMUint8 CVMinvokeVirtualTransitionCode[] = {
    opc_invokevirtual_quick_w, 0, 1, opc_exittransition
};
static const CVMUint8 CVMinvokeNonVirtualTransitionCode[] = {
    opc_invokenonvirtual_quick, 0, 1, opc_exittransition
};
static const CVMUint8 CVMinvokeInterfaceTransitionCode[] = {
    opc_invokeinterface_quick, 0, 1, 0, 0, opc_exittransition
};

/*
 * CVMinvokeInterfaceError - throws the proper exception for
 * invokeinterface_quick.
 */
static void
CVMinvokeInterfaceError(CVMExecEnv *ee, CVMClassBlock *cb, 
			CVMClassBlock *intfcb)
{ 
    CVM_RESET_INTERPRETER_TOS(CVMeeGetCurrentFrame(ee)->topOfStack, 
			      CVMeeGetCurrentFrame(ee));
    CVMthrowIncompatibleClassChangeError(
        ee, "class %C does not implement interface %C", cb, intfcb);
}

/*
 * CVMgcUnsafeExecuteJavaMethod
 *
 * The real deal. This is where byte codes actually get interpreted.
 * Basically it's a big while loop that iterates until we return from
 * the method passed in.
 *
 * As long as java code is being executed, the interpreter is iterative,
 * not recursive. However, if native code is invoked, then it may re-enter
 * executeJavaMethod. In this case there will already be frames on
 * the stack.
 *
 * Upon entry:
 *   -ee->interpreterStack.currentFrame must be a transition
 *    frame that holds the method arguments.
 *   -currentFrame->topOfStack must point after the arguments.
 */
void
CVMgcUnsafeExecuteJavaMethod(CVMExecEnv* ee, CVMMethodBlock* mb,
			     CVMBool isStatic, CVMBool isVirtual)
{


    CVMFrame*         frame = NULL;      /* actually it's an CVMJavaFrame */
    CVMFrame*         initialframe = NULL;  /* the initial transition frame
					that holds the arguments */

    CVMStack*         stack = &ee->interpreterStack;
    CVMStackVal32*    topOfStack = NULL; /* access with STACK_ and _TOS macros */    
    CVMSlotVal32*     locals = 0; /* area in the frame for local variables */
    CVMUint8*         pc = NULL;
    CVMUint8          opcode = 0;
    CVMConstantPool*  cp = NULL;
    CVMTransitionConstantPool   transitioncp;
    CVMClassBlock*    cb;
    CVMObjectICell*   receiverObjICell; /* the object we are dispatching on or
					   the Class object for static calls */
    CVMCheckedExceptions* checks; // Kalpesh //
    CVMClassBlock*  exclass;
    CVMClassTypeID  classID;
    char tmpState;
    static CVMClassBlock* baseClass=NULL;
    static CVMClassBlock* aieClass=NULL;
    static CVMClassBlock* ioClass = NULL;

    // variables for executing <clinit> in Immortal
    int clinitStackDepth = 0;
//    CVMObjectICell oldMemoryArea;
    CVMBool oldCheckConstraints = CVM_FALSE;
    DECLARE_VARS_FOR_MA_SLIDING(ee);
    
#ifdef CVM_CLASSLOADING
    CVMBool clobbersCpIndex; /* true if quickening clobbers the cp index */
#endif
#ifdef CVM_DEBUG
    char              trBuf[30];   /* needed by GET_LONGCSTRING */
#endif

#ifdef USELABELS
#include "generated/javavm/include/opcodeLabels.h"
    const static void* const opclabels_data[256] = { 
	&&opc_0, &&opc_1, &&opc_2, &&opc_3, &&opc_4, &&opc_5,
	&&opc_6, &&opc_7, &&opc_8, &&opc_9, &&opc_10, &&opc_11,
	&&opc_12, &&opc_13, &&opc_14, &&opc_15, &&opc_16, &&opc_17,
	&&opc_18, &&opc_19, &&opc_20, &&opc_21, &&opc_22, &&opc_23,
	&&opc_24, &&opc_25, &&opc_26, &&opc_27, &&opc_28, &&opc_29,
	&&opc_30, &&opc_31, &&opc_32, &&opc_33, &&opc_34, &&opc_35,
	&&opc_36, &&opc_37, &&opc_38, &&opc_39, &&opc_40, &&opc_41,
	&&opc_42, &&opc_43, &&opc_44, &&opc_45, &&opc_46, &&opc_47,
	&&opc_48, &&opc_49, &&opc_50, &&opc_51, &&opc_52, &&opc_53,
	&&opc_54, &&opc_55, &&opc_56, &&opc_57, &&opc_58, &&opc_59,
	&&opc_60, &&opc_61, &&opc_62, &&opc_63, &&opc_64, &&opc_65,
	&&opc_66, &&opc_67, &&opc_68, &&opc_69, &&opc_70, &&opc_71,
	&&opc_72, &&opc_73, &&opc_74, &&opc_75, &&opc_76, &&opc_77,
	&&opc_78, &&opc_79, &&opc_80, &&opc_81, &&opc_82, &&opc_83,
	&&opc_84, &&opc_85, &&opc_86, &&opc_87, &&opc_88, &&opc_89,
	&&opc_90, &&opc_91, &&opc_92, &&opc_93, &&opc_94, &&opc_95,
	&&opc_96, &&opc_97, &&opc_98, &&opc_99, &&opc_100, &&opc_101,
	&&opc_102, &&opc_103, &&opc_104, &&opc_105, &&opc_106, &&opc_107,
	&&opc_108, &&opc_109, &&opc_110, &&opc_111, &&opc_112, &&opc_113,
	&&opc_114, &&opc_115, &&opc_116, &&opc_117, &&opc_118, &&opc_119,
	&&opc_120, &&opc_121, &&opc_122, &&opc_123, &&opc_124, &&opc_125,
	&&opc_126, &&opc_127, &&opc_128, &&opc_129, &&opc_130, &&opc_131,
	&&opc_132, &&opc_133, &&opc_134, &&opc_135, &&opc_136, &&opc_137,
	&&opc_138, &&opc_139, &&opc_140, &&opc_141, &&opc_142, &&opc_143,
	&&opc_144, &&opc_145, &&opc_146, &&opc_147, &&opc_148, &&opc_149,
	&&opc_150, &&opc_151, &&opc_152, &&opc_153, &&opc_154, &&opc_155,
	&&opc_156, &&opc_157, &&opc_158, &&opc_159, &&opc_160, &&opc_161,
	&&opc_162, &&opc_163, &&opc_164, &&opc_165, &&opc_166, &&opc_167,
	&&opc_168, &&opc_169, &&opc_170, &&opc_171, &&opc_172, &&opc_173,
	&&opc_174, &&opc_175, &&opc_176, &&opc_177, &&opc_178, &&opc_179,
	&&opc_180, &&opc_181, &&opc_182, &&opc_183, &&opc_184, &&opc_185,
	&&opc_186, &&opc_187, &&opc_188, &&opc_189, &&opc_190, &&opc_191,
	&&opc_192, &&opc_193, &&opc_194, &&opc_195, &&opc_196, &&opc_197,
	&&opc_198, &&opc_199, &&opc_200, &&opc_201, &&opc_202, &&opc_203,
	&&opc_204, &&opc_205, &&opc_206, &&opc_207, &&opc_208, &&opc_209,
	&&opc_210, &&opc_211, &&opc_212, &&opc_213, &&opc_214, &&opc_215,
	&&opc_216, &&opc_217, &&opc_218, &&opc_219, &&opc_220, &&opc_221,
	&&opc_222, &&opc_223, &&opc_224, &&opc_225, &&opc_226, &&opc_227,
	&&opc_228, &&opc_229, &&opc_230, &&opc_231, &&opc_232, &&opc_233,
	&&opc_234, &&opc_235, &&opc_236, &&opc_237, &&opc_238, &&opc_239,
	&&opc_240, &&opc_241, &&opc_242, &&opc_243, &&opc_244, &&opc_245,
	&&opc_246, &&opc_247, &&opc_248, &&opc_249, &&opc_250, &&opc_251,
	&&opc_252, &&opc_253, &&opc_254, &&opc_255
    };
    const void* const *opclabels = &opclabels_data[0];
#endif /* USELABELS */

    /* C stack redzone check */
    if (!CVMCstackCheckSize(ee, CVM_REDZONE_ILOOP, 
        "CVMgcUnsafeExecuteJavaMethod", CVM_TRUE)) {
	return;
    }

    CVMassert(CVMD_isgcUnsafe(ee));

    /* The initialframe is always a transition frame. Its topOfStack
     * always points just after the arguments. 
     */
    CACHE_FRAME();
    initialframe = frame;
   
		  
   /* We reach this point each time a new transtion frame has been
     * pushed. This happens just before CVMgcUnsafeExecuteJavaMethod
     * is first invoked, by the init_class: label, and if
     * CVM_NEW_TRANSITION_FRAME is returned from an CVM native method.
     */
new_transition:
    CVMassert(CVMframeIsTransition(frame));
    CACHE_TOS();   /* topOfStack points after the arguments. */

    
    /* Get the correct transtion code pointer. */

     
    if (isStatic) {
	/* static method */
	pc = (CVMUint8*)CVMinvokeStaticTransitionCode;
    } else {
	if (CVMcbIs(CVMmbClassBlock(mb), INTERFACE)) {
	    /* interface method */
	    pc = (CVMUint8*)CVMinvokeInterfaceTransitionCode;
	} else if (isVirtual && !CVMmbIs(mb, PRIVATE) &&
		   !(CVMtypeidIsConstructor(CVMmbNameAndTypeID(mb)))) {
	    /* virtual method */
	    pc = (CVMUint8*)CVMinvokeVirtualTransitionCode;
	} else {
	    /* nonvirtual method */ 
	    pc = (CVMUint8*)CVMinvokeNonVirtualTransitionCode;
	}
    }

    /* The constant pool for the transition code has just one entry
     * which points to the resolved mb of the method we want to call.
     */
    CVMinitTransitionConstantPool(&transitioncp, mb);
    cp = (CVMConstantPool*)&transitioncp;

    

    TRACE_METHOD_CALL(frame, CVM_FALSE);
  
#ifdef PREFETCH_OPCCODE
    opcode = *pc;  /* prefetch first opcode */
#endif

    TRACESTATUS();

#ifndef USELABELS
    while (1)
#endif
    {
#ifndef PREFETCH_OPCCODE
	opcode = *pc;
#endif
	/* Using this labels avoids double breakpoints when quickening and
	 * when returing from transition frames.
	 */
    opcode_switch:

#ifdef USELABELS
	goto *opclabels[opcode];
#else
	switch (opcode)
#endif
	{
	CASE opc_nop:
	    TRACE(("\t%s\n", CVMopnames[opcode]));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

	    /* Push miscellaneous constants onto the stack. */

	CASE opc_aconst_null:
	    CVMID_icellSetNull(&STACK_ICELL(0));
	    TRACE(("\t%s\n", CVMopnames[opcode]));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);

#undef  OPC_CONST_n
#define OPC_CONST_n(opcode, const_type, value) 				\
	CASE opcode: 							\
	    STACK_INFO(0).const_type = value; 				\
	    TRACE(("\t%s\n", CVMopnames[opcode])); 			\
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);

	    OPC_CONST_n(opc_iconst_m1,   i,       -1);
	    OPC_CONST_n(opc_iconst_0,    i,        0);
	    OPC_CONST_n(opc_iconst_1,    i,        1);
	    OPC_CONST_n(opc_iconst_2,    i,        2);
	    OPC_CONST_n(opc_iconst_3,    i,        3);
	    OPC_CONST_n(opc_iconst_4,    i,        4);
	    OPC_CONST_n(opc_iconst_5,    i,        5);
	    OPC_CONST_n(opc_fconst_0,    f,      0.0);
	    OPC_CONST_n(opc_fconst_1,    f,      1.0);
	    OPC_CONST_n(opc_fconst_2,    f,      2.0);

#undef  OPC_CONST2_n
#define OPC_CONST2_n(opcname, value, key)	     			\
        CASE opc_##opcname:						\
        {                                                               \
           CVM##key##2Jvm(&STACK_INFO(0).raw,				\
	       CVM##key##Const##value());	   			\
	   TRACE(("\t%s\n", CVMopnames[opcode])); 			\
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, 2);                        \
        }

           OPC_CONST2_n(dconst_0, Zero, double);
           OPC_CONST2_n(dconst_1, One,  double);
           OPC_CONST2_n(lconst_0, Zero, long);
           OPC_CONST2_n(lconst_1, One,  long);

	   /* Load constant from constant pool: */

	CASE opc_aldc_ind_quick: { /* Indirect String (loaded classes) */
	    CVMObjectICell* strICell = CVMcpGetStringICell(cp, pc[1]);
	    CVMID_icellAssignDirect(ee, &STACK_ICELL(0), strICell);
            TRACE(("\t%s #%d => 0x%x\n", CVMopnames[opcode],
		   pc[1], STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, 1);
	}

	CASE opc_aldc_ind_w_quick: { /* Indirect String (loaded classes) */
	    CVMObjectICell* strICell =
		CVMcpGetStringICell(cp, GET_INDEX(pc + 1));
	    CVMID_icellAssignDirect(ee, &STACK_ICELL(0), strICell);
            TRACE(("\t%s #%d => 0x%x\n", CVMopnames[opcode],
		   pc[1], STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);
	}

        CASE opc_aldc_quick: /* aldc loads direct String refs (ROM only) */
        CASE opc_ldc_quick:
	    STACK_INFO(0) = CVMcpGetVal32(cp, pc[1]);
            TRACE(("\t%s #%d => 0x%x\n", CVMopnames[opcode],
		   pc[1], STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, 1);

        CASE opc_aldc_w_quick: /* aldc_w loads direct String refs (ROM only) */
        CASE opc_ldc_w_quick:
            STACK_INFO(0) = CVMcpGetVal32(cp, GET_INDEX(pc + 1));
            TRACE(("\t%s #%d => 0x%x\n", CVMopnames[opcode],
		   GET_INDEX(pc + 1), STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);

        CASE opc_ldc2_w_quick:
 	{
	    CVMJavaVal64 r;
	    CVMcpGetVal64(cp, GET_INDEX(pc+1), r);
	    CVMmemCopy64(&STACK_INFO(0).raw, r.v); 
            TRACE(("\topc_ldc2_w_quick #%d => long=%s double=%g\n", 
		   GET_INDEX(pc + 1),
                   GET_LONGCSTRING(STACK_LONG(0)), STACK_DOUBLE(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 2);
	}

	    /* Push a 1-byte signed integer value onto the stack. */
	CASE opc_bipush:
	    STACK_INT(0) = (CVMInt8)(pc[1]);
	    TRACE(("\tbipush %d\n", STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, 1);

	    /* Push a 2-byte signed integer constant onto the stack. */
	CASE opc_sipush:
	    STACK_INT(0) = CVMgetInt16(pc + 1);
	    TRACE(("\tsipush %d\n", STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);

	    /* load from local variable */

	CASE opc_aload:
        /* *** RTSJ *** */ 
        /*   Reference checking support.   
           Check if local variable assignment is allowed. */ 
#if 0
        { 
            extern int JVMCeiInitialized; 
            if(JVMCeiInitialized) 
            { 
				if(CVMisNoHeapRealtimeThread(ee) == CVM_TRUE) 
				{
                    DECACHE_PC();
                    DECACHE_TOS();
                	if(CVMJTisHeapReference(ee, CVMID_icellDirect(ee, &locals[pc[1]].j.r)) == CVM_TRUE) 
                	{ 
#ifdef CVM_DEBUG
                    	CVMconsolePrintf("RTSJ: (aload) Throwing a MemoryAccessError. \
								memory area = %O stack object = %O\n", 
								CVMID_icellDirect(ee, CVMmemoryAreaICell(ee)),
							    CVMID_icellDirect(ee, &locals[pc[1]].j.r));
#endif
                    	CVM_JAVA_ERROR(MemoryAccessError, "NoHeapRealtimeThread tried to access heap."); 
                	}
				}	
            } 
        } 
#endif
        /* *** JSTR *** */ 
	CASE opc_iload:
    CASE opc_fload:

	    STACK_SLOT(0) = locals[pc[1]];
	    TRACEIF(opc_aload, ("\taload locals[%d](%p) =>\n",
				pc[1], STACK_OBJECT(0)));
	    TRACEIF(opc_iload, ("\tiload locals[%d](%d) =>\n", 
				pc[1], STACK_INT(0)));
	    TRACEIF(opc_fload, ("\tfload locals[%d](%f) =>\n", 
				pc[1], STACK_FLOAT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, 1);

        CASE opc_lload:
        CASE opc_dload:
	    CVMmemCopy64(&STACK_INFO(0).raw, &locals[pc[1]].j.raw);
	    TRACEIF(opc_dload, ("\tdload locals[%d](%f) =>\n",
				pc[1], STACK_DOUBLE(0)));
	    TRACEIF(opc_lload, ("\tlload locals[%d](%s) =>\n",
				pc[1], GET_LONGCSTRING(STACK_LONG(0))));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, 2);

#undef  OPC_LOAD_n
#define OPC_LOAD_n(num)							\
	CASE opc_aload_##num:						\
    /* *** RTSJ ***  \
       Reference checking support.   \
       Check if local variable assignment is allowed. */ \
     /*{ \
            extern int JVMCeiInitialized; \
            if(JVMCeiInitialized) \
            { \
				if(CVMisNoHeapRealtimeThread(ee) == CVM_TRUE) { \
                    DECACHE_PC(); \
                    DECACHE_TOS(); \
                	if(CVMJTisHeapReference(ee, CVMID_icellDirect(ee, &locals[num].j.r)) == CVM_TRUE) \
                	{ \
                    	CVMconsolePrintf("RTSJ: (aload_<n>) Throwing an IllegalAssignmentError. \
								memory area = %O stack object = %O\n", \
								CVMID_icellDirect(ee, CVMmemoryAreaICell(ee)), \
								CVMID_icellDirect(ee, &locals[num].j.r)); \
                    	CVM_JAVA_ERROR(MemoryAccessError, "Illegal local variable load"); \
                	} \
				} \
            } \
        } */\
        /* *** JSTR *** */ \
	CASE opc_iload_##num:						\
	CASE opc_fload_##num:						\
	    STACK_SLOT(0) = locals[num];				\
	    TRACEIF(opc_aload_##num, ("\t%s locals[%d](%p) =>\n",	\
				      CVMopnames[opcode], ##num,	\
				      STACK_OBJECT(0)));		\
	    TRACEIF(opc_iload_##num, ("\t%s locals[%d](%d) =>\n",	\
				      CVMopnames[opcode], ##num,	\
				      STACK_INT(0)));			\
	    TRACEIF(opc_fload_##num, ("\t%s locals[%d](%f) =>\n",	\
				      CVMopnames[opcode], ##num,	\
				      STACK_FLOAT(0)));			\
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);			\
									\
	CASE opc_lload_##num:						\
	CASE opc_dload_##num:						\
	    CVMmemCopy64(&STACK_INFO(0).raw, &locals[num].j.raw);	\
	    TRACEIF(opc_dload_##num, ("\t%s locals[%d](%f) =>\n",	\
				      CVMopnames[opcode], ##num,	\
				      STACK_DOUBLE(0)));		\
	    TRACEIF(opc_lload_##num, ("\t%s locals[%d](%s) =>\n",	\
				      CVMopnames[opcode], ##num,	\
				      GET_LONGCSTRING(STACK_LONG(0)))); \
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 2);

	    OPC_LOAD_n(0);
	    OPC_LOAD_n(1);
	    OPC_LOAD_n(2);
	    OPC_LOAD_n(3);

	    /* store to a local variable */
	   
	CASE opc_astore:
        /* *** RTSJ *** */
        /* Reference checking support.
         * Check if local variable assignment is allowed.
         */
#if 0
        {
            extern int JVMCeiInitialized;
            if(JVMCeiInitialized)
            {
                // CVMJTrcAllowLocalVarAsignment might go into gc-safe 
                // state so uncache our local state.
	            DECACHE_PC();
        	    DECACHE_TOS();
                if(CVMJTrcAllowLocalVarAssignment(ee, STACK_OBJECT(-1)) == CVM_FALSE)
                {
#ifdef CVM_DEBUG
                    //CVMconsolePrintf("RTSJ: (astore) Throwing an IllegalAssignmentError. memory area = %O stack object = %O\n", CVMID_icellDirect(ee, CVMmemoryAreaICell(ee)), STACK_OBJECT(-1));
	                CVMconsolePrintf("RTSJ: \tastore %p => locals[%d]\n",
				            STACK_OBJECT(-1), pc[1]);
#endif
                    CVM_JAVA_ERROR(IllegalAssignmentError, "Illegal local variable store");
                }
            }
        }
#endif
        /* *** JSTR *** */
	CASE opc_istore:
	CASE opc_fstore:
	    locals[pc[1]] = STACK_SLOT(-1);
	    TRACEIF(opc_astore, ("\tastore %p => locals[%d]\n",
				 STACK_OBJECT(-1), pc[1]));
	    TRACEIF(opc_istore,
		    ("\tistore %d ==> locals[%d]\n", STACK_INT(-1), pc[1]));
	    TRACEIF(opc_fstore,
		    ("\tfstore %f ==> locals[%d]\n", STACK_FLOAT(-1), pc[1]));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, -1);

	CASE opc_lstore:
	CASE opc_dstore:
	    CVMmemCopy64(&locals[pc[1]].j.raw, &STACK_INFO(-2).raw);
	    TRACEIF(opc_dstore, ("\tdstore %f => locals[%d]\n",
				 STACK_DOUBLE(-2), pc[1]));
	    TRACEIF(opc_lstore, ("\tlstore %s => locals[%d]\n",
				 GET_LONGCSTRING(STACK_LONG(-2)), pc[1]));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, -2);

        CASE opc_wide: {
            CVMUint16 reg = GET_INDEX(pc + 2);

	    opcode = pc[1];
            switch(opcode) {
                case opc_aload:
			    /* *** RTSJ *** */ 
        		/*   Reference checking support.   
           			Check if local variable assignment is allowed. */ 
        		{ 
            		extern int JVMCeiInitialized; 
            		if(JVMCeiInitialized) 
            		{ 
						if(CVMisNoHeapRealtimeThread(ee) == CVM_TRUE) 
						{
                            DECACHE_PC();
                            DECACHE_TOS();
                			if(CVMJTisHeapReference(ee, CVMID_icellDirect(ee, &locals[reg].j.r)) == CVM_TRUE)
                			{ 
#ifdef CVM_DEBUG
                    	CVMconsolePrintf("RTSJ: (aload) Throwing an IllegalAssignmentError. memory area = %O stack object = %O\n",CVMID_icellDirect(ee, CVMmemoryAreaICell(ee)),
							    CVMID_icellDirect(ee, &locals[reg].j.r));
#endif
                    			CVM_JAVA_ERROR(MemoryAccessError, "Illegal local variable load"); 
                			}
						}	
            		} 
        		} 
        		/* *** JSTR *** */ 
                case opc_iload:
                case opc_fload:
                    STACK_SLOT(0) = locals[reg]; 
                    TRACEIF(opc_iload, ("\tiload_w locals[%d](%d) =>\n", 
					reg, STACK_INT(0)));
                    TRACEIF(opc_aload, ("\taload_w locals[%d](%p) =>\n", 
					reg, STACK_OBJECT(0)));
                    TRACEIF(opc_fload, ("\tfload_w locals[%d](%f) =>\n",
                            	        reg, STACK_FLOAT(0)));
            	    UPDATE_PC_AND_TOS_AND_CONTINUE(4, 1);
                case opc_lload:
                case opc_dload:
	  	    CVMmemCopy64(&STACK_INFO(0).raw, &locals[reg].j.raw);
                    TRACEIF(opc_lload, ("\tlload_w locals[%d](%s) =>\n",
                            		reg, GET_LONGCSTRING(STACK_LONG(0))));
                    TRACEIF(opc_dload, ("\tdload_w locals[%d](%f) =>\n",
                            		reg, STACK_DOUBLE(0)));
            	    UPDATE_PC_AND_TOS_AND_CONTINUE(4, 2);
                case opc_istore:
                case opc_astore:
                    /* *** RTSJ *** */
                    /* Reference checking support.
                     * Check if local variable assignment is allowed.
                     */
#if 0
                    {
                        extern int JVMCeiInitialized;
                        if(JVMCeiInitialized)
                        {
                            // CVMJTrcAllowLocalVarAsignment might go into gc-safe 
                            // state so uncache our local state.
                            DECACHE_PC();
                            DECACHE_TOS();
                            if(CVMJTrcAllowLocalVarAssignment(ee, STACK_OBJECT(-1)) == CVM_FALSE)
                            {
#ifdef CVM_DEBUG
                                //CVMconsolePrintf("RTSJ: (astore) Throwing an IllegalAssignmentError. memory area = %O stack object = %O\n", CVMID_icellDirect(ee, CVMmemoryAreaICell(ee)), STACK_OBJECT(-1));
                                CVMconsolePrintf("RTSJ: \tastore %p => locals[%d]\n",
                                        STACK_OBJECT(-1), pc[1]);
#endif
                                CVM_JAVA_ERROR(IllegalAssignmentError, "Illegal local variable store");
                            }
                        }
                    }
#endif
                    /* *** JSTR *** */

                case opc_fstore:
		    locals[reg] = STACK_SLOT(-1);
                    TRACEIF(opc_istore,
			    ("\tistore_w %d ==> locals[%d]\n", STACK_INT(-1), reg));
                    TRACEIF(opc_astore, ("\tastore_w %p => locals[%d]\n",
                                 STACK_OBJECT(-1), reg));
                    TRACEIF(opc_fstore,
			    ("\tfstore_w %f ==> locals[%d]\n", STACK_FLOAT(-1), reg));
            	    UPDATE_PC_AND_TOS_AND_CONTINUE(4, -1);
                case opc_lstore:
                case opc_dstore:
		    CVMmemCopy64(&locals[reg].j.raw, &STACK_INFO(-2).raw);
                    TRACEIF(opc_lstore, ("\tlstore_w %s => locals[%d]\n",
                                 GET_LONGCSTRING(STACK_LONG(-2)), reg));
                    TRACEIF(opc_dstore, ("\tdstore_w %f => locals[%d]\n",
                                 STACK_DOUBLE(-2), reg));
            	    UPDATE_PC_AND_TOS_AND_CONTINUE(4, -2);
                case opc_iinc: {
		    CVMInt16 offset = CVMgetInt16(pc+4); 
		    locals[reg].j.i += offset; 
            	    TRACE(("\tiinc_w locals[%d]+%d => %d\n", reg, 
                          offset, locals[reg].j.i));
            	    UPDATE_PC_AND_TOS_AND_CONTINUE(6, 0);
	        }
                case opc_ret:
            	    TRACE(("\tret_w %d (%#x)\n", reg, locals[reg].a));
                    pc = locals[reg].a;
            	    UPDATE_PC_AND_TOS_AND_CONTINUE(0, 0);
                default:
                    CVM_JAVA_ERROR(InternalError, "undefined opcode");
            }
        }


#undef  OPC_STORE_n
#define OPC_STORE_n(num)						\
	CASE opc_astore_##num:						\
        /* *** RTSJ ***  \
           Reference checking support.   \
           Check if local variable assignment is allowed. */ \
        /*{ \
            extern int JVMCeiInitialized; \
            if(JVMCeiInitialized) \
            { \
                 CVMJTrcAllowLocalVarAsignment might go into gc-safe  \
                   state so uncache our local state.  \
	            DECACHE_PC(); \
        	    DECACHE_TOS(); \
                if(CVMJTrcAllowLocalVarAssignment(ee, STACK_OBJECT(-1)) == CVM_FALSE) \
                { \
                    CVMconsolePrintf("RTSJ: (astore_<n>) Throwing an IllegalAssignmentError. memory area = %O stack object = %O\n", CVMID_icellDirect(ee, CVMmemoryAreaICell(ee)), STACK_OBJECT(-1)); \
                    CVM_JAVA_ERROR(IllegalAssignmentError, "Illegal local variable store"); \
                } \
            } \
        } */\
        /* *** JSTR *** */ \
	CASE opc_istore_##num:						\
	CASE opc_fstore_##num:						\
	    locals[num] = STACK_SLOT(-1);				\
	    TRACEIF(opc_astore_##num, ("\t%s %p => locals[%d]\n",	\
		    CVMopnames[opcode], STACK_OBJECT(-1), num));	\
	    TRACEIF(opc_istore_##num, ("\t%s %d => locals[%d]\n",	\
		    CVMopnames[opcode], STACK_INT(-1), num));		\
	    TRACEIF(opc_fstore_##num, ("\t%s %f => locals[%d]\n",	\
		    CVMopnames[opcode], STACK_FLOAT(-1), num));		\
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);

	    OPC_STORE_n(0);
	    OPC_STORE_n(1);
	    OPC_STORE_n(2);
	    OPC_STORE_n(3);

#undef  OPC_DSTORE_n
#define OPC_DSTORE_n(num)						\
	CASE opc_dstore_##num:						\
	CASE opc_lstore_##num:						\
	    CVMmemCopy64(&locals[num].j.raw, &STACK_INFO(-2).raw);	\
	    TRACEIF(opc_dstore_##num, ("\t%s %f => locals[%d]\n",	\
		    CVMopnames[opcode], STACK_DOUBLE(-2), num));	\
	    TRACEIF(opc_lstore_##num, ("\t%s %s => locals[%d]\n",	\
		    CVMopnames[opcode], 				\
		    GET_LONGCSTRING(STACK_LONG(-2)), num));		\
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -2);

	    OPC_DSTORE_n(0);
	    OPC_DSTORE_n(1);
	    OPC_DSTORE_n(2);
	    OPC_DSTORE_n(3);

	    /* stack pop, dup, and insert opcodes */

	CASE opc_nonnull_quick:  /* pop stack, and error if it is null */
	    TRACE(("\tnonnull_quick\n"));
	    CHECK_NULL(STACK_OBJECT(-1));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);

	   
	CASE opc_pop:		 /* Discard the top item on the stack */
	    TRACE(("\tpop\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);

	   
	CASE opc_pop2:		 /* Discard the top 2 items on the stack */
	    TRACE(("\tpop2\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -2);

	    
	CASE opc_dup:		/* Duplicate the top item on the stack */
	    STACK_INFO(0) = STACK_INFO(-1);
	    TRACE(("\tdup\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);

	CASE opc_dup2:		/* Duplicate the top 2 items on the stack */
	    STACK_INFO(0) = STACK_INFO(-2);
	    STACK_INFO(1) = STACK_INFO(-1);
	    TRACE(("\tdup2\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 2);

	CASE opc_dup_x1:	/* insert top word two down */
	    STACK_INFO(0) = STACK_INFO(-1);
	    STACK_INFO(-1) = STACK_INFO(-2);
	    STACK_INFO(-2) = STACK_INFO(0);
	    TRACE(("\tdup_x1\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);

	CASE opc_dup_x2:	/* insert top word three down  */
	    STACK_INFO(0) = STACK_INFO(-1);
	    STACK_INFO(-1) = STACK_INFO(-2);
	    STACK_INFO(-2) = STACK_INFO(-3);
	    STACK_INFO(-3) = STACK_INFO(0);
	    TRACE(("\tdup_x2\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);

	CASE opc_dup2_x1:	/* insert top double three down */
	    STACK_INFO(1) = STACK_INFO(-1);
	    STACK_INFO(0) = STACK_INFO(-2);
	    STACK_INFO(-1) = STACK_INFO(-3);
	    STACK_INFO(-2) = STACK_INFO(1);
	    STACK_INFO(-3) = STACK_INFO(0);
	    TRACE(("\tdup2_x1\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 2);

	CASE opc_dup2_x2:	/* insert top double four down */
	    STACK_INFO(1) = STACK_INFO(-1);
	    STACK_INFO(0) = STACK_INFO(-2);
	    STACK_INFO(-1) = STACK_INFO(-3);
	    STACK_INFO(-2) = STACK_INFO(-4);
	    STACK_INFO(-3) = STACK_INFO(1);
	    STACK_INFO(-4) = STACK_INFO(0);
	    TRACE(("\tdup2_x2\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 2);

	CASE opc_swap: {        /* swap top two elements on the stack */
	    CVMJavaVal32 j = STACK_INFO(-1);
	    STACK_INFO(-1) = STACK_INFO(-2);
	    STACK_INFO(-2) = j;
	    TRACE(("\tswap\n"));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);
	}

	    /* Perform various binary integer operations */

#undef  OPC_INT_BINARY 
#define OPC_INT_BINARY(opcname, opname, test)				\
	CASE opc_i##opcname:						\
	    if (test && (STACK_INT(-1) == 0)) {				\
                CVM_JAVA_ERROR(ArithmeticException, "/ by int zero");	\
	    }								\
            STACK_INT(-2) = 						\
		CVMint##opname(STACK_INT(-2), STACK_INT(-1));		\
	    TRACE(("\t%s => %d\n", CVMopnames[opcode], STACK_INT(-2)));	\
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);			\
	CASE opc_l##opcname:						\
	{								\
	    CVMJavaLong l1, l2, r1;					\
	    l2 = CVMjvm2Long(&STACK_INFO(-2).raw);			\
	    if (test && CVMlongEqz(l2)) {				\
                CVM_JAVA_ERROR(ArithmeticException, "/ by long zero");\
	    }								\
	    l1 = CVMjvm2Long(&STACK_INFO(-4).raw);			\
	    r1 = CVMlong##opname(l1, l2);				\
	    CVMlong2Jvm(&STACK_INFO(-4).raw, r1);			\
	    TRACE(("\t%s => %s\n", CVMopnames[opcode],			\
		   GET_LONGCSTRING(STACK_LONG(-4))));		       	\
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -2);			\
	}

	OPC_INT_BINARY(add, Add, 0);
	OPC_INT_BINARY(sub, Sub, 0);
	OPC_INT_BINARY(mul, Mul, 0);
	OPC_INT_BINARY(and, And, 0);
	OPC_INT_BINARY(or,  Or,  0);
	OPC_INT_BINARY(xor, Xor, 0);
	OPC_INT_BINARY(div, Div, 1);
	OPC_INT_BINARY(rem, Rem, 1);


        /* Perform various binary floating number operations */
 
#undef  OPC_FLOAT_BINARY 
#define OPC_FLOAT_BINARY(opcname, opname)				   \
	CASE opc_d##opcname: {						   \
            CVMJavaDouble l1, l2, r;					   \
            l1 = CVMjvm2Double(&STACK_INFO(-4).raw);			   \
            l2 = CVMjvm2Double(&STACK_INFO(-2).raw);			   \
            r = CVMdouble##opname(l1, l2);				   \
            CVMdouble2Jvm(&STACK_INFO(-4).raw, r);			   \
            TRACE(("\t%s => %f\n", CVMopnames[opcode], STACK_DOUBLE(-4))); \
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -2);			   \
        }								   \
        CASE opc_f##opcname:						   \
            STACK_FLOAT(-2) =						   \
		CVMfloat##opname(STACK_FLOAT(-2), STACK_FLOAT(-1));	   \
            TRACE(("\t%s => %f\n", CVMopnames[opcode], STACK_FLOAT (-2))); \
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);

 
       OPC_FLOAT_BINARY(add, Add);
       OPC_FLOAT_BINARY(sub, Sub);
       OPC_FLOAT_BINARY(mul, Mul);
       OPC_FLOAT_BINARY(div, Div);
       OPC_FLOAT_BINARY(rem, Rem);

 	/* Shift operations				   
	 * Shift left int and long: ishl, lshl           
	 * Logical shift right int and long w/zero extension: iushr, lushr
	 * Arithmetic shift right int and long w/sign extension: ishr, lshr
	 */

#undef  OPC_SHIFT_BINARY
#define OPC_SHIFT_BINARY(opcname, opname)				\
        CASE opc_i##opcname:						\
	   STACK_INT(-2) = CVMint##opname(STACK_INT(-2), STACK_INT(-1));\
           TRACE(("\t%s => %d\n", CVMopnames[opcode], STACK_INT(-2)));	\
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);			\
        CASE opc_l##opcname:						\
	{								\
	   CVMJavaLong v, r;						\
	   v = CVMjvm2Long(&STACK_INFO(-3).raw);			\
	   r = CVMlong##opname(v, STACK_INT(-1));			\
	   CVMlong2Jvm(&STACK_INFO(-3).raw, r);				\
           TRACE(("\t%s => %s\n", CVMopnames[opcode],			\
		    GET_LONGCSTRING(STACK_LONG(-3)))); 			\
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);			\
	}
      
        OPC_SHIFT_BINARY(shl, Shl);
	OPC_SHIFT_BINARY(shr, Shr);
	OPC_SHIFT_BINARY(ushr, Ushr);

       /* Increment local variable by constant */ 
        CASE opc_iinc: 
	{
	    locals[pc[1]].j.i += (CVMInt8)(pc[2]);
	    TRACE(("\tiinc locals[%d]+%d => %d\n", pc[1], 
		   (CVMInt8)(pc[2]), locals[pc[1]].j.i)); 
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
	}

	/* Check if an object is an instance of given type. Throw an
	 * exception if not. */
        CASE opc_checkcast_quick:
	{
	    CVMObject* directObj;
	    directObj = STACK_OBJECT(-1);
	    cb = CVMcpGetCb(cp, GET_INDEX(pc+1));   /* cb of cast type */ 
	    if (!CVMgcUnsafeIsInstanceOf(ee, directObj, cb)) {
		cb = CVMobjectGetClass(directObj);
		CVM_JAVA_ERROR_WITH_CLASSNAME(ClassCastException, cb);
	    } 
	    TRACE(("\t%s %C\n", CVMopnames[opcode], cb));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
	}

	/* Check if an object is an instance of given type and store
	 * result on the stack. */
        CASE opc_instanceof_quick:
	{
	    CVMObject* directObj;
	    directObj = STACK_OBJECT(-1);
	    cb = CVMcpGetCb(cp, GET_INDEX(pc+1));  /* cb of casttype */ 
	    STACK_INT(-1) = (directObj != NULL) &&
		CVMgcUnsafeIsInstanceOf(ee, directObj, cb); 
	    TRACE(("\t%s %C => %d\n", CVMopnames[opcode], cb, STACK_INT(-1)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
 	}

       /* negate the value on the top of the stack */

        CASE opc_ineg:
	   STACK_INT(-1) = CVMintNeg(STACK_INT(-1));
           TRACE(("\t%s => %d\n", CVMopnames[opcode], STACK_INT(-1)));
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

        CASE opc_fneg:
	   STACK_FLOAT(-1) = CVMfloatNeg(STACK_FLOAT(-1));
           TRACE(("\t%s => %f\n", CVMopnames[opcode], STACK_FLOAT(-1)));
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

        CASE opc_lneg:
	{
 	   CVMJavaLong r;
	   r = CVMjvm2Long(&STACK_INFO(-2).raw);
	   r = CVMlongNeg(r);
	   CVMlong2Jvm(&STACK_INFO(-2).raw, r);
           TRACE(("\t%s => %s\n", CVMopnames[opcode],
		  GET_LONGCSTRING(STACK_LONG(-2))));
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);
	}

        CASE opc_dneg:
	{
 	   CVMJavaDouble r;
	   r = CVMjvm2Double(&STACK_INFO(-2).raw);
	   r = CVMdoubleNeg(r);
	   CVMdouble2Jvm(&STACK_INFO(-2).raw, r);
           TRACE(("\t%s => %f\n", CVMopnames[opcode], STACK_DOUBLE(-2)));
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);
	}

	/* Conversion operations */

        CASE opc_i2f:	/* convert top of stack int to float */
	   STACK_FLOAT(-1) = CVMint2Float(STACK_INT(-1));
           TRACE(("\ti2f => %f\n", STACK_FLOAT(-1)));
           UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

        CASE opc_i2l:	/* convert top of stack int to long */
	{
	    CVMJavaLong r;
	    r = CVMint2Long(STACK_INT(-1));
	    CVMlong2Jvm(&STACK_INFO(-1).raw, r);
            TRACE(("\ti2l => %s\n", GET_LONGCSTRING(STACK_LONG(-1))));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);
	}

        CASE opc_i2d:	/* convert top of stack int to double */
	{
	    CVMJavaDouble r;
	    r = CVMint2Double(STACK_INT(-1));
	    CVMdouble2Jvm(&STACK_INFO(-1).raw, r);
            TRACE(("\ti2d => %f\n", STACK_DOUBLE(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);
	}

        CASE opc_l2i:	/* convert top of stack long to int */
	{
   	    CVMJavaLong r; 
	    r = CVMjvm2Long(&STACK_INFO(-2).raw);
	    STACK_INT(-2) = CVMlong2Int(r); 
            TRACE(("\tl2i => %d\n", STACK_INT(-2)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	}
	
        CASE opc_l2f:   /* convert top of stack long to float */
	{
   	    CVMJavaLong r; 
	    r = CVMjvm2Long(&STACK_INFO(-2).raw);
	    STACK_FLOAT(-2) = CVMlong2Float(r); 
            TRACE(("\tl2f => %f\n", STACK_FLOAT(-2)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	}

        CASE opc_l2d:	/* convert top of stack long to double */
	{
   	    CVMJavaLong r1; 
	    CVMJavaDouble r2;
	    r1 = CVMjvm2Long(&STACK_INFO(-2).raw);
	    r2 = CVMlong2Double(r1); 
	    CVMdouble2Jvm(&STACK_INFO(-2).raw, r2);
            TRACE(("\tl2d => %f\n", STACK_DOUBLE(-2)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);
	}

        CASE opc_f2i:  /* Convert top of stack float to int */
            STACK_INT(-1) = CVMfloat2Int(STACK_FLOAT(-1)); 
            TRACE(("\tf2i => %d\n", STACK_INT(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

        CASE opc_f2l:  /* convert top of stack float to long */
	{
	    CVMJavaLong r;
 	    r = CVMfloat2Long(STACK_FLOAT(-1));
	    CVMlong2Jvm(&STACK_INFO(-1).raw, r);
            TRACE(("\tf2l => %s\n", GET_LONGCSTRING(STACK_LONG(-1))));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);
	}

        CASE opc_f2d:  /* convert top of stack float to double */
	{
	    CVMJavaDouble r;
	    r = CVMfloat2Double(STACK_FLOAT(-1));
	    CVMdouble2Jvm(&STACK_INFO(-1).raw, r);
            TRACE(("\tf2d => %f\n", STACK_DOUBLE(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 1);
        }

        CASE opc_d2i: /* convert top of stack double to int */
   	{
	    CVMJavaDouble r1;
	    CVMJavaInt r2;
	    r1 = CVMjvm2Double(&STACK_INFO(-2).raw);
	    r2 = CVMdouble2Int(r1);
	    STACK_INT(-2) = r2;
            TRACE(("\td2i => %d\n", STACK_INT(-2)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	}

        CASE opc_d2f: /* convert top of stack double to float */
   	{
	    CVMJavaDouble r;
	    r = CVMjvm2Double(&STACK_INFO(-2).raw);
	    STACK_FLOAT(-2) = CVMdouble2Float(r);
            TRACE(("\td2f => %f\n", STACK_FLOAT(-2)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	}

        CASE opc_d2l: /* convert top of stack double to long */
	{
	    CVMJavaDouble r1;
	    CVMJavaLong r2;
	    r1 = CVMjvm2Double(&STACK_INFO(-2).raw);
	    r2 = CVMdouble2Long(r1);
	    CVMlong2Jvm(&STACK_INFO(-2).raw, r2);
            TRACE(("\td2l => %s\n", GET_LONGCSTRING(STACK_LONG(-2))));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);
	}

        CASE opc_i2b:
	    STACK_INT(-1) = CVMint2Byte(STACK_INT(-1));
            TRACE(("\ti2b => %d\n", STACK_INT(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

        CASE opc_i2c:
	    STACK_INT(-1) = CVMint2Char(STACK_INT(-1));
            TRACE(("\ti2c => %d\n", STACK_INT(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

        CASE opc_i2s:
	    STACK_INT(-1) = CVMint2Short(STACK_INT(-1));
            TRACE(("\ti2s => %d\n", STACK_INT(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);

	/* comparison operators */

#define COMPARISON_OP(name, comparison)					     \
	CASE opc_if_icmp##name: {					     \
            int skip = (STACK_INT(-2) comparison STACK_INT(-1))		     \
	                ? CVMgetInt16(pc + 1) : 3;			     \
            TRACE(("\t%s goto +%d (%staken)\n",				     \
		   CVMopnames[opcode],					     \
		   CVMgetInt16(pc + 1),				    	     \
		   (STACK_INT(-2) comparison STACK_INT(-1)) ? "" : "not ")); \
            UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(skip, -2);   \
        }								     \
	CASE opc_if##name: {						     \
            int skip = (STACK_INT(-1) comparison 0)			     \
                        ? CVMgetInt16(pc + 1) : 3;			     \
            TRACE(("\t%s goto +%d (%staken)\n",				     \
		   CVMopnames[opcode],					     \
		   CVMgetInt16(pc + 1),				             \
		   (STACK_INT(-1) comparison 0) ? "" : "not "));	     \
            UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(skip, -1);   \
        }

#define COMPARISON_OP2(name, comparison)				\
        COMPARISON_OP(name, comparison)					\
	CASE opc_if_acmp##name: {					\
	    int skip = (STACK_OBJECT(-2) comparison STACK_OBJECT(-1))   \
			 ? CVMgetInt16(pc + 1) : 3;			\
            TRACE(("\t%s goto +%d (%staken)\n",				\
		   CVMopnames[opcode],					\
		   CVMgetInt16(pc + 1),					\
                   (STACK_OBJECT(-2) comparison STACK_OBJECT(-1))       \
		   ? "" : "not "));					\
            UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(skip, -2);\
        }

#define NULL_COMPARISON_OP(name, not)				\
	CASE opc_if##name: {					\
            int skip = (##not##(STACK_OBJECT(-1) == 0))		\
			? CVMgetInt16(pc + 1) : 3;		\
            TRACE(("\t%s goto +%d (%staken)\n",			\
		   CVMopnames[opcode],				\
		   CVMgetInt16(pc + 1),				\
		   (##not##(STACK_OBJECT(-1) == 0))		\
		   ? "" : "not "));				\
            UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(skip, -1);\
        }

	COMPARISON_OP(lt, <);
	COMPARISON_OP(gt, >);
	COMPARISON_OP(le, <=);
	COMPARISON_OP(ge, >=);
	COMPARISON_OP2(eq, ==);  /* include ref comparison */
        COMPARISON_OP2(ne, !=);  /* include ref comparison */
	NULL_COMPARISON_OP(null, );
	NULL_COMPARISON_OP(nonnull, !);

	/* Goto pc at specified offset in switch table. */

	CASE opc_tableswitch: {
	    CVMInt32* lpc  = (CVMInt32*)CVMalignWordUp(pc+1);
	    CVMInt32  key  = STACK_INT(-1);
	    CVMInt32  low  = CVMgetAlignedInt32(&lpc[1]);
	    CVMInt32  high = CVMgetAlignedInt32(&lpc[2]);
	    CVMInt32  skip;
	    TRACE(("\ttableswitch %d [%d-%d]\n", key, low, high));
	    key -= low;
	    skip = ((CVMUint32) key > high - low)
		        ? CVMgetAlignedInt32(&lpc[0])
		        : CVMgetAlignedInt32(&lpc[key + 3]);
	    UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(skip, -1);
	}

        /* Goto pc whose table entry matches specified key */

	CASE opc_lookupswitch: {
	    CVMInt32* lpc  = (CVMInt32*)CVMalignWordUp(pc+1);
	    CVMInt32  key  = STACK_INT(-1);
	    CVMInt32  skip = CVMgetAlignedInt32(lpc); /* default amount */
	    CVMInt32  npairs = CVMgetAlignedInt32(&lpc[1]);
	    TRACE(("\tlookupswitch %d\n", key));
	    while (--npairs >= 0) {
		lpc += 2;
		if (key == CVMgetAlignedInt32(lpc)) {
		    skip = CVMgetAlignedInt32(&lpc[1]);
		    break;
		}
	    }
	    UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(skip, -1);
	}

        CASE opc_fcmpl:
        CASE opc_fcmpg:
	{
	    STACK_INT(-2) = CVMfloatCompare(STACK_FLOAT(-2), STACK_FLOAT(-1), 
					    (opcode == opc_fcmpl ? -1 : 1));
            TRACE(("\t%s => %d\n", CVMopnames[opcode], STACK_INT(-2)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	}

        CASE opc_dcmpl:
        CASE opc_dcmpg:
	{
 	    CVMJavaDouble value1, value2; 
            value1 = CVMjvm2Double(&STACK_INFO(-4).raw);
            value2 = CVMjvm2Double(&STACK_INFO(-2).raw);
	    STACK_INT(-4) = CVMdoubleCompare(value1, value2,
					     (opcode == opc_dcmpl ? -1 : 1));
            TRACE(("\t%s => %d\n", CVMopnames[opcode], STACK_INT(-4)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -3);
	}

        CASE opc_lcmp:
	{
 	    CVMJavaLong value1, value2; 
            value1 = CVMjvm2Long(&STACK_INFO(-4).raw);
            value2 = CVMjvm2Long(&STACK_INFO(-2).raw);
	    STACK_INT(-4) = CVMlongCompare(value1, value2);
            TRACE(("\t%s => %d\n", CVMopnames[opcode], 
				   STACK_INT(-4)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -3);
	}

	/* invoke a static method */

	CASE opc_invokestatic_quick: {
	    mb = CVMcpGetMb(cp, GET_INDEX(pc + 1));
	    cb = CVMmbClassBlock(mb);
	    DECACHE_TOS(); /* must include arguments for proper gc */
	    topOfStack -= CVMmbArgsSize(mb);
	    receiverObjICell = CVMcbJavaInstance(cb);
	    goto callmethod;
	}
 
	/* invoke a romized static method that has a static initializer */

	CASE opc_invokestatic_checkinit_quick: {
	    mb = CVMcpGetMb(cp, GET_INDEX(pc + 1));
	    cb = CVMmbClassBlock(mb);
	    INIT_ROMCLASS_IF_NEEDED(ee, cb);
	    DECACHE_TOS(); /* must include arguments for proper gc */
	    topOfStack -= CVMmbArgsSize(mb);
	    receiverObjICell = CVMcbJavaInstance(cb);
	    goto callmethod;
	}
  
	/* invoke a virtual method */

	CASE opc_invokevirtual_quick:
	CASE opc_ainvokevirtual_quick:
	CASE opc_dinvokevirtual_quick:
	CASE opc_vinvokevirtual_quick: {
	    CVMObject* directObj;
	    DECACHE_TOS(); /* must include arguments for proper gc */
	    topOfStack -= pc[2];
	    receiverObjICell = &STACK_ICELL(0);
	    directObj = STACK_OBJECT(0);
	    CHECK_NULL(directObj);
	    mb = CVMcbMethodTableSlot(CVMobjectGetClass(directObj), pc[1]);
	    cb = CVMmbClassBlock(mb);
	    goto callmethod;
	}

	CASE opc_invokevirtual_quick_w: {
	    CVMObject* directObj;
	    mb = CVMcpGetMb(cp, GET_INDEX(pc + 1));
	    DECACHE_TOS(); /* must include arguments for proper gc */
	    topOfStack -= CVMmbArgsSize(mb);
	    receiverObjICell = &STACK_ICELL(0);
	    directObj = STACK_OBJECT(0);
	    CHECK_NULL(directObj);
	    /* JDK uses the equivalent of CVMobjMethodTableSlot, because
	     * in lossless mode you quicken into opc_invokevirtual_quick_w
	     * instead of opc_invokevirtualobject_quick. When we don't
	     * care about lossless mode, we use the faster
	     * CVMcbMethodTableSlot version.
	     */
#ifdef CVM_NO_LOSSY_OPCODES
	    mb = CVMobjMethodTableSlot(directObj, CVMmbMethodTableIndex(mb));
#else
	    mb = CVMcbMethodTableSlot(CVMobjectGetClass(directObj),
				      CVMmbMethodTableIndex(mb));
#endif /* CVM_NO_LOSSY_OPCODES */
	    cb = CVMmbClassBlock(mb);
	    goto callmethod;
	}
  
	/* invoke a virtual method of java/lang/Object */
	/* 
	 * NOTE: we can get rid of opc_invokevirtualobject_quick now because
	 * arrays have there own methodTablePtr (they just use Object's
	 * methodTablePtr), and CVMobjMethodTableSlot() has been optimized
	 * to no longer check if it's dealing with an array type, so we
	 * could just as well use opc_invokevirtual_quick.
	 */

	CASE opc_invokevirtualobject_quick: {
	    CVMObject* directObj;
	    DECACHE_TOS(); /* must include arguments for proper gc */
	    topOfStack -= pc[2];
	    receiverObjICell = &STACK_ICELL(0);
	    directObj = STACK_OBJECT(0);
	    CHECK_NULL(directObj);
	    mb = CVMobjMethodTableSlot(directObj, pc[1]);
	    cb = CVMmbClassBlock(mb);
	    goto callmethod;
	}

	/* invoke a nonvirtual method */

	CASE opc_invokenonvirtual_quick: {
	    CVMObject* directObj;
	    mb = CVMcpGetMb(cp, GET_INDEX(pc + 1));
	    cb = CVMmbClassBlock(mb);
	    DECACHE_TOS(); /* must include arguments for proper gc */
	    topOfStack -= CVMmbArgsSize(mb); // This is how we can get rid of transition frame -Kalpesh.
	    receiverObjICell = &STACK_ICELL(0);
	    directObj = STACK_OBJECT(0);
	    CHECK_NULL(directObj);
	    goto callmethod;
	}

	/* invoke a superclass method */

	CASE opc_invokesuper_quick: {
	    CVMObject* directObj;
	    cb = CVMmbClassBlock(frame->mb);
	    mb = CVMcbMethodTableSlot(CVMcbSuperclass(cb), GET_INDEX(pc + 1));
	    DECACHE_TOS(); /* must include arguments for proper gc */
	    topOfStack -= CVMmbArgsSize(mb);
	    receiverObjICell = &STACK_ICELL(0);
	    directObj = STACK_OBJECT(0);
	    CHECK_NULL(directObj);
	    cb = CVMmbClassBlock(mb);
	    goto callmethod;
	}

	/* ignore invocation. This is a result of inlining a method
	 * that does nothing.
	 */
#define CVM_INLINING
#ifdef CVM_INLINING
        CASE opc_invokeignored_quick: {
	    CVMObject* directObj;
	    TRACE(("\tinvokeignored_quick\n"));
	    topOfStack -= pc[1];
	    receiverObjICell = &STACK_ICELL(0);
	    directObj = STACK_OBJECT(0);
	    if (pc[2]) {
		CHECK_NULL(directObj);
	    }
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
	}
#endif

	/* inovke an interface method */

        CASE opc_invokeinterface_quick: {
	    CVMObject*      directObj;
	    CVMMethodBlock* imb;  /* interface mb for method we are calling */
	    CVMClassBlock*  icb;  /* interface cb for method we are calling */
	    CVMClassBlock*  ocb;  /* cb of the object */
	    CVMUint16 methodTableIndex;/* index into ocb's methodTable */
	    int guess = pc[4];

	    DECACHE_TOS(); /* must include arguments for proper gc */

	    imb = CVMcpGetMb(cp, GET_INDEX(pc + 1));
	    icb = CVMmbClassBlock(imb);

	    /* don't use pc[3] because transtion mb's don't set it up */
	    topOfStack -= CVMmbArgsSize(imb);

	    receiverObjICell = &STACK_ICELL(0);
	    directObj = STACK_OBJECT(0);
	    CHECK_NULL(directObj);
	    ocb = CVMobjectGetClass(directObj);

	    /*
	     * Search the objects interface table, looking for the entry
	     * whose interface cb matches the cb of the inteface we
	     * are invoking.
	     */
	    if (guess < 0 || guess >= CVMcbInterfaceCount(ocb)
		          || CVMcbInterfacecb(ocb, guess) != icb) {
		for (guess = CVMcbInterfaceCount(ocb) - 1; ; guess--) {
		    if (guess < 0) {
			CVMinvokeInterfaceError(ee, ocb, icb);
			DECACHE_PC();  /* required for handle_exception */
			goto handle_exception;
		    }
		    /* Update the guess if it was wrong, the code is not
		     * for a romized class, and the code is not transition
		     * code, which is also in ROM.
		     */
		    if (CVMcbInterfacecb(ocb, guess) == icb) {
		        if (!CVMisROMPureCode(CVMmbClassBlock(frame->mb)) && 
			    !CVMframeIsTransition(frame))
			    pc[4] = guess;
			break;
		    }
		}
	    }

	    /*
	     * We know CVMcbInterfacecb(ocb, guess) is for the interface that 
	     * we are invoking. Use it to convert the index of the method in 
	     * the interface's methods array to the index of the method 
	     * in the class' methodTable array.
	     */
	    methodTableIndex = CVMcbInterfaceMethodTableIndex(
                ocb, guess, CVMmbMethodSlotIndex(imb));

	    /*
	     * Fetch the proper mb and it's cb.
	     */
	    mb = CVMcbMethodTableSlot(ocb, methodTableIndex);
	    cb = CVMmbClassBlock(mb);

	    goto callmethod;
	}

        /* Return from a method */

	CASE opc_areturn:
	CASE opc_ireturn:
	CASE opc_freturn:
        {
	    CVMJavaVal32 result;
	    /* 
	     * Offer a GC-safe point. The current frame is always up
	     * to date, so don't worry about de-caching frame.  
	     */
	    CVMD_gcSafeCheckPoint(ee, {
		DECACHE_PC();    /* only needed if we gc */
		DECACHE_TOS();   /* only needed if we gc */
	    }, {});
	    result = STACK_INFO(-1);
	    CACHE_PREV_TOS();
	    STACK_INFO(0) = result;
	    topOfStack++;
	    TRACEIF(opc_ireturn, ("\tireturn %d\n", STACK_INT(-1)));
	    TRACEIF(opc_areturn, ("\tareturn %p\n", STACK_OBJECT(-1)));
	    TRACEIF(opc_freturn, ("\tfreturn %f\n", STACK_FLOAT(-1)));
	    goto handle_return;
        }

	CASE opc_lreturn:
	CASE opc_dreturn:
        {
	    CVMJavaVal64 result;
	    /* 
	     * Offer a GC-safe point. The current frame is always up
	     * to date, so don't worry about de-caching frame.  
	     */
	    CVMD_gcSafeCheckPoint(ee, {
		DECACHE_PC();    /* only needed if we gc */
		DECACHE_TOS();   /* only needed if we gc */
	    }, {});
	    CVMmemCopy64(result.v, &STACK_INFO(-2).raw);
	    CACHE_PREV_TOS();
	    CVMmemCopy64(&STACK_INFO(0).raw, result.v);
	    topOfStack += 2;
	    TRACEIF(opc_dreturn, ("\tfreturn1 %f\n", STACK_DOUBLE(-2)));
	    TRACEIF(opc_lreturn, ("\tlreturn %s\n",
				  GET_LONGCSTRING(STACK_LONG(-2))));
	    goto handle_return;
        }

	CASE opc_return: {
	    /* 
	     * Offer a GC-safe point. The current frame is always up
	     * to date, so don't worry about de-caching frame.  
	     */
	    CVMD_gcSafeCheckPoint(ee, {
		DECACHE_PC();    /* only needed if we gc */
		DECACHE_TOS();   /* only needed if we gc */
	    }, {});
	    CACHE_PREV_TOS();
	    TRACE(("\treturn\n"));
	    goto handle_return;
	}

	CASE opc_exittransition: {
	    TRACE(("\texittransition\n"));
	    if (frame == initialframe) {
		/* We get here when we've reached the transition frame that
		 * was used to enter CVMgcUnsafeExecuteJavaMethod. All we
		 * to do is exit. The caller is reponsible for cleaning
		 * up the transiton frame.
		 */
		goto finish;
	    } else {
		/* We get here when we've reached a transition frame
		 * that was setup by the init_class: label to invoke
		 * Class.runStaticInitializers, or by an CNI method that
		 * returned CNI_NEW_TRANSITION_FRAME. Method.invoke[BCI...]
		 * is currently the only method that causes this type of
		 * transition frame to be setup. See cjp07/13/99 to see why
		 * CNI methods may want to setup a transtion frame and
		 * return CNI_NEW_TRANSITION_FRAME.
		 */
		char 		retType;
		CVMBool         incrementPC;

		TRACE_METHOD_RETURN(frame);

		/* There should never be an outstanding local
		 * exception because handle_exception skips transition
		 * frames (except for the initialframe). (Note
		 * when callmethod is done it goes to
		 * check_for_exception after invoking JNI and CVM
		 * methods. Java methods, of course, can only raise
		 * exceptions using athrow.)
		 */
		CVMassert(!CVMlocalExceptionOccurred(ee));

		/* See whether there was a return value from the
		 * invoked method. If there was, copy it onto the
		 * caller's frame. This makes it possible to use
		 * transition frames to implement the CVM methods
		 * called by Method.invoke(); the return value from
		 * the invoked method will be copied onto the "real"
		 * invoker's stack, which in this case is the
		 * Java-based Method.invoke() implementation.
		 */
		retType = 
		    CVMtypeidGetReturnType(CVMmbNameAndTypeID(frame->mb));

		/* There are only two cases we need to check: void and
		 * long/double. Every other return type is 32 bit. */
		if (retType == CVM_TYPEID_VOID) {
		    CACHE_PREV_TOS();
		} else if ((retType == CVM_TYPEID_LONG) ||
			   (retType == CVM_TYPEID_DOUBLE)) {
		    /* NOTE where topOfStack points and where each
		     * frame->topOfStack points. topOfStack is cached and
		     * points just above the return argument. 
		     * frame->topOfStack in the transition frame points just
		     * BELOW the return argument.
		     * CVMframePrev(frame)->topOfStack points just BELOW the
		     * arguments to whatever (CNI) method caused this 
		     * transition frame to be pushed.
		     */
		    CVMJavaVal64 result;
		    CVMmemCopy64(result.v, &STACK_INFO(-2).raw);
		    CACHE_PREV_TOS();
		    CVMmemCopy64(&STACK_INFO(0).raw, result.v);
		    topOfStack += 2;
		} else {
		    CVMJavaVal32 result = STACK_INFO(-1);
		    CACHE_PREV_TOS();
		    STACK_INFO(0) = result;
		    topOfStack++;
		}

		/* Check if we will need to increment the pc after popping
		 * the transition frame. We need to do this before
		 * the frame is popped and we lose the transition frame.
		 */
		incrementPC = CVMframeIncrementPcFlag(frame);

		CVMpopFrame(stack, frame);   /* pop the transition frame */
		CACHE_PC();                  /* fetch pc from new frame */
		locals = CVMframeLocals(frame);
		cp = CVMframeCp(frame);
		if (incrementPC) {
		    pc += (*pc == opc_invokeinterface_quick ? 5 : 3);
		    CONTINUE;
		} else {
		    opcode = *pc;
#ifdef CVM_JVMDI
		    /* Prevent double breakpoints. If the opcode we are
		     * returning to is an opc_breakpoint, the we've
		     * already notified the debugger of the breakpoint
		     * once. We don't want to do this again.
		     */
		    if (opcode == opc_breakpoint) {
			DECACHE_PC();
			DECACHE_TOS();
			CVMD_gcSafeExec(ee, {
			    opcode = CVMjvmdiGetBreakpointOpcode(ee, pc,
								 CVM_FALSE);
			});
		    }
#endif
		    goto opcode_switch;
		}
	    }
	}

	/* Quick static field access. All of these opcodes are lossy, so
	 * they aren't used when in lossless mode.
	 */

#ifndef CVM_NO_LOSSY_OPCODES
	CASE opc_agetstatic_quick:
	CASE opc_getstatic_quick: {
	    STACK_INFO(0) = *CVMcpGetStatic(cp, GET_INDEX(pc + 1));
	    TRACE(("\t%s *0x%p(0x%X) ==>\n", CVMopnames[opcode],
		   CVMcpGetStatic(cp, GET_INDEX(pc + 1)),
		   STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);
	}

	CASE opc_getstatic2_quick: {
	    CVMmemCopy64(&STACK_INFO(0).raw,
			 &CVMcpGetStatic(cp, GET_INDEX(pc + 1))->raw);
	    TRACE(("\t%s *0x%p(0x%X,0x%X) ==>\n", CVMopnames[opcode],
		   CVMcpGetStatic(cp, GET_INDEX(pc + 1)),
		   STACK_INT(0), STACK_INT(1)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 2);
	}

	CASE opc_aputstatic_quick:
	CASE opc_putstatic_quick: {
	    *CVMcpGetStatic(cp, GET_INDEX(pc + 1)) = STACK_INFO(-1);
	    TRACE(("\t%s 0x%X ==> 0x%p\n", CVMopnames[opcode],
		   STACK_INT(-1), CVMcpGetStatic(cp, GET_INDEX(pc + 1))));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, -1);
	}

	CASE opc_putstatic2_quick: {
	    CVMmemCopy64(&CVMcpGetStatic(cp, GET_INDEX(pc + 1))->raw,
			 &STACK_INFO(-2).raw);
	    TRACE(("\t%s (0x%X,0x%X) ==> 0x%p\n", CVMopnames[opcode],
		   STACK_INT(-2), STACK_INT(-1),
		   CVMcpGetStatic(cp, GET_INDEX(pc + 1))));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, -2);
	}
#endif

	/* Quick static field access for romized classes with initializers.
	 * Also used for all static field access when in lossless mode
	 */

	CASE opc_agetstatic_checkinit_quick:
	CASE opc_getstatic_checkinit_quick: {
	    CVMFieldBlock* fb = CVMcpGetFb(cp, GET_INDEX(pc+1));
	    cb = CVMfbClassBlock(fb);
	    INIT_ROMCLASS_IF_NEEDED(ee, cb);
#ifdef CVM_JVMDI
	    if (CVMglobals.jvmdiWatchingFieldAccess) {
		DECACHE_PC();
		DECACHE_TOS();

		CVMD_gcSafeExec(ee, {
		    CVMjvmdiNotifyDebuggerOfFieldAccess(ee, NULL, fb);
		});
	    }
#endif
	    STACK_INFO(0) = CVMfbStaticField(fb);
	    TRACE(("\t%s #%d %C.%F[%d] (0x%X) ==>\n",
		   CVMopnames[opcode], GET_INDEX(pc+1), cb, fb, 
		   CVMfbOffset(fb), STACK_INT(0)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);
	}

	CASE opc_getstatic2_checkinit_quick: {
	    CVMFieldBlock* fb = CVMcpGetFb(cp, GET_INDEX(pc+1));
	    cb = CVMfbClassBlock(fb);
	    INIT_ROMCLASS_IF_NEEDED(ee, cb);
#ifdef CVM_JVMDI
	    if (CVMglobals.jvmdiWatchingFieldAccess) {
		DECACHE_PC();
		DECACHE_TOS();
		
		CVMD_gcSafeExec(ee, {
		    CVMjvmdiNotifyDebuggerOfFieldAccess(ee, NULL, fb);
		});
	    }
#endif
	    CVMmemCopy64(&STACK_INFO(0).raw, &CVMfbStaticField(fb).raw);
	    TRACE(("\t%s #%d %C.%F[%d] ((0x%X,0x%X) ==>\n",
		   CVMopnames[opcode], GET_INDEX(pc+1), cb, fb,
		   CVMfbOffset(fb), STACK_INT(0), STACK_INT(1)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 2);
	}

	CASE opc_aputstatic_checkinit_quick:
	CASE opc_putstatic_checkinit_quick: {
	    CVMFieldBlock* fb = CVMcpGetFb(cp, GET_INDEX(pc+1));
	    cb = CVMfbClassBlock(fb);
	    INIT_ROMCLASS_IF_NEEDED(ee, cb);
#ifdef CVM_JVMDI
	    if (CVMglobals.jvmdiWatchingFieldModification) {
		DECACHE_PC();
		DECACHE_TOS();

		if (CVMtypeidFieldIsRef(CVMfbNameAndTypeID(fb))) {
		    jvalue val;
		    val.l = &STACK_ICELL(-1);
		    CVMD_gcSafeExec(ee, {
			CVMjvmdiNotifyDebuggerOfFieldModification(ee, NULL,
								  fb, val);
		    });
		} else {
		    jvalue val;
		    
		    /* Fetch 32 bits from stack (as integer; doesn't matter) */
		    val.i = STACK_INT(-1);
		    CVMD_gcSafeExec(ee, {
			CVMjvmdiNotifyDebuggerOfFieldModification(
			    ee, NULL, fb, val
			);
		    });
		}
	    }
#endif
	    CVMfbStaticField(fb) = STACK_INFO(-1);
	    TRACE(("\t%s #%d (0x%X) ==> %C.%F[%d]\n",
		   CVMopnames[opcode], GET_INDEX(pc+1), STACK_INT(-1), 
		   cb, fb, CVMfbOffset(fb)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, -1);
	}

	CASE opc_putstatic2_checkinit_quick: {
	    CVMFieldBlock* fb = CVMcpGetFb(cp, GET_INDEX(pc+1));
	    cb = CVMfbClassBlock(fb);
	    INIT_ROMCLASS_IF_NEEDED(ee, cb);
#ifdef CVM_JVMDI
	    if (CVMglobals.jvmdiWatchingFieldModification) {
		jvalue val;
		DECACHE_PC();
		DECACHE_TOS();
		CVMassert(CVMfbIsDoubleWord(fb));
		if (CVMtypeidGetType(CVMfbNameAndTypeID(fb))
		    == CVM_TYPEID_LONG)
		{
		    val.j = CVMjvm2Long(&STACK_INFO(-2).raw);
		} else /* CVM_TYPEID_DOUBLE */ {
		    val.d = CVMjvm2Double(&STACK_INFO(-2).raw);
		}

		CVMD_gcSafeExec(ee, {
		    CVMjvmdiNotifyDebuggerOfFieldModification(ee, NULL,
							      fb, val);
		});
	    }
#endif
	    CVMmemCopy64(&CVMfbStaticField(fb).raw, &STACK_INFO(-2).raw);
	    TRACE(("\t%s #%d (0x%X,0x%X) ==> %C.%F[%d]\n",
		   CVMopnames[opcode], GET_INDEX(pc+1), 
		   STACK_INT(-2), STACK_INT(-1),
		   cb, fb, CVMfbOffset(fb)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, -2);
	}

	/* Quick object field access byte-codes. The non-wide version are
	 * all lossy and are not needed when in losslessmode.
	 */

#ifndef CVM_NO_LOSSY_OPCODES
        CASE opc_getfield_quick:
	{
	    CVMObject* directObj = STACK_OBJECT(-1);
	    CHECK_NULL(directObj);
	    CVMD_fieldRead32(directObj, pc[1], STACK_INFO(-1));
	    TRACE(("\t%s %O[%d](0x%X) ==>\n", CVMopnames[opcode],
		   directObj, pc[1], STACK_INT(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
        }

        CASE opc_putfield_quick:
	{
	    CVMObject* directObj = STACK_OBJECT(-2);
	    CHECK_NULL(directObj);
	    CVMD_fieldWrite32(directObj, pc[1], STACK_INFO(-1));
	    TRACE(("\t%s (0x%X) ==> %O[%d]\n", CVMopnames[opcode],
		   STACK_INT(-1), directObj, pc[1]));
            UPDATE_PC_AND_TOS_AND_CONTINUE(3, -2);
        }

        CASE opc_getfield2_quick:
	{
	    CVMObject* directObj = STACK_OBJECT(-1);
	    CHECK_NULL(directObj);
	    CVMD_fieldRead64(directObj, pc[1], &STACK_INFO(-1));
	    TRACE(("\t%s %O[%d](0x%X, 0x%X) ==>\n", CVMopnames[opcode],
		   directObj, pc[1], STACK_INT(-1), STACK_INT(0)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);
        }

        CASE opc_putfield2_quick:
	{
	    CVMObject* directObj = STACK_OBJECT(-3);
	    CHECK_NULL(directObj);
	    CVMD_fieldWrite64(directObj, pc[1], &STACK_INFO(-2));
	    TRACE(("\t%s (0x%X, 0x%X) ==> %O[%d]\n", CVMopnames[opcode],
		   STACK_INT(-1), STACK_INT(0), directObj, pc[1]));
            UPDATE_PC_AND_TOS_AND_CONTINUE(3, -3);
        }

	CASE opc_agetfield_quick:
	{
	    CVMObject* fieldObj;
	    CVMObject* directObj = STACK_OBJECT(-1);
	    CHECK_NULL(directObj);
	    CVMD_fieldReadRef(directObj, pc[1], fieldObj);
	    CVMID_icellSetDirect(ee, &STACK_ICELL(-1), fieldObj); 
	    TRACE(("\t%s %O[%d](0x%p) ==>\n", CVMopnames[opcode],
		   directObj, pc[1], STACK_OBJECT(-1)));
            UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
        }

	CASE opc_aputfield_quick:
	{
	    CVMObject* directObj = STACK_OBJECT(-2);
	    CHECK_NULL(directObj);
        
        /* *** RTSJ *** */
        /* Reference checking support.
         * Check if reference assignment is allowed.
         */
        {
            extern int JVMCeiInitialized;
            if(JVMCeiInitialized)
            {
                // CVMJTrcAllowReferenceAsignment might go into gc-safe 
                // state so uncache our local state.
	            DECACHE_PC();
        	    DECACHE_TOS();
                if(CVMJTrcAllowReferenceAssignment(ee, directObj, STACK_OBJECT(-1)) == CVM_FALSE)
                {
#ifdef CVM_DEBUG
                    CVMconsolePrintf("RTSJ: (putfield) Got a CVM_FALSE for reference checking\n");
//	                CVMconsolePrintf("RTSJ: \t%s, (0x%X) ==> %O[%d]\n", CVMopnames[opcode],
//           		   STACK_INT(-1), directObj, pc[1]);
                    //CVMconsolePrintf("RTSJ: (putfield) Throwing an IllegalAssignmentError. src object = %O dest object = %O\n", STACK_OBJECT(-1), directObj);
#endif
                    CVM_JAVA_ERROR(IllegalAssignmentError, "Illegal reference assignment"); 
                }
            }
        }
        /* *** JSTR *** */
	    
        CVMD_fieldWriteRef(directObj, pc[1], STACK_OBJECT(-1));
	    TRACE(("\t%s (0x%p) ==> %O[%d]\n", CVMopnames[opcode],
		   STACK_OBJECT(-1), directObj, pc[1]));
            UPDATE_PC_AND_TOS_AND_CONTINUE(3, -2);
        }
#endif /* CVM_NO_LOSSY_OPCODES */

        CASE opc_getfield_quick_w:
	{
	    CVMFieldBlock* fb;
	    CVMObject* directObj = STACK_OBJECT(-1);
	    CHECK_NULL(directObj);
	    fb = CVMcpGetFb(cp, GET_INDEX(pc+1));
#ifdef CVM_JVMDI
	    if (CVMglobals.jvmdiWatchingFieldAccess) {
		DECACHE_PC();
		DECACHE_TOS();
		CVMD_gcSafeExec(ee, {
		    CVMjvmdiNotifyDebuggerOfFieldAccess(ee,
							&STACK_ICELL(-1),
							fb);
		});
	    }
#endif
	    if (CVMfbIsDoubleWord(fb)) {
		CVMD_fieldRead64(directObj, CVMfbOffset(fb), &STACK_INFO(-1));
		TRACE(("\t%s %O[%d](0x%X, 0x%X) ==>\n", CVMopnames[opcode],
		       directObj, GET_INDEX(pc+1), 
		       STACK_INT(-1), STACK_INT(0)));
		UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);
	    } else {
		if (CVMfbIsRef(fb)) {
		    CVMD_fieldReadRef(directObj, CVMfbOffset(fb),
				     STACK_OBJECT(-1));
		} else {
		    CVMD_fieldRead32(directObj, CVMfbOffset(fb),
				     STACK_INFO(-1));
		}
		TRACE(("\t%s %O[%d](0x%X) ==>\n", CVMopnames[opcode],
		       directObj, GET_INDEX(pc+1), STACK_INT(-1)));
		UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
	    }
        }

        CASE opc_putfield_quick_w:
	{
	    CVMObject* directObj;
	    CVMFieldBlock* fb;
	    fb = CVMcpGetFb(cp, GET_INDEX(pc+1));
	    if (CVMfbIsDoubleWord(fb)) {
		directObj = STACK_OBJECT(-3);
		CHECK_NULL(directObj);
#ifdef CVM_JVMDI
		if (CVMglobals.jvmdiWatchingFieldModification) {
		    jvalue val;
		    DECACHE_PC();
		    DECACHE_TOS();

		    if (CVMtypeidGetType(CVMfbNameAndTypeID(fb))
			== CVM_TYPEID_LONG)
		    {
			val.j = CVMjvm2Long(&STACK_INFO(-2).raw);
		    } else /* CVM_TYPEID_DOUBLE */ {
			val.d = CVMjvm2Double(&STACK_INFO(-2).raw);
		    }

		    CVMD_gcSafeExec(ee, {
			CVMjvmdiNotifyDebuggerOfFieldModification(
			    ee, &STACK_ICELL(-3), fb, val
			);
		    });
		}
#endif
		CVMD_fieldWrite64(directObj, CVMfbOffset(fb), &STACK_INFO(-2));
		TRACE(("\t%s (0x%X, 0x%X) ==> %O[%d]\n", CVMopnames[opcode],
		       STACK_INT(-1), STACK_INT(0),
		       directObj, GET_INDEX(pc+1)));
		UPDATE_PC_AND_TOS_AND_CONTINUE(3, -3);
	    } else {
		directObj = STACK_OBJECT(-2);	
		CHECK_NULL(directObj);
		if (CVMfbIsRef(fb)) {
#ifdef CVM_JVMDI
		    if (CVMglobals.jvmdiWatchingFieldModification) {
			jvalue val;

			DECACHE_PC();
			DECACHE_TOS();
			val.l = &STACK_ICELL(-1);
			CVMD_gcSafeExec(ee, {
			    CVMjvmdiNotifyDebuggerOfFieldModification(
				ee, &STACK_ICELL(-2), fb, val
			    );
			});
		    }
#endif

		    CVMD_fieldWriteRef(directObj, CVMfbOffset(fb),
				       STACK_OBJECT(-1));
		} else {
#ifdef CVM_JVMDI
		    if (CVMglobals.jvmdiWatchingFieldModification) {
			jvalue val;

			DECACHE_PC();
			DECACHE_TOS();
			/* Fetch 32 bits from stack (as integer;
			   doesn't matter) */
			val.i = STACK_INT(-1);
			CVMD_gcSafeExec(ee, {
			    CVMjvmdiNotifyDebuggerOfFieldModification(
				ee, &STACK_ICELL(-2), fb, val
			    );
			});
		    }
#endif

		    CVMD_fieldWrite32(directObj, CVMfbOffset(fb),
				      STACK_INFO(-1));
		}
		TRACE(("\t%s (0x%X) ==> %O[%d]\n", CVMopnames[opcode],
		       STACK_INT(-1), directObj, GET_INDEX(pc+1)));
		UPDATE_PC_AND_TOS_AND_CONTINUE(3, -2);
	    }
        }

	/* Array access byte-codes */

	/* Every array access byte-code starts out like this */
#define ARRAY_INTRO(T, arrayOff)				      	      \
        CVMArrayOf##T* arrObj = (CVMArrayOf##T*)STACK_OBJECT(arrayOff);       \
	CVMJavaInt     index  = STACK_INT(arrayOff + 1);		      \
        CHECK_NULL(arrObj);						      \
        /* This clever idea is from ExactVM. No need to test index >= 0.      \
	   Instead, do an unsigned comparison. Negative indices will appear   \
	   as positive numbers >= 2^31, which is just out of the range	      \
	   of legal Java array indices */				      \
	if ((CVMUint32)index >= CVMD_arrayGetLength(arrObj)) {		      \
	    CVM_JAVA_ERROR(ArrayIndexOutOfBoundsException, "");	      \
	}

	/* 32-bit loads. These handle conversion from < 32-bit types */
#define ARRAY_LOADTO32(T, format, stackRes)				      \
        {								      \
	    ARRAY_INTRO(T, -2);						      \
	    CVMD_arrayRead##T(arrObj, index, stackRes(-2));		      \
	    TRACE(("\t%s %O[%d](" format ") ==>\n", CVMopnames[opcode],	      \
		   arrObj, index, stackRes(-1))); 			      \
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);			      \
        }

	/* 64-bit loads */
#define ARRAY_LOADTO64(T,T2)						\
        {								\
	    CVMJava##T temp64;						\
            ARRAY_INTRO(T, -2);						\
	    CVMD_arrayRead##T(arrObj, index, temp64);			\
	    CVM##T2##2Jvm(&STACK_INFO(-2).raw, temp64);			\
	    TRACE(("\t%s %O[%d](0x%X, 0x%X) ==>\n", CVMopnames[opcode],	\
		   arrObj, index, STACK_INT(-2), STACK_INT(-1)));      	\
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);			\
        }

        CASE opc_iaload:
	    ARRAY_LOADTO32(Int,   "%d",   STACK_INT);
        CASE opc_faload:
	    ARRAY_LOADTO32(Float, "%f",   STACK_FLOAT);
        CASE opc_aaload:
       /* *** RTSJ *** */ 
        /*   Reference checking support.   
           Check if local variable assignment is allowed. */ 
#if 0
        { 
            extern int JVMCeiInitialized; 
            if(JVMCeiInitialized) 
            { 
				if(CVMisNoHeapRealtimeThread(ee) == CVM_TRUE) 
				{
                    DECACHE_PC();
                    DECACHE_TOS();
                	if(CVMJTisHeapReference(ee, STACK_OBJECT(-2)) == CVM_TRUE) 
                	{ 
#ifdef CVM_DEBUG
                    	CVMconsolePrintf("RTSJ: (aaload) Throwing a MemoryAccessError. memory area = %O stack object = %O\n", CVMID_icellDirect(ee, CVMmemoryAreaICell(ee)), STACK_OBJECT(-2)); 
#endif
                    	CVM_JAVA_ERROR(MemoryAccessError, "Illegal array element load"); 
                	}
				}	
            } 
        } 
#endif
        /* *** JSTR *** */ 
	    ARRAY_LOADTO32(Ref,   "0x%p", STACK_OBJECT);
        CASE opc_baload:
	    ARRAY_LOADTO32(Byte,  "%d",   STACK_INT);
        CASE opc_caload:
	    ARRAY_LOADTO32(Char,  "%d",   STACK_INT);
        CASE opc_saload:
	    ARRAY_LOADTO32(Short, "%d",   STACK_INT);
        CASE opc_laload:
	    ARRAY_LOADTO64(Long,long);
        CASE opc_daload:
	    ARRAY_LOADTO64(Double,double);

	/* 32-bit stores. These handle conversion to < 32-bit types */
#define ARRAY_STOREFROM32(T, format, stackSrc)				      \
        {								      \
	    ARRAY_INTRO(T, -3);						      \
	    CVMD_arrayWrite##T(arrObj, index, stackSrc(-1));		      \
	    TRACE(("\t%s " format " ==> %O[%d] ==>\n", CVMopnames[opcode],    \
		   stackSrc(-1), arrObj, index)); 			      \
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -3);			      \
        }

	/* 64-bit stores */
#define ARRAY_STOREFROM64(T)						    \
        {								    \
	    CVMJava##T temp64;						    \
            ARRAY_INTRO(T, -4);						    \
	    temp64 = CVMjvm2##T(&STACK_INFO(-2).raw);			    \
	    CVMD_arrayWrite##T(arrObj, index, temp64);			    \
	    TRACE(("\t%s (0x%X, 0x%X) ==> %O[%d]\n", CVMopnames[opcode],    \
		   STACK_INT(-2), STACK_INT(-1), arrObj, index));      	    \
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -4);			    \
        }

        CASE opc_iastore:
	    ARRAY_STOREFROM32(Int,   "%d",   STACK_INT);
        CASE opc_fastore:
	    ARRAY_STOREFROM32(Float, "%f",   STACK_FLOAT);
	/*
	 * This one looks different because of the assignability check
	 */
        CASE opc_aastore: {
	    CVMObject* rhsObject = STACK_OBJECT(-1);
	    ARRAY_INTRO(Ref, -3);
	    if (rhsObject != 0) {
		/* Check assignability of rhsObject into arrObj */
		CVMClassBlock* elemType =
		    CVMarrayElementCb(CVMobjectGetClass(arrObj));
		CVMClassBlock* rhsType = CVMobjectGetClass(rhsObject);
		if (rhsType != elemType) { /* Try a quick check first */
		    if (!CVMisAssignable(ee, rhsType, elemType)) {
			if (!CVMlocalExceptionOccurred(ee)) {
			    CVM_JAVA_ERROR(ArrayStoreException, 0);
			}
			else {
			    /* When C stack overflow is thrown */
			    goto handle_exception;
			}
		    }
		}
	    }
        /* *** RTSJ *** */
        /* Reference checking support.
         * Check if reference assignment is allowed.
         */
#if 0
        {
            extern int JVMCeiInitialized;
            if(JVMCeiInitialized)
            {
                // Since an array type can be casted to CVMObject, the 
                // casting for parameter 2 below is valid.
                // i.e. CVMArrayOfAnyType* can be casted to CVMObject*.
                // 
                // CVMJTrcAllowReferenceAsignment might go into gc-safe 
                // state so uncache our local state.
	            DECACHE_PC();
        	    DECACHE_TOS();
                if(CVMJTrcAllowReferenceAssignment(ee, (CVMObject*)arrObj, rhsObject) == CVM_FALSE)
                {
#ifdef CVM_DEBUG
                    CVMconsolePrintf("RTSJ: (aastore) Got a CVM_FALSE for reference checking\n");
#endif
                    CVM_JAVA_ERROR(IllegalAssignmentError, "Illegal reference assignment to an array element"); 
                }
            }
        }
#endif
        /* *** JSTR *** */
	    CVMD_arrayWriteRef(arrObj, index, rhsObject);
	    TRACE(("\t%s 0x%p ==> %O[%d] ==>\n",
		   CVMopnames[opcode], rhsObject, arrObj, index));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -3);
	}
        CASE opc_bastore:
	    ARRAY_STOREFROM32(Byte,  "%d",   STACK_INT);
        CASE opc_castore:
	    ARRAY_STOREFROM32(Char,  "%d",   STACK_INT);
        CASE opc_sastore:
	    ARRAY_STOREFROM32(Short, "%d",   STACK_INT);
        CASE opc_lastore:
	    ARRAY_STOREFROM64(Long);
        CASE opc_dastore:
	    ARRAY_STOREFROM64(Double);

        CASE opc_arraylength:
	{
	    CVMArrayOfAnyType* arrObj = (CVMArrayOfAnyType*)STACK_OBJECT(-1);
	    CHECK_NULL(arrObj);
	    STACK_INT(-1) = CVMD_arrayGetLength(arrObj);
	    TRACE(("\t%s %O.length(%d) ==>\n", CVMopnames[opcode],
		   arrObj, STACK_INT(-1)));
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);
	}

	/* monitorenter and monitorexit for locking/unlocking an object -Kalpesh*/

        CASE opc_monitorenter: {
	    {
		CVMObject* directObj = STACK_OBJECT(-1);
		CHECK_NULL(directObj);
		TRACE(("\topc_monitorenter(%O)\n", directObj));
                /* %comment lamm002 */
	   
		if (CVMobjectTryLock(ee, directObj)) {
			
		    /* Non blocking attempt,..so sequence doesn't  matter much..
		     * -Kalpesh.
		     */

		    //set flag to sync_deferred and increament counter//
		    ATC_MONITOR_ENTER(frame);
		    
		    UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
		    
		}
	    }
	    /* non-blocking attempt failed... */

	    DECACHE_PC();
	    DECACHE_TOS();
	    if (!CVMobjectLock(ee, &STACK_ICELL(-1))) {
	      /* Dio:
	       * we will throw an exception only if 
	       * no previous exception occured, i.e., ResourceLimitError
	       */
	      if (!CVMexceptionOccurred(ee))
		CVM_JAVA_ERROR(OutOfMemoryError, NULL)
	      else
		CVM_JAVA_ERROR_PASS
	    }

	    /* Here we are setting Deferred state b'coz 
	     * if thread waits to aquire lock on object still
	     * its not in synchronized block, hense waiting thread is
	     * not in Deferred state yet if it was in Non-deferred state 
	     * previously.
	     *
	     * -Kalpesh.
	     */

	    //set flag to sync_deferred and increament counter//
	    ATC_MONITOR_ENTER(frame);
	    
            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	}

        CASE opc_monitorexit: {
	    CVMObject* directObj = STACK_OBJECT(-1);
	    CHECK_NULL(directObj);
            TRACE(("\topc_monitorexit(%O)\n", directObj));
           /* %comment lamm003 */
	    if (CVMobjectTryUnlock(ee, directObj)) {
		    
	      //decreament counter, if counter is zero then replace with old state -Kalpesh//
	      ATC_MONITOR_EXIT(frame);
	      
	      UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	    }
	    /* non-blocking attempt failed... */
	    DECACHE_PC();
	    DECACHE_TOS();
	    if (!CVMobjectUnlock(ee, &STACK_ICELL(-1))) {
	      CVMdebugPrintf(("THROWING EXCEPTION FROM MONITOREXIT OPCODE\n"));
	      CVMdebugPrintf(("Method Name = %M, pc offset = %d\n",
		      mb, (CVMUint16)(pc - CVMjmdCode(CVMmbJmd(mb)) )));
		CVM_JAVA_ERROR(IllegalMonitorStateException,
		    "current thread not owner");
	    }
	    
	    //decreament counter, if counter is zero then replace with old state//
	    ATC_MONITOR_EXIT(frame);

            UPDATE_PC_AND_TOS_AND_CONTINUE(1, -1);
	}

	/* All of the non-quick opcodes. */

#ifdef CVM_CLASSLOADING
	/* -Set clobbersCpIndex true if the quickened opcode clobbers the
	 *  constant pool index in the instruction.
	 */
        CASE opc_invokevirtual:
        CASE opc_invokespecial:
	CASE opc_putfield:
	CASE opc_getfield:
	    clobbersCpIndex = CVM_TRUE;
	    goto quicken_opcode;
        CASE opc_invokestatic:
	CASE opc_getstatic:
        CASE opc_putstatic:
        CASE opc_new:
	CASE opc_invokeinterface:
        CASE opc_anewarray:
        CASE opc_multianewarray:
        CASE opc_checkcast:
	CASE opc_instanceof:
        CASE opc_ldc:
        CASE opc_ldc_w:
	CASE opc_ldc2_w:
	    clobbersCpIndex = CVM_FALSE;
	quicken_opcode: {	    
	    CVMQuickenReturnCode retCode;
	    CVMUint8 quickening[3];
	    DECACHE_TOS();
	    DECACHE_PC();
	    TRACE(("\t%s ==> quickening...\n", CVMopnames[opcode]));
	    CVMD_gcSafeExec(ee, {
		/*
		 * Quicken the opcode. Possible return values are:
		 *
		 *    CVM_QUICKEN_SUCCESS_OPCODE_ONLY:
		 *        quickening succeeded, only opcode changed.
		 *    CVM_QUICKEN_SUCCESS_OPCODE_AND_OPERANDS
		 *        quickening succeeded, opcode and operands changed.
		 *    CVM_QUICKEN_NEED_TO_RUN_STATIC_INITIALIZERS
		 *        need to run static initializers, p_cb updated
		 *    CVM_QUICKEN_ALREADY_QUICKENED
		 *        someone else already raced us to it
		 *            OR 
		 *        in the JVMDI case, the instruction stream is already
		 *        updated in CVMquickenOpcode().
		 *    CVM_QUICKEN_ERROR
		 *        an error occurred, exception pending
		 *
		 * Upon success, either quickening[] holds the instruction
		 * sequence to replace the current one (max. length 3),
		 * or we need to run static initializers, in which case
		 * "cb" will contain the CVMClassBlock* to initialize.
		 *
		 * If an error occurred, we go to handle the exception.
		 */
		quickening[0] = opcode;
		retCode = CVMquickenOpcode(ee, quickening, pc, cp, &cb,
					   clobbersCpIndex);
		opcode = quickening[0];
	    });
#ifdef CVM_JVMDI
	    /* These two should never happen with JVMDI. CVMquickenOpcode()
	     * should take care of updating the instruction stream
	     */
	    CVMassert(retCode != CVM_QUICKEN_SUCCESS_OPCODE_AND_OPERANDS);
	    CVMassert(retCode != CVM_QUICKEN_SUCCESS_OPCODE_ONLY);
#endif
	    /*
	     * We should write to the instruction stream only when GC-unsafe.
	     * Otherwise, a GC may occur after the instruction stream
	     * has been updated with the quick opcode, and the stackmap
	     * computer would skip this GC-safe point. GC points without
	     * stackmaps are very bad.
	     *
	     * (See bug #4353843)
	     */
	    switch (retCode) {
	    case CVM_QUICKEN_SUCCESS_OPCODE_ONLY: {
		CVM_WRITE_CODE_BYTE(pc, 0, opcode);
		goto opcode_switch;
	    }
	    case CVM_QUICKEN_SUCCESS_OPCODE_AND_OPERANDS: {
		/* If we have to modify the instruction operands, then
		 * the instruction must be quickened within the code
		 * lock. Also, we must always modify the operands
		 * before the opcode so the interpreter will never
		 * fetch a quick opcode before the operands are
		 * written. */
	        CVM_CODE_LOCK(ee);
		CVM_WRITE_CODE_BYTE(pc, 1, quickening[1]);
		CVM_WRITE_CODE_BYTE(pc, 2, quickening[2]);
		CVM_WRITE_CODE_BYTE(pc, 0, opcode);
                CVM_CODE_UNLOCK(ee);
		CVMassert(opcode == quickening[0]);
		goto opcode_switch;
	    }
	    case CVM_QUICKEN_ALREADY_QUICKENED:
		/* This can happen with or without JVMDI */
		goto opcode_switch;
	    case CVM_QUICKEN_NEED_TO_RUN_STATIC_INITIALIZERS:
		goto init_class;
	    case CVM_QUICKEN_ERROR: {
		CVMassert(CVMexceptionOccurred(ee));
		goto handle_exception;
	    }
	    default:
		CVMassert(CVM_FALSE); /* We should never get here */
	    }
	}
#endif /* CVM_CLASSLOADING */

	/* Allocate memory for a new java object. */

	CASE opc_new_checkinit_quick: {
	    cb = CVMcpGetCb(cp, GET_INDEX(pc + 1));
	    INIT_ROMCLASS_IF_NEEDED(ee, cb);
	    goto new_quick;
	}

	CASE opc_new_quick: {
	    CVMObject* directObj;
	    cb = CVMcpGetCb(cp, GET_INDEX(pc + 1));

	    /*
	     * CVMgcAllocNewInstance may block and initiate GC, so we must
	     * de-cache our state for GC first.
	     */
	new_quick:
	    DECACHE_PC();
	    DECACHE_TOS();
			
	    directObj = CVMgcAllocNewInstance(ee, cb);
	    if (directObj == 0) {
		CVM_JAVA_ERROR_WITH_CLASSNAME(OutOfMemoryError, cb);
	    }

	    CVMID_icellSetDirect(ee, &STACK_ICELL(0), directObj);
	    TRACE(("\tnew_quick %C => 0x%p\n", cb, directObj));
	    
	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 1);

	}

        CASE opc_newarray: {
	    CVMJavaInt         arrLen;
	    CVMBasicType       typeCode;
	    CVMClassBlock*     arrCb;
	    CVMArrayOfAnyType* directArr;

	    arrLen  = STACK_INT(-1);
	    if (arrLen < 0) {
		CVM_JAVA_ERROR(NegativeArraySizeException, 0);
	    }
	    typeCode = pc[1];
	    arrCb    = (CVMClassBlock*)CVMbasicTypeArrayClassblocks[typeCode];
	    CVMassert(arrCb != 0);

	    /*
	     * CVMgcAllocNewArray may block and initiate GC, so we must
	     * de-cache our state for GC first.
	     */
	    DECACHE_PC();
	    DECACHE_TOS();
	    directArr = (CVMArrayOfAnyType*)
		CVMgcAllocNewArray(ee, typeCode, arrCb, arrLen);
	    if (directArr == 0) {
		CVM_JAVA_ERROR_WITH_CLASSNAME(OutOfMemoryError, arrCb);
	    }
	    CVMID_icellSetDirect(ee, &STACK_ICELL(-1), (CVMObject*)directArr);
	    /*
	     * Make sure we created the right beast
	     */
	    CVMassert(CVMisArrayClassOfBasicType(CVMobjectGetClass(directArr),
						 CVMbasicTypeID[typeCode]));

	    TRACE(("\tnewarray %C.%d => 0x%p\n", arrCb, arrLen, directArr));

	    UPDATE_PC_AND_TOS_AND_CONTINUE(2, 0);
	}

        CASE opc_anewarray_quick: {
	    CVMJavaInt     arrLen;
	    CVMClassBlock* arrCb;
	    CVMArrayOfRef* directArr;

	    arrLen  = STACK_INT(-1);
	    if (arrLen < 0) {
		CVM_JAVA_ERROR(NegativeArraySizeException, 0);
	    }
	    /*
	     * This doesn't have to run static initializers, so we
	     * don't have to do INIT_ROMCLASS_IF_NEEDED()
	     */
	    cb      = CVMcpGetCb(cp, GET_INDEX(pc + 1));

	    /*
	     * CVMclassGetArrayOf may block. CVMgcAllocNewArrayOf may block
	     * and initiate GC. So we must de-cache our state for GC
	     * first.  
	     */
	    DECACHE_PC();
	    DECACHE_TOS();

	    CVMD_gcSafeExec(ee, {
		arrCb = CVMclassGetArrayOf(ee, cb);
	    });
	    if (arrCb == NULL) {
		goto handle_exception;
	    }

	    directArr = (CVMArrayOfRef*)
		CVMgcAllocNewArray(ee, CVM_T_CLASS, arrCb, arrLen);
	    if (directArr == NULL) {
		CVM_JAVA_ERROR_WITH_CLASSNAME(OutOfMemoryError, arrCb);
	    }
	    CVMID_icellSetDirect(ee, &STACK_ICELL(-1), (CVMObject*)directArr);

	    TRACE(("\tanewarray_quick %C => 0x%p\n", arrCb, directArr));

	    UPDATE_PC_AND_TOS_AND_CONTINUE(3, 0);
	}

        CASE opc_multianewarray_quick: {
	    CVMClassBlock* arrCb;
	    CVMInt32       nDimensions;
	    CVMInt32       dimCount; /* in case we encounter a 0 */
	    CVMInt32       effectiveNDimensions;
	    CVMObjectICell* resultCell;

	    arrCb         = CVMcpGetCb(cp, GET_INDEX(pc + 1));
	    nDimensions   = pc[3];
	    effectiveNDimensions = nDimensions;

	    DECACHE_TOS(); /* it doesn't matter when we DECACHE_TOS really */

	    /*
	     * Point to beginning of dimension elements instead
	     * of the end, for convenience.
	     */
	    topOfStack   -= nDimensions;

	    /*
	     * Now go through the dimensions. Check for negative dimensions.
	     * Also check for whether there is a 0 in the dimensions array
	     * which would prevent the allocation from continuing.
	     */
	    for (dimCount = 0; dimCount < nDimensions; dimCount++) {
		CVMInt32 dim = STACK_INT(dimCount);
		if (dim <= 0) {
		    /* Remember the first 0 in the dimensions array */
		    if ((dim == 0) && (effectiveNDimensions == nDimensions)) {
			effectiveNDimensions = dimCount + 1;
			/* ... and keep checking for -1's despite a 0 */
		    } else if (dim < 0) {
			CVM_JAVA_ERROR(NegativeArraySizeException, 0);
		    }
		}
	    }

	    /*
	     * CVMmultiArrayAlloc() will do all the work
	     *
	     * Be paranoid about GC in this case (too many intermediate
	     * results).
	     */
	    DECACHE_PC();
	    resultCell = CVMmiscICell(ee);
	    CVMassert(CVMID_icellIsNull(resultCell));
	    CVMD_gcSafeExec(ee, {
		CVMmultiArrayAlloc(ee, effectiveNDimensions,
				   (CVMInt32*)topOfStack,
				   arrCb, resultCell);
	    });

	    if (CVMID_icellIsNull(resultCell)) {
		/*
		 * CVMmultiArrayAlloc() returned a null. This only happens
		 * if an allocation failed, so throw an OutOfMemoryError
		 */
		CVM_JAVA_ERROR(OutOfMemoryError, 0);
	    }
	    CVMID_icellAssignDirect(ee, &STACK_ICELL(0), resultCell);
	    /*
	     * Set this to null when we're done. The misc icell is
	     * needed elsewhere, and the nullness is used to prevent
	     * errors.
	     */
	    CVMID_icellSetNull(resultCell);

	    TRACE(("\tmultianewarray_quick %C "
		   "(dimensions = %d, effective = %d) => 0x%p\n",
		   arrCb, nDimensions, effectiveNDimensions, STACK_OBJECT(0)));

	    UPDATE_PC_AND_TOS_AND_CONTINUE(4, 1);
	}

	/* Throw an exception. */

	CASE opc_athrow: {
	    CVMObject* directObj = STACK_OBJECT(-1);

	    /* ROBBINS SEZ:
	     * new rule, if we "throw" (with athrow) AIE then it gets
	     * the modified AIE semantics
	     */

	    /* ROBBINS SEZ: orginal code */
	    CHECK_NULL(directObj);
	    
	    /* ROBBINS SEZ: new code again */
	    if(CVMgcUnsafeIsInstanceOf(ee, directObj, aieClass)){
	      CVMdebugPrintf(("THROWING AN AIE WITH ATHROW!\n"));
	      /* we aquire a mutex on the current thread object so that */
	      /* posting threads can't change the exception data on us */
	      CVMgcUnsafeLockThreadMutex(ee,{
		DECACHE_PC();DECACHE_TOS();},{
		CVM_JAVA_ERROR(OutOfMemoryError, NULL);
	      });
	      
	      /* now we are synch on the thread object, so post the
	       * exception */
	      
	      /* okay, we are guarenteed that AIE's only get posted 
	       * when there precedence is higher than either pending
	       * or current, so if there is a pending, we take it and
	       * ignore the directObj AIE. This also has the effect of
	       * making an explicitly thrown AIE lower in precedence than 
	       * remotly thrown ones.
	       */
	      if(!CVMpendingAIException(ee)){
		CVMgcUnsafeSetPendingAIException(ee,ee,directObj);
	      }

	      /* now that we've (conditionally) set the exception pending
	       * we unlock the thread object mutex*/
	      CVMgcUnsafeUnlockThreadMutex(ee,{
		DECACHE_PC();DECACHE_TOS();},{
		CVM_JAVA_ERROR(IllegalMonitorStateException, "current thread not owner");
	      });
	      
	      /* ROBBINS SEZ:
	       * we MUST jump to the handler at this point, as the compiler
 	       * expect this, regardless of our deferred state */
 	      DECACHE_PC();
	      goto handle_exception;
	    }
	    else{
	      /* this was a normal exception, throw as usual */
	      TRACE(("\tathrow => %O\n", directObj));
	      DECACHE_PC();  /* required for handle_exception */
	      CVMgcUnsafeThrowLocalException(ee, directObj);
	      goto handle_exception;
	    }
 	}

	/* goto and jsr. They are exactly the same except jsr pushes
	 * the address of the next instruction first.
	 */

        CASE opc_jsr: {
	    /* push return address on the stack */
	    STACK_ADDR(0) = pc + 3;
	    topOfStack++;
	    /* FALL THROUGH */
	}

        CASE opc_goto:
	{
	    CVMInt16 offset = CVMgetInt16(pc + 1);
	    TRACE(("\t%s %#x (offset=%d)\n",CVMopnames[opcode],
		   pc + offset, offset));
	    UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(offset, 0);
	}

        CASE opc_jsr_w: {
	    /* push return address on the stack */
	    STACK_ADDR(0) = pc + 5;
	    topOfStack++;
	    /* FALL THROUGH */
	}

        CASE opc_goto_w:
	{
	    CVMInt32 offset = CVMgetInt32(pc + 1);
	    TRACE(("\t%s %#x (offset=%d)\n", CVMopnames[opcode],
		   pc + offset, offset));
	    UPDATE_PC_AND_TOS_AND_CONTINUE_WITH_BACKWARDS_CHECK(offset, 0);
	}

	/* return from a jsr or jsr_w */

        CASE opc_ret: {
	    TRACE(("\tret %d (%#x)\n", pc[1], locals[pc[1]].a));
	    pc = locals[pc[1]].a;
	    UPDATE_PC_AND_TOS_AND_CONTINUE(0, 0);
	}

	/* debugger breakpoint */

        CASE opc_breakpoint: {
	    TRACE(("\tbreakpoint\n"));
#ifdef CVM_JVMDI
	    DECACHE_TOS();
	    DECACHE_PC();
	    CVMD_gcSafeExec(ee, {
		opcode = CVMjvmdiGetBreakpointOpcode(ee, pc, CVM_TRUE);
	    });
	    goto opcode_switch;
#else
	    UPDATE_PC_AND_TOS_AND_CONTINUE(1, 0);    /* treat as opc_nop */
#endif
	}

#ifdef CVM_NO_LOSSY_OPCODES
	CASE opc_agetstatic_quick:
	CASE opc_getstatic_quick:
	CASE opc_getstatic2_quick:
	CASE opc_aputstatic_quick:
	CASE opc_putstatic_quick:
	CASE opc_putstatic2_quick:
        CASE opc_getfield_quick:
        CASE opc_putfield_quick:
        CASE opc_getfield2_quick:
        CASE opc_putfield2_quick:
	CASE opc_agetfield_quick:
	CASE opc_aputfield_quick:
#endif

#ifndef CVM_CLASSLOADING
        CASE opc_ldc:
        CASE opc_ldc_w:
        CASE opc_ldc2_w:
        CASE opc_getfield:
        CASE opc_putfield:
        CASE opc_getstatic:
        CASE opc_putstatic:
        CASE opc_new:
        CASE opc_anewarray:
        CASE opc_multianewarray:
        CASE opc_checkcast:
        CASE opc_instanceof:
        CASE opc_invokevirtual:
        CASE opc_invokespecial:
        CASE opc_invokestatic:
        CASE opc_invokeinterface:
#endif

#ifndef CVM_INLINING
	CASE opc_invokeignored_quick:
#endif
        CASE opc_xxxunusedxxx:
	DEFAULT:
	    CVMdebugPrintf(("\t*** Unimplemented opcode: %d = %s\n",
		   opcode, CVMopnames[opcode]));
	    goto finish;

	} /* switch(opc) */

    callmethod: {
	    /* callmethod handles the method invoking for all invoke
	     * opcodes. Upon entry, the following locals must be setup:
	     *    mb:            the method to invoke
	     *    cb:            CVMClassBlock* of mb
	     *    pc:            points to the invoke opcode
	     *    topOfStack:    points before the method arguments
	     *    receiverObjICell: the object we are dispatching on or
	     *			 the Class object for static calls
	     */
	    CVMFrame* prev = frame;
	    int invokerIdx;
	    //prevframe = prev;

            TRACE(("\t%s %C.%M\n", CVMopnames[opcode], cb, mb));

	    TRACESTATUS();

	    /* Decache the interpreter state before pushing the frame.
	     * We leave frame->topOfStack pointing after the args,
	     * so they will get scanned it gc starts up during the pushFrame.
	     */
	    DECACHE_PC();

    new_mb:

	    /*Set thread's ATC state deferred/nondeferred -Kalpesh*/
            exclass = CVMmbClassBlock(mb);
	    //CVMATCSetPermission(frame,DEFERRED);
	    tmpState = DEFERRED;

	    if((CVMmbIs(mb,NATIVE))){
		 //   printf("Native Method\n");
	    }
	    else if((CVMmbIs(mb,SYNCHRONIZED))){
		  //  CVMATCSetPermission(frame,SYNCH_DEFERRED);
		  //  printf("SynchDeffered Method\n");
		  tmpState = SYNCH_DEFERRED;
	    }
	    else{
		    
		if(aieClass==NULL){
			  CVMD_gcSafeExec(ee,{
		  	        baseClass = CVMclassLookupByNameFromClass(ee,"java/lang/Object",CVM_FALSE,exclass);
				aieClass = CVMclassLookupByNameFromClass(ee,"javax/realtime/AsynchronouslyInterruptedException",CVM_FALSE,exclass);
				ioClass = CVMclassLookupByNameFromClass(ee,"java/io/PrintStream",CVM_FALSE,exclass);
			  });
		}
		  
	    	checks = CVMmbCheckedExceptions(mb);
		if(checks){
		    CVMUint16* vmexcs;
		    int i,length;
		    CVMBool isAIE = CVM_FALSE;
		    CVMClassBlock* cbRes = NULL;
		    CVMConstantPool* cptmp = CVMcbConstantPool(exclass);
				    
		    vmexcs = &checks->exceptions[0];
		    length = checks->numExceptions;
    	    
		    for(i=0;i<length;i++){
			       CVMD_gcSafeExec(ee,{
					   
				   if (CVMcpResolveEntryFromClass(ee,exclass,CVMcbConstantPool(exclass),vmexcs[i])){
 				        cbRes = CVMcpGetCb(cptmp,vmexcs[i]);
				   }
		               });
					
		       	        isAIE = CVM_FALSE;
				classID = cbRes->classNameX;
				//We traverse whole chain of superclass to find if this exception is AIE or child of AIE -Kalpesh
			        do{
				    if(aieClass==cbRes){
			       		    isAIE = CVM_TRUE;
					    break;
			   	    }
			  	     cbRes = CVMcbSuperclass(cbRes);
					    
			        }while(baseClass!=cbRes);

			        if(isAIE==CVM_TRUE){
					// CVMATCSetPermission(frame,NONDEFERRED);	
				   	 tmpState = NONDEFERRED;
					// printf("This method throws AIE :)) \n");
					 break;
				}
					
		   }//endof for
		
	       }//endof if
				    
	    }//endof else
	    /*
	     * Here we will set dirtyAie flag to indicate if we are
	     * calling method of io package from this AI method.
	     */
	     if(CVMATCdirty(ee)>0){
	     	if(CVMstackDepth(prev) < CVMATCdirty(ee))
			CVMATCsetDirty(ee,0);
	     }
	     if(CVMATCPermission(prev)==NONDEFERRED){
	     	 if(CVMtypeidIsSameClassPackage(CVMcbClassName(cb),CVMcbClassName(ioClass))){
			 CVMATCsetDirty(ee, CVMstackDepth(prev));
			 //printf("Entering IO calls from Ai method == %d.\n",CVMATCdirty(ee));
	 	 }
		 else{
		     CVMATCsetDirty(ee,0);
		 }
	     }
	      
	    
#ifdef CVM_DEBUG	    
	  /* {
	      int x;
	      
	      if(CVMframeIsTransition(frame)){
		printf("Yes this is transition..\n");
	      }
	      for(x=0;x<CVMstackDepth(frame);x++){
		printf("_");
	      }
	    }
	    switch(tmpState){
	     	case DEFERRED :
				printf("D\n");
				break;
				
		case SYNCH_DEFERRED:

				printf("S\n");
				break;
			
		case NONDEFERRED:
				
				printf("#\n");
				break;

		default :
				printf("!\n");
	     
	     }*/
#endif
	     
	    /* ^ Added by Kalpesh*/

	    invokerIdx = CVMmbInvokerIdx(mb);
	    if (invokerIdx < CVM_INVOKE_CVM_METHOD) {
		/*
		 * It's a Java method. Push the frame and setup our state 
		 * for the new method.
		 */
		CVMJavaMethodDescriptor* jmd = CVMmbJmd(mb);

		/* update the new locals value so our pushframe logic
		 * is easier. */
		locals = &topOfStack->s;

		/*
		 * pushFrame() might have to become GC-safe if it needs to 
		 * expand the stack. frame->topOfStack for the current frame 
		 * includes the arguments, and the stackmap does as well, so
		 * that the GC-safe action is really safe.
		 */
		CVMpushFrame(ee, stack, frame, topOfStack,
			     CVMjmdCapacity(jmd), CVMjmdMaxLocals(jmd),
			     CVMjavaFrameScanner,
		/* action if pushFrame fails */
		{
		    CACHE_FRAME();  /* pushframe set it to NULL */
		    locals = CVMframeLocals(frame);
		    goto handle_exception; /* exception already thrown */
		},
		/* action to take if stack expansion occurred */
		{
		    TRACE(("pushing JavaFrame caused stack expansion\n"));
		    /* Since the stack expanded, we need to set locals
		     * to the new frame minus the number of locals.
		     * Also, we need to copy the arguments, which
		     * topOfStack points to, to the new locals area.
		     */
		    locals = (CVMSlotVal32*)frame - CVMjmdMaxLocals(jmd);
		    memcpy((void*)locals, (void*)topOfStack,
			       CVMmbArgsSize(mb) * sizeof(CVMSlotVal32));
		});

		/*
		 * GC-safety stage 2: Update caller frame's topOfStack to
		 * exclude the arguments. Once we update our new frame
		 * state, the callee frame's stackmap will cover the arguments
		 * as incoming locals.  
		 */
		prev->topOfStack = topOfStack;

		/* set topOfStack to start of new operand stack */
		CVM_RESET_JAVA_TOS(topOfStack, frame);

		/* We have to cache frame->locals if this method ever calls
		 * calls another method (which we don't know), and in most
		 * cases it's cheaper just to cache it here rather then cache
		 * it every time we make a method call from this method. The
		 * same is true of frame->mb and frame->constantpool.
		 */
		CVMframeLocals(frame) = locals;
		pc = CVMjmdCode(jmd);
		frame->mb = mb;

		CVMATCSetPermission(frame,tmpState); // Added by Kalpesh: Macro says everything..isn't it..

		/* ROBBINS SEZ:
		 * All code that we add to existing CVM files should be commented with
		 * the name of the developer to facilitate tracking of our changes to the
		 * source tree, regardless of how self-documenting the added or moded code
		 * is. - heh, I think I'm the only one who does this.
		 */
		
		cp = CVMcbConstantPool(cb);
		CVMframeCp(frame) = cp;
		
		TRACE_METHOD_CALL(frame, CVM_FALSE);
		TRACESTATUS();

		CVMID_icellAssignDirect(ee, 
					&CVMframeReceiverObj(frame),
					receiverObjICell);
		
		/* %comment cjp002 */
		if (CVMmbIs(mb, SYNCHRONIZED)) {  
                    /* The method is sync, so lock the object. */
                    /* %comment lamm002 */
                    if (!CVMobjectTryLock(
			    ee, CVMID_icellDirect(ee, receiverObjICell))) {
                        DECACHE_PC();
                        DECACHE_TOS();
                        if (!CVMobjectLock(ee, receiverObjICell)) {
			    CVMpopFrame(stack, frame); /* pop the java frame */
			    if (!CVMexceptionOccurred(ee))
			      CVMthrowOutOfMemoryError(ee, NULL);
			    goto handle_exception;
			}
                    }
		}

#ifdef CVM_JVMDI
		/* %comment kbr001 */
		/* Decache all curently uncached interpreter state */
		if (CVMglobals.jvmdiDebuggingEnabled) {
		    DECACHE_PC();
		    DECACHE_TOS();
		    CVMD_gcSafeExec(ee, {
			CVMjvmdiNotifyDebuggerOfFramePush(ee);
		    });
		}
#endif

		/*
		 * GC-safety stage 3: Now that we have "committed" the new 
		 * frame state, we can request a GC-safe point. At this point,
		 * the topOfStack of the caller frame has been decremented,
		 * and it is the callee that will be responsible for the 
		 * arguments
		 *
		 * Decache the new interpreter state if there's a gc request.
		 */
		CVMD_gcSafeCheckPoint(ee, {
		    DECACHE_PC();
		    DECACHE_TOS();
		}, {});

		
	    } else if (invokerIdx < CVM_INVOKE_JNI_METHOD) {

		/*
		 * It's an CNI method. See cjp07/13/99 in interpreter.h for
		 * details on how CNI methods work.
		 */

		CNINativeMethod *f = (CNINativeMethod *)CVMmbNativeCode(mb);
		CNIResultCode ret = (*f)(ee, topOfStack, &mb);

		/* 
		 * Look at the result code to see what the CNI method did.
		 */
		if ((int)ret >= 0) {
		    /* This is a normal return from an CNI method. The only
		     * positive result codes are CNI_VOID, CNI_SINGLE, and
		     * CNI_DOUBLE, all of which represent the actual size
		     * in words of the result. Bump up the pc to point
		     * beyond the invoke opcode and increment the TOS by
		     * the size of the method result.
		     */
		    topOfStack += (int)ret;
		    pc += (*pc == opc_invokeinterface_quick ? 5 : 3);
		    CONTINUE;
		} else if (ret == CNI_NEW_TRANSITION_FRAME) {
		    /* The CNI method pushed a transition frame. This is
		     * used by CVM.invoke(). The transition frame holds
		     * the arguments for the method to be invoked. We
		     * need to cache the new frame and then act like we just
		     * entered CVMgcUnsafeExecuteJavaMethod (almost).
		     */
		    DECACHE_TOS(); /* pop invoker's arguments. Must be done
				    * before CACHE_FRAME() */
		    CACHE_FRAME(); /* get the transition frame. */

		    /* Set the flag that says opc_exittransition should
		     * increment the pc.
		     */
		    CVMframeIncrementPcFlag(frame) = CVM_TRUE;

		    isStatic = CVMmbIs(mb, STATIC);
		    isVirtual = CVM_FALSE;

		    /* no gcsafe checkpoint is needed since new_transition
		     * will eventually lead to one.
		     */
		    goto new_transition;
#if 0 /* currently no one uses this feature.*/
		} else if (ret == CNI_NEW_JAVA_FRAME) {
		    /* No one uses this yet. It's used if you want to do 
		     * push a JavaFrame and have exectuion resume at the
		     * pc you stored in the JavaFrame. 
		     */
		    CACHE_FRAME();
		    CACHE_PC();
		    CACHE_TOS();
		    locals = CVMframeLocals(frame);
		    cp = CVMframeCp(frame);
		    TRACE_METHOD_CALL(frame, CVM_TRUE);

		    CVMD_gcSafeCheckPoint(ee, {
			DECACHE_PC();    /* only needed if we gc */
			DECACHE_TOS();   /* only needed if we gc */
		    }, {});
#endif
		} else if (ret == CNI_NEW_MB) {
		    /* This is used by CVM.executeClinit. It's eliminates
		     * C recursion by simply returning the mb of the <clinit>
		     * method that needs to be called. topOfStack currently
		     * points before the arguments, which is what we want.
		     * However, we need to update frame->topOfStack to point
		     * after the arguments, since the number of arguments
		     * may have changed.
		     */
		    DECACHE_TOS();
		    frame->topOfStack += CVMmbArgsSize(mb);
		    cb = CVMmbClassBlock(mb);

            /* *** RTSJ *** */
            // Magic: I need to ensure that static initializers get 
            // invoked in ImmortalMemory. So this is how I do it.
            // 1. Set a flag indicating the stack depth at which I 
            // am executing the <clinit.
            if(clinitStackDepth == 0) 
            {
                extern int immortalInitialized;
                if(immortalInitialized)
                {
                    clinitStackDepth = CVMstackDepth(frame);
               //     CVMID_icellAssignDirect(ee, &oldMemoryArea, CVMmemoryAreaICell(ee));
               //     CVMID_icellAssignDirect(ee, CVMmemoryAreaICell(ee), jtImmortalMemory);
                    oldCheckConstraints = CVMcheckForConstraints(ee);
                    CVMdisableCheckForConstraints(ee);
                    SLIDE_UP_TO_MA(ee, jtImmortalMemory);
                }
            }
            //else I am already running a static initializers
            /* *** JSTR *** */
            
		    /* no gcsafe checkpoint is needed since new_mb
		     * will eventually lead to one.
		     */
		    goto new_mb;
		} else if (ret == CNI_EXCEPTION) {
		    /* This is for when an CNI method throws an exception. */
		    goto handle_exception;
		} else {
		    CVMassert(CVM_FALSE);
		}

	    } else if (invokerIdx < CVM_INVOKE_ABSTRACT_METHOD) {
		/*
		 * It's a JNI method.
		 */
		union {
		    CVMJNIReturnValue jni;
		    CVMJavaVal32     i;
		    CVMObjectICell * o;
		} returnValue;
		
		/* Result from CVMjniInvokeNative. It indicates the type
		 * of result the java method returned.
		 */
		CVMInt8 returnCode;

		/*
		 * pushFrame() might have to become GC-safe if it needs to 
		 * expand the stack. frame->topOfStack for the current frame
		 * includes the arguments, and the stackmap does as well, so
		 * that the GC-safe action is really safe.
		 */
		CVMpushFrame(ee, stack, frame, topOfStack,
			     (CVMoffsetof(CVMJNIFrame, vals) / 
			      sizeof(CVMStackVal32)) + CVM_JNI_MIN_CAPACITY,
			     CVMmbArgsSize(mb), /* skip arguments */
			     CVMjniFrameScanner, 
		/* action if pushFrame fails */
		{
		    CACHE_FRAME();  /* pushframe set it to NULL */
		    goto handle_exception; /* exception already thrown */
		},
		/* action to take if stack expansion occurred */
		{
		    TRACE(("pushing JNIFrame caused stack expansion\n"));
		    /* No need to copy arguments for JNI invocations.
		     * The caller's frame will continue to hold the arguments
		     * for us.
		     */ 
		});

		/* Update the new frame's state */
		CVM_RESET_JNI_TOS(frame->topOfStack, frame);
		CVMjniFrameInit(CVMgetJNIFrame(frame));
		frame->mb = mb;

		TRACE_METHOD_CALL(frame, CVM_FALSE);
		TRACESTATUS();

		/* Our stack state at this point is as follows:
		 *    topOfStack        <- points before the arguments
		 *    prev->topOfStack  <- points after the arguments
		 *    frame->topOfStack <- points to space for jni local refs
		 *
		 * We want prev->topOfStack to point after the arguments so
		 * they will be scanned. topOfStack will continue to point
		 * before the arguments so we can store our result there.
		 */

		/* If the method is sync, then then lock the object. */
		if (CVMmbIs(mb, SYNCHRONIZED)) { 
                    /* %comment lamm002 */
                    if (!CVMobjectTryLock(
                            ee, CVMID_icellDirect(ee, receiverObjICell))) {
                        if (!CVMobjectLock(ee, receiverObjICell)) {
			    CVMpopFrame(stack, frame); /* pop the jni frame */
			    if (!CVMexceptionOccurred(ee))
			      CVMthrowOutOfMemoryError(ee, NULL);
			    goto handle_exception;
			}
                    }
		}

#ifdef CVM_JVMDI
		/* %comment kbr001 */
		if (CVMglobals.jvmdiDebuggingEnabled) {
		    CVMD_gcSafeExec(ee, {
			CVMjvmdiNotifyDebuggerOfFramePush(ee);
		    });
		}
#endif

		/* Call the JNI method. topOfStack still points just below
		 * the arguments in the caller's frame, so you can still
		 * use it to access the arguments.
		 */
		CVMD_gcSafeExec(ee, {
		    CVMterseSig nativeSig;
		    CVMtypeidGetTerseSignature(CVMmbNameAndTypeID(mb),
					       &nativeSig );
		    returnCode = CVMjniInvokeNative(CVMexecEnv2JniEnv(ee),
			    CVMmbNativeCode(mb),
			    &topOfStack->s.j.raw,
			    nativeSig.datap,
			    CVMmbArgsSize(mb),
			    CVMmbIs(mb, STATIC) ? CVMcbJavaInstance(cb) : 0,
			    &returnValue.jni);
		});
		
		TRACE_METHOD_RETURN(frame);

#ifdef CVM_JVMDI
		/** The JVMDI spec is unclear about whether the frame
		    pop event is sent just before or just after the
		    frame is popped. The JDK implementation sends the
		    event just before the frame is popped. Also,
		    Bugid 4191820 seems to imply that the callback must
		    be done before the frame is popped, and that no
		    local variable slots should have been overwritten
		    by the return value. */
		if (CVMglobals.jvmdiDebuggingEnabled) {
		    CVMD_gcSafeExec(ee, {
			CVMjvmdiNotifyDebuggerOfFramePop(ee);
		    });
		}
#endif
		/*
		 * Unlock the sync object if necessary, before the return
		 * value tramples over receiveObjICell.
		 */
		if (CVMmbIs(mb, SYNCHRONIZED)) {
                    /* %comment lamm003 */
                    CVMBool success;
                    success = CVMobjectTryUnlock(
                        ee, CVMID_icellDirect(ee, receiverObjICell));
                    if (!success) {
                        /* NOTE: CVMobjectUnlock may become GC safe. */
                        success = CVMobjectUnlock(ee, receiverObjICell);
			if (!success) {
			    /* 
			     * We need to clear any exception that JNI may have
			     * returned and throw a new one.
			     */
			    CVMclearLocalException(ee);
			    CVMpopFrame(stack, frame); /* pop the jni frame */
			    CVMthrowIllegalMonitorStateException(
                                ee, "current thread not owner");
			    goto handle_exception;
			}
                    }
		}

		CVMpopFrame(stack, frame);   /* pop the jni frame */

		   
		/* Grab the result and store it. topOfStack still points 
		 * just below the arguments in the caller's frame, so you 
		 * can still use the STACK_ macros with an offset of 0
		 * to save the result.
		 *
		 * CVMjniInvokeNative stored the result in the local
		 * variable "returnValue".
		 */
		{
		    switch (returnCode) {
		    case 0:
			break;
		    case -1:
			{
			    CVMObjectICell *o = returnValue.o;
			    if (o != 0) {
				CVMID_icellAssignDirect(ee, &STACK_ICELL(0), o);
			    } else {
				CVMID_icellSetNull(&STACK_ICELL(0));
			    }
			    topOfStack += 1;
			    break;
			}
		    case 1:
			STACK_INFO(0) = returnValue.i;
			topOfStack += 1;
			break;
		    case 2:
			CVMmemCopy64(&STACK_INFO(0).raw, returnValue.jni.v64);
			topOfStack += 2;
			break;
		    default:
			/* This should never happen. */
			CVMassert(CVM_FALSE);
		    }
		}	    
	
		/* topOfStack points just after the result at this point,
		 * which is exactly what we want. We don't care what
		 * frame->topOfStack points to since it will get flushed
		 * at the next gcsafe checkpoint.
		 */

		/* If an exception occured, then we don't want to bump up
		 * the pc, so check first.
		 */
		if (CVMexceptionOccurred(ee)) {
		    goto handle_exception;
		}

		/* Bump up the pc to point beyond the invoke opcode in
		 * the transition frame.
		 */
		pc += (*pc == opc_invokeinterface_quick ? 5 : 3);

		CONTINUE;
	    } else if (invokerIdx == CVM_INVOKE_ABSTRACT_METHOD) {
		/*
		 * It's an abstract method.
		 */
		CVM_JAVA_ERROR_WITH_METHODNAME(AbstractMethodError, mb);
#ifdef CVM_CLASSLOADING
	    } else if (invokerIdx == CVM_INVOKE_NONPUBLIC_MIRANDA_METHOD) {
		/*
		 * It's a miranda method created to deal with a non-public
		 * method with the same name as an interface method.
		 */
		CVM_JAVA_ILLEGAL_ACCESS_ERROR(cb, CVMmbMirandaInterfaceMb(mb));
	    } else if (invokerIdx == 
		       CVM_INVOKE_MISSINGINTERFACE_MIRANDA_METHOD) {
		/*
		 * It's a miranda method created to deal with a missing
		 * interface method.
		 */
		CVM_JAVA_ERROR_WITH_METHODNAME(AbstractMethodError, mb);
	    } else if (invokerIdx == CVM_INVOKE_LAZY_JNI_METHOD) {
		/*
		 * It's a native method of a dynamically loaded class.
		 * We still need to lookup the native code.
		 */
		CVMBool result;
		CVMD_gcSafeExec(ee, {
		    result = CVMlookupNativeMethodCode(ee, mb);
		});
		if (!result) {
		    goto handle_exception;
		} else {
		    /*
		     * CVMlookupNativeMethod() stored the pointer to the
		     * native method and also changed the invoker index, so
		     * just branch to new_mb and the correct native
		     * invoker will be used.
		     */
		    goto new_mb;
		}
#endif
	    } else {
		CVMdebugPrintf(("Unkown method invoker: %d\n", invokerIdx));
		CVMassert(CVM_FALSE);
	    }
	};  /* callmethod: */
	
	CONTINUE;
	
    handle_return: {
	  
	  
#ifdef CVM_JVMDI
	    /** The JVMDI spec is unclear about whether the frame pop
                event is sent just before or just after the frame is
                popped. The JDK implementation sends the event just
                before the frame is popped. (NOTE: putting in
                decaching code here causes an assertion failure.) */
	    /** Bugid 4191820 seems to imply that the callback must
		be done before the frame is popped, and that no
		local variable slots should have been overwritten
		by the return value. */
	    if (CVMglobals.jvmdiDebuggingEnabled) {
		/* See comment below at next DECACHE_PC(). There are special
		 * gcsafety concerns here. */
		DECACHE_PC();
		CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);

		/* See comment below at next check for opc_areturn. There
		 * are special gcsafety comment here. */
		if (opcode == opc_areturn) {
		    CVMassert(CVMID_icellIsNull(CVMmiscICell(ee)));
		    CVMID_icellSetDirect(ee, CVMmiscICell(ee),
					 STACK_OBJECT(-1));
		}

		CVMD_gcSafeExec(ee, {
		    CVMjvmdiNotifyDebuggerOfFramePop(ee);
		});

		if (opcode == opc_areturn) {
		    CVMID_icellSetDirect(
                        ee, &STACK_ICELL(-1),
			CVMID_icellDirect(ee, CVMmiscICell(ee)));
		    CVMID_icellSetNull(CVMmiscICell(ee));
		}
	    }
#endif

	    /* %comment cjp003 */
	    /* Unlock the receiverObj if this was a sync method call. */
	    if (CVMmbIs(frame->mb, SYNCHRONIZED)) { //Deffered flag decreament! -Kalpesh
                /* %comment lamm003 */
                CVMObjectICell *objICell = &CVMframeReceiverObj(frame);
                CVMBool success;
                success = 
		    CVMobjectTryUnlock(ee, CVMID_icellDirect(ee, objICell));
                if (!success) {
                    /* Need to decache because CVMobjectUnlock may become
		     * safe. topOfStack currently points to the previous
		     * frame's topOfStack, so we can't use DECACHE_TOS().
		     * It turns out that just reseting the frame's
		     * topOfStack works for the current frame since it 
		     * can't possibly contain anything we need to scan.
		     */
                    DECACHE_PC();
                    CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);
		    if (opcode == opc_areturn) {
			/* We could become gcsafe during the unlock, and gc
			 * will not be scanning the result on the previous
			 * frame's stack. We need to grab the result, store
			 * it in a safe place, and the put it back on the
			 * stack after the unlock completes.
			 */
			CVMassert(CVMID_icellIsNull(CVMmiscICell(ee)));
			CVMID_icellSetDirect(ee, CVMmiscICell(ee), 
					     STACK_OBJECT(-1));
			success = CVMobjectUnlock(ee, objICell);
			CVMID_icellSetDirect(
                            ee, &STACK_ICELL(-1),
			    CVMID_icellDirect(ee, CVMmiscICell(ee)));
			CVMID_icellSetNull(CVMmiscICell(ee));
		    } else {
			success = CVMobjectUnlock(ee, objICell);
		    }
		    if (!success) {
			CVMthrowIllegalMonitorStateException(
			    ee, "current thread not owner");
			CVMpopFrame(stack, frame);
			goto handle_exception;
		    }
                }
	    }

	    TRACESTATUS();
	    TRACE_METHOD_RETURN(frame);
	    CVMpopFrame(stack, frame);

	    /* 
	     * Restore our state. We need to recache all of the local
	     * variables that got trounced when we made the method call.
	     * topOfStack was restored before handlereturn was entered.
	     */
	    CACHE_PC();   /* restore our pc from before the call */
        
        // Memory stuff. revert back to old memory area if <clinit> got
        // over.
        // -Pratik.
        if(clinitStackDepth != 0)
        {
            if(clinitStackDepth == CVMstackDepth(frame))
            {
                // revert back to old memory area
               // CVMID_icellAssignDirect(ee, CVMmemoryAreaICell(ee), &oldMemoryArea);
                SLIDE_BACK_DOWN(ee);
                clinitStackDepth = 0;
                CVMsetCheckForConstraints(ee, oldCheckConstraints);
            }
        }

	    
	    /* ROBBINS SEZ: Below is no longer valid, see above comment */
	    /* if the current frame permits it, re-throw the exception NOW*/
	    //CHECK_FOR_REMOTE_AND_AI_EXCEPTIONS();
	    
	    pc += (*pc == opc_invokeinterface_quick ? 5 : 3);
	    /* The following two lines can cause a read from
               uninitialized memory in the case of popping the frame
               just above a transition frame, which doesn't have
               locals or a constant pool. This is harmless, since
               these values won't be used in this case, but in debug
               mode we'll leave in the check to avoid problems when
               using dbx's memory checking. */
#ifdef CVM_DEBUG
	    if (CVMframeIsJava(frame)) {
#endif
		locals = CVMframeLocals(frame);
		cp = CVMframeCp(frame);
#ifdef CVM_DEBUG
	    }
#endif

	    /* We don't restore mb for performance reasons. The only
	     * quick opcode that needs it is opc_invokesuper_quick and
	     * we'll get better performance if it handles the
	     * frame->mb overhead instead of doing it on every return.
	     */
	    /* mb = frame->mb;*/

	    TRACESTATUS();
	    
#ifdef CVM_REMOTE_EXCEPTIONS_SUPPORTED
	    /* %comment cjp005 */
	    goto check_for_exception;
#else

	    CONTINUE;
#endif
	} /* handlereturn: */

    check_for_exception: {
	    if (!CVMexceptionOccurred(ee)) {
		CONTINUE;
	    }
	    /* We will be gcsafe soon, so flush our state. */
	    DECACHE_PC();
	    goto handle_exception;
	}

    handle_exception: {
      /* ROBBINS SEZ: modified to reflect that we could be
       * here due to an AI exception
       */
      CVMassert(CVMexceptionOccurred(ee) || CVMpendingAIException(ee));

	    /* When we get here we are guaranteed that all callers have
	     * made sure that the PC has been flushed. However, we must
	     * still reset frame->topOfStack per the Java Spec.
	     */
	    CVM_RESET_INTERPRETER_TOS(frame->topOfStack, frame);
	  
#ifdef CVM_JVMDI
	    if (CVMglobals.jvmdiDebuggingEnabled) {
		CVMD_gcSafeExec(ee, {
		    CVMjvmdiNotifyDebuggerOfException(ee, pc,
			(CVMlocalExceptionOccurred(ee) ?
			     CVMlocalExceptionICell(ee) :
			     CVMremoteExceptionICell(ee)));
		});
	    }
#endif

	    /* Get the frame that handles the exception. frame->pc will
	     * also get updated.
	     */
	    frame = CVMgcUnsafeHandleException(ee, frame, initialframe);

	    /* If we didn't find a handler for the exception, then just
	     * return to the CVMgcUnsafeExecuteJavaMethod caller and 
	     * let it handle the exception. Do not attempt to bump up
	     * the pc in the transition code and execute the 
	     * exittransition opcode. This will cause gc grief
	     * because it will think it needs to scan the result
	     * if the method returns a ref.
	     */
	    if (frame == initialframe) { //May be we need to modify something here for propagation -Kalpesh.
		goto finish;
	    }

        // Memory stuff. revert back to old memory area if <clinit> got popped
        // while handling exception
        // -Pratik.
        if(clinitStackDepth != 0)
        {
            if(clinitStackDepth >= CVMstackDepth(frame))
            {
                // revert back to old memory area
                //CVMID_icellAssignDirect(ee, CVMmemoryAreaICell(ee), &oldMemoryArea);
                SLIDE_BACK_DOWN(ee);
                clinitStackDepth = 0;
            }
        }


	    /* The exception was caught. Cache our new state. */
//	    printf("$$$$ Exception frame values :: saved state : %d, depth: %d",frame->ATCflags.val.old_permission,CVMstackDepth(frame));
//	    printf("$$$$ Exception frame->prevX values :: saved state : %d, depth: %d",frame->prevX->ATCflags.val.old_permission,CVMstackDepth(frame->prevX));
	    
	    CACHE_PC();
	    CVM_RESET_JAVA_TOS(topOfStack, frame);
	    locals = CVMframeLocals(frame);
	    cp = CVMframeCp(frame);

	    /* Push the exception back on stack per vm spec */
	    CVMID_icellSetDirect(ee, &STACK_ICELL(0), 
				 CVMgetCurrentExceptionObj(ee));
	    CVMsetCurrentExceptionObj(ee, 0);
	    topOfStack++;

#ifdef CVM_JVMDI
	    if (CVMglobals.jvmdiDebuggingEnabled) {
		DECACHE_PC();
		DECACHE_TOS();
		CVMD_gcSafeExec(ee, {
		    CVMjvmdiNotifyDebuggerOfExceptionCatch(ee, pc,
							   &STACK_ICELL(0));
		});
	    }
#endif
	    CONTINUE;   /* continue interpreter loop */
	}  /* handle_exception: */

    init_class: {
	    /* init_class will run static initializers for the specified class.
	     * It does this in a way that won't involve C recursion.
	     */
            int result;
	    TRACE(("\t%s ==> running static initializers...\n",
		   CVMopnames[opcode]));
	    DECACHE_PC();
	    DECACHE_TOS();
        CVMD_gcSafeExec(ee, {
        result = CVMclassInitNoCRecursion(ee, cb, &mb);
        });
        
	    /* If the result is 0, then the intializers have already been run.
	     * If the result is 1, then mb was setup to run the initializers.
	     * If the result is -1, then an exception was thrown.
	     */
	    if (result == 0) {
#ifdef CVM_NO_LOSSY_OPCODES
		if (CVMcbRomIndex(cb) == -1) {
		    CVMcbRomIndex(cb) = -2;
		}
#endif
		CONTINUE;
	    } else  if (result == 1) {
		CACHE_FRAME();
		isStatic = CVM_FALSE;
		isVirtual = CVM_FALSE;
		goto new_transition;
	    } else {
		goto check_for_exception;
	    }
        }
    } /* while (1) interpreter loop */

 finish:
    TRACE(("Exiting interpreter\n"));
    DECACHE_TOS();
    DECACHE_PC();

    return;
}
