/*
 * @(#)fixedspace.c	1.6 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#include "javavm/include/defs.h"
#include "javavm/include/objects.h"
#include "javavm/include/directmem.h"
#include "javavm/include/interpreter.h"
#include "javavm/include/stackwalk.h"
#include "javavm/include/utils.h"

#include "javavm/include/localroots.h"
#include "javavm/include/globalroots.h"

#include "javavm/include/gc_common.h"
#include "javavm/include/gc/gc_impl.h"

#include "javavm/include/globals.h"
#include "javavm/include/basictypes.h"
#include "javavm/include/indirectmem.h"

#define CVMfixedspaceAllocAndZeroOut(sizeInBytes) \
    calloc(1, sizeInBytes)

/*
 * Allocate a new instance in "fixed space"
 */
CVMObjectICell*
CVMfixedAllocNewInstance(CVMExecEnv* ee, CVMClassBlock* cb)
{
    void* newSpace;
    CVMObject* newInstance;
    CVMObjectICell* fakeICell;

    CVMassert(CVMD_isgcSafe(ee));

    newSpace = CVMfixedspaceAllocAndZeroOut(CVMcbInstanceSize(cb) +
					    sizeof(CVMObjectICell));

    /*
     * Caller responsible for handling failed allocation
     */
    if (newSpace == 0) {
	return 0;
    }
    newInstance = (CVMObject*)((char*)newSpace + sizeof(CVMObjectICell));
    fakeICell   = (CVMObjectICell*)newSpace;

    /* 
     * Set header of new object. Make sure to indicate fixed space
     * object.
     */
    newInstance->hdr.clas = (CVMClassBlock*)
	((CVMUint32)cb | CVM_OBJECT_IN_ROM_MASK);

    /* 
     * Set the 'fake' indirection cell to point to the object 
     */
    CVMD_gcUnsafeExec(ee, {
	CVMID_icellSetDirect(ee, fakeICell, newInstance);
    });
    return fakeICell;
}


/*
 * Allocate fixedspace array of length 'len'. The array type is 'arrayCb'.
 */
CVMArrayOfAnyTypeICell*
CVMfixedAllocNewArray(CVMExecEnv* ee, CVMBasicType typeCode,
		      CVMClassBlock* arrayCb, CVMJavaInt arrayLen)
{
    void*              newSpace;
    CVMArrayOfAnyType* newArray;
    CVMJavaInt         arrayObjectSize;
    CVMArrayOfAnyTypeICell*    fakeICell;

    CVMassert(CVMD_isgcSafe(ee));
    /*
     * The size of the fixedspace object to accommodate this array
     */
    arrayObjectSize = CVMalignWordUp(CVMoffsetof(CVMArrayOfAnyType, elems) +
	CVMbasicTypeSizes[typeCode] * arrayLen);

    newSpace = CVMfixedspaceAllocAndZeroOut(arrayObjectSize +
					    sizeof(CVMArrayOfAnyTypeICell));

    /*
     * Caller responsible for handling failed allocation
     */
    if (newSpace == 0) {
	return 0;
    }
    newArray = (CVMArrayOfAnyType*)((char*)newSpace +
				    sizeof(CVMArrayOfAnyTypeICell));
    fakeICell = (CVMArrayOfAnyTypeICell*)newSpace;

    /* Set header of new object */
    newArray->hdr.clas = (CVMClassBlock*)
	((CVMUint32)arrayCb | CVM_OBJECT_IN_ROM_MASK);
    newArray->length   = arrayLen;

    /* 
     * Set the 'fake' indirection cell to point to the object 
     */
    CVMD_gcUnsafeExec(ee, {
	CVMID_icellSetDirect(ee,
			     (CVMObjectICell*)fakeICell,
			     (CVMObject*)newArray);
    });
    return fakeICell;
}
