/*
 * @(#)ThreadSuspend.java	1.3 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

import java.util.Random;

class ThreadSuspend extends Thread {
    static boolean verbose = false;

    static public void main(String args[]) {
	int threads = 20;
	int index = 0;
	while (index < args.length) {
	    if (args[index].equals("-verbose")) {
		verbose = true;
	    } else {
		break;
	    }
	    ++index;
	}
	if (index < args.length ) {
	    threads = Integer.parseInt(args[index]);
	}

	long delay = 1000;
	long total;
	while (true) {
	    long s = System.currentTimeMillis();
	    spin(delay);
	    total =  System.currentTimeMillis() - s;
	    if (total >= 500) {
		delay = (delay * 500) / total;
		break;
	    } else {
		delay = delay * 2;
	    }
	}
	long s = System.currentTimeMillis();
	spin(delay);
	total =  System.currentTimeMillis() - s;
	if (verbose) {
	    System.err.println("Delay of " + delay + " for " + total + " ms");
	}

	Thread[] threadArray = new Thread[threads];
	for (int i = 0; i < threads; ++i) {
	    threadArray[i] = new ThreadSuspend(delay);
	    threadArray[i].start();
	}

	Random r = new Random();

	while (true) {

	    int i = r.nextInt(threads);
	    if (verbose) {
		System.err.println("ThreadSuspend suspending thread " + i);
	    }
	    /*threadArray[i].suspend();
	    if (verbose) {
		System.err.println("ThreadSuspend performing GC");
	    }
	    */
	    System.gc();
	    if (verbose) {
		System.err.println("ThreadSuspend resuming thread " + i);
	    }
	    /*
	    threadArray[i].resume();
	    if (verbose) {
		System.err.println("ThreadSuspend performing GC");
	    }
	    System.gc();
	    */

	    try {
		if (verbose) {
		    System.err.println("ThreadSuspend sleeping...");
		}
		Thread.sleep(10 * threads);
		if (verbose) {
		    System.err.println("ThreadSuspend done sleeping...");
		}
	    } catch (Exception e) {
	    }
	}
    }

    private long delay;

    public ThreadSuspend(long delay) {
	this.delay = delay;
    }

    public static void spin(long v) {
	for (int i = 0; i < v; ++i) {
	    continue;
	}
    }

    public void run() {
	setPriority(5);
	Random r = new Random();
	while (true) {
	    if (r.nextBoolean()) {
		if (verbose) {
		    System.err.println("Thread " + this + " spinning...");
		}
		// spin (gc-unsafe)
		spin(delay);
	    } else {
		// sleep (gc-safe)
		if (verbose) {
		    System.err.println("Thread " + this + " sleeping...");
		}
		try {
		    Thread.sleep(500);
		} catch (Exception e) {
		}
	    }
	}
    }
}
