/*
 * @(#)CVMWriter.java	1.8 00/11/09
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */
package runtime;
import Const;
import CVMConst;
import components.*;
import vm.*;
import jcc.Util;
import util.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.OutputStream;
import sun.misc.Compare;
import sun.misc.Sort;



/*
 * The CoreImageWriter for the Embedded VM
 */

public class CVMWriter implements CoreImageWriter, Const, CVMConst {

    /* INSTANCE DATA */
    protected  String    outputFileName;
    protected  Exception failureMode = null; // only interesting on falure


    protected CCodeWriter 	  out;
    protected OutputStream        xxx;

    protected CVMStringTable	  stringTable;

    protected String 		  headerFileName = null;
    protected BufferedPrintStream headers;
    protected OutputStream         yyy;

    private   VMClassFactory     classMaker = new CVMClassFactory();

    //CConstants sc;
    boolean	   formatError = false;
    boolean	   verbose     = false;
    boolean	   classDebug  = false;

    private static final String		staticInitThreadStoreName="StaticInitThreadStorage";
    private static final String		stringArrayName = "CVM_ROMStrings";
    private static final String		masterStringArrayName = "CVM_masterROMStrings";
    private static final String		staticStoreName = "CVM_staticData";
    private static final String		masterStaticStoreName = "CVM_masterStaticData";

    private ClassnameFilterList         nativeTypes;
    private ClassnameFilter		invisibleClassList = new ClassnameFilter();

    private boolean			classLoading = true; // by default.

    private boolean			doShared;
    private String			sharedConstantPoolName;
    private int				sharedConstantPoolSize;

    private CVMMethodType[]		methodTypes;
    private int[]			nMethodTypes;
    private int				totalMethodTypes;

    //
    // to keep track of which variants of classblock have been declared
    // see declareClassblock
    //
    private BitSet		classblockSizes = new BitSet();
    // to keep track of which variants of method arrays have been declared
    // see declareMethodArray
    private BitSet		methodArraySizes = new BitSet();
    // to keep track of which variants of field arrays have been declared
    // see declareFieldArray
    private BitSet		fieldArraySizes = new BitSet();

    /*
     * Per-class data that is built up as we
     * print out the various data structures.
     * Rather than trying to pass around both name and size of each,
     * we collect the information here.
     */
    private String			classBlockName;
    private String			constantPoolName;
    private int				constantPoolSize;
    private String			methodTableName;
    private int				methodTableSize;
    private String			checkedExceptionName;
    private String			interfaceTableName;
    private int				interfaceTableSize;
    private String			fieldTableName;
    private int				fieldTableSize;


    /* for statistics only */
    int  ncodebytes;
    int  ncatchframes;
    int  nmethods;
    int  nfields;
    int  nconstants;
    int	 njavastrings;

    public CVMWriter( ){ 
	stringTable = new CVMStringTable();
    }

    public void init( boolean classDebug, ClassnameFilterList nativeTypes, boolean verbose, int maxSegmentSize ){
	this.verbose = verbose;
	this.classDebug = classDebug;
	this.nativeTypes = nativeTypes;
    }

    // If attribute starts with attributeName, then parse the rest of the
    // string as a list of classes.
    private boolean parseClassListAttribute(String attributeName, 
					    String attribute, 
					    ClassnameFilter filter) { 
        if (!attribute.startsWith(attributeName))
	    return false;
	String val = attribute.substring(attributeName.length() );
	java.util.StringTokenizer tkn = 
	    new java.util.StringTokenizer(val, " ,", false );
	while ( tkn.hasMoreTokens() ){
	    String classname = tkn.nextToken().replace('.', '/');
	    filter.includeName(classname);
	}
	return true;
    }

    private boolean
    isClassInvisible( String classname ) {
        return invisibleClassList.accept(null, classname);
    }

    private boolean
    isClassCNI( String classname) { 
	// compare against interned string
	return nativeTypes.isType( classname, "CNI" );
    }

    private boolean
    isClassJNI( String classname) { 
	// compare against interned string
	return nativeTypes.isType( classname, "JNI" );
    }

    public boolean setAttribute( String attribute ){
	if ( parseClassListAttribute("invisible=", attribute,
				     invisibleClassList)){
	    return true;
	} else if ( attribute.equals("noClassLoading" ) ){
	    classLoading = false;
	    return true;
	} else
	    return false; // no attribute by this name.
    }


    public boolean open( String filename ){
	if ( out != null ) close();
	outputFileName = filename;
	if ( filename == null ){
	    xxx = System.out;
	    out = new CCodeWriter( xxx );
	    headerFileName = "jcc.output.h";
	} else {
	    try {
		xxx = new java.io.FileOutputStream( filename );
		out = new CCodeWriter( xxx );
	    } catch ( java.io.IOException e ){
		failureMode = e;
		return false;
	    }
	    headerFileName = filename+".h";
	}
	try {
	    yyy = new java.io.FileOutputStream( headerFileName );
	    headers = new BufferedPrintStream( yyy );
	} catch ( java.io.IOException e ){
	    failureMode = e;
	    return false;
	}
	return true;
    }

    public void close(){
	if ( out == null ) return;
	out.close();
	outputFileName = null;
	out = null;
	return;
    }

    public void printError( java.io.PrintStream o ){
	if ( failureMode != null ){
	    failureMode.printStackTrace( o );
	} else {
	    if ( out != null && out.checkError() )
		o.println(outputFileName+": Output write error");
	}
    }


    protected void declare( String declaration ){
	headers.print( declaration );
	headers.write(';');
	headers.write('\n');
	out.print( declaration );
    }

    private void writeCode(CVMMethodInfo meth) {
	MethodInfo mi = meth.method;
	LineNumberTableEntry[] lntab = mi.getLineNumberTable();
	LocalVariableTableEntry[] locvartab = mi.getLocalVariableTable();
	// for a Java method, write the CVMJavaMethodDescriptor 
	// and the code.
	int codeLengthRoundup = (mi.code.length+3)&~3;
	int nTryCatch = (mi.exceptionTable==null) ? 0 
	       					  : mi.exceptionTable.length;
	int nLineNumbers = (classDebug && lntab != null) ? lntab.length : 0;
	int nLocals = (classDebug && locvartab != null) ? locvartab.length : 0;
	out.println("\n/* Code block for "+meth.method.qualifiedName()+" */");
	out.print("STATIC ");
	/* For now, methods are all writable since stackmap
           computation is done at runtime. */
	if (false) try {
	    if ( meth.isCodePure() ) out.print( "const ");
	} catch ( DataFormatException e ){
	    // ignore.
	}
	out.println("struct {\n    union {");
	out.println("        CVMJavaMethodDescriptor jmd;");
	try {
	    out.println("        CVMUint" +
			((meth.alignment() == 1) ? "8" : "32") +
			" align;\n    } u;");
	} catch (DataFormatException e) {
	    // What is the right thing to do here?
	}
	out.println("    CVMUint8 data["+codeLengthRoundup+"];");
	if ( nTryCatch != 0 ){
	    out.println("    CVMExceptionHandler exceptionTable["+
			nTryCatch+"];");
	}
	if ( nLineNumbers != 0 && classDebug ){
	    out.println("    CVMLineNumberEntry lineNumberTable["+
			nLineNumbers+"];");
	}
	if ( nLocals != 0 && classDebug){
	    out.println("    CVMLocalVariableEntry localVariableTable["+
			nLocals+"];");
	}
	out.print("} "+meth.getNativeName()+
		  " = {\n    { CVM_INIT_JAVA_METHOD_DESCRIPTOR(");
	/*
	 * CVM Note: make sure locals is big enough for any return value.
	 */
	out.print(mi.stack+", "+Math.max(2,mi.locals)+", ");
	out.print(nTryCatch+", "+mi.code.length+", 0");
	out.print(", "+nLineNumbers+", "+nLocals);
	out.print(") },\n    {");

	// output the code array
	int j = 99;
	byte[] methodCode = mi.code;
	int l = methodCode.length;
	for ( int i = 0; i <l; i++ ){
	    if ( j >= 12 ){
		out.print("\n\t");
		j = 1;
	    } else {
		j+= 1;
	    }
	    out.printHexInt( methodCode[i]& 0xff );
	    out.print(",");
	}
	out.print("\n    }");
	ncodebytes += mi.code.length;

	// output the CVMExceptionHandler[] (catch frames)
	if ( nTryCatch != 0 ){
	    out.println(",\n    {");
	    for ( int i = 0; i < nTryCatch; i++ ){
		ExceptionEntry e = mi.exceptionTable[i];
		out.println("\t{ "+e.startPC+", "+e.endPC+", "+e.handlerPC+", "
			    +(e.catchType == null ? 0 : e.catchType.index)
			    +"},");
	    }
	    out.print("    }");
	    ncatchframes += nTryCatch;
	}

	// output the CVMLineNumberEntry[]
	if ( nLineNumbers != 0 ){
	    out.println(",\n    {");
	    for ( int i = 0; i < nLineNumbers; i++ ){
		LineNumberTableEntry e = lntab[i];
		out.println("\t{ "+e.startPC+", "+e.lineNumber+"},");
	    }
	    out.print("    }");
	}

	// output the CVMLocalVariableEntry[]
	if ( nLocals != 0 ){
	    out.println(",\n    {");
	    for ( int i = 0; i < nLocals; i++ ){
		LocalVariableTableEntry e = locvartab[i];
		int typeid = CVMDataType.parseSignature(e.sig.string);
		int nameid = CVMMemberNameEntry.lookupEnter(e.name.string);
		String nameAndType = "0x"+Integer.toHexString((nameid<<16)+typeid);
		out.println("\t{ "+e.pc0+", "+e.length+", "+e.slot+
			    ", "+ nameAndType+"},");
	    }
	    out.print("    }");
	}

	out.println("\n};");
    }


    private void writeImplements(ClassClass[] classes) 
                         { 
				// later
    }

    private void declareMethodArray( CVMClass c ){
	String thisTableName = c.getNativeName()+"_methods";
	int nmethods = c.methods.length;
	String methodArrayFlavor = "struct CVMMethodArray"+nmethods;
	if ( !methodArraySizes.get( nmethods ) ){
	    // need to declare a structure containing a classblock pointer
	    // plus "nmethods" CVMMethodBlock structures
	    headers.println(methodArrayFlavor+" { CVMClassBlock const *cb; CVMMethodBlock  mb[ "+(nmethods)+"]; };");
	    methodArraySizes.set( nmethods );
	    }
	declare( "const "+methodArrayFlavor+" "+thisTableName);
	methodTableName = "&"+thisTableName;
	methodTableSize = nmethods;
    }

    void writeMethods(CVMClass c) {
	CVMMethodInfo m[] = c.methods;
	int nCheckedExceptions = 0;
	int currCheckedException = 0;
	int checkedExceptionVector[];
	if ( (m == null) || (m.length == 0 )){
	    methodTableName = "0";
	    checkedExceptionName = "0";
	    methodTableSize = 0;
	    return;
	}
	int nmethod =  m.length;
	String thisTableName = c.getNativeName()+"_methods";
	String thisExceptionName = null;
	for ( int i = 0; i < nmethod; i++ ){
	    CVMMethodInfo meth = m[i];
	    MethodInfo    mi   = meth.method;
	    if ( mi.exceptionsThrown != null ){
		int n = mi.exceptionsThrown.length;
		if ( n != 0 )
		    nCheckedExceptions += n+1; // "1" is for the count word.
	    }
	    if ( mi.code != null ){
		writeCode( meth );
	    }
	}
	//
	// if any methods declare that they throw any exceptions,
	// we need to construct the exception vector.
	if ( nCheckedExceptions == 0 ){
	    checkedExceptionName = "0";
	    checkedExceptionVector = null;
	} else {
	    thisExceptionName = c.getNativeName()+"_throws";
	    checkedExceptionName = thisExceptionName;
	    // Ensure that item 0 in this array is always unused.
	    // See classes.h.
	    nCheckedExceptions++;
	    checkedExceptionVector = new int[ nCheckedExceptions ];
	    checkedExceptionVector[0] = 0;
	    currCheckedException = 1;
	}
	declareMethodArray( c );
	out.print( " = {\n    &");
	out.print( classBlockName+",\n    {\n" );

	// Include JNI header file if there are native methods
	for ( int i = 0; i < nmethod; i++ ){
	    CVMMethodInfo meth = m[i];
	    int access = meth.CVMflags();
	    if ((access&CVM_METHOD_ACC_NATIVE)!=0) {
		String subdir = isClassCNI(c.ci.className) ? "cni/" : "jni/";
		headers.println("#include \"generated/" + subdir +
		    Util.convertToClassName(c.ci.className)+".h\"");
		break;
	    }
	}

	for ( int i = 0; i < nmethod; i++ ){
	    CVMMethodInfo meth = m[i];
	    int typeid = meth.sigType.entryNo;
	    int nameid = CVMMemberNameEntry.lookupEnter(meth.method.name.string);
	    String nameAndType = "0x"+Integer.toHexString((nameid<<16)+typeid);
	    int access = meth.CVMflags();

	    //System.out.println( nameAndType+" => "+Integer.toHexString(typeid) );

	    boolean synchro = (access&CVM_METHOD_ACC_SYNCHRONIZED)!=0;
	    boolean isNative  = (access&CVM_METHOD_ACC_NATIVE)!=0;
	    boolean isAbstract= (access&CVM_METHOD_ACC_ABSTRACT) != 0;

	    int invoker = isAbstract ? CVM_INVOKE_ABSTRACT_METHOD
			  : isNative ? (
				isClassCNI(c.ci.className) ?
				    CVM_INVOKE_CVM_METHOD : (
				synchro ? CVM_INVOKE_JNI_SYNC_METHOD
					: CVM_INVOKE_JNI_METHOD ))
			  : (
				synchro ? CVM_INVOKE_JAVA_SYNC_METHOD
					: CVM_INVOKE_JAVA_METHOD );

	    //
	    // note the offset in the exception array,
	    // and add to that array as necessary.
	    int exceptions;
	    if ( (meth.method.exceptionsThrown != null) && (meth.method.exceptionsThrown.length!=0) ){
		ClassConstant thrown[] = meth.method.exceptionsThrown;
		int net = thrown.length;
	        exceptions = currCheckedException;
		checkedExceptionVector[currCheckedException++] = net;
		for ( int j = 0; j < net; j++ ){
		    checkedExceptionVector[currCheckedException++] = thrown[j].index;
		}
	    } else {
		// Note we have changed this from ((unsigned short)
		// -1) to 0 to be consistent with the documentation in
		// classes.h. A checkedExceptionsOffset of 0 indicates
		// no checked exceptions for the method. Also see the
		// allocation of the checkedExceptionVector above.
		exceptions = 0;
	    }
	    int argssize   = meth.method.argsSize;
	    int tableIndex = meth.methodOffset();
	    out.print("	CVM_INIT_METHODBLOCK("+nameAndType+","+tableIndex+","+argssize+","+invoker+","+access+","+i+","+exceptions+",");
	    if ( isAbstract ){
		out.print("0),\n");
	    } else if ( isNative ){
		if (isClassCNI(c.ci.className)) {
		    String jniName = meth.method.getNativeName(true);
		    String eniName = "CNI" + jniName.substring(5);
		    out.print("(CVMJavaMethodDescriptor*)&"+eniName+"),\n");
		} else {
		    out.print("(CVMJavaMethodDescriptor*)&"+
			meth.method.getNativeName(true)+"),\n");
		}
	    } else {
		out.print("&("+meth.getNativeName()+".u.jmd)),\n" );
	    }
		
	}
	out.print("}};\n");
        this.nmethods += nmethod;
	//
	// now, if there are any exception-catch clauses
	// that need to be represented, print out the table.
	if ( nCheckedExceptions != 0 ){
	    out.println("STATIC const CVMUint16 "+thisExceptionName+"[] = {");
	    int j = 0;
	    for ( int i = 0; i < nCheckedExceptions; i++ ){
		if ( j >= 8 ){
		    out.print("\n    ");
		    j = 0;
		} else {
		    j += 1;
		}
		out.print( checkedExceptionVector[i]+", ");
	    }
	    out.println("\n};");
	}
    }

    void mungeAllIDsAndWriteExternals(ClassClass classes[]) { 
    }

    private static int CVMDataAccessFlags( int access ){
	int a = 0;
	if ( (access&ACC_PUBLIC) != 0 ) a |= CVM_FIELD_ACC_PUBLIC;
	if ( (access&ACC_PRIVATE) != 0 ) a |= CVM_FIELD_ACC_PRIVATE;
	if ( (access&ACC_PROTECTED) != 0 ) a |= CVM_FIELD_ACC_PROTECTED;
	if ( (access&ACC_STATIC) != 0 ) a |= CVM_FIELD_ACC_STATIC;
	if ( (access&ACC_FINAL) != 0 ) a |= CVM_FIELD_ACC_FINAL;
	if ( (access&ACC_VOLATILE) != 0 ) a |= CVM_FIELD_ACC_VOLATILE;
	if ( (access&ACC_TRANSIENT) != 0 ) a |= CVM_FIELD_ACC_TRANSIENT;
	return a;
    }

    private void declareFieldArray( CVMClass c ){
	// Note: Should deal with > 255 fields.
	// Today is not that day. Just do the simple case.
	FieldInfo m[] = c.ci.fields;
	int nfields = m.length;
	String thisTableName = c.getNativeName()+"_fields";
	String fieldArrayFlavor = "struct CVMFieldArray"+nfields;
	if ( !fieldArraySizes.get( nfields ) ){
	    // need to declare a structure containing a classblock pointer
	    // plus "nfields" CVMFieldBlock structures
	    headers.println(fieldArrayFlavor+" { CVMClassBlock const *cb; CVMFieldBlock  fb[ "+(nfields)+"]; };");
	    fieldArraySizes.set( nfields );
	    }
	declare( "const "+fieldArrayFlavor+" "+thisTableName+"[1]");
	fieldTableName = "&"+thisTableName;
	fieldTableSize = nfields;
    }

    void writeFields(CVMClass c) {
	// Note: Should to deal with > 255 fields.
	// Today is not that day. Just do the simple case.
	FieldInfo m[] = c.ci.fields;
	if ( (m == null) || (m.length== 0)){
	    fieldTableName = "0";
	    fieldTableSize = 0;
	    return;
	}
	int nfields = m.length;
	declareFieldArray( c );
	out.print( " = {{\n    &");
	out.print( classBlockName+",\n    {\n" );
	for ( int i = 0; i < nfields; i++ ){
	    FieldInfo f = m[i];
	    int typeid = CVMDataType.parseSignature(f.type.string);
	    int nameid = CVMMemberNameEntry.lookupEnter(f.name.string);
	    String nameAndType = "0x"+Integer.toHexString((nameid<<16)+typeid);
	    //System.out.println( nameAndType+" => "+Integer.toHexString(typeid) );
	    int access = CVMDataAccessFlags( f.access );
	    int offset;
	    offset = f.instanceOffset;
	    // static members use simple byte offset
	    // instance members include size of header info
	    if ( f.isStaticMember() ){
		out.print("	CVM_INIT_FIELDBLOCK("+nameAndType+","+access+","+i+", STATIC_OFFSET("+offset+")),\n");
	    } else {
		out.print("	CVM_INIT_FIELDBLOCK("+nameAndType+","+access+","+i+", FIELD_OFFSET("+offset+")),\n");
	    }
	}
	out.println("}}};");
	this.nfields += nfields;

    }

    private static String methodBlockAddress( MethodInfo mi ){
	CVMMethodInfo em = (CVMMethodInfo)(mi.vmMethodInfo);
	CVMClass      ec = (CVMClass)(mi.parent.vmClass);
	return "&"+ec.getNativeName()+"_methods.mb["+mi.index+"]"; //  might be interface?
    }

    private static String fieldBlockAddress( FieldInfo f ){
	// pretty disgusting.
	// if this is frequent enough, find a better way!
	ClassInfo c = f.parent;
	FieldInfo cf[] = c.fields;
	int n = f.index;
	//for ( n = 0; cf[n] != f; n++ ) // TRY THIS.
	 //   ;
	// assume we found it.
	String parentName = ((CVMClass)(c.vmClass)).getNativeName();
	return "&"+parentName+"_fields[0].fb["+n+"]";
    }

    private static String staticFieldAddress( FieldInfo f ){
	CVMClass c = (CVMClass)(f.parent.vmClass);
	return "&"+staticStoreName+"[STATIC_OFFSET("+f.instanceOffset+")]";
    }

    private static final int CVMConstantMap[] = {
	0,
	CVM_CONSTANT_Utf8,	CVM_CONSTANT_Unicode,
	CVM_CONSTANT_Integer,	CVM_CONSTANT_Float,
	CVM_CONSTANT_Long,	CVM_CONSTANT_Double,
	CVM_CONSTANT_Class,	CVM_CONSTANT_String,
	CVM_CONSTANT_Fieldref,	CVM_CONSTANT_Methodref,
	CVM_CONSTANT_InterfaceMethodref, CVM_CONSTANT_NameAndType
    };

    private static int CVMconstantTag( ConstantObject c ){
	int t = CVMConstantMap[ c.tag ];
	if ( c.isResolved() ){
	    t |= CVM_CONSTANT_POOL_ENTRY_RESOLVED;
	}
	return t;
    }

    // return address of this type table.
    private String writeTypeTable( ConstantObject constants[], String name ){
	int clen = constants.length;
	String ttname = name+"_tt";
	out.print("CVMUint8 "+ttname+"[] = { 0,");
	int j = 99;
	for ( int i = 1; i < clen; i+=constants[i].nSlots ){
	    if ( j >= 10 ){
		out.print("\n    ");
		j = 1;
	    } else {
		j += 1;
	    }
	    out.print( CVMconstantTag(constants[i])+"," );
	    if ( constants[i].nSlots == 2 )
		out.print(" 0,"); // 2nd word of 2-word constant what should it be?
	}
	out.println("\n};");
	return "&"+ttname;
    }

    // return address of this constant pool.
    private String writeConstants( ConstantObject constants[], String name ){
	boolean doTypeTable = false;
	int clen = constants.length;
	if ( (clen == 0) || (clen == 1 ) ){
	    return "0";
	}
	for ( int i = 1; i < clen; i+=constants[i].nSlots ){
	    if ( ! constants[i].isResolved() ){
		doTypeTable = true;
		break;
	    }
	}
	if ( doTypeTable && !classLoading ){
	    // in future, do something more useful here.
	    System.err.println("A constant pool has unresolved constants, and class loading is not supported!");
	    doTypeTable = false; // ??
	}
	String typeTableName;
	if ( doTypeTable ){
	    typeTableName = writeTypeTable( constants, name );
	}else{
	    typeTableName = "0";
	}
	String constantPoolName = name+"_cp";
	out.print( doTypeTable ? "STATIC " : "STATIC const ");
	out.print("struct { ");
	if ( classLoading ){
	    out.print("CVMUint8 * cpTypes; ");
	}
	out.println("CVMInt32 entries["+(clen)+"]; } "+constantPoolName+" = {");
	if ( classLoading ){
	    out.println("    "+typeTableName+",");
	}
	out.println("    { 0,");
	for ( int i = 1; i < clen; i+=constants[i].nSlots ){
	    out.print("    ");
	    writeConstant( constants[i] );
	    out.println(",");
	}
	out.println("}};");
	nconstants += clen;
	return "&"+constantPoolName;
    }

    private void declareClassblock( CVMClass c, int vtblSize ){
	String myName = c.getNativeName();
	String classblockFlavor = "struct CVMClassBlock"+vtblSize;
	if ( !classblockSizes.get( vtblSize ) ){
	    // need to declare a classblock-like struct with vtbl elements in
	    // its methodblock array
	    if ( vtblSize <= 1 ){
		headers.println(classblockFlavor+" { CVMClassBlock classclass; };");
	    } else {
		headers.println(classblockFlavor+" { CVMClassBlock classclass; CVMMethodBlock * mtab[ "+(vtblSize-1)+"]; };");
	    }
	    classblockSizes.set( vtblSize );
	    }
	declare( "const "+classblockFlavor+" "+myName+"_Classblock");
    }

    // helper for classBitMap
    private static int GCbits( FieldInfo slottable[], int first, int last )
    {
	int mapval = 0;
	for ( int i = first; i < last; i++ ){
	    switch ( slottable[i].type.string.charAt(0) ){
	    case 'L':
	    case '[':
		// this is a reference.
		mapval |= 1<<(i-first);
	    }
	}
	return mapval;
    }

    //
    // compute the GC bit map for a class.
    // if it is big, then write it out as well. In any case,
    // return a string to use to initialize the gcMap field in the CVMClassBlock
    //
    private String classBitMap( CVMClass c ){
	if ( c.isArrayClass() ){
	    // either an array of primitive or an array of objects.
	    ArrayClassInfo ac = (ArrayClassInfo)(c.ci);
	    if ( (ac.depth==1) && (ac.baseType!=T_CLASS) )
		return "0"; // contains no references
	    else
		return "1"; // contains nothing but references.
	}
	ClassInfo ci = c.ci;
	FieldInfo ft[] = ci.slottable;
	if ( (ft==null) || (ft.length==0) ) return "0";
	int nslots;
	if ( (nslots=ft.length) <= 30 ){
	    // easy case. Small object. Map can be represented in a word.
	    return "0x"+Integer.toHexString(GCbits(ft,0,nslots)<<2);
	} else {
	    // hard case.
	    String myName = c.getNativeName()+"_gcBits";
	    int    nWords = (nslots+31)/32;
	    out.print( "STATIC const struct { CVMUint16 maplen; CVMUint32 map["+nWords+"]; } ");
	    out.print( myName );
	    out.print( " = { ");
	    out.print( nWords );
	    out.print( ",\n    {" );
	    for ( int i = 0; i < nslots; i+= 32 ){
		out.print( "0x"+Integer.toHexString(GCbits( ft, i, Math.min( i+32, nslots))) );
		out.print(", ");
	    }
	    out.println("} };");
	    return "((CVMUint32)&"+myName+") + 0x2";
	}
    }

    private void writeClassinfo( int classno, CVMClass c ){

	String	   superName = ( c.ci.superClassInfo == null )? "0" : ("&"+cbName( (CVMClass)(c.ci.superClassInfo.vmClass) ) );
	
	String     myStaticName = (c.nStaticWords==0)?"0":("(CVMJavaVal32*)(&"+staticStoreName+")");
	MethodInfo vtbl[] = c.ci.methodtable;
	int        vtblSize;
	int	   methodTableCount;
	int	   romindex = c.hasStaticInitializer ? ++maxROMindex : 0;
	String	   gcBits;
	/*
	 * Note: If this is a primitive class, the
	 * typecode is stored in the methodTableCount field.
	 * Since these things cannot be instantiated, they need
	 * no virtual methods.
	 */
	if ( c.isPrimitiveClass() ){
	    vtblSize = 0;
	    methodTableCount = c.typecode();
	} else {
	    // vtbl == null may occur for array classes! 
	    vtblSize = (vtbl==null)?0:vtbl.length;
	    methodTableCount = vtblSize;
	}
	// classBitMap may do output, so it needs
	// to be called before declareClassblock!
	gcBits = classBitMap( c );

	declareClassblock( c, vtblSize );

	int classid = CVMDataType.lookupClassname(c.ci.className);
	//System.out.println( c.ci.className+" => "+Integer.toHexString(classid) );

	out.println(" = {\n    CVM_INIT_CLASSBLOCK( "+gcBits+", " +
		    classid+",");
	out.println("    "+superName+",");
	out.println("    "+constantPoolName+", ");
	out.println("    "+interfaceTableName+",");
	out.println("    "+methodTableName+",");
	out.println("    "+fieldTableName+", "+myStaticName+",");
	out.println("    "+constantPoolSize+", "+methodTableSize+", "+
		    fieldTableSize+", "+methodTableCount+
		    ", OBJ_SIZE("+c.instanceSize()+"), "+
		    c.CVMflags()+", "+romindex+", 0,");
	out.println("    (CVMClassICell*)(&CVM_classICell["+classno+"]), "+
		    "&CVM_clinitEE["+classno+"],");
	out.println("    "+checkedExceptionName+", ");
	if (classDebug && c.ci.sourceFileAttr != null) {
	    out.println("    \""+c.ci.sourceFileAttr.sourceName+"\", ");
	} else {
	    out.println("    0, ");
	}

	if ( vtblSize==0 ){
	    // array classes will end here.
	    // unless we copy the java.lang.Object vtbl.
	    // primitive classes, too. They cannot be instantiated.
	    out.println("    0)\n};");
	    return;
	}
	out.print("    (CVMMethodBlock*)"+methodBlockAddress( vtbl[0] )+")");
	if (vtblSize > 1) {
	    out.println(", { ");
	    for ( int i = 1; i < vtblSize; i++ ){
		out.println( "        (CVMMethodBlock*)"+
			   methodBlockAddress( vtbl[i] )+",");
	    }
	    out.println("    }");
	}
	out.println("};");
    }

	
    private void
    writeIntegerValue( int v ){
	 // little things make gcc happy.
	 if (v==0x80000000)
	    out.print( "(CVMInt32)0x80000000" );
	else
	     out.print( v );
    }

    private void
    writeLongValue( String tag, int highval, int lowval ){
	out.print( tag );
	out.write( '(' );
	writeIntegerValue( highval );
	out.print( ", " );
	writeIntegerValue( lowval );
	out.write( ')' );
    }

    private void
    writeConstant( ConstantObject value ){
	switch ( value.tag ){
	case Const.CONSTANT_INTEGER:
	case Const.CONSTANT_FLOAT:
	     writeIntegerValue(((SingleValueConstant)value).value);
	     break;
	case Const.CONSTANT_UTF8:
	    out.print("(CVMInt32)");
	    out.write('"');
	    out.print( Util.unicodeToUTF(((UnicodeConstant)value).string ));
	    out.write('"');
	    // also, not sharing here at all.
	    break;
	case Const.CONSTANT_STRING:
	    out.print("(CVMInt32)&"+stringArrayName+"[");
	    out.print( ((StringConstant)value).unicodeIndex );
	    out.write(']');
	    break;
	case Const.CONSTANT_LONG:{
	    DoubleValueConstant dval = (DoubleValueConstant) value;
	    writeLongValue( "LONG",  dval.highVal, dval.lowVal );
	    }
	    break;
	case Const.CONSTANT_DOUBLE:{
	    DoubleValueConstant dval = (DoubleValueConstant) value;
	    writeLongValue( "DOUBLE",  dval.highVal, dval.lowVal );
	    }
	    break;
	case Const.CONSTANT_CLASS:
	    if ( value.isResolved() ){
		out.print("(CVMInt32)&"+cbName( (CVMClass)(((ClassConstant)value).find().vmClass)));
	    } else {
		out.print("Unresolved class constant: "+value.toString() );
	    }
	    break;
	case Const.CONSTANT_METHOD:
	case Const.CONSTANT_INTERFACEMETHOD:
	    if ( value.isResolved() ){
		out.print("(CVMInt32)");
		out.print(methodBlockAddress( ((MethodConstant)value).find()));
	    } else {
		out.print("Unresolved method constant: "+value.toString() );
	    }
	    break;
	case Const.CONSTANT_FIELD:
	    if ( value.isResolved() ){
		FieldInfo ftarget = ((FieldConstant)value).find();
		ClassInfo ctarget = ftarget.parent;
		out.print("(CVMInt32)");
		if ( ftarget.isStaticMember() && !(ctarget.vmClass.hasStaticInitializer)){
		    out.print(staticFieldAddress( ftarget));
		}else{
		    out.print(fieldBlockAddress( ftarget));
		}
	    } else {
		out.print("Unresolved field constant: "+value.toString() );
	    }
	    break;
	default:
	    // should never get here.
	}
    }

    /*
     * Merge the static store for all classes into one area.
     * Sort static cells, refs first, and assign offsets.
     * Write the resulting master data glob and arrange for
     * it to be copied to writable at startup.
     */
    void writeStaticStore( ClassClass cvec[] ){
	int nclass = cvec.length;
	int nStaticWords = 1; // 1 header word assumed
	int nRef    = 0;
	/*
	 * Count all statics.
	 * Count refs, and count number of words.
	 */
	for ( int cno = 0; cno < nclass; cno++ ){
	    CVMClass c = (CVMClass)(cvec[cno]);
	    c.orderStatics();
	    nRef += c.nStaticRef;
	    nStaticWords += c.nStaticWords;
	    FieldInfo f[] = c.statics;
	    if ( (f == null) || (f.length == 0 ) ) continue; // this class has none
	}
	/*
	 * Allocate writable space for the static store.
	 * Start writing the static storage header.
	 * Remember that the first word is the number of refs.
	 * And all those refs get initialized to zero.
	 */
	out.println("CVMInt32 "+staticStoreName+"["+nStaticWords+"];");
	out.print("STATIC const CVMInt32 "+masterStaticStoreName+"[] = {\n    ");
	out.print( nRef );
	out.println(",");
	int nRefInit = nRef;
	for( ; nRefInit>=8; nRefInit-=8 ){
	    out.println("    0, 0, 0, 0, 0, 0, 0, 0,");
	}
	int nConstPerLine = nRefInit;
	if ( nRefInit>0 ){
	    out.print("    ");
	    for( ; nRefInit>0; nRefInit-=1 ){
		out.print("0, ");
	    }
	}
	/*
	 * Assign offsets and at the same time
	 * write any initial values.
	 */
	int refOff = 1;
	int scalarOff = refOff+nRef;

	ConstantObject staticInitialValue[] = new ConstantObject[nStaticWords-scalarOff];
	for ( int cno = 0; cno < nclass; cno++ ){
	    CVMClass c = (CVMClass)(cvec[cno]);
	    FieldInfo f[] = c.statics;
	    if ( (f == null) || (f.length == 0 ) ) continue; // this class has none
	    int nFields = f.length;
	    for ( int i = 0; i < nFields; i++ ){
		FieldInfo fld = f[i];
		char toptype =  fld.type.string.charAt(0);
		if ( (toptype == 'L') || (toptype=='[')){
		    fld.instanceOffset = refOff;
		    refOff += 1;
		} else {
		    fld.instanceOffset = scalarOff;
		    scalarOff += fld.nSlots;
		    if ( fld.value == null ){
			out.print( "0");
			if (fld.nSlots == 2 )
			    out.print( ", 0");
		    } else {
			writeConstant( fld.value );
		    }
		    out.print(", ");
		    nConstPerLine += fld.nSlots;
		    if ( nConstPerLine >= 8 ){
			out.print("\n    ");
			nConstPerLine = 0;
		    }
		}
	    }
	}
	out.println("\n};");
	CVMInitInfo.addInfo( "&"+masterStaticStoreName, "&"+staticStoreName, nStaticWords+"*sizeof(CVMInt32)" );
    }

    int maxROMindex = 0;	// 
    private void
    writeRomMap(String name, String sizename) { 
        // The indices are 0 .. maxROMindex.  All classes that are already
        // initialized have index 0.  
        // So we set up an array in which the element 0 indicates "already
        // initialized", and the remainder are zero.

	out.println("CVMUint8 "+name+"[" + (1+maxROMindex) + "];");
	out.println("const int "+sizename+" = "+(1+maxROMindex)+";");

    }


    private boolean
    isRefClass( ClassInfo c ){
	while ( c != null ){
	    if ( c.className.equals("sun/misc/Ref"))
		return true;
	    c = c.superClassInfo;
	}
	return false;
    }


    /*
     * typedef struct {
     *     char *hash;			 * the string for this entry * 
     *     unsigned long info;		 * high bit set if malloc'ed
     * 				 * low 31 bits are the low 31 bits of hash * 
     * #define MALLOCED_FLAG (1u << 31)
     * #define HASHBIT_MASK (MALLOCED_FLAG - 1)
     * } StrIDhashSlot;
     *
     * typedef struct StrIDhash {
     *     int size;			* number of entries *
     *     hash_fn hash;		* hash function for this table *
     *     struct StrIDhash *next;	* next bucket *
     *     short used;			* number of full entries *
     *     short baseid;		* ID for item in slot[0] *
     *     void **params;		* param table, if needed *
     *     StrIDhashSlot slot[1];	* expanded to be number of slots *
     * } StrIDhash;
     */

    private String idTypeDecl[] = {
       "extern void default_hash();",
       "typedef struct {",
       "    const char *hash;",
       "    unsigned long info;",
       "} StrIDhashSlot;",
       "#define STR_ID_HASH(n)  struct { \\",
       "    int size; \\",
       "    void (*hash)(); \\",
       "    void *next; \\",
       "    short used; \\",
       "    short baseid; \\",
       "    HString **params; \\",
       "    StrIDhashSlot slot[ n ]; \\",
       "} "
    };

    /*
     * These are macros to encapsulate some of the messiness
     * of data definitions. They are, perhaps, compiler-specific.
     */
    private static String stdHeader[] = {
	"#define IN_ROMJAVA 1",
	"#include \"javavm/include/defs.h\"",
	"#include \"javavm/include/jni_impl.h\"",
	"#include \"javavm/include/objects.h\"",
	"#include \"javavm/include/classes.h\"",
	"#include \"javavm/include/interpreter.h\"",
	"#include \"javavm/include/typeid.h\"",
	"#include \"javavm/include/typeid_impl.h\"",
	"#include \"javavm/include/preloader_impl.h\"",

	"#define LONG(high, low) high, low",

	"#define DOUBLE(high, low) high, low",

	"#ifdef _MSC_VER",
	"#define STATIC /* static */",
	"#else",
	"#define STATIC static",
	"#endif",

	"#define SCALARTYPE_NO(n) ((struct scalarTableEntry*)(&CVMFieldTypeTable.data[n]))",
	//"#define MSIG_NO(n) ((struct methodTypeTableEntry*)&CVMMethodSigTable.typeSig[n])",
	// "#define NAME_NO(n) ((struct memberName*)&CVMMemberNames.data[n])",
	"#define SIG_DETAIL_NO(n) (&msig_details[n])",
	"#define PKG_NO(n) &CVM_ROMpackages[n]",

	"",

    };

    /* this is target-dependent and belongs in a header file... */
    private static String targetHeader[] = {
    };
	
    private void writePrologue(){

	for ( int i = 0; i < targetHeader.length; i++ ){
	    out.println( targetHeader[i] );
	}
	for ( int i = 0; i < stdHeader.length; i++ ){
	    out.println( stdHeader[i] );
	}
	if ( headerFileName != null ){
	    out.println("#include \""+headerFileName+"\"");
	}
    }

    private static String cbName( CVMClass c ){
	return c.getNativeName()+"_Classblock.classclass";
    }

    private void writeClass( int classno, CVMClass c ){

	out.println("\n/* ******** Class "+c.getNativeName()+" ******** */");

	classBlockName = cbName( c );

	writeMethods(c);
	// writeInterfaces(c);
	interfaceTableName = (c.inf==null)?"0":("(CVMInterfaces*)&"+c.inf.name);
	interfaceTableSize = 0;

	writeFields(c);

	if ( doShared ){
	    constantPoolName = sharedConstantPoolName;
	    constantPoolSize = sharedConstantPoolSize;
	} else {
	    ConstantObject cpool[] = c.ci.constants;
	    constantPoolName = writeConstants(cpool, c.getNativeName() );
	    constantPoolSize = cpool.length;
	}

	writeClassinfo(classno, c);

    }

    private void processStrings( ConstantObject cp[] ){
	if ( cp == null ) return;
	int n = cp.length;
	for ( int i = 1; i < n; ){
	    ConstantObject obj = cp[i];
	    if ( obj instanceof StringConstant ){
		stringTable.intern( (StringConstant) obj );
	    }
	    i += obj.nSlots;
	}
    }

    private void instantiateAllStrings(){
	njavastrings = stringTable.writeStrings( out, masterStringArrayName);
	out.println("struct java_lang_String "+stringArrayName+"["+njavastrings+"];");
	out.println("const int CVM_nROMStrings = "+njavastrings+";");
	CVMInitInfo.addInfo( "&"+masterStringArrayName, "&"+stringArrayName, njavastrings+"*sizeof(struct java_lang_String)" );

	stringTable.writeStringInternTable( out, "CVMInternTable", stringArrayName ); 
    }

    /*
     * Our representation of method types has four variations:
     *	0: inline form, inline details;
     *  1: inline form, outboard details;
     *  2: outboard form, inline details (rare);
     *  3: outboard form, outboard details, too.
     * For our own convenience, we are going to segregate them
     * by type. This affects the indexing scheme, so we must
     * assign that, too, after the sort.
     */
    private static int
    variety( CVMMethodType mt ){
	int v = 0;
	if ( mt.form.nSyllables> CVMSigForm.SYLLABLESPERDATUM) v+= 2;
	if ( mt.nDetails > 2 ) v += 1;
	return v;
    }

    private void
    enumerateMethodTypes(){
	methodTypes = new CVMMethodType[4];
	nMethodTypes = new int[4];
	Enumeration e = CVMMethodType.methodTypes.elements();
	while ( e.hasMoreElements() ){
	    CVMMethodType mt = (CVMMethodType)(e.nextElement() );
	    int v = variety( mt );
	    mt.listNext = methodTypes[v];
	    methodTypes[v] = mt;
	    nMethodTypes[v] += 1; }
	int curTypeNo = 0;
	for ( int i = 0; i < 4; i++ ){
	    for( CVMMethodType mt = methodTypes[i]; mt!= null; mt = mt.listNext ){
		mt.entryNo = curTypeNo++;
	    }
	}
	totalMethodTypes = curTypeNo;
    }

    /*
     * The CVM type structures are built up as a side effect of
     * turning signatures and names into 16-bit cookies while writing out
     * other structures. Thus they must be written AFTER all the entries are
     * computed. Fortunately there will be no forward-references to them,
     * as all references are by index values, not addresses.
     */
    private void writeCVMNameAndTypeTables(){
	writeCVMDataTypeTable();
	writeCVMpkg();
	writeCVMMethodTypeTable();
	writeCVMMemberNameTable();
    }

    /*
     * The value-type table is one of the larger type structures, as it also
     * holds class names. It has variant records. It contains an internal linked
     * list for lookup-by-name. It has reference counts, but we will set these
     * to the magic maximum value so that this can be READONLY.
     */
    private void writeCVMDataTypeTable(){
	Vector allDataTypes = CVMDataType.dataTypes; 
	int    nData = allDataTypes.size();
	/*
	 * May need to do an iteration through to determine the layout, in case
	 * we have some trouble initializing union types and have to do more
	 * specific declarations!
	 */
	out.println("STATIC struct scalarTableSegment * dummyNextCell;");
	out.println("STATIC const struct scalarTableSegment * masterDummyNext = 0;");
	CVMInitInfo.addInfo( "&masterDummyNext", "&dummyNextCell", "sizeof(dummyNextCell)" );
	out.println("const struct { SCALAR_TABLE_SEGMENT_HEADER struct scalarTableEntry data["+nData+"]; }\nCVMFieldTypeTable = {" );
	out.println("\t"+(CVMTypeCode.CVMtypeLastScalar+1)+", 0, -1,"+nData+", &dummyNextCell, {" );
	for ( int i = 0; i < nData; i++ ){
	    CVMDataType o = (CVMDataType)allDataTypes.elementAt( i );
	    if ( o != null && i != o.entryNo ){
		System.out.println("Datatype entry "+o+" misplaced: is at vector["+i+"] and expected at ["+o.entryNo+"]");
	    }
	    if ( o instanceof CVMClassDataType ){
		CVMClassDataType ctype = (CVMClassDataType) o;
		String next;
		next = (ctype.next == null ) ? "0" : ("SCALARTYPE_NO("+ctype.next.entryNo+")");
		// int nameHash = CVMDataType.computeHash(ctype.classInPackage);
		String UTFname = Util.unicodeToUTF( ctype.classInPackage );
		out.println("	INIT_CLASSTYPE( "+next+", PKG_NO("+ctype.pkg.entryNo+"), \""+UTFname+"\"),");
	    } else {
		CVMArrayDataType atype = (CVMArrayDataType) o;
		String next;
		next = (atype.next == null ) ? "0" : ("SCALARTYPE_NO("+atype.next.entryNo+")");
		out.println("	INIT_ARRAYTYPE( "+next+", "+atype.depth+", "+atype.baseType+"),");
	    }
	}
	out.println("}};");

    }

    /*
     * The package name data structures and hash tables have to go after the
     * general value-type table, as it contains little hash tables 
     * of pointers to the latter.
     * These are used for lookup by class name.
     */
    private void writeCVMpkg(){
	Vector allPackages = CVMpkg.pkgVector; 
	int    nPkg = allPackages.size();

	headers.println("STATIC struct pkg CVM_ROMpackages["+(nPkg)+"];" );
	out.println("STATIC const struct pkg CVM_masterROMpackages["+(nPkg)+"] = {" );
	for ( int i= 0; i < nPkg; i++ ){
	    CVMpkg p = (CVMpkg) (allPackages.elementAt(i));
	    writeAnCVMpkg( p );
	    /* DEBUG */
	    if ( p.entryNo != i ){
		System.out.println("Package entry "+p.pkgName+" misplaced: is at vector["+i+"] and expected at ["+p.entryNo+"]");
	    }
	}
	out.println("};");

	CVMInitInfo.addInfo( "CVM_masterROMpackages", "CVM_ROMpackages", nPkg+"*sizeof(struct pkg)" );

	// that was the packages themselves.
	// this is the hash table that is the basis of looking them up.
	// this needs to be writable, since we add at the front of the
	// linked list.
	out.print("STATIC const struct pkg *CVM_masterPkgHashtable["+CVMpkg.NPACKAGEHASH+"] = {");
	for ( int i = 0; i< CVMpkg.NPACKAGEHASH; i++ ){
	    CVMpkg p = CVMpkg.packages[i];
	    if ( i%5 == 0 ){
		out.print("\n\t");
	    }
	    out.print( (p==null)?"0, ":("PKG_NO("+(p.entryNo)+"), "));
	}
	out.println("\n};");
	out.println("struct pkg *CVM_pkgHashtable["+CVMpkg.NPACKAGEHASH+"];");
	CVMInitInfo.addInfo( "CVM_masterPkgHashtable", "CVM_pkgHashtable", CVMpkg.NPACKAGEHASH+"*sizeof(struct pkg*)" );

	out.println("struct pkg *CVMnullPackage = PKG_NO(0);");
    }

    private void writeAnCVMpkg( CVMpkg p ){
	String next = (p.next==null)?"0" : ("PKG_NO("+p.next.entryNo+")");
	String UTFname = Util.unicodeToUTF( p.pkgName );
	//int    nameHash = CVMDataType.computeHash(p.pkgName);
	out.print("    { \""+UTFname+"\", "+next+", {");
	for( int i = 0; i < CVMpkg.NCLASSHASH; i++ ){
	    String hashp = (p.typeData[i]==null)? "0" : ("SCALARTYPE_NO("+p.typeData[i].entryNo+")");
	    if ( i%3==0){
		out.print("\n\t");
	    }
	    out.print(hashp);
	    out.print(", ");
	}
	out.println("\n    }},");
    }

    /*
     * The CVMMethodTypes are complex, multi-part beasts.
     * The first part is the Form or Terse Signature,
     * Then we need the details array
     * And finally the table of method types. There are 4 kinds of them!:
     *	0: inline form, inline details;
     *  1: inline form, outboard details;
     *  2: outboard form, inline details (rare);
     *  3: outboard form, outboard details, too.
     * They are all the same size!
     */
    public void writeCVMMethodTypeTable(){
	// The first part is the Form or Terse Signature.
	writeCVMForms();

	// then the details array.
	writeCVMMethodDetails();

	// write the indirect dummy.
	out.println("STATIC struct methodTypeTableSegment *methodTypeDummy;");
	out.println("STATIC const struct methodTypeTableSegment *masterMethodTypeDummy=0;");
	CVMInitInfo.addInfo( "&masterMethodTypeDummy", "&methodTypeDummy", "sizeof(methodTypeDummy)");

	// write the specific declaration we need for this
	// set of types.
	out.println("const struct { METHOD_TABLE_SEGMENT_HEADER");
	for ( int i = 0; i < 4; i++ ){
	    if ( nMethodTypes[i] == 0 ) continue;
	    out.print("    struct { METHOD_TYPE_HEADER ");
	    if ( (i&2) == 0 ){
		// inline form
		out.print("unsigned int formdata; ");
	    } else {
		// outline form
		out.print("struct sigForm * formp; ");
	    }
	    if ( (i&1) == 0 ){
		// inline details
		out.print("unsigned short data[2]; ");
	    } else {
		// outline details
		out.print("const short* datap; ");
	    }
	    out.print(" }typesig");
	    out.println( i+"["+nMethodTypes[i]+"];");
	}
	out.println("} CVMMethodTypeTable = {");

	// now the header info
	out.println("    0, 0, -1, "+totalMethodTypes+", &methodTypeDummy,");

	// and the actual data.
	for ( int i = 0; i < 4; i++ ){
	    if ( nMethodTypes[i] == 0 ) continue;
	    out.println("    {");
	    for ( CVMMethodType m = methodTypes[i]; m != null; m = m.listNext ){
		int next = (m.next==null)?-1:m.next.entryNo;
		out.print("	{"+(next == -1 ? "(CVMUint16)":"")+next+", 255, "+m.form.nSyllables);
		CVMSigForm f = m.form;
		if ( (i&2) == 0 ){
		    // inline form
		    out.print(", 0x"+Integer.toHexString( f.data[0] ));
		} else {
		    // outline form
		    out.print(", (struct sigForm*)(&CVM_ROMforms.form"+f.listNo+"["+f.entryNo+"])");
		}
		if ( (i&1) == 0 ){
		    // inline details
		    out.print(", {");
		    for ( int j=0; j<2; j++){
			out.print( (j<m.nDetails)?m.details[j]:0 );
			out.write(',');
		    }
		    out.print("}");
		} else {
		    // outline details
		    out.print(", SIG_DETAIL_NO("+(m.detailOffset)+")");
		}
		out.println("},");
	    }
	    out.println("    },");
	}
	out.println("};");

	/*
	 * A signature search begins with a hash.
	 * The hash table must be writeable.
	 */
	out.println("CVMUint16	CVMMethodTypeHash[NMETHODTYPEHASH];");
	out.print("STATIC CVMUint16 CVMMethodTypeHashMaster[NMETHODTYPEHASH]={");
	for ( int i = 0 ; i < CVMMethodType.NHASH; i++ ){
	    CVMMethodType mt = CVMMethodType.hashTable[i];
	    if ( i%4 == 0 ){
		out.print("\n	");
	    }
	    if ( mt == null ){
		out.print("(CVMUint16)-1, ");
	    } else {
		out.print( mt.entryNo );
		out.print(", ");
	    }
	}
	out.println("};");
	CVMInitInfo.addInfo( "CVMMethodTypeHashMaster", "CVMMethodTypeHash", CVMMethodType.NHASH+"*sizeof(CVMMethodTypeHash[0])" );
    }

    /*
     * Method signature varietes 1 & 3 have details written in an
     * array-of-short.
     */
    private void
    writeCVMMethodDetails( ){
	if (nMethodTypes[1]+nMethodTypes[3] == 0 ) return; // nothing to do.
	int curOffset = 0;
	out.print("STATIC const short msig_details[] = {");
	for ( int i = 1; i<4; i+= 2 ){ // i.e. for i in {1,3}
	    for ( CVMMethodType m = methodTypes[i]; m!=null; m = m.listNext ){
		m.detailOffset = curOffset;
		int ndetail = m.nDetails;
		int details[] = m.details;
		for ( int j = 0; j < ndetail; j++ ){
		    if ( curOffset%6==0){
			out.print("\n	");
		    }
		    out.print(details[j]);
		    out.write(',');
		    out.write(' ');
		    curOffset+= 1;
		}
	    }
	}
	out.println("\n};");
    }


    private void writeCVMForms(){
	/*
	 * Declare a data structure to contain all the "big" forms.
	 * Since they are of varying lengths, this will require a
	 * custom declaration.
	 */
	out.println("STATIC const struct {");
	for ( int i = 1; i < CVMSigForm.listLength.length; i++ ){
	    if ( CVMSigForm.listLength[i] == 0 ) continue; // never mind...
	    out.println("	DECLARE_SIG_FORM_N( /**/, "+(i+1)+") form"+i+"["+CVMSigForm.listLength[i]+"];");
	}
	out.println("} CVM_ROMforms = {");
	/*
	 * Now supply the data. They're all on a single linked list,
	 * which we construct (last-to-first) as we go.
	 */
	String link = "0";
	for ( int i = 1; i < CVMSigForm.listLength.length; i++ ){
	    int n = CVMSigForm.listLength[i];
	    if ( n == 0 ) continue; // never mind...
	    String segname = "&CVM_ROMforms.form"+i;
	    out.println("    {");
	    int fno = 0;
	    for ( CVMSigForm f = CVMSigForm.formList[i]; f!= null; f = f.listNext ){
		out.print("	");
		out.print("{ (struct sigForm*)"+link+", 255, ");
		out.print( f.nSyllables+", { 0x");
		int nWordsLess1 = f.data.length-1;
		for ( int k=0; k<=nWordsLess1; k++ ){
		    out.print( Integer.toHexString( f.data[k] ) );
		    if ( k<nWordsLess1)
			out.print(", 0x");
		}
		out.println("} },");
		//f.listNo  = i;
		f.entryNo = fno;
		link = segname+"["+(fno++)+"]";
	    }
	    out.println("    },");
	}
	out.println("};");

	/*
	 * Finally, the root of the form list.
	 */
	out.println("\nstruct sigForm *	CVMformTable;");
	out.println("STATIC const struct sigForm * CVMformTableMaster = (struct sigForm *)"+link+";");
	CVMInitInfo.addInfo( "&CVMformTableMaster", "&CVMformTable", "sizeof(CVMformTableMaster)" );
    }

    private void
    writeCVMMemberNameTable(){
	int ndata = CVMMemberNameEntry.tableSize();
	out.println("STATIC struct memberNameTableSegment * dummyNextMembernameCell;");
	out.println("STATIC const struct memberNameTableSegment * masterDummyNextMembername = 0;");
	CVMInitInfo.addInfo( "&masterDummyNextMembername", "&dummyNextMembernameCell", "sizeof(dummyNextMembernameCell)" );
	out.println("const struct { MEMBER_NAME_TABLE_SEGMENT_HEADER struct memberName data["+ndata+"]; }\nCVMMemberNames = {" );
	out.println("    0, 0, -1, "+ndata+", &dummyNextMembernameCell, {" );
	for ( int i = 0; i < ndata; i++ ){
	    CVMMemberNameEntry mne = (CVMMemberNameEntry)(CVMMemberNameEntry.table.elementAt( i ));
	    String UTFname = Util.unicodeToUTF( mne.name );
	    int next    = (mne.next==null) ? -1 : mne.next.entryNo;
	    out.println("\t{\""+UTFname+"\", 255, "+(next==-1?"(CVMUint16)":"")+next+"},");
	}
	out.println("}};");


	out.println("CVMUint16 CVMMemberNameHash[NMEMBERNAMEHASH];");
	out.print("STATIC const CVMUint16 CVMMemberNameHashMaster[NMEMBERNAMEHASH]={");
	for ( int i = 0 ; i < CVMMemberNameEntry.NMEMBERNAMEHASH; i++ ){
	    CVMMemberNameEntry mne = CVMMemberNameEntry.hash[i];
	    if ( i%4 == 0 ){
		out.print("\n	");
	    }
	    if ( mne == null ){
		out.print("(CVMUint16)-1, ");
	    } else {
		out.print( mne.entryNo );
		out.print(", ");
	    }
	}
	out.println("};");
	CVMInitInfo.addInfo( "CVMMemberNameHashMaster", "CVMMemberNameHash", CVMMemberNameEntry.NMEMBERNAMEHASH+"*sizeof(CVMMemberNameHash[0])" );
    }

    public void writeClassList( ClassClass classes[] ){
	int n = (classes == null) ? 0 : classes.length;

	out.println("const int CVM_nROMClasses = "+n+";");
	out.println("struct java_lang_Class CVM_ROMClasses["+n+"];");
	out.println("STATIC const struct java_lang_Class CVM_masterROMClasses[] = {" );
	for ( int i = 0; i < n; i++ ){
	    int hashCode = 0;
	    out.println("    CVM_CLASS_INIT(&"+cbName((CVMClass)(classes[i]))+
		","+hashCode+"),");
	}
	out.println("};");
	CVMInitInfo.addInfo( "CVM_masterROMClasses", "CVM_ROMClasses", n+"*sizeof(struct java_lang_Class)" );

	declare("STATIC const CVMClassICell CVM_classICell[]");
	out.println(" = {");
	for ( int i = 0; i < n; i++ ){
	    out.println("    { (CVMObject*)(&CVM_ROMClasses["+i+"])}," );
	}
	out.println("};");
	out.println("STATIC CVMExecEnv* CVM_clinitEE["+n+"];");
	headers.println("STATIC CVMExecEnv* CVM_clinitEE[];");
    }
    
    public boolean writeClasses( ConstantPool consts, 
				 ConstantPool sharedconsts ){

	ClassClass classes[] = ClassClass.getClassVector( classMaker );
	ClassClass.setTypes();
	doShared = sharedconsts!=null;

	enumerateMethodTypes();// after all classes processed once by classMaker
				  // ... but before we write the classes.
	
	// write out some constant pool stuff here,
	// if we're doing one big shared one...
	// gutted out for now...
	if ( doShared ) processStrings( sharedconsts.getConstants() );

	synchronized( xxx ){	
	    synchronized( out ){	    
		writePrologue();
		if (verbose)
		    System.out.println(Localizer.getString("cwriter.writing_classes"));
		try {
		    //mungeAllIDsAndWriteExternals(classes);
		    if ( !doShared ){
			for ( int i = 0; i < classes.length; i++ ){
			    processStrings( classes[i].ci.constants );
			}
		    }
		    instantiateAllStrings();
		    CVMInterfaceMethodTable.writeInterfaceTables(classes, out);
		    writeStaticStore( classes );
		    for ( int i = 0; i < classes.length; i++ ){
			writeClass( i, (CVMClass)classes[i] );
		    }
		    writeClassList( classes );

		    writeCVMNameAndTypeTables();

		    writeRomMap( "CVMROMClassesInfo", "CVM_nROMClassesInfo");
		    CVMInitInfo.write( out, "CVM_preloaderInitTriple", "CVM_preloaderInitMap", "CVM_nPreloaderMapEntries");
		} catch (RuntimeException e) { 
		    out.flush();
		    headers.flush();
		    System.out.println(e);
		    e.printStackTrace(System.out);
		    formatError = true;
		}
		return (!formatError) && (! out.checkError()) && (! headers.checkError());
	    }
	}
    }

    public boolean writeClasses( ConstantPool consts ) {
	return writeClasses(consts, null);
    }


    public void printSpaceStats( java.io.PrintStream o ){
	ClassClass classes[] = ClassClass.getClassVector( classMaker );
	o.println("	"+Localizer.getString("cwriter.total_classes", Integer.toString(classes.length)));
	o.println("		"+Localizer.getString("cwriter.method_blocks", Integer.toString(nmethods)));
	o.println("		"+Localizer.getString("cwriter.bytes_java_code", Integer.toString(ncodebytes)));
	o.println("		"+Localizer.getString("cwriter.catch_frames", Integer.toString(ncatchframes)));
	o.println("		"+Localizer.getString("cwriter.field_blocks", Integer.toString(nfields)));
	o.println("		"+Localizer.getString("cwriter.constant_pool_entries", Integer.toString(nconstants)));
	o.println("		"+Localizer.getString("cwriter.java_strings",Integer.toString(njavastrings)));
	o.println("		Method Signatures: "+totalMethodTypes );
	CVMSigForm.printStats( o );
    }

}
