/*
 * @(#)sync_md.c	1.15 00/11/04
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#include <vxWorks.h>
#include <taskLib.h>
#include <semLib.h>
#include <sysLib.h>
#include <time.h>
#include <assert.h>
#include <stdio.h>
#define VXWORKS_PRIVATE
#include "javavm/include/porting/sync.h"
#include "javavm/include/porting/threads.h"
#include "javavm/include/porting/doubleword.h"
#include "javavm/include/porting/int.h"


CVMBool
CVMmutexInit(CVMMutex * m)
{
    m->m = semMCreate(SEM_INVERSION_SAFE | SEM_Q_PRIORITY);
    return m->m != NULL;
}

void
CVMmutexDestroy(CVMMutex * m)
{
    semDelete(m->m);
}

CVMBool
CVMmutexTryLock(CVMMutex * m)
{
    return semTake(m->m, NO_WAIT) == OK;
}

void
CVMmutexLock(CVMMutex * m)
{
    semTake(m->m, WAIT_FOREVER);
}

void
CVMmutexUnlock(CVMMutex * m)
{
    semGive(m->m);
}

CVMBool
CVMcondvarInit(CVMCondVar * c, CVMMutex * m)
{
    c->sem = semBCreate(SEM_Q_PRIORITY, SEM_EMPTY);
    return c->sem != NULL;
}

void
CVMcondvarDestroy(CVMCondVar * c)
{
    semDelete(c->sem);
}

CVMBool
CVMcondvarWait(CVMCondVar * c, CVMMutex * m, CVMJavaLong millis)
{
    CVMBool result = CVM_TRUE;
    CVMThreadID *self = CVMthreadSelf();
    int timeout = WAIT_FOREVER;

    if (!CVMlongEqz(millis)) {
	/* Round up to nearest tick */
	CVMJavaLong ltmp, ltmp1, clocks, one_thousand, nine99;
	CVMJavaInt itmp;
	one_thousand = CVMint2Long(1000);
	nine99 = CVMint2Long(999);
	clocks = CVMint2Long(CLOCKS_PER_SEC);
	ltmp = CVMlongMul(millis, clocks);
	if (!CVMlongLtz(ltmp)) {
	    ltmp = CVMlongAdd(ltmp, nine99);
	    if (!CVMlongLtz(ltmp)) {
		ltmp = CVMlongDiv(ltmp, one_thousand);
		itmp = CVMlong2Int(ltmp);
		ltmp1 = CVMint2Long(itmp);
		if (CVMlongEq(ltmp, ltmp1)) {
		    /* (CLOCKS_PER_SEC * millis + 999) / 1000 */
		    timeout = itmp;
		}
	    }
	}
    }

    taskLock();

    if (self->interrupted) {
        self->interrupted = CVM_FALSE;
        taskUnlock();
        return CVM_FALSE;
    }
    self->in_wait = CVM_TRUE;

    CVMmutexUnlock(m);

    if (setjmp(self->env) == 0) {
	semTake(c->sem, timeout);
    } else {
	assert(self->interrupted);
        self->interrupted = CVM_FALSE;
        result = CVM_FALSE;
    }

    self->in_wait = CVM_FALSE;

    taskUnlock();

    CVMmutexLock(m);

    return result;
}

void
CVMcondvarNotify(CVMCondVar * c)
{
    taskLock();
    semGive(c->sem);		/* pull one waiter off the queue */
    semTake(c->sem, NO_WAIT);	/* but don't leave it available */
    taskUnlock();
}

void
CVMcondvarNotifyAll(CVMCondVar * c)
{
    semFlush(c->sem);		/* remove all waiters */
}

#include "javavm/include/porting/threads.h"

void
CVMschedLock()
{
    taskLock();
}

void
CVMschedUnlock()
{
    taskUnlock();
}

#include <signal.h>
#include <assert.h>

static void sigfunc0(int sig)
{
    CVMThreadID *self = CVMthreadSelf();
    assert(self->taskID == taskIdSelf());

    if (self->targetMutex == NULL) {
	fprintf(stderr, "Spurious signal %d\n", sig);
    } else {
	CVMBool success = CVMmutexTryLock(self->targetMutex);
	assert(success), (void) success;

	self->targetMutex = NULL;

	semGive(self->setownerCV);

	/* reset priority */
	taskPrioritySet(0, self->lastPriority);

	taskDelay(0);	/* yield */
    }
}

void
CVMmutexSetOwner(CVMThreadID *self, CVMMutex *m, CVMThreadID *t)
{
    int myPriority;

    assert(t != self);

    CVMmutexLock(&t->setOwnerLock);

    t->targetMutex = m;

    taskPriorityGet(t->taskID, &t->lastPriority);
    taskPriorityGet(0, &myPriority);

    taskLock();

    if (myPriority < t->lastPriority) {
	taskPrioritySet(t->taskID, myPriority);
    }

    kill(t->taskID, SIGUSR1);

    taskUnlock();

    semTake(t->setownerCV, WAIT_FOREVER);

    assert(t->targetMutex == NULL);

    CVMmutexUnlock(&t->setOwnerLock);
}

static void sigfunc1(int sig)
{
    CVMThreadID *self = CVMthreadSelf();
    assert(self->taskID == taskIdSelf());
    if (self->in_wait && self->interrupted) {
	longjmp(self->env, 1);
    } else {
	fprintf(stderr, "Spurious signal %d\n", sig);
    }
}

void
vxworksSyncInterruptWait(CVMThreadID *thread)
{
    /* %comment: mam053 */
    kill(thread->taskID, SIGUSR2);
}

void
vxworksSyncInit(CVMThreadID *self)
{
    struct sigaction sa;
    sa.sa_handler = sigfunc0;
    sa.sa_flags = 0;
    sigemptyset(&sa.sa_mask);
    sigaddset(&sa.sa_mask, SIGUSR2);
    sigaction(SIGUSR1, &sa, NULL);
    sa.sa_handler = sigfunc1;
    sigemptyset(&sa.sa_mask);
    sigaddset(&sa.sa_mask, SIGUSR1);
    sigaction(SIGUSR2, &sa, NULL);

    self->setownerCV = semBCreate(SEM_Q_PRIORITY, SEM_EMPTY);
    CVMmutexInit(&self->setOwnerLock);
}

void
vxworksSyncDestroy(CVMThreadID *self)
{
    semDelete(self->setownerCV);
    CVMmutexDestroy(&self->setOwnerLock);
}
