/*
 * -----------------------------------------------------------------
 * CVM.java :
 *
 * Copyright (C) 2001 TimeSys Corporation, All Rights Reserved.
 *
 * This software is subject to the terms and conditions of the
 * accompanying TimeSys Community Source License Agreement from TimeSys
 * in file TCSL.
 *
 * This file is derived from software distributed under the following terms:
 *
 * Sun Community Source License
 * Sun Microsystems
 * For details, see http://www.sun.com/981208/scsl/principles.html
 *
 * Your use of this software indicates your acceptance of these terms.
 * -----------------------------------------------------------------
 */


/*
 * @(#)CVM.java	1.41 00/11/03
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package sun.misc;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import sun.misc.Version;

//ROBBINS SEZ:
// we import javax.realtime to throw AIE's
import javax.realtime.*;

public final class CVM {
    /** WARNING! NO STATIC INITIALIZER IS ALLOWED IN THIS CLASS!

	More precisely, no initial assignment is allowed to either of
	these two variables. */
    private static String mainClassName;
    private static String[] mainArgs;

    // The status of command-line argument parsing
    public static final int ARG_PARSE_UNINIT = 0;
    public static final int ARG_PARSE_OK     = 1;
    public static final int ARG_PARSE_ERR    = 2;
    public static final int ARG_PARSE_EXITVM = 3;

    private static int parseStatus; 

    private static void usage(String nativeOptions) {
	// Print usage statement
	System.err.println(
	    "usage: tjvm " + nativeOptions + " [-D<property=value>...] " +
	    "<main class name> [arguments...]"
	);
    }

    /** Parse command line options handed up from ansiJavaMain through
	JNI_CreateJavaVM. This parses command line options like -D to
	define user-specified properties; it must be called after
	System.initializeSystemClass(). In its current usage it
	receives argv[1..(argc - 1)] in String form from the C
	initialization code, with "-Xcvm" prepended onto the main
	class and its arguments. This allows the building of the
	String array which will later be passed to main() to be done
	in Java. Note that it is not necessary to use this
	functionality. */
    public static int parseCommandLineOptions(String[] args,
						  String nativeOptions,
						  boolean ignoreUnrecognized)
    {
	parseStatus = ARG_PARSE_UNINIT;

	String pathSeparator = System.getProperty("path.separator", ":");

	// Set up some known properties.

	// "net" library is always builtin, for now
	System.setProperty("java.library.builtin.net", "yes");
	// "math" library too...
	System.setProperty("java.library.builtin.math", "yes");
	// and let's not forget "zip"...
	System.setProperty("java.library.builtin.zip", "yes");

	for (int i = 0; i < args.length; i++) {
	    /* %comment: rt039 */

	    if (args[i].startsWith("-D")) {
		if (!addUserProperty(args[i].substring(2))) {
		    System.err.println("Error parsing property " + args[i]);
		    usage(nativeOptions);
		    parseStatus = ARG_PARSE_ERR;
		    return parseStatus;
		}
	    } else if (args[i].startsWith("-version")) {
		Version.print(true); // Long version
		parseStatus = ARG_PARSE_EXITVM; // Don't parse any more
		return parseStatus;
	    } else if (args[i].startsWith("-showversion")) {
		Version.print(true); // Long version
		// continue with VM execution
	    } else if (args[i].startsWith("-fullversion")) {
		Version.print(false); // Short version
		parseStatus = ARG_PARSE_EXITVM; // Don't parse any more
		return parseStatus;
	    } else if (args[i].startsWith("-Xcvm")) {
		// We have to assume that everything else in the
		// command line options is the main class plus its
		// arguments.
		mainClassName = args[i].substring(5);
		if (mainClassName.startsWith("-")) {
		    System.err.println("Main class name \"" +
				       mainClassName + "\" is not valid");
		    usage(nativeOptions);
		    parseStatus = ARG_PARSE_ERR;
		    return parseStatus;
		}
		// Change '.' to '/' -- see getMainClassName, below
		mainClassName = mainClassName.replace('.', '/');
		
		int numMainArgs = args.length - i - 1;
		mainArgs = new String[numMainArgs];
		for (int j = 0; j < numMainArgs; j++) {
		    String arg = args[i + j + 1];
		    if (!arg.startsWith("-Xcvm")) {
			throw new InternalError(
		            "Illegal use of -Xcvm internal " +
			    "command line options"
			);
		    }
		    mainArgs[j] = arg.substring(5);
		}
		break;
	    } else if (args[i].startsWith("-Xbootclasspath=") ||
		       args[i].startsWith("-Xbootclasspath:")) {
		String tmp = args[i];
		if (tmp.length() < 17) {
		    System.err.println("-Xbootclasspath format error");
		    usage(nativeOptions);
		    parseStatus = ARG_PARSE_ERR;
		    return parseStatus;
		}
		System.setProperty("sun.boot.class.path",
				   args[i].substring(16));
	    } else if (args[i].startsWith("-Xbootclasspath/p=") ||
		       args[i].startsWith("-Xbootclasspath/p:")) {
		String tmp = args[i];
		if (tmp.length() < 19) {
		    System.err.println("-Xbootclasspath format error");
		    usage(nativeOptions);
		    parseStatus = ARG_PARSE_ERR;
		    return parseStatus;
		}
		System.setProperty("sun.boot.class.path",
				   args[i].substring(18) + pathSeparator +
				   System.getProperty("sun.boot.class.path"));
	    } else if (args[i].startsWith("-Xbootclasspath/a=") ||
		       args[i].startsWith("-Xbootclasspath/a:")) {
		String tmp = args[i];
		if (tmp.length() < 19) {
		    System.err.println("-Xbootclasspath format error");
		    usage(nativeOptions);
		    parseStatus = ARG_PARSE_ERR;
		    return parseStatus;
		}
		System.setProperty("sun.boot.class.path",
				   System.getProperty("sun.boot.class.path")
				   + pathSeparator + args[i].substring(18));
// NOTE: do we really want to check all possible options here, or
// should the caller have filtered out the options that it understood?
	    } else {
		if (args[i].startsWith("-")) {
		    if (!ignoreUnrecognized) {
			System.err.println("Unrecognized option " + args[i]);
			usage(nativeOptions);
			parseStatus = ARG_PARSE_ERR;
			return parseStatus;
		    }
		} else {
		    // NOTE: I don't think this code path should be
		    // encountered anymore since we have -Xcvm
		    System.err.println("Unrecognized option " + args[i]);
		    usage(nativeOptions);
		    parseStatus = ARG_PARSE_ERR;
		    return parseStatus;
		}
	    }
	}

	if (mainClassName == null) {
	    System.err.println("Main class name missing.");
	    usage(nativeOptions);
	    parseStatus = ARG_PARSE_ERR;
	    return parseStatus;
	}

	parseStatus = ARG_PARSE_OK;
	return parseStatus;
    }

    /** Helper function for parseCommandLineOptions(), above */
    private static boolean addUserProperty(String property) {
	// Cut it at the '=' into key, value
	int equalsIdx = property.indexOf('=');
	if (equalsIdx == -1) {
	    return false;
	}
	String key = property.substring(0, equalsIdx);
	String value = property.substring(equalsIdx + 1, property.length());
	System.setProperty(key, value);
	return true;
    }

    /* This is called by ansi_java_md.c using the JNI after
       parseCommandLineOptions is called. Note that calling this is
       optional -- the user's code may instead parse argv to find the
       main class and its arguments. This returns the VM-internal,
       "slashified" class name, because this is a VM-internal
       method. */
    public static int getParseStatus() {
	return parseStatus;
    }

    public static String getMainClassName() {
	return mainClassName;
    }
    
    /* This is called by ansi_java_md.c using the JNI after
       parseCommandLineOptions is called. Note that calling this is
       optional -- the user's code may instead parse argv to find the
       main class and its arguments. */
    public static String[] getMainArguments() {
	return mainArgs;
    }

    /* 
     * Debug flags: The debug build of CVM has many debugging features that
     * can be enabled or disabled at runtime. There is a separate
     * flag for each feature. Most of them or for enabling or disabling
     * debugging trace statements.
     */

    public static final int DEBUGFLAG_TRACE_OPCODE     	= 0x00000001;
    public static final int DEBUGFLAG_TRACE_METHOD     	= 0x00000002;
    public static final int DEBUGFLAG_TRACE_STATUS     	= 0x00000004;
    public static final int DEBUGFLAG_TRACE_FASTLOCK    = 0x00000008;
    public static final int DEBUGFLAG_TRACE_DETLOCK    	= 0x00000010;
    public static final int DEBUGFLAG_TRACE_MUTEX      	= 0x00000020;
    public static final int DEBUGFLAG_TRACE_CS		= 0x00000040;
    public static final int DEBUGFLAG_TRACE_GCSTARTSTOP = 0x00000080;
    public static final int DEBUGFLAG_TRACE_GCSCAN     	= 0x00000100;
    public static final int DEBUGFLAG_TRACE_GCSCANOBJ	= 0x00000200;
    public static final int DEBUGFLAG_TRACE_GCALLOC    	= 0x00000400;
    public static final int DEBUGFLAG_TRACE_GCCOLLECT	= 0x00000800;
    public static final int DEBUGFLAG_TRACE_GCSAFETY	= 0x00001000;
    public static final int DEBUGFLAG_TRACE_CLINIT     	= 0x00002000;
    public static final int DEBUGFLAG_TRACE_EXCEPTIONS	= 0x00004000;
    public static final int DEBUGFLAG_TRACE_MISC       	= 0x00008000;
    public static final int DEBUGFLAG_TRACE_BARRIERS	= 0x00010000;
    public static final int DEBUGFLAG_TRACE_STACKMAPS	= 0x00020000;
    public static final int DEBUGFLAG_TRACE_CLASSLOADING= 0x00040000;
    public static final int DEBUGFLAG_TRACE_CLASSLOOKUP = 0x00080000;
    public static final int DEBUGFLAG_TRACE_TYPEID      = 0x00100000;
    public static final int DEBUGFLAG_TRACE_VERIFIER    = 0x00200000;
    public static final int DEBUGFLAG_TRACE_WEAKREFS    = 0x00400000;
    public static final int DEBUGFLAG_TRACE_CLASSUNLOAD = 0x00800000;
    public static final int DEBUGFLAG_TRACE_CLASSLINK   = 0x01000000;

    // NOTE
    // Some of these public native methods are quite dangerous.
    // We need to make sure that applets cannot call them.

    /*
     * Methods for checking, setting, and clearing the state of debug
     * flags.  All of the following methods return the previous state of 
     * the flags.
     *
     * You can pass in more than one flag at a time to any of the methods.
     */
    public native static int checkDebugFlags(int flags);
    public native static int setDebugFlags(int flags);
    public native static int clearDebugFlags(int flags);
    public native static int restoreDebugFlags(int flags, int oldvalue);

    /*
     * Used by Class.runStaticInitializers() to execute the <clinit> method
     * of the specified class.
     */
    public native static void executeClinit(Class c);

    /*
     * Use by Class.runStaticInitializers() to free up the code for the 
     * specified class' <clinit> method.
     */
    public native static void freeClinit(Class c);

    /*
     * Manage remote exceptions 
     */
    public native static void disableRemoteExceptions();
    public native static void enableRemoteExceptions();
    public native static void throwRemoteException(Thread target, Throwable t);

    /*
     * Allows system methods to bypass compiler checks and throw
     * exceptions.  Useful for rewriting native methods in Java.
     */
    public native static Error throwLocalException(Throwable t);

    /*
     * Some heap dumping utilties.
     */
    public native static void gcDumpHeapSimple();
    public native static void gcDumpHeapVerbose();
    public native static void gcDumpHeapStats();

}
