/*
 * @(#)SystemThread.java	1.5 00/11/07
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package sun.misc;

import java.util.Vector;

public class SystemThread {

    public static void add(Thread t) {
	synchronized (staticLock) {
	    systemThreads.addElement(t);
	    dprintln("New SystemThread " + t.getName());
	}
    }

    public static void remove(Thread t) {
	synchronized (staticLock) {
	    systemThreads.removeElement(t);
	    dprintln("SystemThread " + t.getName() + " removed");
	}
    }

    public static void waitAllSystemThreadsExit() {
	dprintln("Asking all SystemThreads to exit");

	allExitRequestedFlag = true;
	Vector threadList;

	while (true) {
	    synchronized (staticLock) {
		threadList = (Vector)systemThreads.clone();
	    }
	    int n = threadList.size();
	    if (n == 0) {
		break;
	    } else {
		count = n;
		for (int i = 0; i < n; ++i) {
		    Thread t = (Thread)threadList.elementAt(i);
		    dprintln("Asking SystemThread " + t.getName() + " to exit");
		    t.interrupt();
		}
		for (int i = 0; i < n; ++i) {
		    Thread t = (Thread)threadList.elementAt(i);
		    dprintln("Waiting for SystemThread " +
			 t.getName() + " to exit");

		    while (t.isAlive()) {
			try {
			    t.join();
			    break;
			} catch (InterruptedException ie) {
			}
		    }
		}
	    }
	}
    }

    public static boolean exitRequested() {
	if (allExitRequestedFlag) {
	    dprintln("SystemThread " + Thread.currentThread().getName() +
		" has recognized the request to exit");
	}
	return allExitRequestedFlag;
    }

    private static void dprintln(String s) {
	if (CVM.checkDebugFlags(CVM.DEBUGFLAG_TRACE_MISC) != 0) {
	    // Be careful of initialization order problems
	    if (System.err != null) {
		if (debugBuffer != null) {
		    int n = debugBuffer.size();
		    for (int i = 0; i < n; ++i) {
			String str = (String)debugBuffer.elementAt(i);
			System.err.println(str);
		    }
		}
		debugBuffer = null;
		System.err.println(s);
	    } else {
		if (debugBuffer == null) {
		    debugBuffer = new Vector();
		}
		debugBuffer.addElement(s);
	    }
	}
    }

    private static Object staticLock = new Object();
    private static boolean allExitRequestedFlag = false;
    private static Vector systemThreads = new Vector();
    private static int count;
    private static Vector debugBuffer;
}
