/*
 * @(#)gc_impl.h	1.12 00/09/05
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
 * This file includes functions that need to be implemented by a particular
 * GC implementation. Common VM GC code will call these when necessary.
 */

#ifndef _INCLUDED_GC_IMPL_H
#define _INCLUDED_GC_IMPL_H

#include "javavm/include/gc_common.h"

/*
 * This file is generated from the GC choice given at build time.
 */
#include "generated/javavm/include/gc_config.h"

#include "javavm/include/clib.h"

/*
 * Initialize GC global state
 */
extern void
CVMgcimplInitGlobalState(CVMGCGlobalState* globalState);

/*
 * Initialize a heap of at least 'minBytes' bytes, and at most 'maxBytes'
 * bytes.
 */
extern CVMBool
CVMgcimplInitHeap(CVMGCGlobalState* globalState,
		  CVMUint32 minBytes, 
		  CVMUint32 maxBytes);

/*
 * Allocate uninitialized heap object of size numBytes
 */
extern CVMObject*
CVMgcimplAllocObject(CVMExecEnv* ee, CVMUint32 numBytes);

/*
 * This routine is called by the common GC code after all locks are
 * obtained, and threads are stopped at GC-safe points. It's the
 * routine that needs a snapshot of the world while all threads are
 * stopped (typically at least a root scan).
 *
 * The goal to free is 'numBytes' bytes.
 */
void
CVMgcimplDoGC(CVMExecEnv* ee, CVMUint32 numBytes);

/*
 * Return the number of bytes free in the heap. 
 */
CVMUint32
CVMgcimplFreeMemory(CVMExecEnv* ee);

/*
 * Return the amount of total memory in the heap, in bytes.
 */
CVMUint32
CVMgcimplTotalMemory(CVMExecEnv* ee);

/*
 * Destroy GC global state
 */
extern void
CVMgcimplDestroyGlobalState(CVMGCGlobalState* globalState);

/*
 * Destroy heap
 */
extern CVMBool
CVMgcimplDestroyHeap(CVMGCGlobalState* globalState);


/*
 * The time of the last full GC of the heap
 */
extern CVMInt64
CVMgcimplTimeOfLastMajorGC();

#ifdef CVM_DEBUG
/*
 * Heap dump support from the GC implementation
 */
extern void
CVMgcimplIterateHeap(CVMExecEnv* ee, CVMObjectCallbackFunc cback, void* data);
#endif /* CVM_DEBUG */

/* Sample read and write barriers. Put these in the gc_config.h file
   for a given GC implementation to get verbose output on when the
   barriers are hit. */

/*#define VERBOSE_BARRIERS */

#ifdef VERBOSE_BARRIERS
#define CVMgcimplReadBarrierByte(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of byte %p[%d] (0x%p) -> 0x%x\n",	\
		     objRef, fieldLoc_ - (CVMJavaByte*)objRef, \
		     fieldLoc_, *(fieldLoc_)));

#define CVMgcimplReadBarrierShort(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of short %p[%d] (0x%p) -> 0x%x\n",	\
		     objRef, fieldLoc_ - (CVMJavaShort*)objRef, \
		     fieldLoc_, *(fieldLoc_)));

#define CVMgcimplReadBarrierInt(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of int %p[%d] (0x%p) -> 0x%x\n",	\
		     objRef, fieldLoc_ - (CVMJavaInt*)objRef, \
		     fieldLoc_, *(fieldLoc_)));

#define CVMgcimplReadBarrier64(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of 64-bit %p[%d] (0x%p) -> (0x%x,0x%x)\n", \
		      objRef, fieldLoc_ - (CVMJavaVal32*)objRef, \
		      fieldLoc_, fieldLoc_[0].i, fieldLoc_[1].i));

#define CVMgcimplReadBarrierRef(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of Ref %p[%d] (0x%p) -> 0x%x\n",	\
		      objRef, fieldLoc_ - (CVMObject**)objRef, \
		      fieldLoc_, *(fieldLoc_)));

#define CVMgcimplWriteBarrierByte(objRef, fieldLoc_, rhs) \
    CVMtraceBarriers(("BARRIER: Write to byte %p[%d] (0x%p) <- 0x%x\n", \
		      objRef, fieldLoc_ - (CVMJavaByte*)objRef, \
		      fieldLoc_, rhs));

#define CVMgcimplWriteBarrierShort(objRef, fieldLoc_, rhs) \
    CVMtraceBarriers(("BARRIER: Write to short %p[%d] (0x%p) <- 0x%x\n", \
		      objRef, fieldLoc_ - (CVMJavaShort*)objRef, \
		      fieldLoc_, rhs));

#define CVMgcimplWriteBarrierInt(objRef, fieldLoc_, rhs) \
    CVMtraceBarriers(("BARRIER: Write to int %p[%d] (0x%p) <- 0x%x\n", \
		      objRef, fieldLoc_ - (CVMJavaInt*)objRef, \
		      fieldLoc_, rhs));

#define CVMgcimplWriteBarrier64(objRef, fieldLoc_, location) \
    CVMtraceBarriers(("BARRIER: Write of 64-bit %p[%d] (0x%p) <- (0x%x,0x%x)\n", \
		      objRef, fieldLoc_ - (CVMJavaVal32*)objRef, \
		      fieldLoc_, location[0].i, location[1].i));

#define CVMgcimplWriteBarrierRef(objRef, fieldLoc_, rhs) \
    CVMtraceBarriers(("BARRIER: Write to Ref %p[%d] (0x%p) <- 0x%x\n", \
		      objRef, fieldLoc_ - (CVMObject**)objRef, \
		      fieldLoc_, rhs));

#define CVMgcimplReadBarrierBool(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of bool %p[%d] (0x%p) -> 0x%x\n",	\
		      objRef, fieldLoc_ - (CVMJavaBool*)objRef, \
		      fieldLoc_, *(fieldLoc_)));

#define CVMgcimplReadBarrierChar(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of char %p[%d] (0x%p) -> 0x%x\n",	\
		      objRef, fieldLoc_ - (CVMJavaChar*)objRef, \
		      fieldLoc_, *(fieldLoc_)));

#define CVMgcimplReadBarrierFloat(objRef, fieldLoc_) \
    CVMtraceBarriers(("BARRIER: Read of float %p[%d] (0x%p) -> %f\n",	\
		      objRef, fieldLoc_ - (CVMJavaFloat*)objRef, \
		      fieldLoc_, *(fieldLoc_)));

#define CVMgcimplWriteBarrierBool(objRef, fieldLoc_, rhs) \
    CVMtraceBarriers(("BARRIER: Write to bool %p[%d] (0x%p) <- 0x%x\n", \
		      objRef, fieldLoc_ - (CVMJavaBool*)objRef, \
		      fieldLoc_, rhs));

#define CVMgcimplWriteBarrierChar(objRef, fieldLoc_, rhs) \
    CVMtraceBarriers(("BARRIER: Write to char %p[%d] (0x%p) <- 0x%x\n", \
		      objRef, fieldLoc_ - (CVMJavaChar*)objRef, \
		      fieldLoc_, rhs));

#define CVMgcimplWriteBarrierFloat(objRef, fieldLoc_, rhs) \
    CVMtraceBarriers(("BARRIER: Write to float %p[%d] (0x%p) <- %f\n", \
		      objRef, fieldLoc_ - (CVMJavaFloat*)objRef, \
		      fieldLoc_, rhs));

#endif /* VERBOSE_BARRIERS */

#endif /* _INCLUDED_GC_IMPL_H */


