/*
 * @(#)named_sys_monitor.h	1.3 00/09/07
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#ifndef _INCLUDED_NAMED_SYS_MONITOR_H
#define _INCLUDED_NAMED_SYS_MONITOR_H

/* NOTE: This module defines the CVMNamedSysMonitor which is currently solely
         used by the JVMDI and JVMPI APIs.  CVMNamedSysMonitor inherits from
         CVMSysMonitor (which is defined in sync.h), and adds a name field.
*/

#if (defined(CVM_JVMDI) || defined(CVM_JVMPI))

#include "javavm/include/indirectmem.h"
#include "javavm/include/sync.h"

/*=============================================== class CVMNamedSysMonitor ==*/

/* Purpose: "raw" named object-less monitor for the use of JVMDI & JVMPI. */
/* NOTE: Make sure to use CVMnamedSysMonitorComputeSize() to compute the
         amount of memory in bytes to be allocated for this monitor.  This is
         necessary because the name of the monitor is stored immediately after
         the monitor part.  This allows the needed memory to be allocated in
         one single request and simplifies the handling of "Out of Memory"
         errors by JVMDI & JVMPI.
*/
/* NOTE: The CVMNamedSysMonitor implementation describes the following
         abstraction:

    class CVMNamedSysMonitor extends CVMSysMonitor {
        private static CVMUint32 computeSize(const char *name);
        public const char *getName();

        public static CVMNamedSysMonitor *init(const char *name);
        public void destroy();
        public void enter(CVMExecEnv *current_ee);
        public void exit(CVMExecEnv *current_ee);

        public CVMBool notify(CVMExecEnv *current_ee);
        public CVMBool notifyAll(CVMExecEnv *current_ee);

        public CVMWaitStatus wait(CVMExecEnv *current_ee, CVMInt64 millis);
        public CVMExecEnv *getOwner();
        public CVMUint32 getCount();

        public CVMBool tryEnter(CVMExecEnv *current_ee);
        public void enterUnsafe(CVMExecEnv *current_ee);
        public void enterSafe(CVMExecEnv *current_ee);
    };
*/

struct CVMNamedSysMonitor {
    CVMSysMonitor _super;
    char name[1];
};

/* Purpose: Gets the name of this monitor. */
/* inline const char *CVMnamedSysMonitorGetName(CVMNamedSysMonitor *self); */
#define CVMnamedSysMonitorGetName(self) \
    ((const char *)(self)->name)

/* Purpose: Constructor. */
extern CVMNamedSysMonitor *CVMnamedSysMonitorInit(const char *name);

/* Purpose: Destructor. */
void CVMnamedSysMonitorDestroy(CVMNamedSysMonitor *self);

/* Purpose: Enters the monitor. */
/* inline void CVMnamedSysMonitorEnter(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee); */
#define CVMnamedSysMonitorEnter(self, current_ee) \
    CVMsysMonitorEnter((current_ee), (&(self)->_super))

/* Purpose: Exits the monitor. */
/* inline void CVMnamedSysMonitorExit(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee); */
#define CVMnamedSysMonitorExit(self, current_ee) {     \
    CVMsysMonitorExit((current_ee), (&(self)->_super)); \
}

/* Purpose: Notifies the next thread waiting on this monitor. */
/* Returns: If the current_ee passed in is not the owner of the monitor,
            CVM_FALSE will be returned.  Else, returns CVM_TRUE. */
/* inline CVMBool CVMnamedSysMonitorNotify(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee); */
#define CVMnamedSysMonitorNotify(self, current_ee) \
    CVMsysMonitorNotify(current_ee, &(self)->_super)

/* Purpose: Notifies all threads waiting on this monitor. */
/* Returns: If the current_ee passed in is not the owner of the monitor,
            CVM_FALSE will be returned.  Else, returns CVM_TRUE. */
/* inline CVMBool CVMnamedSysMonitorNotifyAll(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee); */
#define CVMnamedSysMonitorNotifyAll(self, current_ee) \
    CVMsysMonitorNotifyAll(current_ee, &(self)->_super)

/* Purpose: Wait for the specified time period (in milliseconds).  If the
            period is 0, then wait forever. */
/* Returns: CVM_WAIT_NOT_OWNER if current_ee is not the owner of the monitor,
            CVM_WAIT_OK if the thread was woken up by a timeout, and
            CVM_WAIT_INTERRUPTED is the thread was woken up by a notify. */
/* NOTE:    Thread must be in a GC safe state before calling this. */
/* inline CVMWaitStatus CVMnamedSysMonitorWait(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee, CVMInt64 millis); */
#define CVMnamedSysMonitorWait(self, current_ee, millis) \
    CVMsysMonitorWait(current_ee, (&(self)->_super), millis)

/* Purpose: Gets the current owner ee of this monitor. */
/* inline CVMExecEnv *CVMnamedSysMonitorGetOwner(CVMNamedSysMonitor *self); */
#define CVMnamedSysMonitorGetOwner(self) CVMsysMonitorOwner((&(self)->_super))

/* Purpose: Gets the current lock count of this monitor. */
/* inline CVMUint32 CVMnamedSysMonitorGetCount(CVMNamedSysMonitor *self); */
#define CVMnamedSysMonitorGetCount(self) CVMsysMonitorCount((&(self)->_super))

/* Purpose: Tries to enter the monitor. */
/* inline CVMBool CVMnamedSysMonitorTryEnter(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee); */
#define CVMnamedSysMonitorTryEnter(self, current_ee) \
    CVMsysMonitorTryEnter((current_ee), (&(self)->_super))

/* Purpose: Enters monitor while thread is in a GC unsafe state. */
/* inline void CVMnamedSysMonitorEnterUnsafe(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee); */
#define CVMnamedSysMonitorEnterUnsafe(self, current_ee) {   \
    CVMsysMonitorEnterUnsafe(current_ee, (&(self)->_super)); \
}

/* Purpose: Enters monitor while thread is in a GC safe state. */
/* inline void CVMnamedSysMonitorEnterSafe(CVMNamedSysMonitor *self,
            CVMExecEnv *current_ee); */
#define CVMnamedSysMonitorEnterSafe(self, current_ee) {   \
    CVMsysMonitorEnterSafe(current_ee, (&(self)->_super)); \
}

#endif

#endif
