/*
 * @(#)porting_debug.h	1.7 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#ifndef _INCLUDED_PORTING_DEBUG_H
#define _INCLUDED_PORTING_DEBUG_H

#if defined(CVM_DEBUG) && !defined(CVM_PORTING_DEBUG_IMPL)

#include "javavm/include/porting/ansi/stddef.h"
#include "javavm/include/porting/vm-defs.h"

/*
 * This file allows a way to redirect interfaces through
 * debugging stubs.  Add stubs as needed.
 */

void * CVMCmemsetStub(void * s, int c, size_t n);
void * CVMCmemcpyStub(void * d, const void * s, size_t n);
void * CVMCmemmoveStub(void * d, const void * s, size_t n);
int CVMCstrcmpStub(const char * s1, const char * s2);
int CVMCstrncmpStub(const char * s1, const char * s2, size_t n);
size_t CVMCstrlenStub(const char* s);
char * CVMCstrcpyStub(char * dst, const char * src);
char * CVMCstrncpyStub(char * dst, const char * src, size_t n);
char * CVMCstrdupStub(const char * str);
char * CVMCstrcatStub(char * dst, const char * src);
char * CVMCstrncatStub(char * dst, const char * src, size_t n);
char * CVMCstrstrStub(const char * s1, const char * s2);
char * CVMCstrchrStub(const char * s, int c);
char * CVMCstrrchrStub(const char * s, int c);
long CVMCstrtolStub(const char * str, char ** endptr, int base);
void * CVMCmallocStub(size_t size);
void * CVMCcallocStub(size_t nelem, size_t elsize);
void CVMCfreeStub(void * ptr);
void * CVMCreallocStub(void * ptr, size_t size);
int CVMCisspaceStub(int c);
int CVMCisalnumStub(int c);
int CVMCisprintStub(int c);
void CVMCqsortStub(void * array, size_t numElements, size_t elementSize,
		   int (* compareFunc)(const void *, const void *));

#undef memset
#define memset CVMCmemsetStub
#undef memcpy
#define memcpy CVMCmemcpyStub
#undef memmove
#define memmove CVMCmemmoveStub
#undef strcmp
#define strcmp CVMCstrcmpStub
#undef strncmp
#define strncmp CVMCstrncmpStub
#undef strlen
#define strlen CVMCstrlenStub
#undef strcpy
#define strcpy CVMCstrcpyStub
#undef strncpy
#define strncpy CVMCstrncpyStub
#undef strdup
#define strdup CVMCstrdupStub
#undef strcat
#define strcat CVMCstrcatStub
#undef strncat
#define strncat CVMCstrncatStub
#undef strstr
#define strstr CVMCstrstrStub
#undef strchr
#define strchr CVMCstrchrStub
#undef strrchr
#define strrchr CVMCstrrchrStub
#undef strtol
#define strtol CVMCstrtolStub
#undef malloc
#define malloc CVMCmallocStub
#undef calloc
#define calloc CVMCcallocStub
#undef free
#define free CVMCfreeStub
#undef realloc
#define realloc CVMCreallocStub
#undef isspace
#define isspace CVMCisspaceStub
#undef isalnum
#define isalnum CVMCisalnumStub
#undef isprint
#define isprint CVMCisprintStub
#undef qsort
#define qsort CVMCqsortStub

#include "javavm/include/porting/ansi/setjmp.h"

#undef longjmp
#define longjmp CVMlongjmpStub
void CVMlongjmpStub(jmp_buf env, int val);

#include "javavm/include/porting/ansi/time.h"

#undef mktime
#define mktime CVMmktimeStub
time_t CVMmktimeStub(struct tm * timeptr);

#include "javavm/include/porting/doubleword.h"

#undef CVMlongDiv
#define CVMlongDiv(op1, op2)	\
	(CVMlongDivStub((op1), (op2)))
CVMJavaLong CVMlongDivStub(CVMJavaLong op1, CVMJavaLong op2);

#undef CVMjvm2Long
#define CVMjvm2Long(location)	\
	CVMjvm2LongStub((location))
CVMJavaLong CVMjvm2LongStub(const CVMUint32 location[2]);

#undef CVMlong2Jvm
#define CVMlong2Jvm(location, val)	\
	CVMlong2JvmStub((location), (val))
void CVMlong2JvmStub(CVMUint32 location[2], CVMJavaLong val);

#undef CVMjvm2Double
#define CVMjvm2Double(location)	\
	CVMjvm2DoubleStub((location))
CVMJavaDouble CVMjvm2DoubleStub(const CVMUint32 location[2]);

#undef CVMdouble2Jvm
#define CVMdouble2Jvm(location, val)	\
	CVMdouble2JvmStub((location), (val))
void CVMdouble2JvmStub(CVMUint32 location[2], CVMJavaDouble val);

#undef CVMmemCopy64
#define CVMmemCopy64(to, from)	\
	CVMmemCopy64Stub((to), (from))
void CVMmemCopy64Stub(CVMUint32 to[2], const CVMUint32 from[2]);

#endif

#endif /* _INCLUDED_PORTING_DEBUG_H */
