/*
 * @(#)string.h	1.9 00/09/29
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
 * String-related routines.
 * For use by Java_java_lang_String_intern, classloading,
 * and garbage collection.
 */

#ifndef CVM_STRING_H
#define CVM_STRING_H

#include "javavm/include/defs.h"
#include "javavm/export/jni.h"

/*
 * Initialize on VM start-up.
 */
void 
CVMinternInit();

/*
 * The JNI entry point: java.lang.String.intern.
 * by way of JVM_InternString, which is re-mapped to 
 * this function:
 */
JNIEXPORT jobject JNICALL
CVMstringIntern(JNIEnv *env, jstring this);

/*
 * CVMinternUTF called by the class loader to turn a null-terminated,
 * UTF8-encoded C string into a Java String, put it in the intern table,
 * and return a pointer to the intern table cell.
 * Will lock the intern table during search, unlock at end.
 */
CVMStringICell *
CVMinternUTF( CVMExecEnv *, const CVMUtf8 * );

/*
 * Also for use by the class loader, when unloading a class.
 * If an interned String ref is found in the constant pool, call
 * this on it.
 */
void
CVMinternDecRef( CVMExecEnv * ee, CVMStringICell * target );


/*
 * CVMscanInternedStrings is used to scan the intern table,
 * calling the func on every live (non-null, non-deleted) entry.
 *
 * Used by GC.
 * Assumes intern table already locked, so it is safe to iterate
 * without further locking.
 */
void
CVMscanInternedStrings( CVMRefCallbackFunc func, void * data );

/*
 * CVMdeleteUnreferencedStrings called during garbage collection:
 * iterate over all the intern table entries. For any that have a 0
 * reference count, discover if they have Java references by calling
 * queryStringReferenced. If not, then the String will be deleted from
 * the intern table.
 *
 * Assumes intern table already locked, so it is safe to change
 * entries.
 */
void
CVMdeleteUnreferencedStrings(CVMRefLivenessQueryFunc isLive,
			     void* isLiveData,
			     CVMRefCallbackFunc transitiveScanner,
			     void* transitiveScannerData);

/*
 * give back dynamically allocated stuff when shutting down
 * the VM.
 */
void
CVMinternDestroy();

#endif /* CVM_STRING_H */
