/*
 * @(#)CVMStringTable.java	1.23 00/11/09
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package runtime;
import java.util.Enumeration;
import Const;
import components.*;
import vm.*;
import util.*;

public class CVMStringTable extends vm.StringTable {

    CVMStringIntern internTable;

    final boolean writeStringData( String dataName, String charArrayClassBlockName, CCodeWriter out ){
	int n = arrangeStringData();

	if ( n == 0 ) return false; // nothing to do here.

	char v[] = new char[n];
	data.getChars( 0, n, v, 0 );

	//
	// first typedef the structure we're about to create
	out.print("STATIC const struct { CVMObjectHeader hdr;\n    CVMJavaInt	length;\n    CVMJavaChar stringData["+n+"]; } ");
	out.print( dataName );
	out.println( " = {" );
	//
	// fill in the header information
	// hdr.various = {{0}} ??
	int hashCode = 0;
	out.println("	CVM_ROM_OBJECT_HDR_INIT0("+charArrayClassBlockName+
	    ","+hashCode+"),");
	out.println("	"+n+", {");
	//
	// finally write the char array.
	int mod = 0;
	for (int i = 0; i < n; i++) { 
	    if (mod == 0) 
	        out.write('\t');
	    out.printHexInt(v[i]);
	    out.write(',');
	    if (++mod == 12) { 
	        out.write('\n');
		mod = 0;
	    }
	} 
	if (mod != 0)
	    out.write('\n');
	out.print(" } };\n\n");

	return true;

    }

    private final static int maxCom = 20;
    private static byte cbuf[] = new byte[maxCom + 3];
    private void commentary( String s, CCodeWriter out ){
	int i;
	int m = Math.min( maxCom, s.length() );
	for ( i = 0; i < m; i++ ){
	    char c = s.charAt(i);
	    if ( ' ' <= c && c <= '~' ) {
		if (c == '*') {
		    /*
		     * Prevent embedded comments by escaping * with \
		     */
		    cbuf[i++] = (byte)'\\';
		}
		cbuf[i] = (byte) c;
	    } else {
		cbuf[i] = (byte)'?';
	    }
	}
	out.write( out.commentLeader, 0, out.commentLeader.length );
	out.write( cbuf, 0, m );
	if ( m < s.length() ){
	    out.write('.');
	    out.write('.');
	    out.write('.');
	}
	out.write( out.commentTrailer, 0, out.commentTrailer.length );
    }

    public int writeStrings(CCodeWriter out, String tableName ) {
	String dataName = tableName+"_data";

	ClassInfo charArrayClass = ClassInfo.lookupClass("[C");
	if ( charArrayClass == null ){
	    System.err.println(Localizer.getString("javacodecompact.cannot_find_array_of_char"));
	    return 0;
	}

	if ( ! writeStringData( dataName, ((CVMClass)(charArrayClass.vmClass)).getNativeName()+"_Classblock" , out ) )
	    return 0;

	out.println("\n#define CVM_INIT_JAVA_STRING(hashCode,offset,count)\\\n"+
	    "    CVM_ROM_STRING_INIT({(CVMArrayOfChar*)&"+dataName+"},"+
	    "offset,count,hashCode)");

	int offset;
	out.println("STATIC const struct java_lang_String " + tableName + "[] = {");
	offset = 0;

	// unfortunately, order matters.
	// unfortunately, the order that matters is not the order
	// available throught the Enumeration.
	// Thus we sort first, output second.
	StringConstant stringTable[] = new StringConstant[ internedStringCount() ];

	for (Enumeration e = allStrings(); e.hasMoreElements(); offset++) { 
	    StringConstant s = (StringConstant)e.nextElement();
	    stringTable[ s.unicodeIndex ] = s;
	}
	internTable = new CVMStringIntern( offset );
	for ( int i = 0; i < offset; i++ ){
	    StringConstant s = stringTable[i];
	    String string = s.str.string;
	    int hashCode = 0;
	    out.print("\tCVM_INIT_JAVA_STRING(");
	    out.printHexInt(hashCode);
	    out.write(',');
	    out.printHexInt(s.unicodeOffset);
	    out.write(',');
	    out.printHexInt(string.length());
	    out.print("),");
	    commentary(string, out);
	    internTable.internStringConstant( s );
	}
	out.println("};\n\n");

	return offset;
    }

    public void writeStringInternTable( CCodeWriter out, String tableName, String stringArrayName ){
	internTable.writeInternTable( out, tableName, stringArrayName );
    }
}



class CVMInternSegment {
    int 	     capacity;
    int		     content = 0;
    StringConstant   data[];

    private static final int primeFactors[] = {
	3, 5, 7, 11, 13, 37 };
    private static final int nPrimeFactors = primeFactors.length;

    CVMInternSegment( int size ){
	/*
	 * apply our usual emptyness parameter.
	 * then make sure its relatively prime to
	 * the secondary hash h1, where 1 <= h1 <= 16
	 * and also to 37. This means testing against
	 * 2, 3, 5, 7, 11, 13, and 37.
	 * This loop WILL terminate, because eventually
	 * we will hit either a relative prime to all these
	 * factors, or an actual prime.
	 */
	size = ( size*100 ) / 65; 
	size |= 1; /* not a multiple of 2 */
	boolean changed = true;
	while ( changed ) {
	    changed = false;
	    for ( int i = 0; i < nPrimeFactors; i++ ){
		if (size%primeFactors[i] == 0 ){
		    size += 2;
		    changed = true;
		    break; /* out of for loop */
		}
	    }
	}
	capacity = size;
	data = new StringConstant[ size ];
    }

    void writeSegment(
	String segmentName,
	String stringArrayName,
	String link,
	CCodeWriter out )
    {
	//
	// first typedef the array we're about to create
	out.print("const struct { CVM_INTERN_SEGMENT_HEADER\n\tCVMStringICell data[");
	out.print( capacity );
	out.print( "];\n\tCVMUint8	refCount[" );
	out.print( capacity );
	out.print( "]; } " );
	out.print( segmentName );
	out.print(" = {\n    &" );
	out.print(link);
	out.print(", ");
	out.print(capacity);
	out.print(", ");
	out.print(content);
	out.println(", 1, {");

	//
	// write the String reference array.
	for (int i = 0; i < capacity; i++) { 
	    StringConstant sc = data[i];
	    if ( sc == null ){
		out.println("    {0},");
	    } else {
		out.println("    {(CVMObject*)&"+stringArrayName+"["+sc.unicodeIndex+"]},");
	    }
	}
	out.print("}, {\n    ");
	// now the reference counts: all either unused or sticky
	int perLine = 0;
	for (int i = 0; i < capacity; i++) { 
	    StringConstant sc = data[i];
	    if ( sc == null ){
		out.print("StrU, ");
	    } else {
		out.print("StrS, ");
	    }
	    if (++perLine > 8 ){
		out.print("\n    ");
		perLine = 0;
	    }
	}
	out.println("\n}};");
    }

}

class CVMStringIntern{

    CVMInternSegment internTable;

    CVMStringIntern( int maxSize ){
	internTable = new CVMInternSegment( maxSize );
    }

    StringConstant internStringConstant( StringConstant val ){
	String s = val.str.string;
	CVMInternSegment curseg = internTable;

	int h = 0, n;
	// Hash function.
	// Identical to the one in the runtime interning code.
	n = Math.min( s.length(), 16 );
	for ( int i = 0; i < n; i++ ){
	    h = (h*37) + s.charAt(i);
	}


	int h1 = (h&0x7fffffff) % curseg.capacity;
	int h2 = (h&15)+1;
	int i = h1;
	StringConstant candidate;

	while ( (candidate=curseg.data[i]) != null ){
	    if ( val.equals( candidate ) ){
		return candidate;
	    }
	    i += h2;
	    if ( i >= curseg.capacity ){
		i -= curseg.capacity;
	    }
	}
	// not found in this table.
	// insert at current location.
	curseg.data[i] = val;
	curseg.content += 1;
	return val;
    }

    public void writeInternTable(
	CCodeWriter out,
	String tableName,
	String stringArrayName )
    {
	/* the StringIntern class has no EMV equivalent -- it becomes
	 * part of java.lang.String. Only the InternSegment exists.
	 * Go write it out.
	 */
	String linkname;
	// Note : define more compact representation of
	// CVMInternUnused and CVMInternSticky to reduce our
	// output volumn
	out.print("#undef StrS\n#undef StrU\n");
	out.print("#define StrS	CVMInternSticky\n");
	out.print("#define StrU	CVMInternUnused\n");
	//
	// write out the indirect next cell, and see that it gets
	// initialized.
	linkname = tableName+"NextCell";
	out.println("struct CVMInternSegment * "+linkname+";");
	CVMInitInfo.addInfo("NULL", "&"+linkname, "sizeof "+linkname );
	internTable.writeSegment( tableName, stringArrayName, linkname, out );
    }
}
