/*
 * @(#)AccessController.c	1.8 00/06/15
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#include "javavm/include/interpreter.h"
#include "javavm/include/directmem.h"
#include "javavm/include/indirectmem.h"
#include "javavm/include/globalroots.h"
#include "generated/offsets/java_lang_Thread.h"

CNIResultCode
CNIjava_security_AccessController_pushContext(CVMExecEnv* ee,
    CVMStackVal32 *arguments, CVMMethodBlock **p_mb)
{
    CVMObjectICell *context = &arguments[0].j.r;
    CVMPrivilegeItem *p;
    CVMBool success = CVM_FALSE;
    CVMD_gcSafeExec(ee, {
	p = (CVMPrivilegeItem *)malloc(sizeof *p);
	if (p != NULL) {
	    p->context = CVMID_getGlobalRoot(ee);
	    if (p->context != NULL) {
		success = CVM_TRUE;
	    } else {
		free(p);
	    }
	}
    });
    if (success) {
	/* NOTE: this method only works if called from doPrivileged0 */
	CVMFrame *frame = CVMeeGetCurrentFrame(ee);
	p->frame = frame;
	p->clazz = CVMmbClassBlock(frame->mb);
	p->next = ee->privilege;
	CVMID_icellAssignDirect(ee, p->context, context);
	ee->privilege = p;
    }
    arguments[0].j.i = success;
    return CNI_SINGLE;
}

CNIResultCode
CNIjava_security_AccessController_popContext(CVMExecEnv* ee,
    CVMStackVal32 *arguments, CVMMethodBlock **p_mb)
{
    CVMPrivilegeItem *p = ee->privilege;
    CVMassert(p != NULL);
    CVMassert(p->frame == CVMeeGetCurrentFrame(ee));
    ee->privilege = p->next;
    CVMD_gcSafeExec(ee, {
	CVMID_freeGlobalRoot(ee, p->context);
	free(p);
    });
    return CNI_VOID;
}

CNIResultCode
CNIjava_security_AccessController_computeContext(CVMExecEnv* ee,
    CVMStackVal32 *arguments, CVMMethodBlock **p_mb)
{
    CVMInt32 n = 0;
    CVMArrayOfBooleanICell *isPrivilegedRef =
	(CVMArrayOfBooleanICell *)&arguments[0].j.r;
    CVMArrayOfRefICell *contextRef =
	(CVMArrayOfRefICell *)&arguments[1].j.r;
    CVMObjectICell *privContext = NULL;
    CVMObjectICell *pd, *prevPD = NULL;
    CVMFrame *frame = CVMeeGetCurrentFrame(ee);
    CVMBool isPrivileged = CVM_FALSE;

    /* CNI policy: offer a gc-safe checkpoint */
    CVMD_gcSafeCheckPoint(ee, {}, {});

    do {
	if (ee->privilege && ee->privilege->frame == frame) {
	    isPrivileged = CVM_TRUE;
	    privContext = ee->privilege->context;
	    pd = CVMcbProtectionDomain(ee->privilege->clazz);
	} else {
	    CVMMethodBlock *mb = frame->mb;
	    CVMClassBlock *cb = (mb == NULL) ? NULL : CVMmbClassBlock(mb);
	    if (cb != NULL) {
		pd = CVMcbProtectionDomain(cb);
	    } else {
		pd = NULL;
	    }
	}

	if (pd != NULL) {
	    CVMBool isSamePD;
	    if (prevPD == NULL) {
		isSamePD = CVM_FALSE;
	    } else {
		CVMObject *prevPDObject = CVMID_icellDirect(ee, prevPD);
		CVMObject *pdObject = CVMID_icellDirect(ee, pd);
		isSamePD = (prevPDObject == pdObject);
	    }
	    if (!isSamePD) {
		++n;
		prevPD = pd;
	    }
	}
	if (isPrivileged) {
	    break;
	}
	frame = CVMframePrev(frame);
    } while (frame != NULL);
    {
	CVMArrayOfBoolean *isPrivArr = CVMID_icellDirect(ee, isPrivilegedRef);
	CVMArrayOfRef *ctxArr = CVMID_icellDirect(ee, contextRef);
	CVMObject *ctx = (privContext == NULL) ? NULL :
	    CVMID_icellDirect(ee, privContext);

	CVMD_arrayWriteRef(ctxArr, 0, ctx);
	CVMD_arrayWriteBoolean(isPrivArr, 0, isPrivileged);
    }
    arguments[0].j.i = n;
    return CNI_SINGLE;
}


CNIResultCode
CNIjava_security_AccessController_fillInContext(CVMExecEnv* ee,
    CVMStackVal32 *arguments, CVMMethodBlock **p_mb)
{
    CVMArrayOfRefICell *context =
	(CVMArrayOfRefICell *)&arguments[0].j.r;
    CVMJavaInt count = arguments[1].j.i;
    CVMFrame *frame = CVMeeGetCurrentFrame(ee);
    CVMBool isPrivileged = CVM_FALSE;
    CVMObjectICell *prevPD = NULL;
    CVMInt32 i = 0;

    /* CNI policy: offer a gc-safe checkpoint */
    CVMD_gcSafeCheckPoint(ee, {}, {});

    while (i < count) {
	CVMObjectICell *pd;
	if (ee->privilege && ee->privilege->frame == frame) {
	    isPrivileged = CVM_TRUE;
	    pd = CVMcbProtectionDomain(ee->privilege->clazz);
	} else {
	    CVMMethodBlock *mb = frame->mb;
	    CVMClassBlock *cb = (mb == NULL) ? NULL : CVMmbClassBlock(mb);
	    if (cb != NULL) {
		pd = CVMcbProtectionDomain(cb);
	    } else {
		pd = NULL;
	    }
	}

	if (pd != NULL) {
	    CVMBool isSamePD;
	    if (prevPD == NULL) {
		isSamePD = CVM_FALSE;
	    } else {
		CVMObject *prevPDObject = CVMID_icellDirect(ee, prevPD);
		CVMObject *pdObject = CVMID_icellDirect(ee, pd);
		isSamePD = (prevPDObject == pdObject);
	    }
	    if (!isSamePD) {
		CVMArrayOfRef *ctx = CVMID_icellDirect(ee, context);
		CVMObject *pdObject = CVMID_icellDirect(ee, pd);
		CVMD_arrayWriteRef(ctx, i, pdObject);
		++i;
		prevPD = pd;
	    }
	}
	if (isPrivileged) {
	    break;
	}
	frame = CVMframePrev(frame);
	if (frame == NULL) {
	    break;
	}
    }
    CVMassert(i == count);
    return CNI_VOID;
}

CNIResultCode
CNIjava_security_AccessController_getInheritedAccessControlContext(
    CVMExecEnv* ee, CVMStackVal32 *arguments, CVMMethodBlock **p_mb)
{
    CVMThreadICell *threadRef = CVMcurrentThreadICell(ee);

    /* CNI policy: offer a gc-safe checkpoint */
    CVMD_gcSafeCheckPoint(ee, {}, {});

    {
	CVMObject *thread = CVMID_icellDirect(ee, threadRef);
	CVMObject *obj;

	CVMD_fieldReadRef(thread,
	    CVMoffsetOfjava_lang_Thread_inheritedAccessControlContext,
	    obj);
	CVMID_icellSetDirect(ee, &arguments[0].j.r, obj);
	return CNI_SINGLE;
    }
}
