/*
 * @(#)ClassDictionary.java	1.2 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package dependenceAnalyzer;
import util.*;
import java.io.FilenameFilter;
import java.util.*;
import java.io.InputStream;
import components.ClassInfo;

/*
 * A ClassDictionary is used by the dependence analyser
 * to encapsulate all class name lookup operations. Thus
 * it contains both a hash table and a ClassFileFinder,
 * to lookup, respectively, classes we've already read, and
 * those not yet read. We have also bundled in a FilenameFilter,
 * to know when a class should be excluded rather than found
 * and read in.
 *
 * Note that this is not a public class: it is only for use
 * within the dependenceAnalyzer package.
 */

class ClassDictionary {
    /*
     * public interfaces.
     */
    
    public
    ClassDictionary( ClassFileFinder find, FilenameFilter filt ){
	finder = find;
	filter = filt;
	cdict  = new Hashtable();
    }

    public ClassEntry lookup( String cname ){
	return (ClassEntry)(cdict.get( cname ) );
    }

    public ClassEntry lookupAdd( String cname ){
	ClassEntry ce = (ClassEntry)(cdict.get( cname ) );
	if ( ce == null ){
	    ce = new ClassEntry( cname );
	    if ( filter.accept( null, cname ) ){
		ce.flags |= ClassEntry.EXCLUDED;
	    }
	    cdict.put( cname, ce );
	}
	return ce;
    }

    public Enumeration elements(){
	return cdict.elements();
    }

    /*
     * and as a public service to ClassEntry...
     */
    ClassInfo findClassInfo( String cname ){
	InputStream fin = finder.findClassFile( cname );
	if ( fin == null ) return null;
	ClassFile f = new ClassFile( cname, fin, false );
	if ( ! f.readClassFile( null )){
	    f.dump(System.err);
	    return null;
	}
	return f.clas;
    }

    /*
     * private state parts.
     */
    private ClassFileFinder finder;
    private FilenameFilter  filter;
    private Hashtable       cdict;
}
