/*
 * @(#)MemberName.java	1.2 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

/*
 * A member name has 3 parts:
 * 1) a pointer to the ClassEntry for the class holding the member.
 * 2) the member name
 * 3) a method's signature.
 * The last two are concatenated into the name field. There is
 * apparently little value in separating them.
 *
 * The only value to a signature is to resolve overloading.
 * Since data fields do not overload, we do not carry a signature
 * with them. And since even methods only overload based on parameter
 * list, we do not need to carry the return-type part of their signatures.
 *
 * We construct these for ease of comparison, as we will often
 * be doing lookup operations.
 */

package dependenceAnalyzer;
import util.Localizer;

public class MemberName implements Cloneable {
    public ClassEntry	classEntry;
    public String	name; // and type signature

    public MemberName( ClassEntry c, String nm ){
	classEntry = c;
	name = nm.intern();
    }

    public boolean equals ( Object other ){
	try {
	    MemberName otherName = (MemberName)other;
	    return (
		(this.classEntry == otherName.classEntry) && 
		(this.name == otherName.name) );
	} catch ( ClassCastException e ){
	    return false;
	}
    }

    public int hashCode(){
	return name.hashCode();
    }

    public Object clone(){ 
	try {
	    MemberName newName = (MemberName) super.clone();
	    return newName;
	}catch( CloneNotSupportedException e ){
	    e.printStackTrace();
	    throw new Error(Localizer.getString("membername.could_not_clone"));
	}
    }

    public String toString(){
	String result;
	if ( classEntry == null ){
	    result = "<noclass>";
	} else {
	    result = classEntry.name().toString();
	}
	result += "."+name;
	return result;
    }
}
