/*
 * @(#)CInterfaceMethodTable.java	1.3 00/11/09
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package runtime;
import vm.*;
import components.*;
import Const;
import java.util.Hashtable;

/*
 * This is the C-specific subclass of vm.InterfaceMethodTable
 * It knowns how to write out these data structures in C
 */

class CInterfaceMethodTable extends vm.InterfaceMethodTable {

    private static vm.InterfaceMethodFactory imf = 
        new vm.InterfaceMethodFactory() { 
	    public  InterfaceMethodTable 
	    newInterfaceMethodTable(ClassClass c, String n, 
				    InterfaceVector v[] ){
	        return new CInterfaceMethodTable( c, n, v );
	    }
        };
  
    public
    CInterfaceMethodTable( ClassClass c, String n, InterfaceVector v[] ){
	super( c, n, v );
    }

    static void writeInterfaceTables(ClassClass[] classes, 
				     CWriter writer, CCodeWriter out) {
        CWriter.Alias ivAlias = writer.new Alias("ROMInterfaceVectors");

        int n = classes.length;
	// generate the tables
	for ( int i = 0; i < n; i++ ){
	    ClassClass c = classes[i];
	    c.inf = vm.InterfaceMethodTable.generateInterfaceTable(c, imf );
	}

	// print the vectors
	out.println( "static const unsigned long " + ivAlias.name() + "[] = {");
	int offset = 0;
	Hashtable hash = new Hashtable();
	for (int i = 0; i < n; i++) { 
	    ClassClass c = classes[i];
	    CInterfaceMethodTable simt = (CInterfaceMethodTable)c.inf;
	    if (simt == null || simt.parent != c) 
	        continue;
	    InterfaceVector iv[] = simt.iv;
	    for (int j = 0; j < iv.length; j++) { 
	        InterfaceVector vec = iv[j];
		if ( !needVector( vec ) ) 
		    continue;
		if (vec.parent != simt.parent ) 
		    continue;
		short vector[] = vec.v;
		int num = vector.length;
		int t_offset = 
		       (num == 1) ? writer.checkDuplication(hash, offset, 
							    vector[0])
		     : (num == 2) ? writer.checkDuplication(hash, offset,
							   vector[0], vector[1])
		     : offset;
		out.comment(simt.parent.ci.className + 
			        " " + vec.intf.className + 
			        " (" + t_offset  + ")" );
		// vectorName is an artifact.  We really should be using
		// new ClassPair(simt.parent.ci, vec.intf) as the key.
		String vectorName = simt.parent.ci.className + "." + vec.intf.className;
		ivAlias.define(vectorName, t_offset);
		if (t_offset != offset)
		    continue;
		for (int k = 0, mod = 0; k < num; k++) { 
		    if (mod == 0) out.write('\t');
		    out.printHexInt( vector[k] );
		    out.write(',');
		    if (++mod == 8) { 
		        out.write('\n'); mod = 0;
		    }
		}
		if ((num % 8) != 0) 
		  out.write('\n');
		offset += num;
	    }
	}
	out.println("};\n\n");
	
	for (int i = 0; i < n; i++) { 
	    ClassClass c = classes[i];
	    CInterfaceMethodTable simt = (CInterfaceMethodTable)c.inf;
	    if ( simt != null ){
	        simt.write(writer, out, c, ivAlias);
	    }
	}
    }

    private static boolean
    needVector( InterfaceVector vec ){
	//if ( vec == null ) return false; // nothing to do.
	if ( vec.v == null || vec.v.length == 0 ) return false; // don't bother.
	return true;
    }
	

    public void
    write(CWriter writer, CCodeWriter out, ClassClass c, CWriter.Alias ivAlias){
        if (this.parent != c) 
	    return;
	boolean tableEmpty = iv.length == 0;
	// struct imethodtable { 
	//     int icount;
	//     struct { 
	// 	ClassClass *classdescriptor;
	// 	unsigned long *offsets;
	//     } itable[1];
	// }
	out.print("static const struct { int icount; struct { const ClassClass *classdescriptor; const unsigned long *offsets; } itable["+
		  (tableEmpty ? 1 : iv.length) +
		  "];\n\t} ");
	out.print( name );
	out.print(" = {\n\t");
	out.print( iv.length );
	out.print(", {\n");
	for( int i = 0; i < iv.length; i++ ){
	    InterfaceVector entry = iv[i];
	    ClassInfo intf = entry.intf;
	    String intfName = intf.getGenericNativeName();
	    out.print("\t{ ");
	    writer.classAlias.write(intf, out);
	    out.write(',');
	    if ( needVector( entry ) ){
		String vectorName = entry.parent.ci.className + "."  
                                   + intf.className;
		ivAlias.write(vectorName, out);
	    } else {
		out.write('0');
	    }
	    out.print(" },\n");
	}
	if (tableEmpty) {
	    out.println("\t{ NULL, 0 }");
	}
	out.print("} };\n\n");
    }
}
