/*
 * @(#)CStringTable.java	1.2 00/05/19
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

package runtime;
import java.util.Enumeration;
import Const;
import components.*;
import java.io.PrintStream;

public class CStringTable extends vm.StringTable {

    private final static String dataName = "unicodeData";
    private final static String dataHandleName = "unicodeHandle";

    private final static int maxCom = 20;
    private static byte cbuf[] = new byte[maxCom + 3];
    private void commentary( String s, CCodeWriter out ){
	int i;
	int m = Math.min( maxCom, s.length() );
	for ( i = 0; i < m; i++ ){
	    char c = s.charAt(i);
	    if ( ' ' <= c && c <= '~' ){
		cbuf[i] = (byte) c;
	    } else
		break;
	}
	out.write( out.commentLeader, 0, out.commentLeader.length );
	out.write( cbuf, 0, m );
	if ( m < s.length() ){
	    out.write('.');
	    out.write('.');
	    out.write('.');
	}
	out.write('\n');
	
    }
    final void writeStringData( CCodeWriter out ){
	int n = arrangeStringData();
	char v[] = new char[n];
	data.getChars( 0, n, v, 0 );

	//
	// first write the char array.
	out.print("static const short ");
	out.print( dataName );
	out.print("["+n+"] = {\n");
	int mod = 0;
	for (int i = 0; i < n; i++) { 
	    if (mod == 0) 
	        out.write('\t');
	    out.printHexInt(v[i]);
	    out.write(',');
	    if (++mod == 12) { 
	        out.write('\n');
		mod = 0;
	    }
	} 
	if (mod != 0)
	    out.write('\n');
	out.print(" };\n\n");

	//
	// then write the handle for that array.
	out.print("static const HArrayOfChar ");
	out.print( dataHandleName );
	out.print(" = {\n\t(ArrayOfChar*)");
	out.print( dataName );
	out.print(", (struct methodtable *)");
	out.printHexInt( (n<<5) + Const.OBJ_FLAG_CHAR_ARRAY );
	out.print(" };\n\n");
    }



    public void writeStrings(CCodeWriter out, CWriter writer ) {
	writeStringData( out );
	int offset;
	CWriter.Alias stringAlias = writer.stringAlias;
	String tableName = stringAlias.name();

	out.println("typedef struct { Hjava_lang_String handle; Classjava_lang_String body; } StringInfoType;\n");


	out.println("#define STRING(i, offset, count ) \\");
	out.println("   { { (Classjava_lang_String *)(&"+tableName+"[i].body),\\");
	out.println("       (struct methodtable *)java_lang_StringMethodtable},\\"); 
	out.println("     { (HArrayOfChar*)&unicodeHandle, offset, count } }\n");

	out.println("static const StringInfoType " + tableName + "[] = {");
	offset = 0;

	for (Enumeration e = allStrings(); e.hasMoreElements(); offset++) { 
	    StringConstant s = (StringConstant)e.nextElement();
	    String string = s.str.string;
	    stringAlias.define(s, offset);
	    out.print("\tSTRING(");
	    out.print(offset);
	    out.write(',');
	    out.printHexInt(s.unicodeOffset);
	    out.write(',');
	    out.printHexInt(string.length());
	    out.print("),");
	    CConstants.stringComment(string, out);
	    out.println();
	}
	writer.njavastrings += offset;
	out.println("};\n\n");
    }
}
