/*
 * @(#)FileOutputStream.c	1.12 00/08/25
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */

#include "jni.h"
#include "jni_util.h"
#include "jvm.h"

#include "io_util.h"
#include "java_io_FileOutputStream.h"

/*******************************************************************/
/*  BEGIN JNI ********* BEGIN JNI *********** BEGIN JNI ************/
/*******************************************************************/

jfieldID fos_fd; /* id for jobject 'fd' in java.io.FileOutputStream */

/**************************************************************
 * static methods to store field ID's in initializers
 */

JNIEXPORT void JNICALL 
Java_java_io_FileOutputStream_initIDs(JNIEnv *env, jclass fdClass) {
    fos_fd = (*env)->GetFieldID(env, fdClass, "fd", "Ljava/io/FileDescriptor;");
}

/**************************************************************
 * Output stream
 */

JNIEXPORT void JNICALL
Java_java_io_FileOutputStream_open(JNIEnv *env, jobject this, jstring path) {
    fileOpen(env, this, path, fos_fd, O_WRONLY | O_CREAT | O_TRUNC);
}

JNIEXPORT void JNICALL
Java_java_io_FileOutputStream_openAppend(JNIEnv *env, jobject this, jstring path) {
    fileOpen(env, this, path, fos_fd, O_WRONLY | O_CREAT | O_APPEND);
}

JNIEXPORT void JNICALL
Java_java_io_FileOutputStream_write(JNIEnv *env, jobject this, jint byte) {
    writeSingle(env, this, byte, fos_fd);
}

JNIEXPORT void JNICALL
Java_java_io_FileOutputStream_writeBytes(JNIEnv *env,
    jobject this, jbyteArray bytes, jint off, jint len) {
    writeBytes(env, this, bytes, off, len, fos_fd);
#if 0
    /* %comment hinkmond001 */
    CVMExecEnv *ee = CVMjniEnv2ExecEnv(env);
#   define BUFSIZE 128
    char cbuf[BUFSIZE+1], *buf;

    if (len > BUFSIZE) {
	buf = (char *)malloc(len + 1);
    } else {
	buf = cbuf;
    }
    CVMD_gcUnsafeExec(ee, {
	int i = 0;
	CVMArrayOfByte* arrayobj =
	    (CVMArrayOfByte*)CVMID_icellDirect(ee, bytes);
	while (i < len) {
	    size_t l = (len - i) > BUFSIZE ? BUFSIZE : (len - i);
	    /* %comment hinkmond002 */
	    memcpy(&buf[i], &arrayobj->elems[off + i], l);
	    i += l;
	}
	buf[len] = '\0';
    });
    CVMconsolePrintf("%s", buf);
    if (buf != cbuf) {
	free(buf);
    }
#endif
}

JNIEXPORT void JNICALL
Java_java_io_FileOutputStream_close(JNIEnv *env, jobject this) {
    int fd = GET_FD(fos_fd);
    SET_FD(-1, fos_fd);
    JVM_Close(fd);
}
